@@ -27,19 +27,23 @@
  *      Maros Barabas <mbarabas@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libxml/xmlreader.h>
 #include <libxml/xmlwriter.h>
 #include <libxml/tree.h>
 #include <string.h>
 
-#include "cpelang_priv.h"
 #include "public/cpelang.h"
+#include "cpelang_priv.h"
 
-#include "../common/util.h"
-#include "../common/list.h"
-#include "../common/text_priv.h"
-#include "../common/elements.h"
-#include "../_error.h"
+#include "common/util.h"
+#include "common/list.h"
+#include "common/text_priv.h"
+#include "common/elements.h"
+#include "common/_error.h"
 
 /***************************************************************************/
 /* Variable definitions
@@ -181,6 +185,15 @@ struct cpe_testexpr *cpe_testexpr_new()
 	return ret;
 }
 
+struct cpe_testexpr * cpe_testexpr_clone(struct cpe_testexpr * old_expr)
+{
+        struct cpe_testexpr * new_expr = cpe_testexpr_new();
+        new_expr->oper = old_expr->oper;
+        new_expr->meta.cpe = cpe_name_clone(old_expr->meta.cpe);
+        new_expr->meta.expr = oscap_list_clone(old_expr->meta.expr, (oscap_clone_func) cpe_testexpr_clone);
+        return new_expr;
+}
+
 struct cpe_lang_model *cpe_lang_model_new()
 {
 
