@@ -38,7 +38,7 @@ OSCAP_HIDDEN_START;
 
 #define OVAL_ENUMERATION_INVALID (-1)
 
-#define OVAL_DEFAULT_VERSION "5.5"
+#define OVAL_SUPPORTED "5.8"
 
 #define OVAL_COMMON_NAMESPACE      BAD_CAST "http://oval.mitre.org/XMLSchema/oval-common-5"
 #define OVAL_SYSCHAR_NAMESPACE     BAD_CAST "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5"
