@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2011 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -26,13 +26,18 @@
  * Authors:
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  *      "Peter Vrabec" <pvabec@redhat.com>
+ *      "Tomas Heinrich" <theinric@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
-#include "config.h"
+
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
@@ -119,7 +124,8 @@ typedef struct oval_component_OBJECTREF {
 	struct oval_definition_model *model;
 	oval_component_type_t type;
 	struct oval_object *object;	/*type==OVAL_COMPONENT_OBJECTREF */
-	char *object_field;	/*type==OVAL_COMPONENT_OBJECTREF */
+	char *item_field;
+	char *record_field;
 } oval_component_OBJECTREF_t;
 
 typedef struct oval_component_VARREF {
@@ -256,26 +262,20 @@ struct oval_value *oval_component_get_literal_value(struct oval_component *compo
 
 void oval_component_set_literal_value(struct oval_component *component, struct oval_value *value) {
 	/* type == OVAL_COMPONENT_LITERAL */
-	if (component && !oval_component_is_locked(component)) {
-		if (oval_component_get_type(component) == OVAL_COMPONENT_LITERAL) {
-			((struct oval_component_LITERAL *)component)->value = value;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(component);
+	if (oval_component_get_type(component) == OVAL_COMPONENT_LITERAL) {
+		((struct oval_component_LITERAL *)component)->value = value;
+	}
 }
 
 void oval_component_set_type(struct oval_component *component, oval_component_type_t type)
 {
-
-	if (component && !oval_component_is_locked(component)) {
-		component->type = type;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(component);
+	component->type = type;
 }
 
 struct oval_object *oval_component_get_object(struct oval_component *component)
 {
-	/* type == OVAL_COMPONENT_OBJECTREF */
 	struct oval_object *object = NULL;
 	if (oval_component_get_type(component) == OVAL_COMPONENT_OBJECTREF) {
 		object = ((struct oval_component_OBJECTREF *)component)->object;
@@ -284,32 +284,43 @@ struct oval_object *oval_component_get_object(struct oval_component *component)
 }
 
 void oval_component_set_object(struct oval_component *component, struct oval_object *object) {
-	/* type == OVAL_COMPONENT_OBJECTREF */
-	if (component && !oval_component_is_locked(component)) {
-		if (oval_component_get_type(component) == OVAL_COMPONENT_OBJECTREF) {
-			((struct oval_component_OBJECTREF *)component)->object = object;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(component);
+	if (oval_component_get_type(component) == OVAL_COMPONENT_OBJECTREF) {
+		((struct oval_component_OBJECTREF *)component)->object = object;
+	}
 }
 
-char *oval_component_get_object_field(struct oval_component *component) {
-	/* type == OVAL_COMPONENT_OBJECTREF */
+char *oval_component_get_item_field(struct oval_component *component) {
 	char *field = NULL;
 	if (oval_component_get_type(component) == OVAL_COMPONENT_OBJECTREF) {
-		field = ((struct oval_component_OBJECTREF *)component)->object_field;
+		field = ((struct oval_component_OBJECTREF *)component)->item_field;
 	}
 	return field;
 }
 
-void oval_component_set_object_field(struct oval_component *component, char *field) {
-	if (component && !oval_component_is_locked(component)) {
-		/* type == OVAL_COMPONENT_OBJECTREF */
-		if (oval_component_get_type(component) == OVAL_COMPONENT_OBJECTREF) {
-			((struct oval_component_OBJECTREF *)component)->object_field = oscap_strdup(field);
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_component_set_item_field(struct oval_component *component, char *field) {
+	__attribute__nonnull__(component);
+	if (oval_component_get_type(component) == OVAL_COMPONENT_OBJECTREF) {
+		((struct oval_component_OBJECTREF *)component)->item_field = oscap_strdup(field);
+	}
+}
+
+char *oval_component_get_record_field(struct oval_component *component) {
+	if (oval_component_get_type(component) != OVAL_COMPONENT_OBJECTREF)
+		return NULL;
+
+	return ((struct oval_component_OBJECTREF *) component)->record_field;
+}
+
+void oval_component_set_record_field(struct oval_component *component, char *field) {
+	__attribute__nonnull__(component);
+
+	if (oval_component_get_type(component) != OVAL_COMPONENT_OBJECTREF) {
+		dW("Wrong component type: %d.\n", oval_component_get_type(component));
+		return;
+	}
+
+	((struct oval_component_OBJECTREF *) component)->record_field = oscap_strdup(field);
 }
 
 struct oval_variable *oval_component_get_variable(struct oval_component *component) {
@@ -352,15 +363,15 @@ oval_arithmetic_operation_t oval_component_get_arithmetic_operation(struct oval_
 	return operation;
 }
 
-void oval_component_set_arithmetic_operation(struct oval_component *component, oval_arithmetic_operation_t operation) {
-	if (component && !oval_component_is_locked(component)) {
-		/* type == OVAL_COMPONENT_ARITHMETIC */
-		if (component->type == OVAL_FUNCTION_ARITHMETIC) {
-			oval_component_ARITHMETIC_t *arithmetic = (oval_component_ARITHMETIC_t *) component;
-			arithmetic->operation = operation;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_component_set_arithmetic_operation(struct oval_component *component, oval_arithmetic_operation_t operation) 
+{
+	__attribute__nonnull__(component);
+	
+	/* type == OVAL_COMPONENT_ARITHMETIC */
+	if (component->type == OVAL_FUNCTION_ARITHMETIC) {
+		oval_component_ARITHMETIC_t *arithmetic = (oval_component_ARITHMETIC_t *) component;
+		arithmetic->operation = operation;
+	}
 }
 
 char *oval_component_get_prefix(struct oval_component *component)
@@ -377,15 +388,15 @@ char *oval_component_get_prefix(struct oval_component *component)
 	return character;
 }
 
-void oval_component_set_prefix(struct oval_component *component, char *character) {
-	if (component && !oval_component_is_locked(component)) {
-		/* type == OVAL_COMPONENT_BEGIN */
-		if (component->type == OVAL_FUNCTION_BEGIN) {
-			oval_component_BEGEND_t *begin = (oval_component_BEGEND_t *) component;
-			begin->character = oscap_strdup(character);
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_component_set_prefix(struct oval_component *component, char *character) 
+{
+	__attribute__nonnull__(component);
+
+	/* type == OVAL_COMPONENT_BEGIN */
+	if (component->type == OVAL_FUNCTION_BEGIN) {
+		oval_component_BEGEND_t *begin = (oval_component_BEGEND_t *) component;
+		begin->character = oscap_strdup(character);
+	}
 }
 
 char *oval_component_get_suffix(struct oval_component *component)
@@ -452,14 +463,13 @@ int oval_component_get_substring_start(struct oval_component *component)
 
 void oval_component_set_substring_start(struct oval_component *component, int start)
 {
-	if (component && !oval_component_is_locked(component)) {
-		/* type==OVAL_COMPONENT_SUBSTRING */
-		if (component->type == OVAL_FUNCTION_SUBSTRING) {
-			oval_component_SUBSTRING_t *substring = (oval_component_SUBSTRING_t *) component;
-			substring->start = start;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(component);
+	
+	/* type==OVAL_COMPONENT_SUBSTRING */
+	if (component->type == OVAL_FUNCTION_SUBSTRING) {
+		oval_component_SUBSTRING_t *substring = (oval_component_SUBSTRING_t *) component;
+		substring->start = start;
+	}
 }
 
 int oval_component_get_substring_length(struct oval_component *component)
@@ -476,15 +486,15 @@ int oval_component_get_substring_length(struct oval_component *component)
 	return length;
 }
 
-void oval_component_set_substring_length(struct oval_component *component, int length) {
-	if (component && !oval_component_is_locked(component)) {
-		/* type == OVAL_COMPONENT_SUBSTRING */
-		if (component->type == OVAL_FUNCTION_SUBSTRING) {
-			oval_component_SUBSTRING_t *substring = (oval_component_SUBSTRING_t *) component;
-			substring->length = length;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_component_set_substring_length(struct oval_component *component, int length) 
+{
+	__attribute__nonnull__(component);
+
+	/* type == OVAL_COMPONENT_SUBSTRING */
+	if (component->type == OVAL_FUNCTION_SUBSTRING) {
+		oval_component_SUBSTRING_t *substring = (oval_component_SUBSTRING_t *) component;
+		substring->length = length;
+	}
 }
 
 oval_datetime_format_t oval_component_get_timedif_format_1(struct oval_component
@@ -502,15 +512,14 @@ oval_datetime_format_t oval_component_get_timedif_format_1(struct oval_component
 	return format;
 }
 
-void oval_component_set_timedif_format_1(struct oval_component *component, oval_datetime_format_t format) {
-	if (component && !oval_component_is_locked(component)) {
-		/* type == OVAL_COMPONENT_TIMEDIF */
-		if (component->type == OVAL_FUNCTION_TIMEDIF) {
-			oval_component_TIMEDIF_t *timedif = (oval_component_TIMEDIF_t *) component;
-			timedif->format_1 = format;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_component_set_timedif_format_1(struct oval_component *component, oval_datetime_format_t format) 
+{
+	__attribute__nonnull__(component);
+	/* type == OVAL_COMPONENT_TIMEDIF */
+	if (component->type == OVAL_FUNCTION_TIMEDIF) {
+		oval_component_TIMEDIF_t *timedif = (oval_component_TIMEDIF_t *) component;
+		timedif->format_1 = format;
+	}
 }
 
 oval_datetime_format_t oval_component_get_timedif_format_2(struct oval_component
@@ -528,15 +537,14 @@ oval_datetime_format_t oval_component_get_timedif_format_2(struct oval_component
 	return format;
 }
 
-void oval_component_set_timedif_format_2(struct oval_component *component, oval_datetime_format_t format) {
-	if (component && !oval_component_is_locked(component)) {
-		/* type == OVAL_COMPONENT_TIMEDIF */
-		if (component->type == OVAL_FUNCTION_TIMEDIF) {
-			oval_component_TIMEDIF_t *timedif = (oval_component_TIMEDIF_t *) component;
-			timedif->format_2 = format;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_component_set_timedif_format_2(struct oval_component *component, oval_datetime_format_t format) 
+{
+	__attribute__nonnull__(component);
+	/* type == OVAL_COMPONENT_TIMEDIF */
+	if (component->type == OVAL_FUNCTION_TIMEDIF) {
+		oval_component_TIMEDIF_t *timedif = (oval_component_TIMEDIF_t *) component;
+		timedif->format_2 = format;
+	}
 }
 
 char *oval_component_get_regex_pattern(struct oval_component *component) {
@@ -551,15 +559,15 @@ char *oval_component_get_regex_pattern(struct oval_component *component) {
 	return pattern;
 }
 
-void oval_component_set_regex_pattern(struct oval_component *component, char *pattern) {
-	if (component && !oval_component_is_locked(component)) {
-		/* type == OVAL_COMPONENT_REGEX_CAPTURE */
-		if (component->type == OVAL_FUNCTION_REGEX_CAPTURE) {
-			oval_component_REGEX_CAPTURE_t *regex = (oval_component_REGEX_CAPTURE_t *) component;
-			regex->pattern = oscap_strdup(pattern);
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_component_set_regex_pattern(struct oval_component *component, char *pattern) 
+{
+	__attribute__nonnull__(component);
+
+	/* type == OVAL_COMPONENT_REGEX_CAPTURE */
+	if (component->type == OVAL_FUNCTION_REGEX_CAPTURE) {
+		oval_component_REGEX_CAPTURE_t *regex = (oval_component_REGEX_CAPTURE_t *) component;
+		regex->pattern = oscap_strdup(pattern);
+	}
 }
 
 struct oval_component *oval_component_new(struct oval_definition_model *model, oval_component_type_t type)
@@ -585,7 +593,8 @@ struct oval_component *oval_component_new(struct oval_definition_model *model, o
 
 			component = (oval_component_t *) objectref;
 			objectref->object = NULL;
-			objectref->object_field = NULL;
+			objectref->item_field = NULL;
+			objectref->record_field = NULL;
 		}
 		break;
 	case OVAL_COMPONENT_VARREF:{
@@ -684,19 +693,6 @@ struct oval_component *oval_component_new(struct oval_definition_model *model, o
 	return component;
 }
 
-bool oval_component_is_valid(struct oval_component * component)
-{
-        oscap_dlprintf(DBG_W, "NOOP.\n");
-	return true;		//TODO
-}
-
-bool oval_component_is_locked(struct oval_component * component)
-{
-	__attribute__nonnull__(component);
-
-	return oval_definition_model_is_locked(component->model);
-}
-
 static void _oval_component_clone_subcomponents
     (struct oval_component *old_component, struct oval_component *new_component, struct oval_definition_model *model) {
 	struct oval_component_iterator *subcomps = oval_component_get_function_components(old_component);
@@ -743,14 +739,18 @@ struct oval_component *oval_component_clone(struct oval_definition_model *new_mo
 		}
 		break;
 	case OVAL_COMPONENT_OBJECTREF:{
+			char *field;
 			struct oval_object *old_object = oval_component_get_object(old_component);
 			if (old_object) {
 				struct oval_object *new_object = oval_object_clone(new_model, old_object);
 				oval_component_set_object(new_component, new_object);
 			}
-			char *field = oval_component_get_object_field(old_component);
+			field = oval_component_get_item_field(old_component);
 			if (field)
-				oval_component_set_object_field(new_component, field);
+				oval_component_set_item_field(new_component, field);
+			field = oval_component_get_record_field(old_component);
+			if (field)
+				oval_component_set_record_field(new_component, field);
 		}
 		break;
 	case OVAL_FUNCTION_REGEX_CAPTURE:{
@@ -767,8 +767,7 @@ struct oval_component *oval_component_clone(struct oval_definition_model *new_mo
 		break;
 	case OVAL_FUNCTION_SUBSTRING:{
 			int length = oval_component_get_substring_length(old_component);
-			if (length > 0)
-				oval_component_set_substring_length(new_component, length);
+			oval_component_set_substring_length(new_component, length);
 			int start = oval_component_get_substring_start(old_component);
 			oval_component_set_substring_start(new_component, start);
 		} break;
@@ -812,9 +811,12 @@ void oval_component_free(struct oval_component *component)
 		break;
 	case OVAL_COMPONENT_OBJECTREF:{
 			oval_component_OBJECTREF_t *objectref = (oval_component_OBJECTREF_t *) component;
-			if (objectref->object_field != NULL)
-				oscap_free(objectref->object_field);
-			objectref->object_field = NULL;
+			if (objectref->item_field != NULL)
+				oscap_free(objectref->item_field);
+			objectref->item_field = NULL;
+			if (objectref->record_field != NULL)
+				oscap_free(objectref->record_field);
+			objectref->record_field = NULL;
 		}
 		break;
 	case OVAL_FUNCTION_BEGIN:
@@ -854,27 +856,26 @@ void oval_component_free(struct oval_component *component)
 	oscap_free(component);
 }
 
-void oval_component_add_function_component(struct oval_component *component, struct oval_component *func_component) {
-	if (component && !oval_component_is_locked(component)) {
-		if (component->type > OVAL_FUNCTION) {
-			oval_component_FUNCTION_t *function = (oval_component_FUNCTION_t *) component;
-			if (func_component)
-				oval_collection_add(function->function_components, func_component);
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_component_add_function_component(struct oval_component *component, struct oval_component *func_component) 
+{
+	__attribute__nonnull__(component);
+	
+	if (component->type > OVAL_FUNCTION) {
+		oval_component_FUNCTION_t *function = (oval_component_FUNCTION_t *) component;
+		if (func_component)
+			oval_collection_add(function->function_components, func_component);
+	}
 }
 
 void oval_component_set_variable(struct oval_component *component, struct oval_variable *variable)
 {
-	if (component && !oval_component_is_locked(component)) {
-		/* type == OVAL_COMPONENT_VARREF */
-		if (component->type == OVAL_COMPONENT_VARREF) {
-			oval_component_VARREF_t *varref = (oval_component_VARREF_t *) component;
-			varref->variable = variable;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(component);
+
+	/* type == OVAL_COMPONENT_VARREF */
+	if (component->type == OVAL_COMPONENT_VARREF) {
+		oval_component_VARREF_t *varref = (oval_component_VARREF_t *) component;
+		varref->variable = variable;
+	}
 }
 
 static void oval_value_consume(struct oval_value *value, void *component)
@@ -893,28 +894,32 @@ static int _oval_component_parse_OBJECTREF_tag(xmlTextReaderPtr reader,
 					       struct oval_parser_context *context, struct oval_component *component)
 {
 
-	struct oval_definition_model *model = oval_parser_context_model(context);
+	struct oval_definition_model *model = context->definition_model;
 	char *objref = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "object_ref");
 	struct oval_object *object = oval_object_get_new(model, objref);
+	char *field;
+
 	oscap_free(objref);
 	objref = NULL;
 	oval_component_set_object(component, object);
 
-	char *objfld = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "item_field");
-	oval_component_set_object_field(component, objfld);
-	if (objfld != NULL) {
-		oscap_free(objfld);
-		objfld = NULL;
-	}
+	field = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "item_field");
+	oval_component_set_item_field(component, field);
+	if (field)
+		oscap_free(field);
+	field = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "record_field");
+	oval_component_set_record_field(component, field);
+	if (field)
+		oscap_free(field);
 
-	return 1;
+	return 0;
 }
 
 static int _oval_component_parse_VARREF_tag(xmlTextReaderPtr reader,
 					    struct oval_parser_context *context, struct oval_component *component)
 {
 
-	struct oval_definition_model *model = oval_parser_context_model(context);
+	struct oval_definition_model *model = context->definition_model;
 	char *varref = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "var_ref");
 	struct oval_variable *variable = oval_variable_get_new(model, varref, OVAL_VARIABLE_UNKNOWN);
 	if (varref != NULL) {
@@ -923,7 +928,7 @@ static int _oval_component_parse_VARREF_tag(xmlTextReaderPtr reader,
 	}
 	oval_component_set_variable(component, variable);
 
-	return 1;
+	return 0;
 }
 
 static void oval_subcomp_consume(struct oval_component *subcomp, void *func)
@@ -1090,120 +1095,14 @@ int oval_component_parse_tag(xmlTextReaderPtr reader,
 	}
 	if (component != NULL)
 		(*consumer) (component, user);
-	if (return_code != 1) {
-		oscap_dlprintf(DBG_I, "Parsing of <%s> terminated by an error at line %d.\n", tagname,
-                            xmlTextReaderGetParserLineNumber(reader));
+
+	if (return_code != 0 ) {
+		dW("Parsing of <%s> terminated by an error at line %d.\n", tagname, xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(tagname);
 	return return_code;
 }
 
-static void function_comp_to_print(struct oval_component *component, char *nxtindent)
-{
-
-	int i;
-	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
-	if (oval_component_iterator_has_more(subcomps)) {
-		for (i = 1; oval_component_iterator_has_more(subcomps); i++) {
-			struct oval_component *subcomp = oval_component_iterator_next(subcomps);
-			oval_component_to_print(subcomp, nxtindent, i);
-		}
-	} else
-		oscap_dprintf("%s FUNCTION_COMPONENTS <<NONE BOUND>>\n", nxtindent);
-	oval_component_iterator_free(subcomps);
-}
-
-void oval_component_to_print(struct oval_component *component, char *indent, int idx)
-{
-	char nxtindent[100];
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%sCOMPONENT.", indent);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%sCOMPONENT[%d].", indent, idx);
-
-	oscap_dprintf("%sTYPE(%lx) = %d\n", nxtindent, (unsigned long)component, oval_component_get_type(component));
-	if (oval_component_get_type(component) > OVAL_COMPONENT_FUNCTION) {
-		/* oval_component_FUNCTION_t *function = (oval_component_FUNCTION_t *) component; */
-	}
-	switch (oval_component_get_type(component)) {
-	case OVAL_COMPONENT_LITERAL:{
-			struct oval_value *value = oval_component_get_literal_value(component);
-			if (value == NULL)
-				oscap_dprintf("%sVALUE <<NOT BOUND>>\n", nxtindent);
-			else
-				oval_value_to_print(value, nxtindent, 0);
-		}
-		break;
-	case OVAL_COMPONENT_OBJECTREF:{
-			oscap_dprintf("%sOBJECT_FIELD %s\n", nxtindent, oval_component_get_object_field(component));
-			struct oval_object *object = oval_component_get_object(component);
-			if (object == NULL)
-				oscap_dprintf("%sOBJECT <<NOT BOUND>>\n", nxtindent);
-			else
-				oval_object_to_print(object, nxtindent, 0);
-		}
-		break;
-	case OVAL_COMPONENT_VARREF:{
-			struct oval_variable *variable = oval_component_get_variable(component);
-			if (variable == NULL)
-				oscap_dprintf("%sVARIABLE <<NOT BOUND>>\n", nxtindent);
-			else
-				oval_variable_to_print(variable, nxtindent, 0);
-		}
-		break;
-	case OVAL_FUNCTION_ARITHMETIC:{
-			oscap_dprintf("%sARITHMETIC_OPERATION %d\n", nxtindent,
-				      oval_component_get_arithmetic_operation(component));
-			function_comp_to_print(component, nxtindent);
-		}
-		break;
-	case OVAL_FUNCTION_BEGIN:{
-			oscap_dprintf("%sBEGIN_CHARACTER %s\n", nxtindent, oval_component_get_prefix(component));
-			function_comp_to_print(component, nxtindent);
-		}
-		break;
-	case OVAL_FUNCTION_END:{
-			oscap_dprintf("%sEND_CHARACTER %s\n", nxtindent, oval_component_get_suffix(component));
-			function_comp_to_print(component, nxtindent);
-		}
-		break;
-	case OVAL_FUNCTION_SPLIT:{
-			oscap_dprintf("%sSPLIT_DELIMITER %s\n", nxtindent,
-				      oval_component_get_split_delimiter(component));
-			function_comp_to_print(component, nxtindent);
-		}
-		break;
-	case OVAL_FUNCTION_SUBSTRING:{
-			oscap_dprintf("%sSUBSTRING_START  %d\n", nxtindent,
-				      oval_component_get_substring_start(component));
-			oscap_dprintf("%sSUBSTRING_LENGTH %d\n", nxtindent,
-				      oval_component_get_substring_length(component));
-			function_comp_to_print(component, nxtindent);
-		}
-		break;
-	case OVAL_FUNCTION_TIMEDIF:{
-			oscap_dprintf("%sTIMEDIF_FORMAT_1  %d\n", nxtindent,
-				      oval_component_get_timedif_format_1(component));
-			oscap_dprintf("%sTIMEDIF_FORMAT_2  %d\n", nxtindent,
-				      oval_component_get_timedif_format_2(component));
-			function_comp_to_print(component, nxtindent);
-		}
-		break;
-	case OVAL_FUNCTION_REGEX_CAPTURE:
-	case OVAL_FUNCTION_ESCAPE_REGEX:
-	case OVAL_FUNCTION_CONCAT:{
-			function_comp_to_print(component, nxtindent);
-		}
-		break;
-	default:
-		break;
-	}
-}
-
 xmlNode *oval_component_to_dom(struct oval_component *component, xmlDoc * doc, xmlNode * parent)
 {
 	oval_component_type_t type = oval_component_get_type(component);
@@ -1233,11 +1132,15 @@ xmlNode *oval_component_to_dom(struct oval_component *component, xmlDoc * doc, x
 		}
 		break;
 	case OVAL_COMPONENT_OBJECTREF:{
+			char *item_field, *record_field;
 			struct oval_object *object = oval_component_get_object(component);
 			char *object_ref = oval_object_get_id(object);
 			xmlNewProp(component_node, BAD_CAST "object_ref", BAD_CAST object_ref);
-			char *item_field = oval_component_get_object_field(component);
+			item_field = oval_component_get_item_field(component);
 			xmlNewProp(component_node, BAD_CAST "item_field", BAD_CAST item_field);
+			record_field = oval_component_get_record_field(component);
+			if (record_field)
+				xmlNewProp(component_node, BAD_CAST "record_field", BAD_CAST record_field);
 		} break;
 	case OVAL_COMPONENT_VARREF:{
 			struct oval_variable *variable = oval_component_get_variable(component);
@@ -1292,9 +1195,12 @@ xmlNode *oval_component_to_dom(struct oval_component *component, xmlDoc * doc, x
 			char *pattern = oval_component_get_regex_pattern(component);
 			xmlNewProp(component_node, BAD_CAST "pattern", BAD_CAST pattern);
 		} break;
+	case OVAL_FUNCTION_SPLIT:{
+			char * delimiter = oval_component_get_split_delimiter(component);
+			xmlNewProp(component_node, BAD_CAST "delimiter", BAD_CAST delimiter);
+		} break;
 	case OVAL_FUNCTION_CONCAT:
 	case OVAL_FUNCTION_ESCAPE_REGEX:
-	case OVAL_FUNCTION_SPLIT:
 		break;
 	default:
 		break;
@@ -1354,23 +1260,61 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(oval_ar
 	}
 
 	if (syschar) {
+		char *ifield_name, *rfield_name;
+
 		flag = oval_syschar_get_flag(syschar);
-		char *field_name = objref->object_field;
+		ifield_name = objref->item_field;
+		rfield_name = objref->record_field;
 		struct oval_sysitem_iterator *sysitems = oval_syschar_get_sysitem(syschar);
 		while (oval_sysitem_iterator_has_more(sysitems)) {
 			struct oval_sysitem *sysitem = oval_sysitem_iterator_next(sysitems);
-			struct oval_sysent_iterator *items = oval_sysitem_get_items(sysitem);
-			while (oval_sysent_iterator_has_more(items)) {
-				struct oval_sysent *item = oval_sysent_iterator_next(items);
-				char *item_name = oval_sysent_get_name(item);
-				if (strcmp(field_name, item_name) == 0) {
-					char *text = oval_sysent_get_value(item);
-					oval_datatype_t datatype = oval_sysent_get_datatype(item);
-					struct oval_value *value = oval_value_new(datatype, text);
-					oval_collection_add(value_collection, value);
+			struct oval_sysent_iterator *sysent_itr = oval_sysitem_get_sysents(sysitem);
+			while (oval_sysent_iterator_has_more(sysent_itr)) {
+				oval_datatype_t dt;
+				struct oval_sysent *sysent = oval_sysent_iterator_next(sysent_itr);
+				char *sysent_name = oval_sysent_get_name(sysent);
+
+				if (strcmp(ifield_name, sysent_name))
+					continue;
+
+				dt = oval_sysent_get_datatype(sysent);
+				if ((dt == OVAL_DATATYPE_RECORD && rfield_name == NULL)
+				    || (dt != OVAL_DATATYPE_RECORD && rfield_name != NULL))
+					/* todo: throw error */
+					continue;
+
+				if (dt == OVAL_DATATYPE_RECORD) {
+					struct oval_record_field_iterator *rf_itr;
+
+					rf_itr = oval_sysent_get_record_fields(sysent);
+					while (oval_record_field_iterator_has_more(rf_itr)) {
+						struct oval_record_field *rf;
+						char *txtval;
+						struct oval_value *val;
+
+						rf = oval_record_field_iterator_next(rf_itr);
+						txtval = oval_record_field_get_name(rf);
+						if (strcmp(rfield_name, txtval))
+							continue;
+
+						dt = oval_record_field_get_datatype(rf);
+						txtval = oval_record_field_get_value(rf);
+						val = oval_value_new(dt, txtval);
+						oval_collection_add(value_collection, val);
+					}
+					oval_record_field_iterator_free(rf_itr);
+					/* todo: throw error if none matched */
+				} else {
+					char *txtval;
+					struct oval_value *val;
+
+					txtval = oval_sysent_get_value(sysent);
+					val = oval_value_new(dt, txtval);
+					oval_collection_add(value_collection, val);
 				}
 			}
-			oval_sysent_iterator_free(items);
+			oval_sysent_iterator_free(sysent_itr);
+			/* todo: throw error if none matched */
 		}
 		oval_sysitem_iterator_free(sysitems);
 	}
@@ -1532,7 +1476,6 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_CONCAT(oval_argu_
 				texts[idx0] = oval_value_get_text(oval_value_iterator_next(comp_values));
 				not_finished = true;
 			} else {
-				oval_value_iterator_free(values[idx0]);
 				oval_collection_free_items(component_colls[idx0],
 							   (oscap_destruct_func) oval_value_free);
 				component_colls[idx0] = NULL;
@@ -1636,11 +1579,10 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_SUBSTRING(oval_ar
 {
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
 	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
-	int start = oval_component_get_substring_start(component) - 1;
-	size_t len = (size_t)oval_component_get_substring_length(component);
-        size_t beg;
+        ssize_t beg = oval_component_get_substring_start(component);
+	ssize_t len = oval_component_get_substring_length(component);
 
-	beg = (start < 0) ? 0 : (size_t)start;
+	beg = (beg < 1) ? 0 : beg - 1;
 
 	if (oval_component_iterator_has_more(subcomps)) {	/*Only first component is considered */
 		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
@@ -1651,27 +1593,21 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_SUBSTRING(oval_ar
 
 		while (oval_value_iterator_has_more(values)) {
 			char *text = oval_value_get_text(oval_value_iterator_next(values));
-			char substr[len + 1];
-                        size_t txtlen, sublen;
+			size_t txtlen, sublen;
 
-                        substr[0] = '\0';
-                        txtlen    = strlen(text);
+			txtlen = strlen(text);
+			sublen = (len < 0 || (size_t) len > txtlen) ? txtlen : (size_t) len;
+			if ((size_t) beg < txtlen) {
+				char substr[sublen + 1];
 
-                        if (beg < txtlen) {
-                                /* Check if we are still inside the string
-                                 * beg: position of start of next substring (from 0! last position is txtlen-1)
-                                 * len: length of susbstring
-                                 * txtlen: length of whole string */
-                                sublen = (beg + len) > txtlen ? 0 : txtlen - (beg + len) + 1;
-                                if (sublen < len)
-                                        len = sublen;
+				strncpy(substr, text + beg, sublen);
+				substr[sublen] = '\0';
 
-                                strncpy(substr, text + beg, len);
-                                substr[len] = '\0';
-                        }
-
-			value = oval_value_new(OVAL_DATATYPE_STRING, substr);
-			oval_collection_add(value_collection, value);
+				value = oval_value_new(OVAL_DATATYPE_STRING, substr);
+				oval_collection_add(value_collection, value);
+			} else {
+				flag = SYSCHAR_FLAG_ERROR;
+			}
 		}
 		oval_value_iterator_free(values);
 		oval_collection_free_items(subcoll, (oscap_destruct_func) oval_value_free);
@@ -2059,7 +1995,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_REGEX_CAPTURE(ova
 		while (oval_value_iterator_has_more(values)) {
 			struct oval_value *value = oval_value_iterator_next(values);
 			char *text = oval_value_get_text(value);
-			char *nval;
+			char *nval = NULL;
 #if defined USE_REGEX_PCRE
 			int i, ovector[60], ovector_len = sizeof (ovector) / sizeof (ovector[0]);
 
