@@ -28,9 +28,14 @@
  *      "Peter Vrabec" <pvrabec@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
@@ -203,79 +208,6 @@ struct oval_criteria_node *oval_criteria_node_new(struct oval_definition_model *
 	return node;
 }
 
-bool oval_criteria_node_is_valid(struct oval_criteria_node * criteria_node)
-{
-	bool is_valid = true;
-	oval_criteria_node_type_t type;
-
-	if (criteria_node == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-	type = oval_criteria_node_get_type(criteria_node);
-	switch (type) {
-	case OVAL_NODETYPE_CRITERIA:
-		{
-			struct oval_criteria_node_iterator *subnodes_itr;
-
-                        if (oval_criteria_node_get_operator(criteria_node) == OVAL_OPERATOR_UNKNOWN) {
-                                oscap_dlprintf(DBG_W, "Argument is not valid: operator == OVAL_OPERATOR_UNKNOWN.\n");
-                                return false;
-                        }
-
-			subnodes_itr = oval_criteria_node_get_subnodes(criteria_node);
-                        if (oval_criteria_node_iterator_has_more(subnodes_itr) == false) {
-                                oscap_dlprintf(DBG_W, "Argument is not valid: there are no subnodes.\n");
-                                return false;
-                        }
-			while (oval_criteria_node_iterator_has_more(subnodes_itr)) {
-				struct oval_criteria_node *subnode;
-
-				subnode = oval_criteria_node_iterator_next(subnodes_itr);
-				if (oval_criteria_node_is_valid(subnode) != true) {
-					is_valid = false;
-					break;
-				}
-			}
-			oval_criteria_node_iterator_free(subnodes_itr);
-			if (is_valid != true)
-				return false;
-		}
-		break;
-	case OVAL_NODETYPE_CRITERION:
-		{
-			struct oval_test *test;
-
-			test = oval_criteria_node_get_test(criteria_node);
-			if (oval_test_is_valid(test) != true)
-				return false;
-		}
-		break;
-	case OVAL_NODETYPE_EXTENDDEF:
-		{
-			struct oval_definition *def;
-
-			def = oval_criteria_node_get_definition(criteria_node);
-			if (oval_definition_is_valid(def) != true)
-				return false;
-		}
-		break;
-	default:
-                oscap_dlprintf(DBG_W, "Argument is not valid: wrong node type: %d.\n", type);
-		return false;
-	}
-
-	return true;
-}
-
-bool oval_criteria_node_is_locked(struct oval_criteria_node * criteria_node)
-{
-	__attribute__nonnull__(criteria_node);
-
-	return oval_definition_model_is_locked(criteria_node->model);
-}
-
 struct oval_criteria_node *oval_criteria_node_clone
     (struct oval_definition_model *new_model, struct oval_criteria_node *old_node) {
 	__attribute__nonnull__(old_node);
@@ -355,78 +287,62 @@ void oval_criteria_set_node_type(struct oval_criteria_node *node, oval_criteria_
 {
 	__attribute__nonnull__(node);
 
-	if (node && !oval_criteria_node_is_locked(node)) {
-		node->type = type;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	node->type = type;
 }
 
 void oval_criteria_node_set_negate(struct oval_criteria_node *node, bool negate)
 {
-	if (node && !oval_criteria_node_is_locked(node)) {
-		node->negate = negate;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(node);
+	node->negate = negate;
 }
 
 void oval_criteria_node_set_comment(struct oval_criteria_node *node, char *comm)
 {
-	if (node && !oval_criteria_node_is_locked(node)) {
-		if (node->comment != NULL)
-			oscap_free(node->comment);
-		node->comment = (comm == NULL) ? NULL : oscap_strdup(comm);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(node);
+	if (node->comment != NULL)
+		oscap_free(node->comment);
+	node->comment = (comm == NULL) ? NULL : oscap_strdup(comm);
 }
 
 void oval_criteria_node_set_operator(struct oval_criteria_node *node, oval_operator_t op)
 {
-	if (node && !oval_criteria_node_is_locked(node)) {
-		if (node->type == OVAL_NODETYPE_CRITERIA) {
-			struct oval_criteria_node_CRITERIA *criteria = (struct oval_criteria_node_CRITERIA *)node;
-			criteria->operator = op;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(node);
+	if (node->type == OVAL_NODETYPE_CRITERIA) {
+		struct oval_criteria_node_CRITERIA *criteria = (struct oval_criteria_node_CRITERIA *)node;
+		criteria->operator = op;
+	}
 }
 
 void oval_criteria_node_add_subnode(struct oval_criteria_node *node, struct oval_criteria_node *subnode)
 {
-	if (node && !oval_criteria_node_is_locked(node)) {
-		if (node->type == OVAL_NODETYPE_CRITERIA) {
-			struct oval_criteria_node_CRITERIA *criteria = (struct oval_criteria_node_CRITERIA *)node;
-			oval_collection_add(criteria->subnodes, (void *)subnode);
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(node);
+	if (node->type == OVAL_NODETYPE_CRITERIA) {
+		struct oval_criteria_node_CRITERIA *criteria = (struct oval_criteria_node_CRITERIA *)node;
+		oval_collection_add(criteria->subnodes, (void *)subnode);
+	}
 }
 
 void oval_criteria_node_set_test(struct oval_criteria_node *node, struct oval_test *test)
 {
-	if (node && !oval_criteria_node_is_locked(node)) {
-		if (node->type == OVAL_NODETYPE_CRITERION) {
-			struct oval_criteria_node_CRITERION *criterion = (struct oval_criteria_node_CRITERION *)node;
-			criterion->test = test;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(node);
+	if (node->type == OVAL_NODETYPE_CRITERION) {
+		struct oval_criteria_node_CRITERION *criterion = (struct oval_criteria_node_CRITERION *)node;
+		criterion->test = test;
+	}
 }
 
 void oval_criteria_node_set_definition(struct oval_criteria_node *node, struct oval_definition *definition)
 {
-	if (node && !oval_criteria_node_is_locked(node)) {
-		if (node->type == OVAL_NODETYPE_EXTENDDEF) {
-			struct oval_criteria_node_EXTENDDEF *extenddef = (struct oval_criteria_node_EXTENDDEF *)node;
-			extenddef->definition = definition;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(node);
+	if (node->type == OVAL_NODETYPE_EXTENDDEF) {
+		struct oval_criteria_node_EXTENDDEF *extenddef = (struct oval_criteria_node_EXTENDDEF *)node;
+		extenddef->definition = definition;
+	}
 }
 
 static void _oval_criteria_subnode_consume(struct oval_criteria_node *subnode, void *criteria)
 {
-	oval_criteria_node_add_subnode((struct oval_criteria_node *)
-				       criteria, subnode);
+	oval_criteria_node_add_subnode((struct oval_criteria_node *) criteria, subnode);
 }
 
 static int _oval_criteria_subnode_consumer(xmlTextReaderPtr reader, struct oval_parser_context *context, void *user)
@@ -437,8 +353,7 @@ static int _oval_criteria_subnode_consumer(xmlTextReaderPtr reader, struct oval_
 }
 
 //typedef void (*oval_criteria_consumer)(struct oval_criteria_node *node, void*);
-int oval_criteria_parse_tag(xmlTextReaderPtr reader,
-			    struct oval_parser_context *context, oval_criteria_consumer consumer, void *user)
+int oval_criteria_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, oval_criteria_consumer consumer, void *user)
 {
 	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
@@ -462,29 +377,26 @@ int oval_criteria_parse_tag(xmlTextReaderPtr reader,
 			comm = NULL;
 		}
 		oval_criteria_node_set_negate(node, oval_parser_boolean_attribute(reader, "negate", 0));
-		return_code = 1;
+		return_code = 0;
 		switch (oval_criteria_node_get_type(node)) {
 		case OVAL_NODETYPE_CRITERIA:{
 				struct oval_criteria_node_CRITERIA *criteria =
 				    (struct oval_criteria_node_CRITERIA *)node;
 				oval_operator_t operator = oval_operator_parse(reader, "operator", OVAL_OPERATOR_AND);
 				oval_criteria_node_set_operator((struct oval_criteria_node *)criteria, operator);
-				return_code =
-				    oval_parser_parse_tag(reader, context, &_oval_criteria_subnode_consumer, criteria);
+				return_code = oval_parser_parse_tag(reader, context, &_oval_criteria_subnode_consumer, criteria);
 			} break;
 		case OVAL_NODETYPE_CRITERION:{
-				char *test_ref = (char *)xmlTextReaderGetAttribute(reader,
-										   BAD_CAST "test_ref");
-				struct oval_definition_model *model = oval_parser_context_model(context);
+				char *test_ref = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "test_ref");
+				struct oval_definition_model *model = context->definition_model;
 				struct oval_test *test = oval_test_get_new(model, test_ref);
 				oscap_free(test_ref);
 				test_ref = NULL;
 				oval_criteria_node_set_test(node, test);
 			} break;
 		case OVAL_NODETYPE_EXTENDDEF:{
-				char *definition_ref = (char *)xmlTextReaderGetAttribute(reader,
-											 BAD_CAST "definition_ref");
-				struct oval_definition_model *model = oval_parser_context_model(context);
+				char *definition_ref = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "definition_ref");
+				struct oval_definition_model *model = context->definition_model;
 				struct oval_definition *definition = oval_definition_get_new(model, definition_ref);
 				oval_criteria_node_set_definition(node, definition);
 				oscap_free(definition_ref);
@@ -496,7 +408,7 @@ int oval_criteria_parse_tag(xmlTextReaderPtr reader,
 		//oval_parser_parse_tag(reader, context,&_oval_criteria_parse_tag,node);
 		(*consumer) (node, user);
 	} else {
-		return_code = 0;
+		return_code = 1;
 		oscap_dlprintf(DBG_W, "Invalid node type: OVAL_NODETYPE_UNKNOWN.\n");
 		oval_parser_skip_tag(reader, context);
 	}
@@ -505,70 +417,6 @@ int oval_criteria_parse_tag(xmlTextReaderPtr reader,
 	return return_code;
 }
 
-void oval_criteria_node_to_print(struct oval_criteria_node *node, char *indent, int idx)
-{
-	__attribute__nonnull__(node);
-
-	char *nodetype = NULL;
-	char nxtindent[100];
-
-	switch (node->type) {
-	case OVAL_NODETYPE_CRITERIA:
-		nodetype = "CRITERIA";
-		break;
-	case OVAL_NODETYPE_CRITERION:
-		nodetype = "CRITERION";
-		break;
-	case OVAL_NODETYPE_EXTENDDEF:
-		nodetype = "EXTEND_DEFINITION";
-		break;
-	case OVAL_NODETYPE_UNKNOWN:
-		nodetype = "UNKNOWN_CRITNODE";
-		break;
-	}
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%s%s.", indent, nodetype);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%s%s[%d].", indent, nodetype, idx);
-
-	oscap_dprintf("%sCOMMENT = %s\n", nxtindent, node->comment);
-	oscap_dprintf("%sNEGATE  = %d\n", nxtindent, node->negate);
-	switch (node->type) {
-	case OVAL_NODETYPE_CRITERIA:{
-			struct oval_criteria_node_CRITERIA *criteria = (struct oval_criteria_node_CRITERIA *)node;
-			oscap_dprintf("%sOPERATOR = %d\n", nxtindent, criteria->operator);
-			struct oval_iterator *subnodes = oval_collection_iterator(criteria->subnodes);
-			for (idx = 1; oval_collection_iterator_has_more(subnodes); idx++) {
-				void *subnode = oval_collection_iterator_next(subnodes);
-				oval_criteria_node_to_print((struct oval_criteria_node *)subnode, nxtindent, idx);
-			}
-			oval_collection_iterator_free(subnodes);
-		} break;
-	case OVAL_NODETYPE_CRITERION:{
-			struct oval_criteria_node_CRITERION *criterion = (struct oval_criteria_node_CRITERION *)node;
-			if (criterion->test != NULL)
-				oval_test_to_print(criterion->test, nxtindent, 0);
-			else
-				oscap_dprintf("%sTEST = <<NONE>>\n", nxtindent);
-		}
-		break;
-	case OVAL_NODETYPE_EXTENDDEF:{
-			struct oval_criteria_node_EXTENDDEF *extenddef = (struct oval_criteria_node_EXTENDDEF *)node;
-			if (extenddef->definition != NULL)
-				oval_definition_to_print(extenddef->definition, nxtindent, 0);
-			else
-				oscap_dprintf("%sDEFINITION = <<NONE>>\n", nxtindent);
-		}
-		break;
-	case OVAL_NODETYPE_UNKNOWN:
-		break;
-	}
-}
-
 static xmlNode *_oval_CRITERIA_to_dom(struct oval_criteria_node *cnode, xmlDoc * doc, xmlNode * parent) {
 	xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
 	xmlNode *criteria_node = xmlNewTextChild(parent, ns_definitions, BAD_CAST "criteria", NULL);
