@@ -27,9 +27,13 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <time.h>
-#include "config.h"
+
 #include "oval_definitions_impl.h"
 #include "oval_agent_api_impl.h"
 #include "oval_parser_impl.h"
@@ -41,6 +45,7 @@
 #include "common/debug_priv.h"
 #include "common/_error.h"
 #include "common/reporter_priv.h"
+#include "common/elements.h"
 
 typedef struct oval_definition_model {
 	struct oval_generator *generator;
@@ -49,7 +54,8 @@ typedef struct oval_definition_model {
 	struct oval_string_map *object_map;
 	struct oval_string_map *state_map;
 	struct oval_string_map *variable_map;
-	bool is_locked;
+	struct oval_collection *bound_variable_models;
+	xmlDoc *metadata_doc;
         char *schema;
 } oval_definition_model_t;
 
@@ -95,6 +101,8 @@ static char * _oval_generate_schema_location(const char * version)
  * */
 struct oval_definition_model *oval_definition_model_new()
 {
+	xmlNode *root;
+	xmlNs *ns;
 	oval_definition_model_t *newmodel = (oval_definition_model_t *) oscap_alloc(sizeof(oval_definition_model_t));
 	if (newmodel == NULL)
 		return NULL;
@@ -105,101 +113,18 @@ struct oval_definition_model *oval_definition_model_new()
 	newmodel->state_map = oval_string_map_new();
 	newmodel->test_map = oval_string_map_new();
 	newmodel->variable_map = oval_string_map_new();
-	newmodel->is_locked = false;
+	newmodel->bound_variable_models = NULL;
         newmodel->schema = strdup(OVAL_DEF_SCHEMA_LOCATION);
+
+	root = xmlNewNode(NULL, BAD_CAST "root");
+	ns = xmlNewNs(root, OVAL_DEFINITIONS_NAMESPACE, NULL);
+	xmlSetNs(root, ns);
+	newmodel->metadata_doc = xmlNewDoc(BAD_CAST "1.0");
+	xmlDocSetRootElement(newmodel->metadata_doc, root);
+
 	return newmodel;
 }
 
-void oval_definition_model_lock(struct oval_definition_model *definition_model)
-{
-	__attribute__nonnull__(definition_model);
-
-	if (definition_model && oval_definition_model_is_valid(definition_model))
-		definition_model->is_locked = true;
-}
-
-bool oval_definition_model_is_locked(struct oval_definition_model *definition_model)
-{
-	__attribute__nonnull__(definition_model);
-
-	return definition_model->is_locked;
-}
-
-bool oval_definition_model_is_valid(struct oval_definition_model * definition_model)
-{
-	bool is_valid = true;
-	struct oval_definition_iterator *definitions_itr;
-	struct oval_test_iterator *tests_itr;
-	struct oval_object_iterator *objects_itr;
-	struct oval_state_iterator *states_itr;
-
-	if (definition_model == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-	/* validate definitions */
-	definitions_itr = oval_definition_model_get_definitions(definition_model);
-	while (oval_definition_iterator_has_more(definitions_itr)) {
-		struct oval_definition *definition;
-
-		definition = oval_definition_iterator_next(definitions_itr);
-		if (oval_definition_is_valid(definition) != true) {
-			is_valid = false;
-			break;
-		}
-	}
-	oval_definition_iterator_free(definitions_itr);
-	if (is_valid != true)
-		return false;
-
-	/* validate tests */
-	tests_itr = oval_definition_model_get_tests(definition_model);
-	while (oval_test_iterator_has_more(tests_itr)) {
-		struct oval_test *test;
-
-		test = oval_test_iterator_next(tests_itr);
-		if (oval_test_is_valid(test) != true) {
-			is_valid = false;
-			break;
-		}
-	}
-	oval_test_iterator_free(tests_itr);
-	if (is_valid != true)
-		return false;
-
-	/* validate objects */
-	objects_itr = oval_definition_model_get_objects(definition_model);
-	while (oval_object_iterator_has_more(objects_itr)) {
-		struct oval_object *obj;
-
-		obj = oval_object_iterator_next(objects_itr);
-		if (oval_object_is_valid(obj) != true) {
-			is_valid = false;
-			break;
-		}
-	}
-	oval_object_iterator_free(objects_itr);
-	if (is_valid != true)
-		return false;
-
-	/* validate states */
-	states_itr = oval_definition_model_get_states(definition_model);
-	while (oval_state_iterator_has_more(states_itr)) {
-		struct oval_state *state;
-		state = oval_state_iterator_next(states_itr);
-		if (oval_state_is_valid(state) != true) {
-			is_valid = false;
-			break;
-		}
-	}
-	oval_state_iterator_free(states_itr);
-	if (is_valid != true)
-		return false;
-
-	return true;
-}
-
 typedef void (*_oval_clone_func) (void *, struct oval_definition_model *);
 
 static void _oval_definition_model_clone(struct oval_string_map *oldmap, 
@@ -234,6 +159,7 @@ struct oval_definition_model *oval_definition_model_clone(struct oval_definition
 	_oval_definition_model_clone(oldmodel->test_map, newmodel, (_oval_clone_func) oval_test_clone);
 	_oval_definition_model_clone
 	    (oldmodel->variable_map, newmodel, (_oval_clone_func) oval_variable_clone);
+	newmodel->metadata_doc = xmlCopyDoc(oldmodel->metadata_doc, 1);
         newmodel->schema = oscap_strdup(oldmodel->schema);
 	return newmodel;
 }
@@ -247,6 +173,11 @@ void oval_definition_model_free(struct oval_definition_model *model)
 	oval_string_map_free(model->state_map, (oscap_destruct_func) oval_state_free);
 	oval_string_map_free(model->test_map, (oscap_destruct_func) oval_test_free);
 	oval_string_map_free(model->variable_map, (oscap_destruct_func) oval_variable_free);
+	if (model->bound_variable_models)
+		oval_collection_free_items(model->bound_variable_models,
+					   (oscap_destruct_func) oval_variable_model_free);
+
+	xmlFreeDoc(model->metadata_doc);
 
         if (model->schema != NULL)
             oscap_free(model->schema);
@@ -256,8 +187,11 @@ void oval_definition_model_free(struct oval_definition_model *model)
 	model->state_map = NULL;
 	model->test_map = NULL;
 	model->variable_map = NULL;
+	model->metadata_doc = NULL;
         model->schema = NULL;
 
+	oval_generator_free(model->generator);
+
 	oscap_free(model);
 }
 
@@ -281,63 +215,51 @@ const char * oval_definition_model_get_schema(struct oval_definition_model * mod
 
 void oval_definition_model_add_definition(struct oval_definition_model *model, struct oval_definition *definition)
 {
-	if (model && !oval_definition_model_is_locked(model)) {
-		char *key = oval_definition_get_id(definition);
-		oval_string_map_put(model->definition_map, key, (void *)definition);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(model);
+	char *key = oval_definition_get_id(definition);
+	oval_string_map_put(model->definition_map, key, (void *)definition);
 }
 
 void oval_definition_model_set_schema(struct oval_definition_model *model, const char *version)
 {
-	if (model && !oval_definition_model_is_locked(model)) {
-                model->schema = oscap_strdup(version);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(model);
+	model->schema = oscap_strdup(version);
 }
 
 void oval_definition_model_add_test(struct oval_definition_model *model, struct oval_test *test)
 {
-	if (model && !oval_definition_model_is_locked(model)) {
-		char *key = oval_test_get_id(test);
-		oval_string_map_put(model->test_map, key, (void *)test);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(model);
+	char *key = oval_test_get_id(test);
+	oval_string_map_put(model->test_map, key, (void *)test);
 }
 
 void oval_definition_model_add_object(struct oval_definition_model *model, struct oval_object *object)
 {
-	if (model && !oval_definition_model_is_locked(model)) {
-		char *key = oval_object_get_id(object);
-		oval_string_map_put(model->object_map, key, (void *)object);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(model);
+	char *key = oval_object_get_id(object);
+	oval_string_map_put(model->object_map, key, (void *)object);
 }
 
 void oval_definition_model_add_state(struct oval_definition_model *model, struct oval_state *state)
 {
-	if (model && !oval_definition_model_is_locked(model)) {
-		char *key = oval_state_get_id(state);
-		oval_string_map_put(model->state_map, key, (void *)state);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(model);
+	char *key = oval_state_get_id(state);
+	oval_string_map_put(model->state_map, key, (void *)state);
 }
 
 void oval_definition_model_add_variable(struct oval_definition_model *model, struct oval_variable *variable)
 {
-	if (model && !oval_definition_model_is_locked(model)) {
-		char *key = oval_variable_get_id(variable);
-		oval_string_map_put(model->variable_map, key, (void *)variable);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(model);
+	char *key = oval_variable_get_id(variable);
+	oval_string_map_put(model->variable_map, key, (void *)variable);
 }
 
 struct oval_definition_model * oval_definition_model_import(const char *file)
 {
         struct oval_definition_model *model = oval_definition_model_new();
         int ret = oval_definition_model_merge(model,file);
-        if (ret != 1) {
-		oscap_dlprintf(DBG_E, "Failed to merge the definition model (%s).\n", file);
+        if (ret == -1 ) {
+		oscap_dlprintf(DBG_E, "Failed to merge the definition model from: %s.\n", file);
                 oval_definition_model_free(model);
                 model = NULL;
         }
@@ -359,8 +281,16 @@ int oval_definition_model_merge(struct oval_definition_model *model, const char
 		return -1;
 	}
 
+	/* setup context */
+	struct oval_parser_context context;
+	context.reader = reader;
+	context.definition_model = model;
+	context.user_data = NULL;
+	xmlTextReaderSetErrorHandler(reader, &libxml_error_handler, &context);
+	/* jump into oval_definitions */
 	xmlTextReaderRead(reader);
-	ret = ovaldef_parser_parse(model, reader, NULL);
+	/* start parsing */
+	ret = oval_definition_model_parse(reader, &context);
 	xmlFreeTextReader(reader);
 
 	return ret;
@@ -404,49 +334,40 @@ struct oval_variable *oval_definition_model_get_variable(struct oval_definition_
 int oval_definition_model_bind_variable_model(struct oval_definition_model *defmodel,
 					       struct oval_variable_model *varmodel)
 {
-	//Bind values to all external variables specified in the variable model.
-	struct oval_variable_iterator *variables = oval_definition_model_get_variables(defmodel);
-	while (oval_variable_iterator_has_more(variables)) {
-		struct oval_variable *variable = oval_variable_iterator_next(variables);
-		if (oval_variable_get_type(variable) == OVAL_VARIABLE_EXTERNAL) {
-			char *varid = oval_variable_get_id(variable);
-			oval_datatype_t var_datatype;
+	struct oval_string_iterator *evar_id_itr;
 
-			if (!oval_variable_model_has_variable(varmodel, varid))
-				continue;
+	if (!defmodel->bound_variable_models)
+		defmodel->bound_variable_models = oval_collection_new();
 
-			var_datatype = oval_variable_model_get_datatype(varmodel, varid);
-			if (var_datatype != OVAL_DATATYPE_UNKNOWN) {	//values are bound in the variable model
-				oval_datatype_t def_datatype = oval_variable_get_datatype(variable);
-				if (def_datatype == var_datatype) {
-					struct oval_string_iterator *values =
-					    oval_variable_model_get_values(varmodel, varid);
-					while (oval_string_iterator_has_more(values)) {
-						char *text = oval_string_iterator_next(values);
-						struct oval_value *value = oval_value_new(var_datatype, text);
-						oval_variable_add_value(variable, value);
-					}
-					oval_string_iterator_free(values);
-				} else {
-					oval_variable_iterator_free(variables);
-					oscap_dlprintf(DBG_E, "Unmatched variable datatypes: id: %s, "
-						       "definition model datatype: %s, "
-						       "variable model datatype: %s.\n",
-						       varid, oval_datatype_get_text(def_datatype),
-						       oval_datatype_get_text(var_datatype));
-					return -1;
-				}
-			} else {
-				oval_variable_iterator_free(variables);
-				oscap_dlprintf(DBG_E, "Unknown variable datatype: id: %s.\n", varid);
-				return -1;
-			}
-		}
+	oval_collection_add(defmodel->bound_variable_models, varmodel);
+
+	/* todo: keep reference count for each variable model if it can be bound to multiple definition models */
+
+	evar_id_itr = oval_variable_model_get_variable_ids(varmodel);
+	while (oval_string_iterator_has_more(evar_id_itr)) {
+		char *evar_id;
+		struct oval_variable *var;
+
+		evar_id = oval_string_iterator_next(evar_id_itr);
+		var = oval_definition_model_get_variable(defmodel, evar_id);
+		if (!var)
+			continue;
+
+		oval_variable_bind_ext_var(var, varmodel, evar_id);
 	}
-	oval_variable_iterator_free(variables);
+	oval_string_iterator_free(evar_id_itr);
+
 	return 0;
 }
 
+struct oval_variable_model_iterator *oval_definition_model_get_variable_models(struct oval_definition_model *model)
+{
+	if (model->bound_variable_models)
+		return (struct oval_variable_model_iterator *) oval_collection_iterator(model->bound_variable_models);
+	else
+		return (struct oval_variable_model_iterator *) oval_collection_iterator_new();
+}
+
 void oval_definition_model_clear_external_variables(struct oval_definition_model *model)
 {
 	struct oval_variable_iterator *vars_itr;
@@ -561,14 +482,19 @@ struct oval_test *oval_test_get_new(struct oval_definition_model *model, const c
 	return test;
 }
 
+xmlDoc *oval_definition_model_get_metadata_doc(struct oval_definition_model *model)
+{
+	return model->metadata_doc;
+}
+
 xmlNode *oval_definitions_to_dom(struct oval_definition_model *definition_model, xmlDocPtr doc, xmlNode * parent)
 {
 
 	xmlNodePtr root_node = NULL;
 
-	if (parent) {
+	if (parent) { /* result file */
 		root_node = xmlNewTextChild(parent, NULL, BAD_CAST "oval_definitions", NULL);
-	} else {
+	} else {      /* definitions file, we are the root */
 		root_node = xmlNewNode(NULL, BAD_CAST "oval_definitions");
 		xmlDocSetRootElement(doc, root_node);
 	}
@@ -582,10 +508,8 @@ xmlNode *oval_definitions_to_dom(struct oval_definition_model *definition_model,
 	xmlSetNs(root_node, ns_xsi);
 	xmlSetNs(root_node, ns_defntns);
 
-	/* Report generator */
-	if (!parent) {
-		oval_generator_to_dom(definition_model->generator, doc, root_node);
-	}
+	/* Always report the generator */
+	oval_generator_to_dom(definition_model->generator, doc, root_node);
 
 	/* Report definitions */
 	struct oval_definition_iterator *definitions = oval_definition_model_get_definitions(definition_model);
