@@ -27,18 +27,24 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
 #include "oval_agent_api_impl.h"
+
+#include "common/assume.h"
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
 
-#define OVAL_SUPPORTED "5.5"
 
 /***************************************************************************/
 /* Variable definitions
@@ -55,6 +61,7 @@ typedef struct oval_definition {
 	struct oval_collection *affected;
 	struct oval_collection *reference;
 	struct oval_collection *notes;
+	xmlNode *metadata;
 	struct oval_criteria_node *criteria;
 } oval_definition_t;
 
@@ -139,6 +146,13 @@ struct oval_string_iterator *oval_definition_get_notes(struct oval_definition *d
 	    oval_collection_iterator(definition->notes);
 }
 
+xmlNode *oval_definition_get_metadata(struct oval_definition *definition)
+{
+	__attribute__nonnull__(definition);
+
+	return definition->metadata;
+}
+
 struct oval_criteria_node *oval_definition_get_criteria(struct oval_definition
 							*definition)
 {
@@ -149,16 +163,13 @@ struct oval_criteria_node *oval_definition_get_criteria(struct oval_definition
 
 struct oval_definition *oval_definition_new(struct oval_definition_model *model, const char *id)
 {
+	__attribute__nonnull__(model);
 	struct oval_definition *definition;
+	xmlNode *root;
 
-	if (model && oval_definition_model_is_locked(model)) {
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
-		return NULL;
-	}
+	definition = (struct oval_definition *)oscap_talloc(oval_definition_t);
 
-	definition = (struct oval_definition *)oscap_alloc(sizeof(oval_definition_t));;
-	if (definition == NULL)
-		return NULL;
+        assume_r(definition != NULL, /* return */ NULL);
 
 	definition->id = oscap_strdup(id);
 	definition->version = 0;
@@ -169,6 +180,8 @@ struct oval_definition *oval_definition_new(struct oval_definition_model *model,
 	definition->affected = oval_collection_new();
 	definition->reference = oval_collection_new();
 	definition->notes = oval_collection_new();
+	root = xmlDocGetRootElement(oval_definition_model_get_metadata_doc(model));
+	definition->metadata = xmlNewChild(root, NULL, BAD_CAST "metadata", NULL);
 	definition->criteria = NULL;
 	definition->model = model;
 
@@ -177,40 +190,14 @@ struct oval_definition *oval_definition_new(struct oval_definition_model *model,
 	return definition;
 }
 
-bool oval_definition_is_valid(struct oval_definition * definition)
-{
-	struct oval_criteria_node *criteria_node;
-
-	if (definition == NULL) {
-		oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-	}
-
-	if (oval_definition_get_class(definition) == OVAL_CLASS_UNKNOWN) {
-		oscap_dlprintf(DBG_W, "Argument is not valid: class == OVAL_CLASS_UNKNOWN.\n");
-		return false;
-	}
-
-	criteria_node = oval_definition_get_criteria(definition);
-	if (oval_criteria_node_is_valid(criteria_node) != true)
-		return false;
-
-	return true;
-}
-
-bool oval_definition_is_locked(struct oval_definition * definition)
-{
-	__attribute__nonnull__(definition);
-
-	return oval_definition_model_is_locked(definition->model);
-}
-
 struct oval_definition *oval_definition_clone
     (struct oval_definition_model *new_model, struct oval_definition *old_definition) {
 	__attribute__nonnull__(old_definition);
 
 	struct oval_definition *new_definition = oval_definition_model_get_definition(new_model, old_definition->id);
 	if (new_definition == NULL) {
+		xmlNode *root, *metadata;
+
 		new_definition = oval_definition_new(new_model, old_definition->id);
 		oval_definition_set_version(new_definition, old_definition->version);
 		oval_definition_set_class(new_definition, old_definition->class);
@@ -237,6 +224,10 @@ struct oval_definition *oval_definition_clone
 		}
 		oval_string_iterator_free(notes);
 
+		metadata = xmlDocCopyNode(old_definition->metadata, oval_definition_model_get_metadata_doc(new_model), 1);
+		root = xmlDocGetRootElement(oval_definition_model_get_metadata_doc(new_model));
+		xmlAddChild(root, metadata);
+
 		oval_definition_set_criteria(new_definition,
 					     oval_criteria_node_clone(new_model, old_definition->criteria));
 	}
@@ -258,6 +249,8 @@ void oval_definition_free(struct oval_definition *definition)
 	oval_collection_free_items(definition->affected, (oscap_destruct_func) oval_affected_free);
 	oval_collection_free_items(definition->reference, (oscap_destruct_func) oval_reference_free);
 	oval_collection_free_items(definition->notes, (oscap_destruct_func) oscap_free);
+	xmlUnlinkNode(definition->metadata);
+	xmlFreeNode(definition->metadata);
 
 	definition->affected = NULL;
 	definition->criteria = NULL;
@@ -265,6 +258,7 @@ void oval_definition_free(struct oval_definition *definition)
 	definition->id = NULL;
 	definition->reference = NULL;
 	definition->notes = NULL;
+	definition->metadata = NULL;
 	definition->title = NULL;
 	oscap_free(definition);
 }
@@ -295,77 +289,59 @@ void oval_definition_iterator_free(struct
 
 void oval_definition_set_version(struct oval_definition *definition, int version)
 {
-	if (definition && !oval_definition_is_locked(definition)) {
-		definition->version = version;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	definition->version = version;
 }
 
 void oval_definition_set_class(struct oval_definition *definition, oval_definition_class_t class)
 {
-	if (definition && !oval_definition_is_locked(definition)) {
-		definition->class = class;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	definition->class = class;
 }
 
 void oval_definition_set_deprecated(struct oval_definition *definition, bool deprecated)
 {
-	if (definition && !oval_definition_is_locked(definition)) {
-		definition->deprecated = deprecated;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	definition->deprecated = deprecated;
 }
 
 void oval_definition_set_title(struct oval_definition *definition, char *title)
 {
-	if (definition && !oval_definition_is_locked(definition)) {
-		if (definition->title != NULL)
-			oscap_free(definition->title);
-		definition->title = (title == NULL) ? NULL : oscap_strdup(title);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	if (definition->title != NULL)
+		oscap_free(definition->title);
+	definition->title = (title == NULL) ? NULL : oscap_strdup(title);
 }
 
 void oval_definition_set_description(struct oval_definition *definition, char *description)
 {
-	if (definition && !oval_definition_is_locked(definition)) {
-		if (definition->description)
-			oscap_free(definition->description);
-		definition->description = (description == NULL) ? NULL : oscap_strdup(description);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	if (definition->description)
+		oscap_free(definition->description);
+	definition->description = (description == NULL) ? NULL : oscap_strdup(description);
 }
 
 void oval_definition_set_criteria(struct oval_definition *definition, struct oval_criteria_node *criteria)
 {
-	if (definition && !oval_definition_is_locked(definition)) {
-		definition->criteria = criteria;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	definition->criteria = criteria;
 }
 
 void oval_definition_add_affected(struct oval_definition *definition, struct oval_affected *affected)
 {
-	if (definition && !oval_definition_is_locked(definition)) {
-		oval_collection_add(definition->affected, affected);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	oval_collection_add(definition->affected, affected);
 }
 
 void oval_definition_add_reference(struct oval_definition *definition, struct oval_reference *ref)
 {
-	if (definition && !oval_definition_is_locked(definition)) {
-		oval_collection_add(definition->reference, ref);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	oval_collection_add(definition->reference, ref);
 }
 
 void oval_definition_add_note(struct oval_definition *definition, char *note) {
-	if (definition && !oval_definition_is_locked(definition)) {
-		oval_collection_add(definition->notes, note);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	oval_collection_add(definition->notes, note);
 }
 
 static oval_definition_class_t _odaclass(char *class)
@@ -448,8 +424,7 @@ static int _oval_definition_parse_metadata(xmlTextReaderPtr reader, struct oval_
 	if ((strcmp(tagname, "title") == 0)) {
 		return_code = oval_parser_text_value(reader, context, &_oval_definition_title_consumer, definition);
 	} else if (strcmp(tagname, "description") == 0) {
-		return_code =
-		    oval_parser_text_value(reader, context, &_oval_definition_description_consumer, definition);
+		return_code = oval_parser_text_value(reader, context, &_oval_definition_description_consumer, definition);
 	} else if (strcmp(tagname, "affected") == 0) {
 		return_code = oval_affected_parse_tag(reader, context, &_oval_definition_affected_consumer, definition);
 	} else if (strcmp(tagname, "oval_repository") == 0) {	/* NOOP */
@@ -457,11 +432,20 @@ static int _oval_definition_parse_metadata(xmlTextReaderPtr reader, struct oval_
 	} else if (strcmp(tagname, "reference") == 0) {
 		return_code = oval_reference_parse_tag(reader, context, &oval_reference_consume, definition);
 	} else {
-		int depth = xmlTextReaderDepth(reader);
-		if (depth == -1)
-			oscap_setxmlerr(xmlGetLastError());
-		oscap_dlprintf(DBG_I, "Skipping tag: %s, depth: %d, line: %d.\n",
-			      tagname, depth, xmlTextReaderGetParserLineNumber(reader));
+		char *str;
+		xmlDoc *doc;
+		xmlNode *node;
+
+		str = (char *) xmlTextReaderReadOuterXml(reader);
+		doc = xmlReadDoc(BAD_CAST str, NULL, NULL, 0);
+		xmlFree(str);
+
+		node = xmlDocGetRootElement(doc);
+		// todo: reattach instead of copying?
+		node = xmlDocCopyNode(node, oval_definition_model_get_metadata_doc(definition->model), 1);
+		xmlAddChild(definition->metadata, node);
+		xmlFreeDoc(doc);
+
 		return_code = oval_parser_skip_tag(reader, context);
 	}
 	oscap_free(tagname);
@@ -486,20 +470,16 @@ static int _oval_definition_parse_tag(xmlTextReaderPtr reader, struct oval_parse
 	} else if ((strcmp(tagname, "criteria") == 0)) {
 		return_code = oval_criteria_parse_tag(reader, context, &_oval_definition_criteria_consumer, definition);
 	} else {
-		int depth = xmlTextReaderDepth(reader);
-		if (depth == -1)
-			oscap_setxmlerr(xmlGetLastError());
-		oscap_dlprintf(DBG_I, "Skipping tag: %s, depth: %d, line: %d.\n", tagname, depth,
-                              xmlTextReaderGetParserLineNumber(reader));
+		oscap_dlprintf(DBG_I, "Skipping tag: %s.\n", tagname);
 		return_code = oval_parser_skip_tag(reader, context);
 	}
 	oscap_free(tagname);
 	return return_code;
 }
 
-int oval_definition_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context)
+int oval_definition_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr)
 {
-	struct oval_definition_model *model = oval_parser_context_model(context);
+	struct oval_definition_model *model = context->definition_model;
 	char *id = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "id");
 	struct oval_definition *definition = oval_definition_get_new(model, id);
 	oscap_free(id);
@@ -517,49 +497,14 @@ int oval_definition_parse_tag(xmlTextReaderPtr reader, struct oval_parser_contex
 
 	int deprecated = oval_parser_boolean_attribute(reader, "deprecated", 0);
 	oval_definition_set_deprecated(definition, deprecated);
-	int return_code = oval_parser_parse_tag(reader, context, &_oval_definition_parse_tag,
-						definition);
+	int return_code = oval_parser_parse_tag(reader, context, &_oval_definition_parse_tag, definition);
 	return return_code;
 }
 
-void oval_definition_to_print(struct oval_definition *definition, char *indent, int idx)
-{
-	__attribute__nonnull__(definition);
-
-	char nxtindent[100];
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%sDEFINITION.", indent);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%sDEFINITION[%d].", indent, idx);
-
-	oscap_dprintf("%sID          = %s\n", nxtindent, definition->id);
-	oscap_dprintf("%sVERSION     = %d\n", nxtindent, definition->version);
-	oscap_dprintf("%sCLASS       = %d\n", nxtindent, definition->class);
-	oscap_dprintf("%sDEPRECATED  = %d\n", nxtindent, definition->deprecated);
-	oscap_dprintf("%sTITLE       = %s\n", nxtindent, definition->title);
-	oscap_dprintf("%sDESCRIPTION = %s\n", nxtindent, definition->description);
-	struct oval_iterator *affecteds = oval_collection_iterator(definition->affected);
-	for (idx = 1; oval_collection_iterator_has_more(affecteds); idx++) {
-		void *affected = oval_collection_iterator_next(affecteds);
-		oval_affected_to_print(affected, nxtindent, idx);
-	}
-	oval_collection_iterator_free(affecteds);
-	struct oval_iterator *references = oval_collection_iterator(definition->reference);
-	for (idx = 1; oval_collection_iterator_has_more(references); idx++) {
-		void *ref = oval_collection_iterator_next(references);
-		oval_reference_to_print(ref, nxtindent, idx);
-	}
-	oval_collection_iterator_free(references);
-	if (definition->criteria != NULL)
-		oval_criteria_node_to_print(definition->criteria, nxtindent, 0);
-}
 
 xmlNode *oval_definition_to_dom(struct oval_definition *definition, xmlDoc * doc, xmlNode * parent)
 {
+	xmlNode *nodelst;
 	xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
 	xmlNode *definition_node = xmlNewTextChild(parent, ns_definitions, BAD_CAST "definition", NULL);
 
@@ -631,6 +576,9 @@ xmlNode *oval_definition_to_dom(struct oval_definition *definition, xmlDoc * doc
 	}
 	oval_string_iterator_free(notes);
 
+	nodelst = xmlDocCopyNodeList(doc, definition->metadata->children);
+	xmlAddChildList(metadata_node, nodelst);
+
 	struct oval_criteria_node *criteria = oval_definition_get_criteria(definition);
 	if (criteria)
 		oval_criteria_node_to_dom(criteria, doc, definition_node);
