@@ -27,9 +27,14 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
@@ -56,6 +61,7 @@ struct oval_entity {
 struct oval_consume_varref_context {
 	struct oval_definition_model *model;
 	struct oval_variable **variable;
+	struct oval_value **value;
 };
 
 /* End of variable definitions
@@ -154,37 +160,6 @@ struct oval_entity *oval_entity_new(struct oval_definition_model *model)
 	return entity;
 }
 
-bool oval_entity_is_valid(struct oval_entity * entity)
-{
-        if (entity == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-                return false;
-        }
-        /* todo: currently the entity type is not parsed
-        if (oval_entity_get_type(entity) == OVAL_ENTITY_TYPE_UNKNOWN) {
-                oscap_dprintf("WARNING: argument is not valid: type == OVAL_ENTITY_TYPE_UNKNOWN.\n");
-                return false;
-        }
-        */
-        if (oval_entity_get_datatype(entity) == OVAL_DATATYPE_UNKNOWN) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: datatype == OVAL_DATATYPE_UNKNOWN.\n");
-                return false;
-        }
-        if (oval_entity_get_operation(entity) == OVAL_OPERATION_UNKNOWN) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: operation == OVAL_OPERATION_UNKNOWN.\n");
-                return false;
-        }
-
-	return true;
-}
-
-bool oval_entity_is_locked(struct oval_entity * entity)
-{
-	__attribute__nonnull__(entity);
-
-	return oval_definition_model_is_locked(entity->model);
-}
-
 struct oval_entity *oval_entity_clone(struct oval_definition_model *new_model, struct oval_entity *old_entity) {
 	struct oval_entity *new_entity = oval_entity_new(new_model);
 	oval_datatype_t datatype = oval_entity_get_datatype(old_entity);
@@ -227,68 +202,52 @@ void oval_entity_free(struct oval_entity *entity)
 
 void oval_entity_set_type(struct oval_entity *entity, oval_entity_type_t type)
 {
-	if (entity && !oval_entity_is_locked(entity)) {
-		entity->type = type;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(entity);
+	entity->type = type;
 }
 
 void oval_entity_set_datatype(struct oval_entity *entity, oval_datatype_t datatype)
 {
-	if (entity && !oval_entity_is_locked(entity)) {
-		entity->datatype = datatype;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(entity);
+	entity->datatype = datatype;
 }
 
 void oval_entity_set_operation(struct oval_entity *entity, oval_operation_t operation)
 {
-	if (entity && !oval_entity_is_locked(entity)) {
-		entity->operation = operation;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(entity);
+	entity->operation = operation;
 }
 
 void oval_entity_set_mask(struct oval_entity *entity, int mask)
 {
-	if (entity && !oval_entity_is_locked(entity)) {
-		entity->mask = mask;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(entity);
+	entity->mask = mask;
 }
 
 void oval_entity_set_varref_type(struct oval_entity *entity, oval_entity_varref_type_t type)
 {
-	if (entity && !oval_entity_is_locked(entity)) {
-		entity->varref_type = type;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(entity);
+	entity->varref_type = type;
 }
 
 void oval_entity_set_variable(struct oval_entity *entity, struct oval_variable *variable)
 {
-	if (entity && !oval_entity_is_locked(entity)) {
-		entity->variable = variable;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(entity);
+	entity->variable = variable;
 }
 
 void oval_entity_set_value(struct oval_entity *entity, struct oval_value *value)
 {
-	if (entity && !oval_entity_is_locked(entity)) {
-		entity->value = value;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(entity);
+	entity->value = value;
 }
 
 void oval_entity_set_name(struct oval_entity *entity, char *name)
 {
-	if (entity && !oval_entity_is_locked(entity)) {
-		if (entity->name != NULL)
-			oscap_free(entity->name);
-		entity->name = (name == NULL) ? NULL : oscap_strdup(name);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(entity);
+	if (entity->name != NULL)
+		oscap_free(entity->name);
+	entity->name = (name == NULL) ? NULL : oscap_strdup(name);
 }
 
 static void oval_consume_varref(char *varref, void *user)
@@ -298,6 +257,7 @@ static void oval_consume_varref(char *varref, void *user)
 
 	struct oval_consume_varref_context *ctx = user;
 	*(ctx->variable) = oval_variable_get_new((struct oval_definition_model *)ctx->model, varref, OVAL_VARIABLE_UNKNOWN);
+	*(ctx->value) = oval_value_new(OVAL_DATATYPE_STRING, varref);
 }
 
 static void oval_consume_value(struct oval_value *use_value, void *value)
@@ -313,10 +273,10 @@ int oval_entity_parse_tag(xmlTextReaderPtr reader,
 	__attribute__nonnull__(context);
 
 	struct oval_entity *entity = oval_entity_new(context->definition_model);
-	int return_code;
+	int return_code = 0;
 	oval_datatype_t datatype = oval_datatype_parse(reader, "datatype", OVAL_DATATYPE_STRING);
 	oval_operation_t operation = oval_operation_parse(reader, "operation", OVAL_OPERATION_EQUALS);
-	int mask = oval_parser_boolean_attribute(reader, "mask", 1);
+	int mask = oval_parser_boolean_attribute(reader, "mask", 0);
 	oval_entity_type_t type = OVAL_ENTITY_TYPE_UNKNOWN;
 	//The value of the type field vs. the complexity of extracting type is arguable
 	char *varref = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "var_ref");
@@ -326,29 +286,31 @@ int oval_entity_parse_tag(xmlTextReaderPtr reader,
 	oval_entity_varref_type_t varref_type;
 	if (strcmp(name, "var_ref") == 0) {	//special case for <var_ref>
 		if (varref == NULL) {
-			struct oval_definition_model *model = oval_parser_context_model(context);
+			struct oval_definition_model *model = context->definition_model;
 			varref_type = OVAL_ENTITY_VARREF_ELEMENT;
-			struct oval_consume_varref_context ctx = {.model = model,.variable = &variable };
+			struct oval_consume_varref_context ctx = {.model = model, .variable = &variable, .value = &value};
 			return_code = oval_parser_text_value(reader, context, &oval_consume_varref, &ctx);
 		} else {
-			struct oval_definition_model *model = oval_parser_context_model(context);
+			struct oval_definition_model *model = context->definition_model;
 			variable = oval_definition_model_get_variable(model, varref);
 			varref_type = OVAL_ENTITY_VARREF_ATTRIBUTE;
-			return_code = 1;
 			oscap_free(varref);
 			varref = NULL;
+			value = NULL;
 		}
-		value = NULL;
 	} else if (varref == NULL) {
 		variable = NULL;
 		varref_type = OVAL_ENTITY_VARREF_NONE;
-		return_code = oval_value_parse_tag(reader, context, &oval_consume_value, &value);
+		if (datatype == OVAL_DATATYPE_RECORD) {
+			value = NULL;
+		} else {
+			return_code = oval_value_parse_tag(reader, context, &oval_consume_value, &value);
+		}
 	} else {
-		struct oval_definition_model *model = oval_parser_context_model(context);
+		struct oval_definition_model *model = context->definition_model;
 		variable = oval_variable_get_new(model, varref, OVAL_VARIABLE_UNKNOWN);
 		varref_type = OVAL_ENTITY_VARREF_ATTRIBUTE;
 		value = NULL;
-		return_code = 1;
 		oscap_free(varref);
 		varref = NULL;
 	}
@@ -361,80 +323,61 @@ int oval_entity_parse_tag(xmlTextReaderPtr reader,
 	oval_entity_set_variable(entity, variable);
 	oval_entity_set_value(entity, value);
 	(*consumer) (entity, user);
-	if (return_code != 1) {
-		oscap_dlprintf(DBG_I, "Parsing of <%s> terminated by an error at line %d.\n", name,
-			       xmlTextReaderGetParserLineNumber(reader));
+
+	if (return_code != 0) {
+		dW("Parsing of <%s> terminated by an error at line %d.\n", name, xmlTextReaderGetParserLineNumber(reader));
 	}
+
 	oscap_free(name);
 	return return_code;
 }
 
-void oval_entity_to_print(struct oval_entity *entity, char *indent, int idx)
-{
-	char nxtindent[100];
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%sENTITY.", indent);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%sENTITY[%d].", indent, idx);
-
-	oscap_dprintf("%sNAME        = %s\n", nxtindent, oval_entity_get_name(entity));
-	oscap_dprintf("%sTYPE        = %d\n", nxtindent, oval_entity_get_type(entity));
-	if (oval_entity_get_type(entity) > 10) {
-		oscap_dprintf("%s<<WARNING::TYPE OUT OF RANGE>>", nxtindent);
-	} else {
-		oscap_dprintf("%sDATATYPE    = %d\n", nxtindent, oval_entity_get_datatype(entity));
-		oscap_dprintf("%sOPERATION   = %d\n", nxtindent, oval_entity_get_operation(entity));
-		oscap_dprintf("%sMASK        = %d\n", nxtindent, oval_entity_get_mask(entity));
-		oscap_dprintf("%sVARREF_TYPE = %d\n", nxtindent, oval_entity_get_varref_type(entity));
-
-		struct oval_variable *variable = oval_entity_get_variable(entity);
-		if (variable == NULL)
-			oscap_dprintf("%sVARIABLE    = <<NOT SET>>\n", nxtindent);
-		else
-			oval_variable_to_print(variable, nxtindent, 0);
-
-		struct oval_value *value = oval_entity_get_value(entity);
-		if (value == NULL)
-			oscap_dprintf("%sVALUE       = <<NOT SET>>\n", nxtindent);
-		else
-			oval_value_to_print(value, nxtindent, 0);
-	}
-}
 
 xmlNode *oval_entity_to_dom(struct oval_entity *entity, xmlDoc * doc, xmlNode * parent) {
-	char *tagname = oval_entity_get_name(entity);
-	xmlNsPtr *ns_family = xmlGetNsList(doc, parent);
+	
+	xmlNsPtr *ns_parent = xmlGetNsList(doc, parent);
+	xmlNs *ent_ns;
+	xmlNodePtr root_node = xmlDocGetRootElement(doc);
+
+	xmlNode *entity_node = NULL;
+	char *content = NULL;
 
 	struct oval_variable *variable = oval_entity_get_variable(entity);
 	oval_entity_varref_type_t vtype = oval_entity_get_varref_type(entity);
 	struct oval_value *value = oval_entity_get_value(entity);
 
-	char *content = NULL;
-	if (variable && vtype == OVAL_ENTITY_VARREF_ELEMENT) {
+	if (vtype == OVAL_ENTITY_VARREF_ELEMENT) {
 		content = oval_variable_get_id(variable);
 	} else if (value) {
 		content = oval_value_get_text(value);
 	}
 
-	xmlNode *entity_node = xmlNewTextChild(parent, ns_family[0], BAD_CAST tagname, BAD_CAST content);
+	char *tagname = oval_entity_get_name(entity);
+	bool mask = oval_entity_get_mask(entity);
 
-	if(ns_family)
-		xmlFree(ns_family);
+	ent_ns = ns_parent ? ns_parent[0] : NULL;
+
+	/* omit the value and operation used for testing in oval_results if mask=true */
+	if(mask && !xmlStrcmp(root_node->name, (const xmlChar *) "oval_results")){
+		entity_node = xmlNewTextChild(parent, ent_ns, BAD_CAST tagname, BAD_CAST "");
+	}
+	else {
+		entity_node = xmlNewTextChild(parent, ent_ns, BAD_CAST tagname, BAD_CAST content);
+		oval_operation_t operation = oval_entity_get_operation(entity);
+		if (operation != OVAL_OPERATION_EQUALS)
+			xmlNewProp(entity_node, BAD_CAST "operation", BAD_CAST oval_operation_get_text(operation));
+	}
 
 	oval_datatype_t datatype = oval_entity_get_datatype(entity);
 	if (datatype != OVAL_DATATYPE_STRING)
 		xmlNewProp(entity_node, BAD_CAST "datatype", BAD_CAST oval_datatype_get_text(datatype));
-	oval_operation_t operation = oval_entity_get_operation(entity);
-	if (operation != OVAL_OPERATION_EQUALS)
-		xmlNewProp(entity_node, BAD_CAST "operation", BAD_CAST oval_operation_get_text(operation));
-	bool mask = oval_entity_get_mask(entity);
 	if (mask)
 		xmlNewProp(entity_node, BAD_CAST "mask", BAD_CAST "true");
 	if (vtype == OVAL_ENTITY_VARREF_ATTRIBUTE)
 		xmlNewProp(entity_node, BAD_CAST "var_ref", BAD_CAST oval_variable_get_id(variable));
+
+	if(ns_parent)
+		xmlFree(ns_parent);
+
 	return entity_node;
 }
