@@ -27,7 +27,12 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_string_map_impl.h"
 #include "oval_system_characteristics_impl.h"
@@ -50,7 +55,11 @@ static int oval_enumeration_attr(xmlTextReaderPtr reader, char *attname, const s
 static const char *oval_enumeration_get_text(const struct oscap_string_map *map, int val)
 {
 	return oscap_enum_to_string(map, val);
+}
 
+static int oval_enumeration_from_text(const struct oscap_string_map *map, const char *text)
+{
+	return oscap_string_to_enum(map, text);
 }
 
 static const struct oscap_string_map OVAL_SYSCHAR_FLAG_MAP[] = {
@@ -239,6 +248,8 @@ static const struct oscap_string_map OVAL_DATATYPE_MAP[] = {
 	{OVAL_DATATYPE_VERSION, "version"},
 	{OVAL_DATATYPE_INTEGER, "int"},
 	{OVAL_DATATYPE_STRING, "string"},
+	{OVAL_DATATYPE_RECORD, "record"},
+	{OVAL_DATATYPE_SEXP, "SEXP"},
 	{OVAL_ENUMERATION_INVALID, NULL}
 };
 
@@ -252,6 +263,11 @@ const char *oval_datatype_get_text(oval_datatype_t datatype)
 	return oval_enumeration_get_text(OVAL_DATATYPE_MAP, datatype);
 }
 
+oval_datatype_t oval_datatype_from_text(const char *text)
+{
+	return oval_enumeration_from_text(OVAL_DATATYPE_MAP, text);
+}
+
 static const struct oscap_string_map OVAL_EXISTENCE_MAP[] = {
 	{OVAL_ALL_EXIST, "all_exist"},
 	{OVAL_ANY_EXIST, "any_exist"},
@@ -407,6 +423,7 @@ static const struct oscap_string_map OVAL_SUBTYPE_INDEPENDENT_MAP[] = {
 	{OVAL_INDEPENDENT_FILE_MD5, "filemd5"},
 	{OVAL_INDEPENDENT_FILE_HASH, "filehash"},
 	{OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE, "environmentvariable"},
+	{OVAL_INDEPENDENT_SQL57, "sql57"},
 	{OVAL_INDEPENDENT_SQL, "sql"},
 	{OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54"},
 	{OVAL_INDEPENDENT_TEXT_FILE_CONTENT, "textfilecontent"},
@@ -414,6 +431,8 @@ static const struct oscap_string_map OVAL_SUBTYPE_INDEPENDENT_MAP[] = {
 	{OVAL_INDEPENDENT_VARIABLE, "variable"},
 	{OVAL_INDEPENDENT_XML_FILE_CONTENT, "xmlfilecontent"},
 	{OVAL_INDEPENDENT_SYSCHAR_SUBTYPE, "system_info"},
+	{OVAL_INDEPENDENT_LDAP57, "ldap57"},
+	{OVAL_INDEPENDENT_LDAP, "ldap"},
 	{OVAL_SUBTYPE_UNKNOWN, NULL}
 };
 
@@ -432,6 +451,11 @@ static const struct oscap_string_map OVAL_SUBTYPE_LINUX_MAP[] = {
 	{OVAL_LINUX_INET_LISTENING_SERVERS, "inetlisteningservers"},
 	{OVAL_LINUX_RPM_INFO, "rpminfo"},
 	{OVAL_LINUX_SLACKWARE_PKG_INFO_TEST, "slackwarepkginfo"},
+        {OVAL_LINUX_PARTITION, "partition"},
+        {OVAL_LINUX_IFLISTENERS, "iflisteners"},
+        {OVAL_LINUX_RPMVERIFY, "rpmverify"},
+        {OVAL_LINUX_SELINUXBOOLEAN, "selinuxboolean"},
+        {OVAL_LINUX_SELINUXSECURITYCONTEXT, "selinuxsecuritycontext"},
 	{OVAL_SUBTYPE_UNKNOWN, NULL}
 };
 
@@ -468,6 +492,12 @@ static const struct oscap_string_map OVAL_SUBTYPE_UNIX_MAP[] = {
 	{OVAL_UNIX_SHADOW, "shadow"},
 	{OVAL_UNIX_UNAME, "uname"},
 	{OVAL_UNIX_XINETD, "xinetd"},
+	{OVAL_UNIX_DNSCACHE, "dnscache"},
+        {OVAL_UNIX_SYSCTL, "sysctl"},
+	{OVAL_UNIX_PROCESS58, "process58"},
+        {OVAL_UNIX_FILEEXTENDEDATTRIBUTE, "fileextendedattribute"},
+        {OVAL_UNIX_GCONF, "gconf"},
+        {OVAL_UNIX_ROUTINGTABLE, "routingtable"},
 	{OVAL_SUBTYPE_UNKNOWN, NULL}
 };
 
@@ -566,7 +596,7 @@ oval_subtype_t oval_subtype_parse(xmlTextReaderPtr reader)
 		map = OVAL_SUBTYPE_WINDOWS_MAP;
 		break;
 	default:
-		map = NULL;
+		goto cleanup;
 	}
 
 	subtype = oscap_string_to_enum(map, tagname);
