@@ -19,6 +19,11 @@
  * Authors:
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -28,7 +33,6 @@
 # include <pthread.h>
 #endif
 
-#include "config.h"
 #include "oval_probe.h"
 #include "oval_system_characteristics.h"
 #include "common/_error.h"
@@ -50,20 +54,24 @@ oval_subtypedsc_t __s2n_tbl[] = {
         /*  7002 */ {OVAL_INDEPENDENT_FILE_MD5,             "filemd5"             },
         /*  7003 */ {OVAL_INDEPENDENT_FILE_HASH,            "filehash"            },
         /*  7004 */ {OVAL_INDEPENDENT_ENVIRONMENT_VARIABLE, "environmentvariable" },
+	/*  7005 */ {OVAL_INDEPENDENT_SQL,                  "sql"                 },
 	/*  7006 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54"   },
 	/*  7007 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT,    "textfilecontent"     },
         /*  7009 */ {OVAL_INDEPENDENT_VARIABLE,             "variable"            },
 	/*  7010 */ {OVAL_INDEPENDENT_XML_FILE_CONTENT,     "xmlfilecontent"      },
+	/*  7013 */ {OVAL_INDEPENDENT_SQL57,                "sql57"               },
 	/*  9001 */ {OVAL_LINUX_DPKG_INFO,                  "dpkginfo"            },
 	/*  9002 */ {OVAL_LINUX_INET_LISTENING_SERVERS,     "inetlisteningservers"},
 	/*  9003 */ {OVAL_LINUX_RPM_INFO,                   "rpminfo"             },
+        /*  9005 */ {OVAL_LINUX_PARTITION,                  "partition",          },
 	/* 13001 */ {OVAL_UNIX_FILE,                        "file"                },
 	/* 13003 */ {OVAL_UNIX_INTERFACE,                   "interface"           },
 	/* 13004 */ {OVAL_UNIX_PASSWORD,                    "password"            },
 	/* 13005 */ {OVAL_UNIX_PROCESS,                     "process"             },
 	/* 13006 */ {OVAL_UNIX_RUNLEVEL,                    "runlevel"            },
 	/* 13008 */ {OVAL_UNIX_SHADOW,                      "shadow"              },
-	/* 13009 */ {OVAL_UNIX_UNAME,                       "uname"               }
+	/* 13009 */ {OVAL_UNIX_UNAME,                       "uname"               },
+        /* 13012 */ {OVAL_UNIX_SYSCTL,                      "sysctl"              }
 };
 
 #define __s2n_tbl_count (sizeof __s2n_tbl / sizeof(oval_subtypedsc_t))
@@ -84,10 +92,14 @@ oval_subtypedsc_t __n2s_tbl[] = {
 	/*  9002 */ {OVAL_LINUX_INET_LISTENING_SERVERS,     "inetlisteningservers"},
 	/* 13003 */ {OVAL_UNIX_INTERFACE,                   "interface"           },
 	/* 13004 */ {OVAL_UNIX_PASSWORD,                    "password"            },
+        /*  9005 */ {OVAL_LINUX_PARTITION,                  "partition"           },
 	/* 13005 */ {OVAL_UNIX_PROCESS,                     "process"             },
 	/*  9003 */ {OVAL_LINUX_RPM_INFO,                   "rpminfo"             },
 	/* 13006 */ {OVAL_UNIX_RUNLEVEL,                    "runlevel"            },
 	/* 13008 */ {OVAL_UNIX_SHADOW,                      "shadow"              },
+	/*  7005 */ {OVAL_INDEPENDENT_SQL,                  "sql"                 },
+	/*  7013 */ {OVAL_INDEPENDENT_SQL57,                "sql57"               },
+        /* 13012 */ {OVAL_UNIX_SYSCTL,                      "sysctl"              },
         /*     2 */ {OVAL_SUBTYPE_SYSINFO,                  "system_info"         },
 	/*  7007 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT,    "textfilecontent"     },
 	/*  7006 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54"   },
@@ -110,7 +122,7 @@ static int __n2s_tbl_cmp(const char *name, oval_subtypedsc_t *dsc)
  * of memory used by this cache is done at exit using a hook
  * registered with atexit().
  */
-encache_t *OSCAP_GSYM(encache) = NULL;
+probe_ncache_t  *OSCAP_GSYM(encache) = NULL;
 struct id_desc_t OSCAP_GSYM(id_desc);
 #endif
 
@@ -331,19 +343,32 @@ int oval_probe_query_object(oval_probe_session_t *psess, struct oval_object *obj
 	oid = oval_object_get_id(object);
 	model = psess->sys_model;
 
+	dI("Querying object id: \"%s\", flags: %u.\n", oid, flags);
+
 	sysc = oval_syschar_model_get_syschar(model, oid);
 	if (sysc != NULL) {
+		oval_syschar_collection_flag_t sc_flg;
+
+		sc_flg = oval_syschar_get_flag(sysc);
+
+		dI("Syschar already exists, flag: %u, '%s'.\n", sc_flg, oval_syschar_collection_flag_get_text(sc_flg));
+
+		if (sc_flg != SYSCHAR_FLAG_UNKNOWN || (flags & OVAL_PDFLAG_NOREPLY)) {
+			if (out_syschar)
+				*out_syschar = sysc;
+			return 0;
+		}
+	} else {
+		sysc = oval_syschar_new(model, object);
 		if (out_syschar)
 			*out_syschar = sysc;
-		return 0;
 	}
 
-	sysc = oval_syschar_new(model, object);
-        type = oval_object_get_subtype(object);
-        ph   = oval_probe_handler_get(psess->ph, type);
+	type = oval_object_get_subtype(object);
+	ph = oval_probe_handler_get(psess->ph, type);
 
         if (ph == NULL) {
-                char *msg = "OVAL object not supported.\n";
+                char *msg = "OVAL object not supported.";
 
 		dW(msg);
 		oval_syschar_add_new_message(sysc, msg, OVAL_MESSAGE_LEVEL_WARNING);
@@ -363,9 +388,6 @@ int oval_probe_query_object(oval_probe_session_t *psess, struct oval_object *obj
 		oval_string_map_free(vm, NULL);
 	}
 
-	if (out_syschar)
-		*out_syschar = sysc;
-
 	return 0;
 }
 
@@ -394,30 +416,6 @@ int oval_probe_query_sysinfo(oval_probe_session_t *sess, struct oval_sysinfo **o
 
 static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_criteria_node *cnode);
 
-int oval_probe_query_objects(oval_probe_session_t *sess)
-{
-	struct oval_syschar_model * syschar_model;
-	struct oval_definition_model *definition_model;
-
-	syschar_model = sess->sys_model;
-	definition_model = oval_syschar_model_get_definition_model(syschar_model);
-
-	if (definition_model) {
-		struct oval_object_iterator *objects = oval_definition_model_get_objects(definition_model);
-		while (oval_object_iterator_has_more(objects)) {
-			int ret;
-			struct oval_object *object = oval_object_iterator_next(objects);
-			ret = oval_probe_query_object(sess, object, 0, NULL);
-			if (ret != 0) {
-				oval_object_iterator_free(objects);
-				return ret;
-			}
-		}
-		oval_object_iterator_free(objects);
-	}
-	return 0;
-}
-
 int oval_probe_query_definition(oval_probe_session_t *sess, const char *id) {
 
 	struct oval_syschar_model * syschar_model;
@@ -428,8 +426,11 @@ int oval_probe_query_definition(oval_probe_session_t *sess, const char *id) {
 	syschar_model = sess->sys_model;
         definition_model = oval_syschar_model_get_definition_model(syschar_model);
 	definition = oval_definition_model_get_definition(definition_model, id);
-	if (definition == NULL)
+	if (definition == NULL) {
+		dE("No definition with ID: %s in definition model.", id);
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "Unknown definition.");
 		return -1;
+	}
 
 	struct oval_criteria_node * cnode = oval_definition_get_criteria(definition);
 	if (cnode == NULL)
@@ -452,7 +453,7 @@ static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_cri
 		/* There should be a test .. */
 		struct oval_test *test;
 		struct oval_object *object;
-		struct oval_state *state;
+		struct oval_state_iterator *ste_itr;
 
 		test = oval_criteria_node_get_test(cnode);
 		if (test == NULL)
@@ -465,8 +466,9 @@ static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_cri
 		if (ret != 0)
 			return ret;
 		/* probe objects referenced like this: test->state->variable->object */
-		state = oval_test_get_state(test);
-		if (state != NULL) {
+		ste_itr = oval_test_get_states(test);
+		while (oval_state_iterator_has_more(ste_itr)) {
+			struct oval_state *state = oval_state_iterator_next(ste_itr);
 			struct oval_state_content_iterator *contents = oval_state_get_contents(state);
 			while (oval_state_content_iterator_has_more(contents)) {
 				struct oval_state_content *content = oval_state_content_iterator_next(contents);
@@ -478,6 +480,7 @@ static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_cri
 					ret = oval_probe_query_variable(sess, var);
 					if (ret != 0) {
 						oval_state_content_iterator_free(contents);
+						oval_state_iterator_free(ste_itr);
 						return ret;
 					}
 
@@ -488,12 +491,15 @@ static int oval_probe_query_criteria(oval_probe_session_t *sess, struct oval_cri
 						break;
 					default:
 						oval_state_content_iterator_free(contents);
+						oval_state_iterator_free(ste_itr);
 						return 0;
 					}
 				}
 			}
 			oval_state_content_iterator_free(contents);
 		}
+		oval_state_iterator_free(ste_itr);
+
 		return 0;
 
 		}
