@@ -19,12 +19,20 @@
  * Authors:
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <seap.h>
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
 #include <limits.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <stdlib.h>
+
 #include "common/_error.h"
 #include "common/public/alloc.h"
 #include "common/assume.h"
@@ -37,25 +45,34 @@
 
 #define __ERRBUF_SIZE 128
 
+#if defined(__SVR4) && defined (__sun)
+#define __STRING(x)     #x
+#endif
 /* KEEP THIS LIST SORTED! (by subtype) */
 const oval_pdsc_t OSCAP_GSYM(default_pdsc)[] = {
-        /*     2 */ {OVAL_SUBTYPE_SYSINFO,                  "system_info",       "probe_system_info"},
-	/*  7001 */ {OVAL_INDEPENDENT_FAMILY,               "family",            "probe_family"},
-        /*  7002 */ {OVAL_INDEPENDENT_FILE_MD5,             "filemd5",           "probe_filemd5"},
-        /*  7003 */ {OVAL_INDEPENDENT_FILE_HASH,            "filehash",          "probe_filehash"},
-	/*  7006 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54", "probe_textfilecontent54"},
-	/*  7007 */ {OVAL_INDEPENDENT_TEXT_FILE_CONTENT,    "textfilecontent",   "probe_textfilecontent"},
-	/*  7010 */ {OVAL_INDEPENDENT_XML_FILE_CONTENT,     "xmlfilecontent",    "probe_xmlfilecontent"},
-	/*  9001 */ {OVAL_LINUX_DPKG_INFO,                  "dpkginfo",          "probe_dpkginfo"},
-	/*  9002 */ {OVAL_LINUX_INET_LISTENING_SERVERS,     "inetlisteningservers", "probe_inetlisteningservers"},
-	/*  9003 */ {OVAL_LINUX_RPM_INFO,                   "rpminfo",           "probe_rpminfo"},
-	/* 13001 */ {OVAL_UNIX_FILE,                        "file",              "probe_file"},
-	/* 13003 */ {OVAL_UNIX_INTERFACE,                   "interface",         "probe_interface"},
-	/* 13004 */ {OVAL_UNIX_PASSWORD,                    "password",          "probe_password"},
-	/* 13005 */ {OVAL_UNIX_PROCESS,                     "process",           "probe_process"},
-	/* 13006 */ {OVAL_UNIX_RUNLEVEL,                    "runlevel",          "probe_runlevel"},
-	/* 13008 */ {OVAL_UNIX_SHADOW,                      "shadow",            "probe_shadow"},
-	/* 13009 */ {OVAL_UNIX_UNAME,                       "uname",             "probe_uname"}
+        {OVAL_SUBTYPE_SYSINFO,                  "system_info",       "probe_system_info"},
+	{OVAL_INDEPENDENT_FAMILY,               "family",            "probe_family"},
+        {OVAL_INDEPENDENT_FILE_MD5,             "filemd5",           "probe_filemd5"},
+        {OVAL_INDEPENDENT_FILE_HASH,            "filehash",          "probe_filehash"},
+        {OVAL_INDEPENDENT_SQL,                  "sql",               "probe_sql"},
+        {OVAL_INDEPENDENT_SQL57,                "sql57",             "probe_sql57"},
+	{OVAL_INDEPENDENT_TEXT_FILE_CONTENT_54, "textfilecontent54", "probe_textfilecontent54"},
+	{OVAL_INDEPENDENT_TEXT_FILE_CONTENT,    "textfilecontent",   "probe_textfilecontent"},
+	{OVAL_INDEPENDENT_XML_FILE_CONTENT,     "xmlfilecontent",    "probe_xmlfilecontent"},
+        {OVAL_INDEPENDENT_LDAP57,               "ldap57",            "probe_ldap57"},
+	{OVAL_LINUX_DPKG_INFO,                  "dpkginfo",          "probe_dpkginfo"},
+	{OVAL_LINUX_INET_LISTENING_SERVERS,     "inetlisteningservers", "probe_inetlisteningservers"},
+	{OVAL_LINUX_RPM_INFO,                   "rpminfo",           "probe_rpminfo"},
+        {OVAL_LINUX_PARTITION,                  "partition",         "probe_partition"},
+	{OVAL_UNIX_FILE,                        "file",              "probe_file"},
+	{OVAL_UNIX_INTERFACE,                   "interface",         "probe_interface"},
+	{OVAL_UNIX_PASSWORD,                    "password",          "probe_password"},
+	{OVAL_UNIX_PROCESS,                     "process",           "probe_process"},
+	{OVAL_UNIX_RUNLEVEL,                    "runlevel",          "probe_runlevel"},
+	{OVAL_UNIX_SHADOW,                      "shadow",            "probe_shadow"},
+	{OVAL_UNIX_UNAME,                       "uname",             "probe_uname"},
+        {OVAL_UNIX_DNSCACHE,                    "dnscache",          "probe_dnscache"},
+        {OVAL_UNIX_SYSCTL,                      "sysctl",            "probe_sysctl"}
 };
 
 #define DEFAULT_PDSC_COUNT (sizeof OSCAP_GSYM(default_pdsc) / sizeof (oval_pdsc_t))
@@ -77,11 +94,11 @@ oval_pext_t *oval_pext_new(void)
         pext->do_init = true;
         pthread_mutex_init(&pext->lock, NULL);
 
-//#if defined(OVAL_PROBEDIR_ENV)
+#if defined(OVAL_PROBEDIR_ENV)
         pext->probe_dir = getenv("OVAL_PROBE_DIR");
-//#else
-//        pext->probe_dir = NULL;
-//#endif
+#else
+        pext->probe_dir = NULL;
+#endif
         if (pext->probe_dir == NULL)
                 pext->probe_dir = OVAL_PROBE_DIR;
 
@@ -96,6 +113,9 @@ void oval_pext_free(oval_pext_t *pext)
 {
         if (!pext->do_init) {
                 /* free structs */
+		oscap_free(pext->pdsc);
+		pext->pdsc     = NULL;
+		pext->pdsc_cnt = 0;
                 oval_pdtbl_free(pext->pdtbl);
         }
 
@@ -241,12 +261,13 @@ static SEXP_t *oval_probe_cmd_obj_eval(SEXP_t *sexp, void *arg)
 	if (obj == NULL) {
 		oscap_dlprintf(DBG_E, "Can't find obj: id=%s.\n", id_str);
 		oscap_free(id_str);
+                SEXP_free(ret);
 
 		return (NULL);
 	}
 
 	oscap_clearerr();
-	r = oval_probe_query_object(pext->sess_ptr, obj, OVAL_PDFLAG_NOREPLY, &res);
+	r = oval_probe_query_object(pext->sess_ptr, obj, OVAL_PDFLAG_NOREPLY|OVAL_PDFLAG_SLAVE, &res);
 	if (r < 0)
 		ret_code = SEXP_number_newu((unsigned int) SYSCHAR_FLAG_COMPLETE);
 	else
@@ -260,6 +281,7 @@ static SEXP_t *oval_probe_cmd_obj_eval(SEXP_t *sexp, void *arg)
 			       id_str, oscap_err_family(), oscap_err_code(), oscap_err_desc());
 		oscap_clearerr();
 		oscap_free(id_str);
+		SEXP_free(ret);
 
 		return (NULL);
 	}
@@ -293,6 +315,7 @@ static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
 				oscap_dlprintf(DBG_E, "Can't find ste: id: %s.\n", id_str);
 				SEXP_list_free(ste_list);
 				oscap_free(id_str);
+                                SEXP_free(id);
 
 				return (NULL);
 			}
@@ -303,6 +326,7 @@ static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
 					       id_str);
 				SEXP_list_free(ste_list);
 				oscap_free(id_str);
+                                SEXP_free(id);
 
 				return (NULL);
 			}
@@ -317,7 +341,7 @@ static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
 	return (ste_list);
 }
 
-static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj, int noreply, SEXP_t **out_sexp)
+static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj, int flags, SEXP_t **out_sexp)
 {
 	int retry, ret;
 	bool aborted = false;
@@ -365,7 +389,7 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
 		s_omsg = SEAP_msg_new();
 		SEAP_msg_set(s_omsg, (SEXP_t *) s_iobj);
 
-		if (noreply) {
+		if (flags & OVAL_PDFLAG_NOREPLY) {
 			if (SEAP_msgattr_set(s_omsg, "no-reply", NULL) != 0) {
                                 protect_errno {
                                         oscap_dlprintf(DBG_E, "Can't set no-reply attribute.\n");
@@ -386,6 +410,18 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                 oscap_dlprintf(DBG_W, "Can't send message: %u, %s.\n", errno, strerror(errno));
                         }
 
+			if (flags & OVAL_PDFLAG_SLAVE) {
+                                char errbuf[__ERRBUF_SIZE];
+
+                                if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, "Unable to send a message to probe");
+                                else
+					oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, errbuf);
+
+				SEAP_msg_free(s_omsg);
+				return (-1);
+			}
+
 			if (SEAP_close(ctx, pd->sd) != 0) {
                                 char errbuf[__ERRBUF_SIZE];
 
@@ -399,6 +435,7 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                 else
                                         oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECLOSE, errbuf);
 
+				pd->sd = -1;
 				return (-1);
 			}
 
@@ -426,7 +463,7 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
 
 		oscap_dlprintf(DBG_I, "Waiting for reply.\n");
 
- recv_retry:
+		/* recv_retry: */
 		s_imsg = NULL;
 
 		ret = SEAP_recvmsg(ctx, pd->sd, &s_imsg);
@@ -443,16 +480,103 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
 				{
 					SEAP_err_t *err = NULL;
 
-					if (SEAP_recverr_byid(ctx, pd->sd, &err, SEAP_msg_id(s_omsg)) != 0)
-						goto recv_retry;
+					switch(SEAP_recverr_byid(ctx, pd->sd, &err,
+								 SEAP_msg_id(s_omsg)))
+					{
+					case  0:
+						break;
+					case  1: /* no error found */
+						dE("Internal error: An error was signaled on sd=%d but the error queue is empty.\n");
+						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBE,
+							     "SEAP_recverr_byid: internal error: empty error queue.");
+						SEAP_msg_free(s_omsg);
+						return (-1);
+					case -1: /* internal error */
+						dE("Internal error: SEAP_recverr_byid returned -1\n");
+						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBE,
+							     "SEAP_recverr_byid: internal error.");
+						SEAP_msg_free(s_omsg);
+						return (-1);
+					}
 
 					/*
 					 * decide what to do based on the error code/type
 					 */
+					switch (err->type) {
+					case SEAP_ETYPE_USER:
+					{
+						char errmsg[512] = "", *codemsg;
+
+						/*
+						 * Errors of type USER should all be from the probe "namespace" (i.e. only codes
+						 * defined at public/probe-api.h.
+						 */
+						switch (err->code) {
+						case PROBE_EINVAL: codemsg = "Invalid type, value or format";
+							break;
+						case PROBE_ENOELM: codemsg = "Missing element";
+							break;
+						case PROBE_ENOVAL: codemsg = "Missing value";
+							break;
+						case PROBE_ENOATTR: codemsg = "Missing attribute";
+							break;
+						case PROBE_EINIT: codemsg = "Initialization failed";
+							break;
+						case PROBE_ENOMEM: codemsg = "Insufficient memory";
+							break;
+						case PROBE_EOPNOTSUPP: codemsg = "Operation not supported";
+							break;
+						case PROBE_ERANGE: codemsg = "Value out of range";
+							break;
+						case PROBE_EDOM: codemsg = "Value out of domain";
+							break;
+						case PROBE_EFAULT: codemsg = "Memory fault or NULL value";
+							break;
+						case PROBE_EACCESS: codemsg = "Operation not permitted";
+							break;
+						case PROBE_ESETEVAL: codemsg = "Set evaluation failed";
+							break;
+						case PROBE_ENOENT: codemsg = "Missing entity";
+							break;
+						case PROBE_EFATAL: codemsg = "Unrecoverable error";
+							break;
+						case PROBE_EUNKNOWN:
+						default:
+							codemsg = "Unknown error";
+						}
+
+						snprintf(errmsg, sizeof errmsg, "probe at sd=%d reported an error: %s", pd->sd, codemsg);
+						dE("Received an error from probe at sd=%d: %u, \"%s\"\n", pd->sd, err->code, errmsg);
+						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBE, errmsg);
+						break;
+					}
+					case SEAP_ETYPE_INT:
+						oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EPROBE, "Internal error");
+						break;
+					}
+
+					SEAP_error_free(err);
+					SEAP_msg_free(s_omsg);
+
+					return (-1);
 				}
 				break;
 			}
 
+			if (flags & OVAL_PDFLAG_SLAVE) {
+				char errbuf[__ERRBUF_SIZE];
+
+                                if (strerror_r (errno, errbuf, sizeof errbuf - 1) != 0)
+                                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, "Unable to receive a message to probe");
+                                else
+					oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBESEND, errbuf);
+
+				SEAP_msg_free(s_imsg);
+				SEAP_msg_free(s_omsg);
+
+				return (-1);
+			}
+
 			if (SEAP_close(ctx, pd->sd) != 0) {
                                 char errbuf[__ERRBUF_SIZE];
 
@@ -467,6 +591,7 @@ static int oval_probe_comm(SEAP_CTX_t *ctx, oval_pd_t *pd, const SEXP_t *s_iobj,
                                 else
                                         oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBECLOSE, errbuf);
 
+				pd->sd = -1;
 				return (-1);
 			}
 
@@ -519,6 +644,11 @@ static int oval_pdsc_typecmp(oval_subtype_t *a, oval_pdsc_t *b)
         return (*a - b->type);
 }
 
+static int oval_pdsc_cmp(oval_pdsc_t *a, oval_pdsc_t *b)
+{
+	return (a->type - b->type);
+}
+
 static oval_pdsc_t *oval_pdsc_lookup(oval_pdsc_t pdsc[], int count, oval_subtype_t type)
 {
 	return oscap_bfind(pdsc, count, sizeof(oval_pdsc_t), &type,
@@ -590,11 +720,6 @@ static int oval_probe_sys_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, struct oval_sysch
                 SEXP_free (val);                                        \
         } while (0)
 
-	if (s_sinf == NULL) {
-		oval_sysinfo_free(sysinf);
-		return (-1);
-	}
-
 	SYSINF_EXT(s_sinf, os_name, sysinf, fail_gen);
 	SYSINF_EXT(s_sinf, os_version, sysinf, fail_gen);
 	SYSINF_EXT(s_sinf, os_architecture, sysinf, fail_gen);
@@ -689,27 +814,40 @@ int oval_probe_sys_handler(oval_subtype_t type, void *ptr, int act, ...)
         }
         case PROBE_HANDLER_ACT_OPEN:
         {
-                char         probe_uri[PATH_MAX + 1];
+                char         probe_uri[PATH_MAX + 1], errmsg[__ERRBUF_SIZE];
                 size_t       probe_urilen;
                 char        *probe_dir;
                 oval_pdsc_t *probe_dsc;
 
-                probe_dir    = pext->probe_dir;
-                probe_dsc    = oval_pdsc_lookup(pext->pdsc, pext->pdsc_cnt, type);
+                probe_dir = pext->probe_dir;
+                probe_dsc = oval_pdsc_lookup(pext->pdsc, pext->pdsc_cnt, type);
+
+		if (probe_dsc == NULL) {
+			snprintf (errmsg, sizeof errmsg, "subtype %u not supported", type);
+			oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
+
+			ret = -1;
+			break;
+		}
+
                 probe_urilen = snprintf(probe_uri, sizeof probe_uri,
                                         "%s://%s/%s", OVAL_PROBE_SCHEME, probe_dir, probe_dsc->file);
 
-                oscap_dlprintf(DBG_I, "URI: %s.\n", probe_uri);
-
-                if (oval_pdtbl_add(pext->pdtbl, type, -1, probe_uri) != 0) {
-                        char errmsg[__ERRBUF_SIZE];
-
-                        snprintf (errmsg, sizeof errmsg, "%s probe not supported", probe_dsc->name);
-                        oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
+                if (probe_urilen >= sizeof probe_uri) {
+                        snprintf (errmsg, sizeof errmsg, "probe URI too long");
+                        oscap_seterr (OSCAP_EFAMILY_GLIBC, ENAMETOOLONG, errmsg);
 
                         ret = -1;
-                }
+                } else {
+                        oscap_dlprintf(DBG_I, "URI: %s.\n", probe_uri);
 
+                        if (oval_pdtbl_add(pext->pdtbl, type, -1, probe_uri) != 0) {
+                                snprintf (errmsg, sizeof errmsg, "%s probe not supported", probe_dsc->name);
+                                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
+
+                                ret = -1;
+                        }
+                }
                 break;
         }
         case PROBE_HANDLER_ACT_INIT:
@@ -746,27 +884,47 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                 pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
 
                 if (pd == NULL) {
-                        char         probe_uri[PATH_MAX + 1];
+                        char         probe_uri[PATH_MAX + 1], errmsg[__ERRBUF_SIZE];
                         size_t       probe_urilen;
                         char        *probe_dir;
                         oval_pdsc_t *probe_dsc;
 
-                        probe_dir    = pext->probe_dir;
-                        probe_dsc    = oval_pdsc_lookup(pext->pdsc, pext->pdsc_cnt, oval_object_get_subtype(obj));
+                        probe_dir = pext->probe_dir;
+                        probe_dsc = oval_pdsc_lookup(pext->pdsc, pext->pdsc_cnt, oval_object_get_subtype(obj));
+
+			if (probe_dsc == NULL) {
+				snprintf (errmsg, sizeof errmsg, "subtype %u not supported", oval_object_get_subtype(obj));
+				oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
+
+				return (-1);
+			}
+
                         probe_urilen = snprintf(probe_uri, sizeof probe_uri,
                                                 "%s://%s/%s", OVAL_PROBE_SCHEME, probe_dir, probe_dsc->file);
 
+                        if (probe_urilen >= sizeof probe_uri) {
+                                snprintf (errmsg, sizeof errmsg, "probe URI too long");
+                                oscap_seterr (OSCAP_EFAMILY_GLIBC, ENAMETOOLONG, errmsg);
+
+                                return (-1);
+                        }
+
                         oscap_dlprintf(DBG_I, "URI: %s.\n", probe_uri);
 
                         if (oval_pdtbl_add(pext->pdtbl, oval_object_get_subtype(obj), -1, probe_uri) != 0) {
-                                char errmsg[__ERRBUF_SIZE];
 
                                 snprintf (errmsg, sizeof errmsg, "%s probe not supported", probe_dsc->name);
                                 oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, errmsg);
 
                                 return (-1);
 			}
+
 			pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
+
+                        if (pd == NULL) {
+                                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "internal error");
+                                return (-1);
+                        }
                 }
 
 		ret = oval_probe_ext_eval(pext->pdtbl->ctx, pd, pext, sys, flags);
@@ -775,18 +933,20 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
 			ret = 0;
 
 		if (ret < 0 && errno == ECONNABORTED) {
-			if (!pext->do_init) {
-				oval_pdtbl_free(pext->pdtbl);
+			if (!(flags & OVAL_PDFLAG_SLAVE)) {
+				if (!pext->do_init) {
+					oval_pdtbl_free(pext->pdtbl);
+				}
+
+				pext->do_init  = true;
+				pext->pdtbl    = NULL;
+				pext->pdsc     = NULL;
+				pext->pdsc_cnt = 0;
+
+				oval_probe_ext_init(pext);
+
+				errno = ECONNABORTED;
 			}
-
-			pext->do_init  = true;
-			pext->pdtbl    = NULL;
-			pext->pdsc     = NULL;
-			pext->pdsc_cnt = 0;
-
-			oval_probe_ext_init(pext);
-
-			errno = ECONNABORTED;
 		}
 
 		return ret;
@@ -858,16 +1018,64 @@ int oval_probe_ext_init(oval_pext_t *pext)
         int ret = 0;
 
         pthread_mutex_lock(&pext->lock);
+
         if (pext->do_init) {
-                pext->pdsc     = (oval_pdsc_t *)OSCAP_GSYM(default_pdsc);
-                pext->pdsc_cnt = DEFAULT_PDSC_COUNT;
-                pext->pdtbl    = oval_pdtbl_new();
+		char curdir[PATH_MAX];
+		struct stat st;
+		register unsigned int i, r;
+
+		if (getcwd(curdir, PATH_MAX) == NULL) {
+			dE("getcwd() failed\n");
+                        ret = -1;
+                        goto _ret;
+		}
+
+		if (chdir(pext->probe_dir) != 0) {
+			dE("Can't chdir to \"%s\"\n", pext->probe_dir);
+                        ret = -1;
+                        goto _ret;
+		}
+
+		pext->pdsc = oscap_alloc(sizeof(oval_pdsc_t) * DEFAULT_PDSC_COUNT);
+
+		for (r = 0, i = 0; i < DEFAULT_PDSC_COUNT; ++i) {
+			if (stat((OSCAP_GSYM(default_pdsc)[i]).file, &st) != 0) {
+				dW("skipped: %s (stat failed, errno=%d)\n", OSCAP_GSYM(default_pdsc)[i].name, errno);
+				continue;
+			}
+
+			if (!S_ISREG(st.st_mode)) {
+				dW("skipped: %s (not a regular file)\n", OSCAP_GSYM(default_pdsc)[i].name);
+				continue;
+			}
+
+			memcpy(pext->pdsc + r, OSCAP_GSYM(default_pdsc) + i, sizeof(oval_pdsc_t));
+			++r;
+		}
+
+		if (r < DEFAULT_PDSC_COUNT)
+			pext->pdsc = oscap_realloc(pext->pdsc, sizeof(oval_pdsc_t) * r);
+
+		pext->pdsc_cnt = r;
+		qsort(pext->pdsc, pext->pdsc_cnt, sizeof(oval_pdsc_t),
+		      (int(*)(const void *, const void *))oval_pdsc_cmp);
+
+		if (chdir(curdir) != 0) {
+			dE("Can't chdir back to \"%s\"\n", curdir);
+			oscap_free(pext->pdsc);
+			pext->pdsc_cnt = 0;
+                        ret = -1;
+                        goto _ret;
+		}
+
+                pext->pdtbl = oval_pdtbl_new();
 
                 if (oval_probe_cmd_init(pext) != 0)
                         ret = -1;
                 else
                         pext->do_init = false;
         }
+_ret:
         pthread_mutex_unlock(&pext->lock);
 
         return(ret);
@@ -890,7 +1098,7 @@ int oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struc
 	if (ret != 0)
 		return (1);
 
-	ret = oval_probe_comm(ctx, pd, s_obj, flags & OVAL_PDFLAG_NOREPLY, &s_sys);
+	ret = oval_probe_comm(ctx, pd, s_obj, flags, &s_sys);
 	SEXP_free(s_obj);
 
 	if (ret != 0) {
@@ -965,7 +1173,10 @@ int oval_probe_ext_abort(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext)
 		sch_pipedata_t *pipeinfo = (sch_pipedata_t *)dsc->scheme_data;
 
 		dI("Sending SIGUSR1 to pid=%u\n", pipeinfo->pid);
-		kill(pipeinfo->pid, SIGUSR1);
+
+		if (kill(pipeinfo->pid, SIGUSR1) != 0)
+			dW("kill(SIGUSR1, %u): %u, %s\n", errno, strerror(errno));
+
 		break;
 	}
 	default:
