@@ -28,12 +28,10 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
-#include "config.h"
 #pragma once
 #ifndef OVAL_PROBE_IMPL_H
 #define OVAL_PROBE_IMPL_H
 
-#include <config.h>
 #ifdef ENABLE_PROBES
 
 #include <seap-types.h>
@@ -58,11 +56,11 @@ OSCAP_HIDDEN_START;
 # define OVAL_PROBE_DIR    "/usr/libexec/openscap"
 #endif
 
-#define OVAL_PROBE_MAXRETRY 3
+#define OVAL_PROBE_MAXRETRY 0
 
 OSCAP_HIDDEN_END;
 
-extern encache_t *OSCAP_GSYM(encache);
+extern probe_ncache_t *OSCAP_GSYM(encache);
 
 #endif /* ENABLE_PROBES */
 
