@@ -27,10 +27,15 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
+
 #include "oval_results_impl.h"
 #include "oval_collection_impl.h"
 #include "common/util.h"
@@ -70,10 +75,11 @@ typedef struct oval_result_criteria_node_EXTENDDEF {
 	struct oval_result_definition *extends;
 } oval_result_criteria_node_EXTENDDEF_t;
 
-struct oval_result_criteria_node *oval_result_criteria_node_new
-    (struct oval_result_system *sys, oval_criteria_node_type_t type, int negate, ...) {
+struct oval_result_criteria_node *oval_result_criteria_node_new(struct oval_result_system *sys, oval_criteria_node_type_t type, int negate, ...) {
+
 	oval_result_criteria_node_t *node = NULL;
 	va_list ap;
+
 	va_start(ap, negate);
 	switch (type) {
 	case OVAL_NODETYPE_CRITERIA:{
@@ -114,7 +120,9 @@ struct oval_result_criteria_node *oval_result_criteria_node_new
 			extenddef->variable_instance = va_arg(ap, int);
 		} break;
 	default:
-		break;
+		va_end(ap);
+		dE("Unsupported criteria node type: %d.\n", type);
+		return NULL;
 	}
 	node->sys = sys;
 	node->negate = negate;
@@ -124,73 +132,6 @@ struct oval_result_criteria_node *oval_result_criteria_node_new
 	return node;
 }
 
-bool oval_result_criteria_node_is_valid(struct oval_result_criteria_node * result_criteria_node)
-{
-	bool is_valid = true;
-	oval_criteria_node_type_t type;
-
-	if (result_criteria_node == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-	type = oval_result_criteria_node_get_type(result_criteria_node);
-	switch (type) {
-	case OVAL_NODETYPE_CRITERIA:
-		{
-			struct oval_result_criteria_node_iterator *subnodes_itr;
-
-			/* validate subnodes */
-			subnodes_itr = oval_result_criteria_node_get_subnodes(result_criteria_node);
-			while (oval_result_criteria_node_iterator_has_more(subnodes_itr)) {
-				struct oval_result_criteria_node *subnode;
-
-				subnode = oval_result_criteria_node_iterator_next(subnodes_itr);
-				if (oval_result_criteria_node_is_valid(subnode) != true) {
-					is_valid = false;
-					break;
-				}
-			}
-			oval_result_criteria_node_iterator_free(subnodes_itr);
-			if (is_valid != true)
-				return false;
-		}
-		break;
-	case OVAL_NODETYPE_CRITERION:
-		{
-			struct oval_result_test *rslt_test;
-
-			/* validate test */
-			rslt_test = oval_result_criteria_node_get_test(result_criteria_node);
-			if (oval_result_test_is_valid(rslt_test) != true)
-				return false;
-		}
-		break;
-	case OVAL_NODETYPE_EXTENDDEF:
-		{
-			struct oval_result_definition *rslt_def;
-
-			/* validate definition */
-			rslt_def = oval_result_criteria_node_get_extends(result_criteria_node);
-			if (oval_result_definition_is_valid(rslt_def) != true)
-				return false;
-		}
-		break;
-	default:
-                oscap_dlprintf(DBG_W, "Argument is not valid: wrong node type: %d.\n", type);
-		return false;
-	}
-
-	return true;
-}
-
-bool oval_result_criteria_node_is_locked(struct oval_result_criteria_node * result_criteria_node)
-{
-	__attribute__nonnull__(result_criteria_node);
-
-	return oval_result_system_is_locked(result_criteria_node->sys);
-}
-
 struct oval_result_criteria_node *oval_result_criteria_node_clone
     (struct oval_result_system *new_system, struct oval_result_criteria_node *old_node) {
 	oval_criteria_node_type_t type = oval_result_criteria_node_get_type(old_node);
@@ -287,7 +228,7 @@ struct oval_result_criteria_node *make_result_criteria_node_from_oval_criteria_n
 			} break;
 		case OVAL_NODETYPE_CRITERION:{
 				struct oval_test *oval_test = oval_criteria_node_get_test(oval_node);
-				struct oval_result_test *rslt_test = get_oval_result_test_new(sys, oval_test);
+				struct oval_result_test *rslt_test = oval_result_system_get_new_test(sys, oval_test);
 				rslt_node = oval_result_criteria_node_new(sys, type, negate, rslt_test, 1);
 			} break;
 		case OVAL_NODETYPE_EXTENDDEF:{
@@ -326,9 +267,7 @@ void oval_result_criteria_node_iterator_free(struct
 				      oc_result_criteria_node);
 }
 
-oval_criteria_node_type_t oval_result_criteria_node_get_type(struct
-							     oval_result_criteria_node
-							     *node)
+oval_criteria_node_type_t oval_result_criteria_node_get_type(struct oval_result_criteria_node *node)
 {
 	__attribute__nonnull__(node);
 
@@ -336,6 +275,13 @@ oval_criteria_node_type_t oval_result_criteria_node_get_type(struct
 	return type;
 }
 
+static struct oval_result_system *oval_result_criteria_get_system(struct oval_result_criteria_node *node)
+{
+        __attribute__nonnull__(node);
+
+        return node->sys;
+}
+
 static oval_result_t _oval_result_negate(bool negate, oval_result_t result)
 {
 	return (negate && result == OVAL_RESULT_TRUE) ? OVAL_RESULT_FALSE :
@@ -447,110 +393,97 @@ struct oval_result_definition *oval_result_criteria_node_get_extends(struct oval
 	return extends;
 }
 
-void oval_result_criteria_node_set_result(struct oval_result_criteria_node *node, oval_result_t result) {
-	if (node && !oval_result_criteria_node_is_locked(node)) {
-		node->result = result;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_criteria_node_set_result(struct oval_result_criteria_node *node, oval_result_t result) 
+{
+	__attribute__nonnull__(node);
+	node->result = result;
 }
 
-void oval_result_criteria_node_set_negate(struct oval_result_criteria_node *node, bool negate) {
-	if (node && !oval_result_criteria_node_is_locked(node)) {
-		node->negate = negate;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_criteria_node_set_negate(struct oval_result_criteria_node *node, bool negate) 
+{
+	__attribute__nonnull__(node);
+	node->negate = negate;
 }
 
-void oval_result_criteria_node_set_operator(struct oval_result_criteria_node *node, oval_operator_t operator) {
-	if (node && !oval_result_criteria_node_is_locked(node)) {
-		/*type==NODETYPE_CRITERIA */
-		if (oval_result_criteria_node_get_type(node) == OVAL_NODETYPE_CRITERIA) {
-			((struct oval_result_criteria_node_CRITERIA *)node)->operator = operator;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_criteria_node_add_subnode(struct oval_result_criteria_node *node, struct oval_result_criteria_node *subnode) 
+{
+	__attribute__nonnull__(node);
+
+       /*type==NODETYPE_CRITERIA */
+	if (oval_result_criteria_node_get_type(node) == OVAL_NODETYPE_CRITERIA) {
+		struct oval_result_criteria_node_CRITERIA *criteria = ((struct oval_result_criteria_node_CRITERIA *)node);
+		oval_collection_add(criteria->subnodes, subnode);
+	}
 }
 
-void oval_result_criteria_node_add_subnode
-    (struct oval_result_criteria_node *node, struct oval_result_criteria_node *subnode) {
-	if (node && !oval_result_criteria_node_is_locked(node)) {
-		/*type==NODETYPE_CRITERIA */
-		if (oval_result_criteria_node_get_type(node) == OVAL_NODETYPE_CRITERIA) {
-			struct oval_result_criteria_node_CRITERIA *criteria
-			    = ((struct oval_result_criteria_node_CRITERIA *)node);
-			oval_collection_add(criteria->subnodes, subnode);
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_criteria_node_set_operator(struct oval_result_criteria_node *node, oval_operator_t operator) 
+{
+	__attribute__nonnull__(node);
+	/*type==NODETYPE_CRITERIA */
+	if (oval_result_criteria_node_get_type(node) == OVAL_NODETYPE_CRITERIA) {
+		((struct oval_result_criteria_node_CRITERIA *)node)->operator = operator;
+	}
 }
 
-void oval_result_criteria_node_set_test(struct oval_result_criteria_node *node, struct oval_result_test *test) {
-	if (node && !oval_result_criteria_node_is_locked(node)) {
-		/*type==NODETYPE_CRITERION */
-		if (oval_result_criteria_node_get_type(node) == OVAL_NODETYPE_CRITERION) {
-			((struct oval_result_criteria_node_CRITERION *)node)->test = test;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_criteria_node_set_test(struct oval_result_criteria_node *node, struct oval_result_test *test) 
+{
+	__attribute__nonnull__(node);
+	/*type==NODETYPE_CRITERION */
+	if (oval_result_criteria_node_get_type(node) == OVAL_NODETYPE_CRITERION) {
+		((struct oval_result_criteria_node_CRITERION *)node)->test = test;
+	}
 }
 
-void oval_result_criteria_node_set_extends
-    (struct oval_result_criteria_node *node, struct oval_result_definition *extends) {
-	if (node && !oval_result_criteria_node_is_locked(node)) {
-		/*type==NODETYPE_EXTENDDEF */
-		if (oval_result_criteria_node_get_type(node) == OVAL_NODETYPE_EXTENDDEF) {
-			extends = ((struct oval_result_criteria_node_EXTENDDEF *)node)->extends = extends;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_criteria_node_set_extends(struct oval_result_criteria_node *node, struct oval_result_definition *extends) 
+{
+	__attribute__nonnull__(node);
+	/*type==NODETYPE_EXTENDDEF */
+	if (oval_result_criteria_node_get_type(node) == OVAL_NODETYPE_EXTENDDEF) {
+		extends = ((struct oval_result_criteria_node_EXTENDDEF *)node)->extends = extends;
+	}
 }
 
-static void _oval_result_criteria_consume_subnode
-    (struct oval_result_criteria_node *subnode, struct oval_result_criteria_node *node) {
+static void _oval_result_criteria_consume_subnode(struct oval_result_criteria_node *subnode, struct oval_result_criteria_node *node) {
 	oval_result_criteria_node_add_subnode(node, subnode);
 }
 
-static int _oval_result_criteria_parse(xmlTextReaderPtr reader, struct oval_parser_context *context, void **args) {
-	struct oval_result_system *sys = (struct oval_result_system *)args[0];
-	struct oval_result_criteria_node *node = (struct oval_result_criteria_node *)args[1];
-	return oval_result_criteria_node_parse
-	    (reader, context, sys, (oscap_consumer_func) _oval_result_criteria_consume_subnode, node);
+static int _oval_result_criteria_parse(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr) {
+	struct oval_result_system *sys = oval_result_criteria_get_system(usr);
+	return oval_result_criteria_node_parse(reader, context, sys, (oscap_consumer_func) _oval_result_criteria_consume_subnode, usr);
 }
 
-int oval_result_criteria_node_parse
-    (xmlTextReaderPtr reader, struct oval_parser_context *context,
-     struct oval_result_system *sys, oscap_consumer_func consumer, void *client) {
-	int return_code = 1;
+int oval_result_criteria_node_parse(xmlTextReaderPtr reader, struct oval_parser_context *context, struct oval_result_system *sys, oscap_consumer_func consumer, void *client) 
+{
+	int return_code = 0;
 	xmlChar *localName = xmlTextReaderLocalName(reader);
 
-	oscap_dlprintf(DBG_I, "Parsing <%s>.\n", localName);
-
 	struct oval_result_criteria_node *node = NULL;
 	if (strcmp((const char *)localName, "criteria") == 0) {
 		oval_operator_t operator = oval_operator_parse(reader, "operator", OVAL_OPERATOR_UNKNOWN);
 		int negate = oval_parser_boolean_attribute(reader, "negate", false);
 		node = oval_result_criteria_node_new(sys, OVAL_NODETYPE_CRITERIA, negate, operator);
-		void *args[] = { sys, node };
-		return_code = oval_parser_parse_tag
-		    (reader, context, (oval_xml_tag_parser) _oval_result_criteria_parse, args);
+		return_code = oval_parser_parse_tag(reader, context, _oval_result_criteria_parse, node);
 	} else if (strcmp((const char *)localName, "criterion") == 0) {
 		xmlChar *test_ref = xmlTextReaderGetAttribute(reader, BAD_CAST "test_ref");
 		int version = oval_parser_int_attribute(reader, "version", 0);
 		int variable_instance = oval_parser_int_attribute(reader, "variable_instance", 1);
 		int negate = oval_parser_boolean_attribute(reader, "negate", false);
+
 		struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
 		struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
 		struct oval_test *oval_test = oval_definition_model_get_test(definition_model, (char *)test_ref);
-		struct oval_result_test *rslt_test = (oval_test)
-		    ? get_oval_result_test_new(sys, oval_test) : NULL;
-		int test_vsn = oval_test_get_version(oval_test);
-		if (test_vsn != version) {
-			oscap_dlprintf(DBG_W, "Unmatched test versions: "
-				       "version: %d, criteria version: %d.\n", test_vsn, version);
+		struct oval_result_test *rslt_test = NULL;
+
+		if (oval_test != NULL) {
+			int tst_ver;
+
+			rslt_test = oval_result_system_get_new_test(sys, oval_test);
+			tst_ver = oval_test_get_version(oval_test);
+			if (tst_ver != version)
+				dW("Unmatched versions: test: %d, criteria: %d.\n", tst_ver, version);
 		}
-		node = oval_result_criteria_node_new
-		    (sys, OVAL_NODETYPE_CRITERION, negate, rslt_test, variable_instance);
-		return_code = 1;
+
+		node = oval_result_criteria_node_new(sys, OVAL_NODETYPE_CRITERION, negate, rslt_test, variable_instance);
 		oscap_free(test_ref);
 	} else if (strcmp((const char *)localName, "extend_definition") == 0) {
 		xmlChar *definition_ref = xmlTextReaderGetAttribute(reader, BAD_CAST "definition_ref");
@@ -558,26 +491,23 @@ int oval_result_criteria_node_parse
 		int negate = oval_parser_boolean_attribute(reader, "negate", false);
 		struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
 		struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
-		struct oval_definition *oval_definition
-		    = oval_definition_model_get_definition(definition_model, (char *)definition_ref);
-		struct oval_result_definition *rslt_definition = (oval_definition)
-		    ? oval_result_system_get_new_definition(sys, oval_definition) : NULL;
-		node = (rslt_definition)
-		    ? oval_result_criteria_node_new
-		    (sys, OVAL_NODETYPE_EXTENDDEF, negate, rslt_definition, variable_instance) : NULL;
-		return_code = 1;
+		struct oval_definition *oval_definition = oval_definition_model_get_definition(definition_model, (char *)definition_ref);
+		struct oval_result_definition *rslt_definition = (oval_definition) ? oval_result_system_get_new_definition(sys, oval_definition) : NULL;
+		node = (rslt_definition) ? oval_result_criteria_node_new(sys, OVAL_NODETYPE_EXTENDDEF, negate, rslt_definition, variable_instance) : NULL;
 		oscap_free(definition_ref);
 	} else {
-		oscap_dlprintf(DBG_E, "unhandled criteria node: <%s>.\n",
-			       (char *)localName);
+		dW("unhandled criteria node: <%s>.\n",(char *)localName);
 		oval_parser_skip_tag(reader, context);
-		return_code = 0;
+		return_code = 1;
 	}
-	if (return_code) {
+
+	if (return_code==0) {
 		oval_result_t result = oval_result_parse(reader, "result", 0);
 		oval_result_criteria_node_set_result(node, result);
 	}
+
 	(*consumer) (node, client);
+
 	oscap_free(localName);
 	return return_code;
 }
@@ -630,9 +560,15 @@ static xmlNode *_oval_result_EXTENDDEF_to_dom(struct oval_result_criteria_node *
 
 	struct oval_result_definition *rslt_definition = oval_result_criteria_node_get_extends(node);
 	struct oval_definition *oval_definition = oval_result_definition_get_definition(rslt_definition);
+
 	char *definition_ref = oval_definition_get_id(oval_definition);
 	xmlNewProp(node_root, BAD_CAST "definition_ref", BAD_CAST definition_ref);
 
+        int version = oval_definition_get_version(oval_definition);
+        char version_att[10] = "";
+        snprintf(version_att, sizeof(version_att), "%d", version);
+        xmlNewProp(node_root, BAD_CAST "version", BAD_CAST version_att);
+
         int instance = ((struct oval_result_criteria_node_EXTENDDEF *) node)->variable_instance;
         if (instance != 1) {
                 char instance_att[10] = "";
