@@ -27,9 +27,14 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_results_impl.h"
 #include "oval_collection_impl.h"
 #include "public/oval_agent_api.h"
@@ -76,41 +81,6 @@ struct oval_result_definition *oval_result_definition_new(struct oval_result_sys
 	return definition;
 }
 
-bool oval_result_definition_is_valid(struct oval_result_definition * result_definition)
-{
-	struct oval_definition *definition;
-	struct oval_result_criteria_node *rslt_criteria_node;
-
-	if (result_definition == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-	if (oval_result_definition_get_system(result_definition) == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: system == NULL.\n");
-		return false;
-        }
-
-	/* validate definition */
-	definition = oval_result_definition_get_definition(result_definition);
-	if (oval_definition_is_valid(definition) != true)
-		return false;
-
-	/* validate criteria */
-	rslt_criteria_node = oval_result_definition_get_criteria(result_definition);
-	if (oval_result_criteria_node_is_valid(rslt_criteria_node) != true)
-		return false;
-
-	return true;
-}
-
-bool oval_result_definition_is_locked(struct oval_result_definition * result_definition)
-{
-	__attribute__nonnull__(result_definition);
-
-	return oval_result_system_is_locked(result_definition->system);
-}
-
 struct oval_result_definition *oval_result_definition_clone
     (struct oval_result_system *new_system, struct oval_result_definition *old_definition) {
 	struct oval_definition *ovaldef = oval_result_definition_get_definition(old_definition);
@@ -203,86 +173,81 @@ oval_result_t oval_result_definition_get_result(struct oval_result_definition *
 	return definition->result;
 }
 
-struct oval_message_iterator *oval_result_definition_get_messages(struct oval_result_definition *definition) {
+struct oval_message_iterator *oval_result_definition_get_messages(struct oval_result_definition *definition) 
+{
 	__attribute__nonnull__(definition);
 
 	return (struct oval_message_iterator *)
 	    oval_collection_iterator(definition->messages);
 }
 
-struct oval_result_criteria_node *oval_result_definition_get_criteria(struct oval_result_definition *definition) {
+struct oval_result_criteria_node *oval_result_definition_get_criteria(struct oval_result_definition *definition) 
+{
 	__attribute__nonnull__(definition);
 
 	return definition->criteria;
 }
 
-void oval_result_definition_set_result(struct oval_result_definition *definition, oval_result_t result) {
-	if (definition && !oval_result_definition_is_locked(definition)) {
-		definition->result = result;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_definition_set_result(struct oval_result_definition *definition, oval_result_t result) 
+{
+	__attribute__nonnull__(definition);
+	definition->result = result;
 }
 
-void oval_result_definition_set_instance(struct oval_result_definition *definition, int instance) {
-	if (definition && !oval_result_definition_is_locked(definition)) {
-		definition->instance = instance;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_definition_set_instance(struct oval_result_definition *definition, int instance) 
+{
+	__attribute__nonnull__(definition);
+	definition->instance = instance;
 }
 
-void oval_result_definition_set_criteria
-    (struct oval_result_definition *definition, struct oval_result_criteria_node *criteria) {
-	if (definition && !oval_result_definition_is_locked(definition)) {
-		if (definition->criteria) {
-			if (oval_result_criteria_node_get_type(criteria) == OVAL_NODETYPE_CRITERIA) {
-				oval_result_criteria_node_free(definition->criteria);
-			}
+void oval_result_definition_set_criteria(struct oval_result_definition *definition, struct oval_result_criteria_node *criteria) 
+{
+	__attribute__nonnull__(definition);
+	if (definition->criteria) {
+		if (oval_result_criteria_node_get_type(criteria) == OVAL_NODETYPE_CRITERIA) {
+			oval_result_criteria_node_free(definition->criteria);
 		}
-		definition->criteria = criteria;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	}
+	definition->criteria = criteria;
 }
 
 void oval_result_definition_add_message(struct oval_result_definition *definition, struct oval_message *message) {
-	if (definition && !oval_result_definition_is_locked(definition)) {
-		if (message)
-			oval_collection_add(definition->messages, message);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(definition);
+	if (message)
+		oval_collection_add(definition->messages, message);
 }
 
-static void _oval_result_definition_consume_criteria
-    (struct oval_result_criteria_node *node, struct oval_result_definition *definition) {
+static void _oval_result_definition_consume_criteria(struct oval_result_criteria_node *node, struct oval_result_definition *definition) {
 	oval_result_definition_set_criteria(definition, node);
 }
 
-static void _oval_result_definition_consume_message
-    (struct oval_message *message, struct oval_result_definition *definition) {
+static void _oval_result_definition_consume_message(struct oval_message *message, struct oval_result_definition *definition) {
 	oval_result_definition_add_message(definition, message);
 }
 
-static int _oval_result_definition_parse
-    (xmlTextReaderPtr reader, struct oval_parser_context *context, struct oval_result_definition *definition) {
-	int return_code = 1;
+static int oval_result_definition_parse(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr) {
+	int return_code = 0;
 	xmlChar *localName = xmlTextReaderLocalName(reader);
-
+	struct oval_result_system  *sys = oval_result_definition_get_system((struct oval_result_definition *) usr);
+	
 	if (strcmp((const char *)localName, "criteria") == 0) {
 		return_code = oval_result_criteria_node_parse
-		    (reader, context, oval_result_definition_get_system(definition),
-		     (oscap_consumer_func) _oval_result_definition_consume_criteria, definition);
+		    (reader, context, sys, (oscap_consumer_func) _oval_result_definition_consume_criteria, usr);
 	} else if (strcmp((const char *)localName, "message") == 0) {
 		return_code = oval_message_parse_tag
-		    (reader, context, (oscap_consumer_func) _oval_result_definition_consume_message, definition);
+		    (reader, context, (oscap_consumer_func) _oval_result_definition_consume_message, usr);
 	}
-
+	oscap_free(localName);
 	return return_code;
 }
 
-int oval_result_definition_parse
-    (xmlTextReaderPtr reader, struct oval_parser_context *context,
-     struct oval_result_system *sys, oscap_consumer_func consumer, void *client) {
-	int return_code = 1;
+int oval_result_definition_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr) {
 
+	struct oval_result_system *sys = (struct oval_result_system *) usr;
+	int return_code = 0;
+	struct oval_definition_model *dmod;
+	struct oval_definition *ddef;
+	struct oval_result_definition *definition;
 	xmlChar *definition_id = xmlTextReaderGetAttribute(reader, BAD_CAST "definition_id");
 	xmlChar *definition_version = xmlTextReaderGetAttribute(reader, BAD_CAST "version");
 	int resvsn = atoi((char *)definition_version);
@@ -291,15 +256,15 @@ int oval_result_definition_parse
 
 	int instance = oval_parser_int_attribute(reader, "variable_instance", 1);
 
-	struct oval_result_definition *definition = oval_result_definition_new(sys, (char *)definition_id);
+	dmod = context->definition_model;
+	ddef = oval_definition_get_new(dmod, (char *) definition_id);
+	definition = oval_result_system_get_new_definition(sys, ddef);
 	if (definition == NULL)
 		return -1;
 
 	int defvsn = oval_definition_get_version(definition->definition);
 	if (defvsn && resvsn != defvsn) {
-		oscap_dlprintf(DBG_W, "Definition versions don't match: "
-			       "definition id: %s, "
-			       "ovaldef vsn: %d, resdef vsn: %d.\n", definition_id, defvsn, resvsn);
+		dW("Definition versions don't match: definition id: %s, ovaldef vsn: %d, resdef vsn: %d.\n", definition_id, defvsn, resvsn);
 	}
 	oval_definition_set_version(definition->definition, resvsn);
 	oval_result_definition_set_instance(definition, instance);
@@ -308,23 +273,15 @@ int oval_result_definition_parse
 	if ((int)result != OVAL_ENUMERATION_INVALID) {
 		oval_result_definition_set_result(definition, result);
 	} else {
-		oscap_dlprintf(DBG_W, "Can't resolve result attribute, "
-			       "definition id: %s.\n", definition_id);
+		dW("Can't resolve result attribute, definition id: %s.\n", definition_id);
 		oval_result_definition_set_result(definition, OVAL_RESULT_UNKNOWN);
 	}
 
-	//Process tag contents
-	oscap_dlprintf(DBG_I, "Processing <definition> contents: "
-		       "id: %s, vsn: %d, "
-		       "result: %d.\n", definition_id, defvsn, result);
-
-	return_code = oval_parser_parse_tag
-	    (reader, context, (oval_xml_tag_parser) _oval_result_definition_parse, definition);
+	return_code = oval_parser_parse_tag(reader, context, oval_result_definition_parse, definition);
 
 	oscap_free(definition_id);
 	oscap_free(definition_version);
 
-	(*consumer) (definition, client);
 	return return_code;
 }
 
@@ -343,15 +300,13 @@ xmlNode *oval_result_definition_to_dom
 	xmlNewProp(definition_node, BAD_CAST "result", BAD_CAST result_att);
 
 	int version = oval_definition_get_version(oval_definition);
-	char version_att[10];
-	*version_att = '\0';
+	char version_att[10] = "";
 	snprintf(version_att, sizeof(version_att), "%d", version);
 	xmlNewProp(definition_node, BAD_CAST "version", BAD_CAST version_att);
 
 	int instance = oval_result_definition_get_instance(definition);
 	if (instance != 1) {
-		char instance_att[10];
-		*instance_att = '\0';
+		char instance_att[10] = "";
 		snprintf(instance_att, sizeof(instance_att), "%d", instance);
 		xmlNewProp(definition_node, BAD_CAST "variable_instance", BAD_CAST instance_att);
 	}
