@@ -27,8 +27,13 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
+
 #include "oval_results_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_system_characteristics_impl.h"
@@ -59,29 +64,6 @@ struct oval_result_item *oval_result_item_new(struct oval_result_system *sys, ch
 	return item;
 }
 
-bool oval_result_item_is_valid(struct oval_result_item * result_item)
-{
-	struct oval_sysitem *sysitem;
-
-	if (result_item == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-	sysitem = oval_result_item_get_sysitem(result_item);
-	if (oval_sysitem_is_valid(sysitem) != true)
-		return false;
-
-	return true;
-}
-
-bool oval_result_item_is_locked(struct oval_result_item * result_item)
-{
-	__attribute__nonnull__(result_item);
-
-	return oval_result_system_is_locked(result_item->sys);
-}
-
 struct oval_result_item *oval_result_item_clone
     (struct oval_result_system *new_system, struct oval_result_item *old_item) {
 	struct oval_sysitem *old_sysitem = oval_result_item_get_sysitem(old_item);
@@ -160,17 +142,14 @@ struct oval_message_iterator *oval_result_item_get_messages(struct oval_result_i
 
 void oval_result_item_set_result(struct oval_result_item *item, oval_result_t result)
 {
-	if (item && !oval_result_item_is_locked(item)) {
-		item->result = result;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(item);
+	item->result = result;
 }
 
-void oval_result_item_add_message(struct oval_result_item *item, struct oval_message *message) {
-	if (item && !oval_result_item_is_locked(item)) {
-		oval_collection_add(item->messages, message);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_item_add_message(struct oval_result_item *item, struct oval_message *message) 
+{
+	__attribute__nonnull__(item);
+	oval_collection_add(item->messages, message);
 }
 
 static void _oval_result_item_message_consumer(struct oval_message *message, struct oval_result_item *item) {
@@ -193,9 +172,6 @@ int oval_result_item_parse_tag
 	oval_result_t result = oval_result_parse(reader, "result", 0);
 	oval_result_item_set_result(item, result);
 
-	oscap_dlprintf(DBG_I, "item_id: %s, result: %d.\n",
-		       oval_sysitem_get_id(oval_result_item_get_sysitem(item)), oval_result_item_get_result(item));
-
 	return_code = oval_parser_parse_tag
 	    (reader, context, (oval_xml_tag_parser) _oval_result_item_message_parse, item);
 
