@@ -28,9 +28,14 @@
  *      "Peter Vrabec" <pvrabec@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_definitions.h"
 #include "oval_agent_api.h"
 #include "oval_results_impl.h"
@@ -45,7 +50,6 @@ typedef struct oval_result_system {
 	struct oval_string_map *definitions;
 	struct oval_string_map *tests;
 	struct oval_syschar_model *syschar_model;
-	bool definitions_initialized;
 } oval_result_system_t;
 
 
@@ -95,13 +99,9 @@ static bool _oval_result_system_resolve_syschar(struct oval_syschar *syschar, st
 struct oval_result_system *oval_result_system_new(struct oval_results_model *model,
 						  struct oval_syschar_model *syschar_model)
 {
+	__attribute__nonnull__(model);
 	oval_result_system_t *sys;
 
-        if (model && oval_results_model_is_locked(model)) {
-                oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
-                return NULL;
-        }
-
 	sys = (oval_result_system_t *) oscap_alloc(sizeof(oval_result_system_t));
 	if (sys == NULL)
 		return NULL;
@@ -109,7 +109,6 @@ struct oval_result_system *oval_result_system_new(struct oval_results_model *mod
 	sys->definitions = oval_string_map_new();
 	sys->tests = oval_string_map_new();
 	sys->syschar_model = syschar_model;
-	sys->definitions_initialized = false;
 	sys->model = model;
 
 	oval_results_model_add_system(model, sys);
@@ -117,63 +116,6 @@ struct oval_result_system *oval_result_system_new(struct oval_results_model *mod
 	return sys;
 }
 
-bool oval_result_system_is_valid(struct oval_result_system * result_system)
-{
-	bool is_valid = true;
-	struct oval_result_definition_iterator *rslt_definitions_itr;
-	struct oval_result_test_iterator *rslt_tests_itr;
-	struct oval_syschar_model *syschar_model;
-
-	if (result_system == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-	/* validate syschar_model */
-	syschar_model = oval_result_system_get_syschar_model(result_system);
-	if (oval_syschar_model_is_valid(syschar_model) != true)
-		return false;
-
-	/* validate result definitions */
-	rslt_definitions_itr = oval_result_system_get_definitions(result_system);
-	while (oval_result_definition_iterator_has_more(rslt_definitions_itr)) {
-		struct oval_result_definition *rslt_definition;
-
-		rslt_definition = oval_result_definition_iterator_next(rslt_definitions_itr);
-		if (oval_result_definition_is_valid(rslt_definition) != true) {
-			is_valid = false;
-			break;
-		}
-	}
-	oval_result_definition_iterator_free(rslt_definitions_itr);
-	if (is_valid != true)
-		return false;
-
-	/* validate result tests */
-	rslt_tests_itr = oval_result_system_get_tests(result_system);
-	while (oval_result_test_iterator_has_more(rslt_tests_itr)) {
-		struct oval_result_test *rslt_test;
-
-		rslt_test = oval_result_test_iterator_next(rslt_tests_itr);
-		if (oval_result_test_is_valid(rslt_test) != true) {
-			is_valid = false;
-			break;
-		}
-	}
-	oval_result_test_iterator_free(rslt_tests_itr);
-	if (is_valid != true)
-		return false;
-
-	return true;
-}
-
-bool oval_result_system_is_locked(struct oval_result_system * result_system)
-{
-	__attribute__nonnull__(result_system);
-
-	return oval_results_model_is_locked(result_system->model);
-}
-
 typedef void (*_oval_result_system_clone_func) (struct oval_result_system *, void *);
 
 static void _oval_result_system_clone
@@ -231,33 +173,9 @@ void oval_result_system_iterator_free(struct oval_result_system_iterator *sys) {
 	oval_collection_iterator_free((struct oval_iterator *)sys);
 }
 
-static void _oval_result_system_initialize(struct oval_result_system *sys) {
-	__attribute__nonnull__(sys);
-
-	sys->definitions_initialized = true;
-	struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(sys->syschar_model);
-
-	struct oval_definition_iterator *oval_definitions = oval_definition_model_get_definitions(definition_model);
-	while (oval_definition_iterator_has_more(oval_definitions)) {
-		struct oval_definition *oval_definition = oval_definition_iterator_next(oval_definitions);
-		oval_result_system_get_new_definition(sys, oval_definition);
-	}
-	oval_definition_iterator_free(oval_definitions);
-
-	struct oval_test_iterator *oval_tests = oval_definition_model_get_tests(definition_model);
-	while (oval_test_iterator_has_more(oval_tests)) {
-		struct oval_test *oval_test = oval_test_iterator_next(oval_tests);
-		get_oval_result_test_new(sys, oval_test);
-	}
-	oval_test_iterator_free(oval_tests);
-}
-
 struct oval_result_definition_iterator *oval_result_system_get_definitions(struct oval_result_system *sys) {
 	__attribute__nonnull__(sys);
 
-	if (!sys->definitions_initialized) {
-		_oval_result_system_initialize(sys);
-	}
 	struct oval_result_definition_iterator *iterator = (struct oval_result_definition_iterator *)
 	    oval_string_map_values(sys->definitions);
 	return iterator;
@@ -266,9 +184,6 @@ struct oval_result_definition_iterator *oval_result_system_get_definitions(struc
 struct oval_result_test_iterator *oval_result_system_get_tests(struct oval_result_system *sys) {
 	__attribute__nonnull__(sys);
 
-	if (!sys->definitions_initialized) {
-		_oval_result_system_initialize(sys);
-	}
 	struct oval_result_test_iterator *iterator = (struct oval_result_test_iterator *)
 	    oval_string_map_values(sys->tests);
 	return iterator;
@@ -277,9 +192,6 @@ struct oval_result_test_iterator *oval_result_system_get_tests(struct oval_resul
 struct oval_result_definition *oval_result_system_get_definition(struct oval_result_system *sys, const char *id) {
 	__attribute__nonnull__(sys);
 
-	if (!sys->definitions_initialized) {
-		_oval_result_system_initialize(sys);
-	}
 	return (struct oval_result_definition *)
 	    oval_string_map_get_value(sys->definitions, id);
 
@@ -288,9 +200,6 @@ struct oval_result_definition *oval_result_system_get_definition(struct oval_res
 struct oval_result_test *oval_result_system_get_test(struct oval_result_system *sys, char *id) {
 	__attribute__nonnull__(sys);
 
-	if (!sys->definitions_initialized) {
-		_oval_result_system_initialize(sys);
-	}
 	return (struct oval_result_test *)
 	    oval_string_map_get_value(sys->tests, id);
 
@@ -310,7 +219,7 @@ struct oval_result_definition *oval_result_system_get_new_definition
 	return rslt_definition;
 }
 
-struct oval_result_test *get_oval_result_test_new(struct oval_result_system *sys, struct oval_test *oval_test) {
+struct oval_result_test *oval_result_system_get_new_test(struct oval_result_system *sys, struct oval_test *oval_test) {
 	char *id = oval_test_get_id(oval_test);
 	struct oval_result_test *rslt_testtest = oval_result_system_get_test(sys, id);
 	if (rslt_testtest == NULL) {
@@ -339,85 +248,79 @@ struct oval_sysinfo *oval_result_system_get_sysinfo(struct oval_result_system *s
 	    ? oval_syschar_model_get_sysinfo(syschar_model) : NULL;
 }
 
-void oval_result_system_add_definition(struct oval_result_system *sys, struct oval_result_definition *definition) {
-	if (sys && !oval_result_system_is_locked(sys)) {
-		if (definition) {
-			struct oval_definition *ovaldef = oval_result_definition_get_definition(definition);
-			char *id = oval_definition_get_id(ovaldef);
-			oval_string_map_put(sys->definitions, id, definition);
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
-}
-
-void oval_result_system_add_test(struct oval_result_system *sys, struct oval_result_test *test) {
-	if (sys && !oval_result_system_is_locked(sys)) {
-		if (test) {
-			struct oval_test *ovaldef = oval_result_test_get_test(test);
-			char *id = oval_test_get_id(ovaldef);
-			oval_string_map_put(sys->tests, id, test);
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
-}
-
-static void _oval_result_system_test_consume(struct oval_result_test *test, struct oval_result_system *sys) {
-	oval_result_system_add_test(sys, test);
-}
-
-static int _oval_result_system_test_parse
-    (xmlTextReaderPtr reader, struct oval_parser_context *context, struct oval_result_system *sys) {
-	return oval_result_test_parse_tag
-	    (reader, context, sys, (oscap_consumer_func) _oval_result_system_test_consume, sys);
-
-}
-
-static void _oval_result_system_definition_consume
-    (struct oval_result_definition *definition, struct oval_result_system *sys) {
-	oval_result_system_add_definition(sys, definition);
-}
-
-static int _oval_result_system_definition_parse
-    (xmlTextReaderPtr reader, struct oval_parser_context *context, struct oval_result_system *sys) {
-	return oval_result_definition_parse
-	    (reader, context, sys, (oscap_consumer_func) _oval_result_system_definition_consume, sys);
-}
-
-static int _oval_result_system_parse
-    (xmlTextReaderPtr reader, struct oval_parser_context *context, struct oval_result_system *sys) {
+void oval_result_system_add_definition(struct oval_result_system *sys, struct oval_result_definition *definition) 
+{
 	__attribute__nonnull__(sys);
+	if (definition) {
+		struct oval_definition *ovaldef = oval_result_definition_get_definition(definition);
+		char *id = oval_definition_get_id(ovaldef);
+		oval_string_map_put(sys->definitions, id, definition);
+	}
+}
 
+void oval_result_system_add_test(struct oval_result_system *sys, struct oval_result_test *test) 
+{
+	__attribute__nonnull__(sys);
+	if (test) {
+		struct oval_test *ovaldef = oval_result_test_get_test(test);
+		char *id = oval_test_get_id(ovaldef);
+		oval_string_map_put(sys->tests, id, test);
+	}
+}
+
+static int oval_result_system_parse(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr) {
+	__attribute__nonnull__(usr);
+
+	struct oval_result_system *sys = (struct oval_result_system *) usr;
 	xmlChar *localName = xmlTextReaderLocalName(reader);
-	oscap_dlprintf(DBG_I, "Parsing <%s>.\n", localName);
+
 	int return_code = 0;
 	if (strcmp((const char *)localName, "definitions") == 0) {
-		return_code = oval_parser_parse_tag
-		    (reader, context, (oval_xml_tag_parser) _oval_result_system_definition_parse, sys);
+                return_code = oval_parser_parse_tag(reader, context, oval_result_definition_parse_tag, sys);
 	} else if (strcmp((const char *)localName, "tests") == 0) {
-		return_code = oval_parser_parse_tag
-		    (reader, context, (oval_xml_tag_parser) _oval_result_system_test_parse, sys);
+		return_code = oval_parser_parse_tag(reader, context, oval_result_test_parse_tag, sys);
 	} else if (strcmp((const char *)localName, "oval_system_characteristics") == 0) {
-		return_code = ovalsys_parser_parse(sys->syschar_model, reader, context->user_data);
-		//return_code = oval_parser_skip_tag(reader, context);
+		return_code = oval_syschar_model_parse(reader, context);
 	} else {
-		return_code = 0;
-		oscap_dlprintf(DBG_W, "Unhandled tag: <%s>.\n", localName);
+                dW("Skipping tag: %s\n", localName);
+                oval_parser_skip_tag(reader, context);
 	}
+
+        if (return_code != 0) {
+                dW("Parsing of <%s> terminated by an error at line %d.\n", localName, xmlTextReaderGetParserLineNumber(reader));
+        }
+
 	oscap_free(localName);
 	return return_code;
 }
 
-int oval_result_system_parse
-    (xmlTextReaderPtr reader, struct oval_parser_context *context, struct oval_syschar_model *syschar_model,
-     oscap_consumer_func consumer, void *client) {
+int oval_result_system_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr) {
 	__attribute__nonnull__(context);
 
-	int return_code = 1;
-	struct oval_result_system *sys = oval_result_system_new(context->results_model, syschar_model);
+        char *tagname = (char *)xmlTextReaderLocalName(reader);
+        char *namespace = (char *)xmlTextReaderNamespaceUri(reader);
+	int return_code = 0;
 
-	return_code = oval_parser_parse_tag(reader, context, (oval_xml_tag_parser) _oval_result_system_parse, sys);
+	int is_ovalres = strcmp((const char *)OVAL_RESULTS_NAMESPACE, namespace) == 0;
+	if (is_ovalres && (strcmp(tagname, "system") == 0)) {
+		/* create new empty syschar for this model */
+		struct oval_syschar_model *syschar_model = oval_syschar_model_new(context->definition_model);
+		context->syschar_model = syschar_model;
+		/* create new results system instance */
+		struct oval_result_system *sys = oval_result_system_new(context->results_model, syschar_model);
+		/* populate results system  */
+		return_code = oval_parser_parse_tag(reader, context, oval_result_system_parse, sys);
+        } else {
+                dW("Skipping tag: %s\n", tagname);
+                oval_parser_skip_tag(reader, context);
+        }
 
-	(*consumer) (sys, client);
+        if (return_code != 0) {
+                dW("Parsing of <%s> terminated by an error at line %d.\n", tagname, xmlTextReaderGetParserLineNumber(reader));
+        }
+
+        oscap_free(tagname);
+        oscap_free(namespace);
 
 	return return_code;
 }
@@ -445,7 +348,7 @@ int oval_result_system_eval(struct oval_result_system *sys)
 	return 0;
 }
 
-oval_result_t oval_result_system_eval_definition(struct oval_result_system *sys, const char *id)
+int oval_result_system_eval_definition(struct oval_result_system *sys, const char *id)
 {
         struct oval_results_model *res_model;
         struct oval_definition_model *definition_model;
@@ -456,9 +359,9 @@ oval_result_t oval_result_system_eval_definition(struct oval_result_system *sys,
 	definition_model = oval_results_model_get_definition_model(res_model);
 	oval_definition = oval_definition_model_get_definition(definition_model, id);
 	if (oval_definition == NULL) {
-		oscap_dlprintf(DBG_E, "No definition with ID: %s in definition model.\n", id);
-		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "No definition with such an ID in definition model.");
-		return OVAL_RESULT_ERROR;
+		dE("No definition with ID: %s in definition model.", id);
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "Unknown definition.");
+		return -1;
 	}
 
         rslt_definition = oval_result_system_get_definition(sys, id);
@@ -467,7 +370,9 @@ oval_result_t oval_result_system_eval_definition(struct oval_result_system *sys,
                 oval_result_system_add_definition(sys, rslt_definition);
         }
 
-	return oval_result_definition_eval(rslt_definition);
+	oval_result_definition_eval(rslt_definition);
+
+	return 0;
 }
 
 
@@ -516,6 +421,7 @@ xmlNode *oval_result_system_to_dom
 	if (oval_result_test_iterator_has_more(result_tests)) {
 		xmlNode *tests_node = xmlNewTextChild(system_node, ns_results, BAD_CAST "tests", NULL);
 		while (oval_result_test_iterator_has_more(result_tests)) {
+			struct oval_state_iterator *ste_itr;
 			struct oval_result_test *result_test = oval_result_test_iterator_next(result_tests);
 			/* report the test */
 			oval_result_test_to_dom(result_test, doc, tests_node);
@@ -540,8 +446,9 @@ xmlNode *oval_result_system_to_dom
 				}
 			}
 			/* look for objects in test->state->... */
-			struct oval_state *state = oval_test_get_state(oval_test);
-			if (state) {
+			ste_itr = oval_test_get_states(oval_test);
+			while (oval_state_iterator_has_more(ste_itr)) {
+				struct oval_state *state = oval_state_iterator_next(ste_itr);
 				char *sttid = oval_state_get_id(state);
 				void *value = oval_string_map_get_value(sttmap, sttid);
 				if (value == NULL) {
@@ -550,6 +457,7 @@ xmlNode *oval_result_system_to_dom
 										      varmap, sysmap);
 				}
 			}
+			oval_state_iterator_free(ste_itr);
 		}
 	}
 	oval_result_test_iterator_free(result_tests);
