@@ -28,15 +28,24 @@
  *      "Peter Vrabec" <pvrabec@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
+#if defined USE_REGEX_PCRE
+#include <pcre.h>
+#elif defined USE_REGEX_POSIX
 #include <regex.h>
+#endif
 #include <ctype.h>
 #include "oval_results_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
+#include "public/oval_types.h"
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
@@ -204,50 +213,6 @@ struct oval_result_test *oval_result_test_new(struct oval_result_system *sys, ch
 	return test;
 }
 
-bool oval_result_test_is_valid(struct oval_result_test * result_test)
-{
-	bool is_valid = true;
-	struct oval_test *test;
-	struct oval_result_item_iterator *rslt_items_itr;
-
-	if (result_test == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-	/* validate test */
-	test = oval_result_test_get_test(result_test);
-	if (oval_test_is_valid(test) != true)
-		return NULL;
-
-	/* validate items */
-	rslt_items_itr = oval_result_test_get_items(result_test);
-	while (oval_result_item_iterator_has_more(rslt_items_itr)) {
-		struct oval_result_item *rslt_item;
-
-		rslt_item = oval_result_item_iterator_next(rslt_items_itr);
-		if (oval_result_item_is_valid(rslt_item) != true) {
-			is_valid = false;
-			break;
-		}
-	}
-	oval_result_item_iterator_free(rslt_items_itr);
-	if (is_valid != true)
-		return false;
-
-	/* validate bindings */
-	// todo
-
-	return true;
-}
-
-bool oval_result_test_is_locked(struct oval_result_test * result_test)
-{
-	__attribute__nonnull__(result_test);
-
-	return oval_result_system_is_locked(result_test->system);
-}
-
 struct oval_result_test *oval_result_test_clone
     (struct oval_result_system *new_system, struct oval_result_test *old_test) {
 	__attribute__nonnull__(old_test);
@@ -364,25 +329,54 @@ static int istrcmp(char *st1, char *st2)
 
 static oval_result_t strregcomp(char *pattern, char *test_str)
 {
-	regex_t re;
-	int status;
-	oval_result_t result;
+	int ret;
+	oval_result_t result = OVAL_RESULT_ERROR;
+#if defined USE_REGEX_PCRE
+	pcre *re;
+	const char *err;
+	int errofs;
 
-	if ( (status = regcomp(&re, pattern, REG_EXTENDED)) != 0 ) {
-		oscap_dlprintf(DBG_E, "Unable to compile regex pattern: %d.\n", status);
+	re = pcre_compile(pattern, PCRE_UTF8, &err, &errofs, NULL);
+	if (re == NULL) {
+		oscap_dlprintf(DBG_E, "Unable to compile regex pattern, "
+			       "pcre_compile() returned error (offset: %d): '%s'.\n", errofs, err);
+		return OVAL_RESULT_ERROR;
+	}
+
+	ret = pcre_exec(re, NULL, test_str, strlen(test_str), 0, 0, NULL, 0);
+	if (ret > -1 ) {
+		result = OVAL_RESULT_TRUE;
+	} else if (ret == -1) {
+		result = OVAL_RESULT_FALSE;
+	} else {
+		oscap_dlprintf(DBG_E, "Unable to match regex pattern, "
+			       "pcre_exec() returned error: %d.\n", ret);
 		result = OVAL_RESULT_ERROR;
 	}
 
-	if ( (status = regexec(&re, test_str, 0, NULL, 0)) == 0) {	// got a match
+	pcre_free(re);
+#elif defined USE_REGEX_POSIX
+	regex_t re;
+
+	ret = regcomp(&re, pattern, REG_EXTENDED);
+	if (ret != 0) {
+		oscap_dlprintf(DBG_E, "Unable to compile regex pattern, "
+			       "regcomp() returned error: %d.\n", ret);
+		return OVAL_RESULT_ERROR;
+	}
+
+	ret = regexec(&re, test_str, 0, NULL, 0);
+	if (ret == 0) {
 		result = OVAL_RESULT_TRUE;
-	} else if (status == REG_NOMATCH) {	// no match, no errror
+	} else if (ret == REG_NOMATCH) {
 		result = OVAL_RESULT_FALSE;
 	} else {
-		oscap_dlprintf(DBG_E, "Unable to match regex pattern: %d.\n", status);
+		oscap_dlprintf(DBG_E, "Unable to match regex pattern: %d.\n", ret);
 		result = OVAL_RESULT_ERROR;
 	}
 
 	regfree(&re);
+#endif
 	return result;
 }
 
@@ -524,13 +518,15 @@ static oval_result_t evaluate(char *sys_data, char *state_data, oval_datatype_t
 		int state_idx = 0;
 		int sys_idx = 0;
 		int result = -1;
-		int is_equal = 1;
+		/* int is_equal = 1; */
 		for (state_idx = 0, sys_idx = 0; (((state_data[state_idx]) || (sys_data[sys_idx])) && (result == -1));) {	// keep going as long as there is data in either the state or sysitem
 			int tmp_state_int, tmp_sys_int;
 			tmp_state_int = atoi(&state_data[state_idx]);	// look at the current data field (if we're at the end, atoi should return 0)
 			tmp_sys_int = atoi(&sys_data[sys_idx]);
+                        /* o rly?
 			if (tmp_state_int != tmp_sys_int)
 				is_equal = 0;	// we might need this later (if we don't terminate early)
+                        */
 			if (operation == OVAL_OPERATION_EQUALS) {
 				if (tmp_state_int != tmp_sys_int)
 					return (OVAL_RESULT_FALSE);
@@ -895,7 +891,7 @@ static oval_result_t eval_item(struct oval_syschar_model *syschar_model, struct
 
 		ores_clear(&ent_ores);
 
-		item_entities_itr = oval_sysitem_get_items(cur_sysitem);
+		item_entities_itr = oval_sysitem_get_sysents(cur_sysitem);
 		while (oval_sysent_iterator_has_more(item_entities_itr)) {
 			struct oval_sysent *item_entity;
 			oval_result_t ent_val_res;
@@ -914,6 +910,10 @@ static oval_result_t eval_item(struct oval_syschar_model *syschar_model, struct
 			if (strcmp(item_entity_name, state_entity_name))
 				continue;
 
+			/* copy mask attribute from state to item */
+			if (oval_entity_get_mask(state_entity))
+				oval_sysent_set_mask(item_entity,1);
+
 			if (state_entity_var != NULL) {
 				struct oresults var_ores;
 				struct oval_value_iterator *val_itr;
@@ -937,6 +937,11 @@ static oval_result_t eval_item(struct oval_syschar_model *syschar_model, struct
 
 						var_val = oval_value_iterator_next(val_itr);
 						state_entity_val_text = oval_value_get_text(var_val);
+						if (state_entity_val_text == NULL) {
+							dE("Found NULL variable value text.\n");
+							ores_add_res(&var_ores, OVAL_RESULT_ERROR);
+							break;
+						}
 						state_entity_val_datatype = oval_value_get_datatype(var_val);
 
 						var_val_res = evaluate(oval_sysent_get_value(item_entity),
@@ -1004,36 +1009,69 @@ static void _oval_test_item_consumer(struct oval_result_item *item, void **args)
 	}
 }
 
-static oval_result_t eval_check_state(struct oval_state *state, oval_check_t check, void **args)
+static oval_result_t eval_check_state(struct oval_test *test, void **args)
 {
 	struct oval_syschar_model *syschar_model;
 	struct oval_result_item_iterator *ritems_itr;
-	struct oresults ores;
+	struct oresults item_ores;
 	oval_result_t result;
+	oval_check_t ste_check;
+	oval_operator_t ste_opr;
 
+	ste_check = oval_test_get_check(test);
+	ste_opr = oval_test_get_state_operator(test);
 	syschar_model = oval_result_system_get_syschar_model(SYSTEM);
-	ores_clear(&ores);
+	ores_clear(&item_ores);
 
 	ritems_itr = oval_result_test_get_items(TEST);
 	while (oval_result_item_iterator_has_more(ritems_itr)) {
 		struct oval_result_item *ritem;
 		struct oval_sysitem *item;
+		oval_syschar_status_t item_status;
+		struct oresults ste_ores;
+		struct oval_state_iterator *ste_itr;
 		oval_result_t item_res;
 
 		ritem = oval_result_item_iterator_next(ritems_itr);
 		item = oval_result_item_get_sysitem(ritem);
 
-		item_res = eval_item(syschar_model, item, state);
-		oval_result_item_set_result(ritem, item_res);
-
-		if (ores_add_res(&ores, item_res)) {
-			oval_result_item_iterator_free(ritems_itr);
-			return OVAL_RESULT_ERROR;
+		item_status = oval_sysitem_get_status(item);
+		switch (item_status) {
+		case SYSCHAR_STATUS_ERROR:
+		case SYSCHAR_STATUS_NOT_COLLECTED:
+			item_res = OVAL_RESULT_ERROR;
+			ores_add_res(&item_ores, item_res);
+			oval_result_item_set_result(ritem, item_res);
+			continue;
+		case SYSCHAR_STATUS_DOES_NOT_EXIST:
+			item_res = OVAL_RESULT_FALSE;
+			ores_add_res(&item_ores, item_res);
+			oval_result_item_set_result(ritem, item_res);
+			continue;
+		default:
+			break;
 		}
+
+		ores_clear(&ste_ores);
+
+		ste_itr = oval_test_get_states(test);
+		while (oval_state_iterator_has_more(ste_itr)) {
+			struct oval_state *ste;
+			oval_result_t ste_res;
+
+			ste = oval_state_iterator_next(ste_itr);
+			ste_res = eval_item(syschar_model, item, ste);
+			ores_add_res(&ste_ores, ste_res);
+		}
+		oval_state_iterator_free(ste_itr);
+
+		item_res = ores_get_result_byopr(&ste_ores, ste_opr);
+		ores_add_res(&item_ores, item_res);
+		oval_result_item_set_result(ritem, item_res);
 	}
 	oval_result_item_iterator_free(ritems_itr);
 
-	result = ores_get_result_bychk(&ores, check);
+	result = ores_get_result_bychk(&item_ores, ste_check);
 
 	return result;
 }
@@ -1103,15 +1141,15 @@ static oval_result_t eval_check_existence(oval_existence_t check_existence, int
 }
 
 static oval_result_t
-_oval_result_test_evaluate_items(struct oval_syschar *syschar_object,
-				 struct oval_state *state,
-				 oval_check_t test_check,
-				 oval_existence_t test_check_existence,
-				 void **args)
+_oval_result_test_evaluate_items(struct oval_test *test, struct oval_syschar *syschar_object, void **args)
 {
 	struct oval_sysitem_iterator *collected_items_itr;
 	oval_result_t result;
 	int exists_cnt, error_cnt;
+	bool hasstate;
+	oval_check_t test_check;
+	oval_existence_t test_check_existence;
+	struct oval_state_iterator *ste_itr;
 
 	exists_cnt = error_cnt = 0;
 	collected_items_itr = oval_syschar_get_sysitem(syschar_object);
@@ -1142,10 +1180,16 @@ _oval_result_test_evaluate_items(struct oval_syschar *syschar_object,
 	}
 	oval_sysitem_iterator_free(collected_items_itr);
 
+	test_check = oval_test_get_check(test);
+	test_check_existence = oval_test_get_existence(test);
+	ste_itr = oval_test_get_states(test);
+	hasstate = oval_state_iterator_has_more(ste_itr);
+	oval_state_iterator_free(ste_itr);
+
 	switch (oval_syschar_get_flag(syschar_object)) {
 	case SYSCHAR_FLAG_ERROR:
 		if (test_check_existence == OVAL_ANY_EXIST
-		    && state == NULL) {
+		    && !hasstate) {
 			result = OVAL_RESULT_TRUE;
 		} else {
 			result = OVAL_RESULT_ERROR;
@@ -1153,7 +1197,7 @@ _oval_result_test_evaluate_items(struct oval_syschar *syschar_object,
 		break;
 	case SYSCHAR_FLAG_NOT_COLLECTED:
 		if (test_check_existence == OVAL_ANY_EXIST
-		    && state == NULL) {
+		    && !hasstate) {
 			result = OVAL_RESULT_TRUE;
 		} else {
 			result = OVAL_RESULT_UNKNOWN;
@@ -1161,7 +1205,7 @@ _oval_result_test_evaluate_items(struct oval_syschar *syschar_object,
 		break;
 	case SYSCHAR_FLAG_NOT_APPLICABLE:
 		if (test_check_existence == OVAL_ANY_EXIST
-		    && state == NULL) {
+		    && !hasstate) {
 			result = OVAL_RESULT_TRUE;
 		} else {
 			result = OVAL_RESULT_NOT_APPLICABLE;
@@ -1178,8 +1222,8 @@ _oval_result_test_evaluate_items(struct oval_syschar *syschar_object,
 	case SYSCHAR_FLAG_COMPLETE:
 		result = eval_check_existence(test_check_existence, exists_cnt, error_cnt);
 		if (result == OVAL_RESULT_TRUE
-		    && state != NULL) {
-			result = eval_check_state(state, test_check, args);
+		    && hasstate) {
+			result = eval_check_state(test, args);
 		}
 		break;
 	case SYSCHAR_FLAG_INCOMPLETE:
@@ -1199,8 +1243,8 @@ _oval_result_test_evaluate_items(struct oval_syschar *syschar_object,
 		}
 
 		if (result == OVAL_RESULT_TRUE
-		    && state != NULL) {
-			result = eval_check_state(state, test_check, args);
+		    && hasstate) {
+			result = eval_check_state(test, args);
 			if (result == OVAL_RESULT_TRUE) {
 				if (test_check != OVAL_CHECK_AT_LEAST_ONE) {
 					result = OVAL_RESULT_UNKNOWN;
@@ -1219,44 +1263,33 @@ _oval_result_test_evaluate_items(struct oval_syschar *syschar_object,
 	return result;
 }
 
-// this function will gather all the necessary ingredients and call 'evaluate_items' when it finds them
+/* this function will gather all the necessary ingredients and call 'evaluate_items' when it finds them */
 static oval_result_t _oval_result_test_result(struct oval_result_test *rtest, void **args)
 {
 	__attribute__nonnull__(rtest);
 
-	// first, let's see if we already did the test
+	/* is the test already evaluated? */
 	if (rtest->result != OVAL_RESULT_NOT_EVALUATED) {
-		oscap_dlprintf(DBG_I, "Found result from previous evaluation: %d, returning without further processing.\n",
-			       rtest->result);
+		dI("Found result from previous evaluation: %d, returning without further processing.\n", rtest->result);
 		return (rtest->result);
 	}
 
-	// let's go looking for the stuff to test
-	struct oval_test *test2check = oval_result_test_get_test(rtest);
-	oval_check_t test_check = oval_test_get_check(test2check);
-	oval_existence_t test_check_existence = oval_test_get_existence(test2check);
-
-	struct oval_object * tmp_obj = oval_test_get_object(test2check);
-	if (tmp_obj == NULL) {
-		oscap_dlprintf(DBG_E, "Object is null.\n");
-		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "Object is null");
-		return OVAL_RESULT_ERROR;
-
-	}
-	char * definition_object_id_string = oval_object_get_id(tmp_obj);
+	/* get syschar of rtest */
+	struct oval_test *test = oval_result_test_get_test(rtest);
+	struct oval_object * object = oval_test_get_object(test);
+	char * object_id = oval_object_get_id(object);
 
 	struct oval_result_system *sys = oval_result_test_get_system(rtest);
 	struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
-	// OK, we have our object ID, now use that to find selected items in the syschar_model
-	struct oval_syschar * syschar_object = oval_syschar_model_get_syschar(syschar_model, definition_object_id_string);
-	if (syschar_object == NULL) {
-		oscap_dlprintf(DBG_E, "System characteristics object is null.\n");
-		return OVAL_RESULT_ERROR;
+
+	struct oval_syschar * syschar = oval_syschar_model_get_syschar(syschar_model, object_id);
+	if (syschar == NULL) {
+		dW("No syschar for object: %s\n", object_id);
+		return OVAL_RESULT_UNKNOWN;
 	}
 
-	struct oval_state *tmp_state = oval_test_get_state(test2check);
-	// FINALLY, we have all the pieces, now figure out the result
-	oval_result_t result = _oval_result_test_evaluate_items(syschar_object, tmp_state, test_check, test_check_existence, args);
+	/* evaluate items */
+	oval_result_t result = _oval_result_test_evaluate_items(test, syschar, args);
 
 	return result;
 }
@@ -1273,12 +1306,15 @@ static void _oval_result_test_initialize_bindings(struct oval_result_test *rslt_
 		struct oval_result_system *sys = oval_result_test_get_system(rslt_test);
 		struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
 		struct oval_syschar *syschar = oval_syschar_model_get_syschar(syschar_model, object_id);
-		struct oval_variable_binding_iterator *bindings = oval_syschar_get_variable_bindings(syschar);
-		while (oval_variable_binding_iterator_has_more(bindings)) {
-			struct oval_variable_binding *binding = oval_variable_binding_iterator_next(bindings);
-			oval_result_test_add_binding(rslt_test, binding);
+		/* no syschar if system characteristics was a subset of definitions */
+		if(syschar) {
+			struct oval_variable_binding_iterator *bindings = oval_syschar_get_variable_bindings(syschar);
+			while (oval_variable_binding_iterator_has_more(bindings)) {
+				struct oval_variable_binding *binding = oval_variable_binding_iterator_next(bindings);
+				oval_result_test_add_binding(rslt_test, binding);
+			}
+			oval_variable_binding_iterator_free(bindings);
 		}
-		oval_variable_binding_iterator_free(bindings);
 	}
 	rslt_test->bindings_initialized = true;
 	rslt_test->bindings_clearable = false;	//bindings are shared from syschar model.
@@ -1289,7 +1325,7 @@ oval_result_t oval_result_test_eval(struct oval_result_test *rtest)
 	__attribute__nonnull__(rtest);
 
 	if (rtest->result == OVAL_RESULT_NOT_EVALUATED) {
-		if ( oval_test_get_subtype( oval_result_test_get_test(rtest) ) != OVAL_INDEPENDENT_UNKNOWN ) {
+		if ((oval_independent_subtype_t)oval_test_get_subtype(oval_result_test_get_test(rtest)) != OVAL_INDEPENDENT_UNKNOWN ) {
 			struct oval_string_map *tmp_map = oval_string_map_new();
 			void *args[] = { rtest->system, rtest, tmp_map };
 			rtest->result = _oval_result_test_result(rtest, args);
@@ -1346,39 +1382,32 @@ struct oval_variable_binding_iterator *oval_result_test_get_bindings(struct oval
 
 void oval_result_test_set_result(struct oval_result_test *test, oval_result_t result)
 {
-	if (test && !oval_result_test_is_locked(test)) {
-		test->result = result;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(test);
+	test->result = result;
 }
 
 void oval_result_test_set_instance(struct oval_result_test *test, int instance)
 {
-	if (test && !oval_result_test_is_locked(test)) {
-		test->instance = instance;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(test);
+	test->instance = instance;
 }
 
-void oval_result_test_add_message(struct oval_result_test *test, struct oval_message *message) {
-	if (test && !oval_result_test_is_locked(test)) {
-		oval_collection_add(test->messages, message);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_test_add_message(struct oval_result_test *test, struct oval_message *message) 
+{
+	__attribute__nonnull__(test);
+	oval_collection_add(test->messages, message);
 }
 
-void oval_result_test_add_item(struct oval_result_test *test, struct oval_result_item *item) {
-	if (test && !oval_result_test_is_locked(test)) {
-		oval_collection_add(test->items, item);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_test_add_item(struct oval_result_test *test, struct oval_result_item *item) 
+{
+	__attribute__nonnull__(test);
+	oval_collection_add(test->items, item);
 }
 
-void oval_result_test_add_binding(struct oval_result_test *test, struct oval_variable_binding *binding) {
-	if (test && !oval_result_test_is_locked(test)) {
-		oval_collection_add(test->bindings, binding);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+void oval_result_test_add_binding(struct oval_result_test *test, struct oval_variable_binding *binding) 
+{
+	__attribute__nonnull__(test);
+	oval_collection_add(test->bindings, binding);
 }
 
 //void(*oscap_consumer_func)(void*, void*);
@@ -1387,7 +1416,7 @@ static void _oval_test_message_consumer(struct oval_message *message, struct ova
 }
 
 static int _oval_result_test_binding_parse(xmlTextReaderPtr reader, struct oval_parser_context *context, void **args) {
-	int return_code = 1;
+	int return_code = 0;
 
 	xmlChar *variable_id = xmlTextReaderGetAttribute(reader, BAD_CAST "variable_id");
 
@@ -1409,22 +1438,18 @@ static int _oval_result_test_binding_parse(xmlTextReaderPtr reader, struct oval_
 }
 
 static int _oval_result_test_parse(xmlTextReaderPtr reader, struct oval_parser_context *context, void **args) {
-	int return_code = 1;
+	int return_code = 0;
 	xmlChar *localName = xmlTextReaderLocalName(reader);
 
-	oscap_dlprintf(DBG_I, "Parsing <%s>.\n", localName);
-
 	if (strcmp((const char *)localName, "message") == 0) {
-		return_code = oval_message_parse_tag
-		    (reader, context, (oscap_consumer_func) _oval_test_message_consumer, TEST);
+		return_code = oval_message_parse_tag(reader, context, (oscap_consumer_func) _oval_test_message_consumer, TEST);
 	} else if (strcmp((const char *)localName, "tested_item") == 0) {
-		return_code = oval_result_item_parse_tag
-		    (reader, context, SYSTEM, (oscap_consumer_func) _oval_test_item_consumer, args);
+		return_code = oval_result_item_parse_tag(reader, context, SYSTEM, (oscap_consumer_func) _oval_test_item_consumer, args);
 	} else if (strcmp((const char *)localName, "tested_variable") == 0) {
 		return_code = _oval_result_test_binding_parse(reader, context, args);
 	} else {
-		oscap_dlprintf(DBG_W, "Unhandled tag: <%s>.\n", localName);
-		return_code = oval_parser_skip_tag(reader, context);
+		dW( "Unhandled tag: <%s>.\n", localName);
+		oval_parser_skip_tag(reader, context);
 	}
 
 	oscap_free(localName);
@@ -1432,13 +1457,18 @@ static int _oval_result_test_parse(xmlTextReaderPtr reader, struct oval_parser_c
 	return return_code;
 }
 
-int oval_result_test_parse_tag
-    (xmlTextReaderPtr reader, struct oval_parser_context *context,
-     struct oval_result_system *sys, oscap_consumer_func consumer, void *client) {
-	int return_code = 1;
+int oval_result_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr) {
 
+	struct oval_result_system *sys = (struct oval_result_system *) usr;
+	int return_code = 0;
+	struct oval_definition_model *dmod;
+	struct oval_test *dtst;
+	struct oval_result_test *test;
 	xmlChar *test_id = xmlTextReaderGetAttribute(reader, BAD_CAST "test_id");
-	struct oval_result_test *test = oval_result_test_new(sys, (char *)test_id);
+
+	dmod = context->definition_model;
+	dtst = oval_test_get_new(dmod, (char *) test_id);
+	test = oval_result_system_get_new_test(sys, dtst);
 	if (test == NULL)
 		return -1;
 
@@ -1476,10 +1506,10 @@ int oval_result_test_parse_tag
 	struct oval_string_map *itemmap = oval_string_map_new();
 	void *args[] = { sys, test, itemmap };
 	return_code = oval_parser_parse_tag(reader, context, (oval_xml_tag_parser) _oval_result_test_parse, args);
+	oval_string_map_free(itemmap, NULL);
 	test->bindings_initialized = true;
 	test->bindings_clearable = true;
 
-	(*consumer) (test, client);
 	oscap_free(test_id);
 	return return_code;
 }
