@@ -37,38 +37,30 @@
 
 OSCAP_HIDDEN_START;
 
-int oval_result_system_parse
-    (xmlTextReaderPtr, struct oval_parser_context *, struct oval_syschar_model *, oscap_consumer_func, void *);
-xmlNode *oval_result_system_to_dom
-    (struct oval_result_system *, struct oval_results_model *, struct oval_result_directives *, xmlDocPtr, xmlNode *);
+int oval_result_system_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, void *);
+xmlNode *oval_result_system_to_dom(struct oval_result_system *, struct oval_results_model *, struct oval_result_directives *, xmlDocPtr, xmlNode *);
 
-struct oval_result_test *get_oval_result_test_new(struct oval_result_system *, struct oval_test *);
+struct oval_result_test *oval_result_system_get_new_test(struct oval_result_system *, struct oval_test *);
 
-int oval_result_definition_parse
-    (xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_system *, oscap_consumer_func, void *);
+int oval_result_definition_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, void *);
 struct oval_result_definition *make_result_definition_from_oval_definition(struct oval_result_system *,
 									   struct oval_definition *);
-xmlNode *oval_result_definition_to_dom(struct oval_result_definition *, oval_result_directive_content_t, xmlDocPtr,
-				       xmlNode *);
+xmlNode *oval_result_definition_to_dom(struct oval_result_definition *, oval_result_directive_content_t, xmlDocPtr, xmlNode *);
 
-struct oval_result_test *make_result_test_from_oval_test
-    (struct oval_result_system *system, struct oval_test *oval_test);
-int oval_result_test_parse_tag
-    (xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_system *, oscap_consumer_func, void *);
+struct oval_result_test *make_result_test_from_oval_test(struct oval_result_system *system, struct oval_test *oval_test);
+
+int oval_result_test_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, void *);
 xmlNode *oval_result_test_to_dom(struct oval_result_test *, xmlDocPtr, xmlNode *);
 
-int oval_result_directives_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_directives *);
+int oval_result_directives_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, void *);
 int oval_result_directives_to_dom(struct oval_result_directives *, xmlDoc *, xmlNode *);
 
-int oval_result_item_parse_tag
-    (xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_system *, oscap_consumer_func, void *);
+int oval_result_item_parse_tag(xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_system *, oscap_consumer_func, void *);
 xmlNode *oval_result_item_to_dom(struct oval_result_item *, xmlDocPtr, xmlNode *);
 
-struct oval_result_criteria_node *make_result_criteria_node_from_oval_criteria_node
-    (struct oval_result_system *, struct oval_criteria_node *);
+struct oval_result_criteria_node *make_result_criteria_node_from_oval_criteria_node(struct oval_result_system *, struct oval_criteria_node *);
 
-int oval_result_criteria_node_parse
-    (xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_system *, oscap_consumer_func, void *);
+int oval_result_criteria_node_parse(xmlTextReaderPtr, struct oval_parser_context *, struct oval_result_system *, oscap_consumer_func, void *);
 xmlNode *oval_result_criteria_node_to_dom(struct oval_result_criteria_node *, xmlDocPtr, xmlNode *);
 
 oval_result_t oval_result_parse(xmlTextReaderPtr, char *, oval_result_t);
