@@ -27,9 +27,14 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
@@ -137,86 +142,6 @@ struct oval_setobject *oval_setobject_new(struct oval_definition_model *model)
 	return set;
 }
 
-bool oval_setobject_is_valid(struct oval_setobject * set_object)
-{
-	bool is_valid = true;
-	oval_setobject_type_t type;
-
-	if (set_object == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-	type = oval_setobject_get_type(set_object);
-	switch (type) {
-	case OVAL_SET_AGGREGATE:
-		{
-			struct oval_setobject_iterator *subsets_itr;
-
-			subsets_itr = oval_setobject_get_subsets(set_object);
-			while (oval_setobject_iterator_has_more(subsets_itr)) {
-				struct oval_setobject *subset;
-
-				subset = oval_setobject_iterator_next(subsets_itr);
-				if (oval_setobject_is_valid(subset) != true) {
-					is_valid = false;
-					break;
-				}
-			}
-			oval_setobject_iterator_free(subsets_itr);
-			if (is_valid != true)
-				return false;
-		}
-		break;
-	case OVAL_SET_COLLECTIVE:
-		{
-			struct oval_object_iterator *objects_itr;
-			struct oval_filter_iterator *filters_itr;
-
-			objects_itr = oval_setobject_get_objects(set_object);
-			while (oval_object_iterator_has_more(objects_itr)) {
-				struct oval_object *object;
-
-				object = oval_object_iterator_next(objects_itr);
-				if (oval_object_is_valid(object) != true) {
-					is_valid = false;
-					break;
-				}
-			}
-			oval_object_iterator_free(objects_itr);
-			if (is_valid != true)
-				return false;
-
-			filters_itr = oval_setobject_get_filters(set_object);
-			while (oval_filter_iterator_has_more(filters_itr)) {
-				struct oval_filter *filter;
-
-				filter = oval_filter_iterator_next(filters_itr);
-				if (oval_filter_is_valid(filter) != true) {
-					is_valid = false;
-					break;
-				}
-			}
-			oval_filter_iterator_free(filters_itr);
-			if (is_valid != true)
-				return false;
-		}
-		break;
-	default:
-                oscap_dlprintf(DBG_W, "Argument is not valid: wrong setobject type: %d.\n", type);
-		return false;
-	}
-
-	return true;
-}
-
-bool oval_setobject_is_locked(struct oval_setobject * setobject)
-{
-	__attribute__nonnull__(setobject);
-
-	return oval_definition_model_is_locked(setobject->model);
-}
-
 struct oval_setobject *oval_setobject_clone
     (struct oval_definition_model *new_model, struct oval_setobject *old_setobject) {
 	struct oval_setobject *new_setobject = oval_setobject_new(new_model);
@@ -286,67 +211,58 @@ void oval_setobject_free(struct oval_setobject *set)
 
 void oval_setobject_set_type(struct oval_setobject *set, oval_setobject_type_t type)
 {
-	if (set && !oval_setobject_is_locked(set)) {
-		set->type = type;
-		switch (type) {
-		case OVAL_SET_AGGREGATE:{
-				oval_set_AGGREGATE_t *aggregate =
-				    (oval_set_AGGREGATE_t *) (set->extension =
-							      oscap_alloc(sizeof(oval_set_AGGREGATE_t)));
-				aggregate->subsets = oval_collection_new();
-			}
-			break;
-		case OVAL_SET_COLLECTIVE:{
-				oval_set_COLLECTIVE_t *collective =
-				    (oval_set_COLLECTIVE_t *) (set->extension =
-							       oscap_alloc(sizeof(oval_set_COLLECTIVE_t)));
-				collective->filters = oval_collection_new();
-				collective->objects = oval_collection_new();
-			}
-			break;
-		case OVAL_SET_UNKNOWN:
-			break;
+	__attribute__nonnull__(set);
+
+	set->type = type;
+	switch (type) {
+	case OVAL_SET_AGGREGATE:{
+			oval_set_AGGREGATE_t *aggregate =
+			    (oval_set_AGGREGATE_t *) (set->extension =
+						      oscap_alloc(sizeof(oval_set_AGGREGATE_t)));
+			aggregate->subsets = oval_collection_new();
 		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+		break;
+	case OVAL_SET_COLLECTIVE:{
+			oval_set_COLLECTIVE_t *collective =
+			    (oval_set_COLLECTIVE_t *) (set->extension =
+						       oscap_alloc(sizeof(oval_set_COLLECTIVE_t)));
+			collective->filters = oval_collection_new();
+			collective->objects = oval_collection_new();
+		}
+		break;
+	case OVAL_SET_UNKNOWN:
+		break;
+	}
 }
 
 void oval_setobject_set_operation(struct oval_setobject *set, oval_setobject_operation_t operation)
 {
-	if (set && !oval_setobject_is_locked(set)) {
-		set->operation = operation;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(set);
+	set->operation = operation;
 }
 
 void oval_setobject_add_subset(struct oval_setobject *set, struct oval_setobject *subset)
 {
-	if (set && !oval_setobject_is_locked(set)) {
-		oval_set_AGGREGATE_t *aggregate = (oval_set_AGGREGATE_t *) set->extension;
-		assert(aggregate != NULL);
-		oval_collection_add(aggregate->subsets, (void *)subset);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(set);
+	oval_set_AGGREGATE_t *aggregate = (oval_set_AGGREGATE_t *) set->extension;
+	assert(aggregate != NULL);
+	oval_collection_add(aggregate->subsets, (void *)subset);
 }
 
 void oval_setobject_add_object(struct oval_setobject *set, struct oval_object *object)
 {
-	if (set && !oval_setobject_is_locked(set)) {
-		oval_set_COLLECTIVE_t *collective = (oval_set_COLLECTIVE_t *) set->extension;
-		assert(collective != NULL);
-		oval_collection_add(collective->objects, (void *)object);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(set);
+	oval_set_COLLECTIVE_t *collective = (oval_set_COLLECTIVE_t *) set->extension;
+	assert(collective != NULL);
+	oval_collection_add(collective->objects, (void *)object);
 }
 
 void oval_setobject_add_filter(struct oval_setobject *set, struct oval_filter *filter)
 {
-	if (set && !oval_setobject_is_locked(set)) {
-		oval_set_COLLECTIVE_t *collective = (oval_set_COLLECTIVE_t *) set->extension;
-		assert(collective != NULL);
-		oval_collection_add(collective->filters, (void *)filter);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(set);
+	oval_set_COLLECTIVE_t *collective = (oval_set_COLLECTIVE_t *) set->extension;
+	assert(collective != NULL);
+	oval_collection_add(collective->filters, (void *)filter);
 }
 
 //typedef int (*oval_xml_tag_parser)(xmlTextReaderPtr, struct oval_parser_context*, void*);
@@ -365,7 +281,7 @@ static void oval_consume_object_ref(char *objref, void *user)
 	__attribute__nonnull__(user);
 
 	struct oval_set_context *ctx = user;
-	struct oval_definition_model *model = oval_parser_context_model(ctx->context);
+	struct oval_definition_model *model = ctx->context->definition_model;
 	struct oval_object *object = oval_object_get_new(model, objref);
 	oval_setobject_add_object(ctx->set, object);
 }
@@ -405,10 +321,11 @@ static int _oval_set_parse_tag(xmlTextReaderPtr reader, struct oval_parser_conte
 			return_code = oval_parser_skip_tag(reader, context);
 		}
 	}
-	if (return_code != 1) {
-		oscap_dlprintf(DBG_I, "Parsing of <%s> terminated by an error at line %d.\n", tagname,
-			       xmlTextReaderGetParserLineNumber(reader));
+
+	if (return_code != 0) {
+		dW("Parsing of <%s> terminated by an error at line %d.\n", tagname, xmlTextReaderGetParserLineNumber(reader));
 	}
+
 	oscap_free(tagname);
 	oscap_free(namespace);
 	return return_code;
@@ -437,53 +354,6 @@ int oval_set_parse_tag(xmlTextReaderPtr reader,
 	return return_code;
 }
 
-void oval_set_to_print(struct oval_setobject *set, char *indent, int idx)
-{
-	char nxtindent[100];
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%sSET.", indent);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%sSET[%d].", indent, idx);
-
-	oscap_dprintf("%sOPERATOR    = %d\n", nxtindent, oval_setobject_get_operation(set));
-	oscap_dprintf("%sTYPE        = %d\n", nxtindent, oval_setobject_get_type(set));
-
-	switch (oval_setobject_get_type(set)) {
-	case OVAL_SET_AGGREGATE:{
-			struct oval_setobject_iterator *subsets = oval_setobject_get_subsets(set);
-			int i;
-			for (i = 1; oval_setobject_iterator_has_more(subsets); i++) {
-				struct oval_setobject *subset = oval_setobject_iterator_next(subsets);
-				oval_set_to_print(subset, nxtindent, i);
-			}
-			oval_setobject_iterator_free(subsets);
-		} break;
-	case OVAL_SET_COLLECTIVE:{
-			struct oval_object_iterator *objects = oval_setobject_get_objects(set);
-			int i;
-			for (i = 1; oval_object_iterator_has_more(objects); i++) {
-				struct oval_object *object = oval_object_iterator_next(objects);
-				oval_object_to_print(object, nxtindent, i);
-			}
-			oval_object_iterator_free(objects);
-			struct oval_filter_iterator *filters = oval_setobject_get_filters(set);
-			for (i = 1; oval_filter_iterator_has_more(filters); i++) {
-				struct oval_filter *filter;
-
-				filter = oval_filter_iterator_next(filters);
-				//oval_filter_to_print(filter, nxtindent, i);
-			}
-			oval_filter_iterator_free(filters);
-		} break;
-	case OVAL_SET_UNKNOWN:
-		break;
-	}
-}
-
 xmlNode *oval_set_to_dom(struct oval_setobject *set, xmlDoc * doc, xmlNode * parent) {
 	xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
 	xmlNode *set_node = xmlNewTextChild(parent, ns_definitions, BAD_CAST "set", NULL);
