@@ -27,9 +27,14 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
@@ -137,13 +142,9 @@ int oval_state_get_operator(struct oval_state *state)
 
 struct oval_state *oval_state_new(struct oval_definition_model *model, const char *id)
 {
+	__attribute__nonnull__(model);
 	oval_state_t *state;
 
-	if (model && oval_definition_model_is_locked(model)) {
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
-		return NULL;
-	}
-
 	state = (oval_state_t *) oscap_alloc(sizeof(oval_state_t));
 	if (state == NULL)
 		return NULL;
@@ -163,42 +164,6 @@ struct oval_state *oval_state_new(struct oval_definition_model *model, const cha
 	return state;
 }
 
-bool oval_state_is_valid(struct oval_state * state)
-{
-	/*
-	bool is_valid = true;
-	struct oval_state_content_iterator *contents_itr;
-	*/
-
-	if (state == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-	/*
-	contents_itr = oval_state_get_contents(state);
-	while (oval_state_content_iterator_has_more(contents_itr)) {
-		struct oval_state_content *content;
-
-		content = oval_state_content_iterator_next(contents_itr);
-		if (oval_state_content_is_valid(content) != true) {
-			is_valid = false;
-			break;
-		}
-	}
-	oval_state_content_iterator_free(contents_itr);
-	if (is_valid != true)
-		return false;
-	*/
-	return true;
-}
-
-bool oval_state_is_locked(struct oval_state * state)
-{
-	__attribute__nonnull__(state);
-
-	return oval_definition_model_is_locked(state->model);
-}
-
 struct oval_state *oval_state_clone(struct oval_definition_model *new_model, struct oval_state *old_state) {
 	__attribute__nonnull__(old_state);
 
@@ -247,59 +212,46 @@ void oval_state_free(struct oval_state *state)
 
 void oval_state_set_subtype(struct oval_state *state, oval_subtype_t subtype)
 {
-	if (state && !oval_state_is_locked(state)) {
-		state->subtype = subtype;
-	} else
-		fprintf(stderr, "WARNING: attempt to update locked content\n %s(%d)\n", __FILE__, __LINE__);
+	__attribute__nonnull__(state);
+	state->subtype = subtype;
 }
 
 void oval_state_add_note(struct oval_state *state, char *notes)
 {
-	if (state && !oval_state_is_locked(state)) {
-		oval_collection_add(state->notes, (void *)strdup(notes));
-	} else
-		fprintf(stderr, "WARNING: attempt to update locked content\n %s(%d)\n", __FILE__, __LINE__);
+	__attribute__nonnull__(state);
+	oval_collection_add(state->notes, (void *)strdup(notes));
 }
 
 void oval_state_set_comment(struct oval_state *state, char *comm)
 {
-	if (state && !oval_state_is_locked(state)) {
-		if (state->comment != NULL)
-			free(state->comment);
-		state->comment = comm == NULL ? NULL : strdup(comm);
-	} else
-		fprintf(stderr, "WARNING: attempt to update locked content\n %s(%d)\n", __FILE__, __LINE__);
+	__attribute__nonnull__(state);
+	if (state->comment != NULL)
+		free(state->comment);
+	state->comment = comm == NULL ? NULL : strdup(comm);
 }
 
 void oval_state_set_deprecated(struct oval_state *state, bool deprecated)
 {
-	if (state && !oval_state_is_locked(state)) {
-		state->deprecated = deprecated;
-	} else
-		fprintf(stderr, "WARNING: attempt to update locked content\n %s(%d)\n", __FILE__, __LINE__);
+	__attribute__nonnull__(state);
+	state->deprecated = deprecated;
 }
 
 void oval_state_set_version(struct oval_state *state, int version)
 {
-	if (state && !oval_state_is_locked(state)) {
-		state->version = version;
-	} else
-		fprintf(stderr, "WARNING: attempt to update locked content\n %s(%d)\n", __FILE__, __LINE__);
+	__attribute__nonnull__(state);
+	state->version = version;
 }
 
 void oval_state_set_operator(struct oval_state *state, oval_operator_t operator)
 {
-	if (state && !oval_state_is_locked(state)) {
-		state->operator = operator;
-	} else
-		fprintf(stderr, "WARNING: attempt to update locked content\n %s(%d)\n", __FILE__, __LINE__);
+	__attribute__nonnull__(state);
+	state->operator = operator;
 }
 
-void oval_state_add_content(struct oval_state *state, struct oval_state_content *content) {
-	if (state && !oval_state_is_locked(state)) {
-		oval_collection_add(state->contents, content);
-	} else
-		fprintf(stderr, "WARNING: attempt to update locked content\n %s(%d)\n", __FILE__, __LINE__);
+void oval_state_add_content(struct oval_state *state, struct oval_state_content *content) 
+{
+	__attribute__nonnull__(state);
+	oval_collection_add(state->contents, content);
 }
 
 static void _oval_note_consumer(char *text, void *state)
@@ -324,28 +276,21 @@ static int _oval_state_parse_tag(xmlTextReaderPtr reader, struct oval_parser_con
 	struct oval_state *state = (struct oval_state *)user;
 	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
-	int return_code = 1;
+	int return_code = 0;
 	if ((strcmp(tagname, "notes") == 0)) {
 		return_code = oval_parser_parse_tag(reader, context, &_oval_state_parse_notes, state);
 	} else {
-		return_code =
-		    oval_state_content_parse_tag
+		return_code = oval_state_content_parse_tag
 		    (reader, context, (oscap_consumer_func) _oval_state_content_consumer, state);
 	}
-	if (return_code != 1) {
-		int line = xmlTextReaderGetParserLineNumber(reader);
-		printf
-		    ("NOTICE: oval_state_parse_tag::parse of %s terminated on error at <%s> line %d\n",
-		     state->id, tagname, line);
-	}
 	free(tagname);
 	free(namespace);
 	return return_code;
 }
 
-int oval_state_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context)
+int oval_state_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr)
 {
-	struct oval_definition_model *model = oval_parser_context_model(context);
+	struct oval_definition_model *model = context->definition_model;
 	char *id = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "id");
 	struct oval_state *state = oval_state_get_new(model, id);
 	free(id);
@@ -365,37 +310,10 @@ int oval_state_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *co
 	oval_operator_t operator = oval_operator_parse(reader, "operator", OVAL_OPERATOR_AND);
 	oval_state_set_operator(state, operator);
 
-	int return_code = oval_parser_parse_tag(reader, context, &_oval_state_parse_tag,
-						state);
+	int return_code = oval_parser_parse_tag(reader, context, &_oval_state_parse_tag, state);
 	return return_code;
 }
 
-void oval_state_to_print(struct oval_state *state, char *indent, int idx)
-{
-	char nxtindent[100];
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%sSTATE.", indent);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%sSTATE[%d].", indent, idx);
-
-	printf("%sID         = %s\n", nxtindent, oval_state_get_id(state));
-	printf("%sFAMILY     = %d\n", nxtindent, oval_state_get_family(state));
-	printf("%sSUBTYPE    = %d\n", nxtindent, oval_state_get_subtype(state));
-	printf("%sVERSION    = %d\n", nxtindent, oval_state_get_version(state));
-	printf("%sCOMMENT    = %s\n", nxtindent, oval_state_get_comment(state));
-	printf("%sDEPRECATED = %d\n", nxtindent, oval_state_get_deprecated(state));
-	printf("%sOPERATOR   = %d\n", nxtindent, oval_state_get_operator(state));
-	struct oval_string_iterator *notes = oval_state_get_notes(state);
-	for (idx = 1; oval_string_iterator_has_more(notes); idx++) {
-		printf("%sNOTE[%d]    = %s\n", nxtindent, idx, oval_string_iterator_next(notes));
-	}
-	oval_string_iterator_free(notes);
-}
-
 xmlNode *oval_state_to_dom(struct oval_state *state, xmlDoc * doc, xmlNode * parent)
 {
 	oval_subtype_t subtype = oval_state_get_subtype(state);
