@@ -26,14 +26,20 @@
  * Authors:
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <string.h>
+#include <time.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_agent_api_impl.h"
 #include "oval_parser_impl.h"
 #include "oval_string_map_impl.h"
 #include "oval_system_characteristics_impl.h"
 #include "oval_results_impl.h"
-#include <string.h>
-#include <time.h>
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
@@ -48,7 +54,6 @@ typedef struct _oval_variable_model_frame {
 typedef struct oval_variable_model {
 	struct oval_generator *generator;
 	struct oval_string_map *varmap;
-	bool is_locked;
 } oval_variable_model_t;
 
 static _oval_variable_model_frame_t *_oval_variable_model_frame_new(char *id, const char *comm, oval_datatype_t datatype);
@@ -75,47 +80,20 @@ static _oval_variable_model_frame_t *_oval_variable_model_frame_new(char *id, co
 	return frame;
 }
 
-bool oval_variable_model_is_valid(struct oval_variable_model *variable_model)
+bool oval_variable_model_iterator_has_more(struct oval_variable_model_iterator *itr)
 {
-        bool is_valid = true;
-	struct oval_string_iterator *varids_itr;
-
-	if (variable_model == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-		return false;
-        }
-
-        varids_itr = oval_variable_model_get_variable_ids(variable_model);
-	while (oval_string_iterator_has_more(varids_itr)) {
-		char *varid = oval_string_iterator_next(varids_itr);
-
-                if (oval_variable_model_get_datatype(variable_model, varid) == OVAL_DATATYPE_UNKNOWN) {
-                        oscap_dlprintf(DBG_W, "Argument is not valid: variable (%s) datatype == OVAL_DATATYPE_UNKNOWN.\n", varid);
-                        is_valid = false;
-                        break;
-                }
-		if (oval_variable_model_get_comment(variable_model, varid) == NULL) {
-                        oscap_dlprintf(DBG_W, "Argument is not valid: variable (%s) comment == NULL.\n", varid);
-                        is_valid = false;
-                        break;
-                }
-	}
-        oval_string_iterator_free(varids_itr);
-
-        return is_valid;
+	return oval_collection_iterator_has_more((struct oval_iterator *) itr);
 }
 
-bool oval_variable_model_is_locked(struct oval_variable_model * variable_model)
+struct oval_variable_model *oval_variable_model_iterator_next(struct oval_variable_model_iterator *itr)
 {
-	__attribute__nonnull__(variable_model);
-	return variable_model->is_locked;
+	return (struct oval_variable_model *) oval_collection_iterator_next((struct oval_iterator *) itr);
 }
 
-void oval_variable_model_lock(struct oval_variable_model *variable_model)
+void oval_variable_model_iterator_free(struct oval_variable_model_iterator *itr)
 {
-	__attribute__nonnull__(variable_model);
-	variable_model->is_locked = true;
-} 
+	oval_collection_iterator_free((struct oval_iterator *) itr);
+}
 
 static void _oval_variable_model_frame_free(_oval_variable_model_frame_t * frame)
 {
@@ -124,7 +102,7 @@ static void _oval_variable_model_frame_free(_oval_variable_model_frame_t * frame
 			oscap_free(frame->id);
 		if (frame->comment)
 			oscap_free(frame->comment);
-		oval_collection_free_items(frame->values, oscap_free);
+		oval_collection_free_items(frame->values, (oscap_destruct_func) oval_value_free);
 		frame->id = NULL;
 		frame->comment = NULL;
 		frame->values = NULL;
@@ -140,7 +118,6 @@ struct oval_variable_model *oval_variable_model_new()
 		return NULL;
 	model->generator = oval_generator_new();
 	model->varmap = oval_string_map_new();
-	model->is_locked = false;
 	return model;
 }
 
@@ -151,12 +128,17 @@ struct oval_variable_model *oval_variable_model_clone(struct oval_variable_model
 	while (oval_string_iterator_has_more(old_varids)) {
 		char *varid = oval_string_iterator_next(old_varids);
 		oval_datatype_t datatype = oval_variable_model_get_datatype(old_model, varid);
-		struct oval_string_iterator *values = oval_variable_model_get_values(old_model, varid);
+		struct oval_value_iterator *values = oval_variable_model_get_values(old_model, varid);
 		const char *comm = oval_variable_model_get_comment(old_model, varid);
-		while (oval_string_iterator_has_more(values)) {
-			char *value = oval_string_iterator_next(values);
-			oval_variable_model_add(new_model, varid, comm, datatype, value);
-		} oval_string_iterator_free(values);
+		while (oval_value_iterator_has_more(values)) {
+			struct oval_value *ov;
+			char *text;
+
+			ov = oval_value_iterator_next(values);
+			text = oval_value_get_text(ov);
+			oval_variable_model_add(new_model, varid, comm, datatype, text);
+		}
+		oval_value_iterator_free(values);
 	} oval_string_iterator_free(old_varids);
 	return new_model;
 }
@@ -166,6 +148,7 @@ void oval_variable_model_free(struct oval_variable_model *model)
 	if (model) {
 		oval_string_map_free(model->varmap, (oscap_destruct_func) _oval_variable_model_frame_free);
 		model->varmap = NULL;
+		oval_generator_free(model->generator);
 		oscap_free(model);
 	}
 }
@@ -184,14 +167,16 @@ void oval_variable_model_set_generator(struct oval_variable_model *model, struct
 void oval_variable_model_add(struct oval_variable_model *model, char *varid, const char *comm,
 			     oval_datatype_t datatype, char *value)
 {
+	struct oval_value *ov;
+
 	struct _oval_variable_model_frame *frame =
 	    (struct _oval_variable_model_frame *)oval_string_map_get_value(model->varmap, varid);
 	if (frame == NULL) {
 		frame = _oval_variable_model_frame_new(varid, comm, datatype);
 		oval_string_map_put(model->varmap, varid, frame);
 	}
-	value = oscap_strdup(value);
-	oval_collection_add(frame->values, value);
+	ov = oval_value_new(datatype, value);
+	oval_collection_add(frame->values, ov);
 }
 
 #define NAMESPACE_VARIABLES "http://oval.mitre.org/XMLSchema/oval-variables-5"
@@ -203,9 +188,12 @@ static int _oval_variable_model_parse_variable_values
 	int return_code;
 	bool is_variable_ns = strcmp(NAMESPACE_VARIABLES, namespace) == 0;
 	if (is_variable_ns && strcmp("value", tagname) == 0) {
+		struct oval_value *ov;
+
 		return_code = xmlTextReaderRead(reader);
 		char *value = (char *)xmlTextReaderValue(reader);
-		oval_collection_add(frame->values, strdup(value));
+		ov = oval_value_new(frame->datatype, value);
+		oval_collection_add(frame->values, ov);
 		oscap_free(value);
 	} else {
 		oscap_dlprintf(DBG_W, "Unprocessed tag: <%s:%s>.\n", namespace, tagname);
@@ -370,12 +358,16 @@ static xmlNode *oval_variable_model_to_dom(struct oval_variable_model * variable
                 xmlNewProp(variable, BAD_CAST "datatype", BAD_CAST oval_datatype_get_text(datatype));
                 xmlNewProp(variable, BAD_CAST "comment", BAD_CAST comm);
 
-		struct oval_string_iterator *values = oval_variable_model_get_values(variable_model, varid);
-		while (oval_string_iterator_has_more(values)) {
-			char *value = oval_string_iterator_next(values);
-                        xmlNewTextChild(variable, ns_variables, BAD_CAST "value", BAD_CAST value);
+		struct oval_value_iterator *value_itr = oval_variable_model_get_values(variable_model, varid);
+		while (oval_value_iterator_has_more(value_itr)) {
+			struct oval_value *value;
+			char *text;
+
+			value = oval_value_iterator_next(value_itr);
+			text = oval_value_get_text(value);
+			xmlNewTextChild(variable, ns_variables, BAD_CAST "value", BAD_CAST text);
 		}
-                oval_string_iterator_free(values);
+                oval_value_iterator_free(value_itr);
 	}
         oval_string_iterator_free(varids);
 
@@ -444,9 +436,17 @@ const char *oval_variable_model_get_comment(struct oval_variable_model *model, c
 	return (frame) ? frame->comment : NULL;
 }
 
-struct oval_string_iterator *oval_variable_model_get_values(struct oval_variable_model *model, char *varid)
+struct oval_value_iterator *oval_variable_model_get_values(struct oval_variable_model *model, char *varid)
 {
 	__attribute__nonnull__(model);
 	_oval_variable_model_frame_t *frame = oval_string_map_get_value(model->varmap, varid);
-	return (frame) ? (struct oval_string_iterator *)oval_collection_iterator(frame->values) : NULL;
+	return (frame) ? (struct oval_value_iterator *) oval_collection_iterator(frame->values) : NULL;
+}
+
+struct oval_collection *oval_variable_model_get_values_ref(struct oval_variable_model *model, char *varid)
+{
+	_oval_variable_model_frame_t *frame;
+
+	frame = oval_string_map_get_value(model->varmap, varid);
+	return (frame) ? frame->values : NULL;
 }
