@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2011 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -27,9 +27,14 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
@@ -39,29 +44,45 @@
 #include "common/_error.h"
 
 typedef struct oval_variable {
-	struct oval_definition_model *model;
-	char *id;
-	char *comment;
-	int version;
-	int deprecated;
-	oval_variable_type_t type;
-	oval_datatype_t datatype;
-	oval_syschar_collection_flag_t flag;
-	struct oval_collection *values;
-} oval_variable_t, oval_variable_CONEXT_t;
+#define VAR_BASE				\
+	struct oval_definition_model *model;	\
+	oval_variable_type_t type;		\
+	oval_syschar_collection_flag_t flag;	\
+	char *id;				\
+	int version;				\
+	oval_datatype_t datatype;		\
+	char *comment;				\
+	int deprecated
 
-typedef struct oval_variable_LOCAL {
-	struct oval_definition_model *model;
-	char *id;
-	char *comment;
-	int version;
-	int deprecated;
-	oval_variable_type_t type;
-	oval_datatype_t datatype;
-	oval_syschar_collection_flag_t flag;
+	VAR_BASE;
+} oval_variable_t;
+
+typedef struct {
+	VAR_BASE;
 	struct oval_collection *values;
-	struct oval_component *component;	//type==OVAL_VARIABLE_LOCAL
-} oval_variable_LOCAL_t, oval_variable_UNKNOWN_t;
+} oval_variable_CONSTANT_t;
+
+typedef struct {
+	VAR_BASE;
+	/* todo: possible value */
+	/* todo: possible restriction */
+	struct oval_collection *values_ref;
+} oval_variable_EXTERNAL_t;
+
+typedef struct {
+	VAR_BASE;
+	struct oval_component *component;
+	struct oval_collection *values;
+} oval_variable_LOCAL_t;
+
+typedef struct {
+	union {
+		oval_variable_t v1;
+		oval_variable_CONSTANT_t v2;
+		oval_variable_EXTERNAL_t v3;
+		oval_variable_LOCAL_t v4;
+	} unused;
+} oval_variable_UNKNOWN_t;
 
 bool oval_variable_iterator_has_more(struct oval_variable_iterator
 				     *oc_variable)
@@ -129,10 +150,36 @@ oval_datatype_t oval_variable_get_datatype(struct oval_variable * variable)
 
 struct oval_value_iterator *oval_variable_get_values(struct oval_variable *variable)
 {
+	struct oval_collection *values;
+
 	__attribute__nonnull__(variable);
 
-	struct oval_collection *value_collection = variable->values;
-	return (value_collection) ? (struct oval_value_iterator *)oval_collection_iterator(variable->values) : NULL;
+	switch (variable->type) {
+	case OVAL_VARIABLE_EXTERNAL: {
+		oval_variable_EXTERNAL_t *var = (oval_variable_EXTERNAL_t *) variable;
+
+		values = var->values_ref;
+		break;
+	}
+	case OVAL_VARIABLE_CONSTANT: {
+		oval_variable_CONSTANT_t *var = (oval_variable_CONSTANT_t *) variable;
+
+		values = var->values;
+		break;
+	}
+	case OVAL_VARIABLE_LOCAL: {
+		oval_variable_LOCAL_t *var = (oval_variable_LOCAL_t *) variable;
+
+		values = var->values;
+		break;
+	}
+	default:
+		values = NULL;
+		break;
+	}
+
+	return (values) ? (struct oval_value_iterator *) oval_collection_iterator(values) : 
+		(struct oval_value_iterator *) oval_collection_iterator_new();
 }
 
 oval_syschar_collection_flag_t oval_variable_get_collection_flag(struct oval_variable *variable) {
@@ -143,26 +190,30 @@ oval_syschar_collection_flag_t oval_variable_get_collection_flag(struct oval_var
 
 int oval_syschar_model_compute_variable(struct oval_syschar_model *sysmod, struct oval_variable *variable)
 {
-	oval_syschar_collection_flag_t flag;
+	oval_variable_LOCAL_t *var;
+	struct oval_component *component;
 	struct oval_value_iterator *val_itr;
 
 	__attribute__nonnull__(variable);
 
-	if (oval_variable_get_collection_flag(variable) != SYSCHAR_FLAG_UNKNOWN)
-                return 0;
+	if (variable->type != OVAL_VARIABLE_LOCAL)
+		return 0;
 
-        variable->values = oval_collection_new();
-        struct oval_component *component = oval_variable_get_component(variable);
+	var = (oval_variable_LOCAL_t *) variable;
+	if (var->flag != SYSCHAR_FLAG_UNKNOWN)
+		return 0;
+
+	component = var->component;
         if (component) {
-                variable->flag = oval_component_compute(sysmod, component, variable->values);
-        } else {
-		oscap_dlprintf(DBG_W, "NULL component bound to a variable, id: %s, type: %s.\n",
-			       oval_variable_type_get_text(variable->type), oval_variable_get_id(variable));
-                return -1;
+		if (!var->values)
+			var->values = oval_collection_new();
+		var->flag = oval_component_compute(sysmod, component, var->values);
+	} else {
+		oscap_dlprintf(DBG_W, "NULL component bound to a variable, id: %s.\n", var->id);
+		return -1;
         }
 
-	flag = oval_variable_get_collection_flag(variable);
-	switch (flag) {
+	switch (var->flag) {
 	case SYSCHAR_FLAG_COMPLETE:
 	case SYSCHAR_FLAG_INCOMPLETE:
 		break;
@@ -172,7 +223,7 @@ int oval_syschar_model_compute_variable(struct oval_syschar_model *sysmod, struc
 
 	val_itr = oval_variable_get_values(variable);
 	if (!oval_value_iterator_has_more(val_itr))
-		variable->flag = SYSCHAR_FLAG_ERROR;
+		var->flag = SYSCHAR_FLAG_ERROR;
 	oval_value_iterator_free(val_itr);
 
         return 0;
@@ -180,27 +231,30 @@ int oval_syschar_model_compute_variable(struct oval_syschar_model *sysmod, struc
 
 int oval_probe_query_variable(oval_probe_session_t *sess, struct oval_variable *variable)
 {
-	oval_datatype_t var_dt;
-	oval_syschar_collection_flag_t flag;
+	oval_variable_LOCAL_t *var;
+	struct oval_component *component;
 	struct oval_value_iterator *val_itr;
 
 	__attribute__nonnull__(variable);
 
-	if (variable->flag != SYSCHAR_FLAG_UNKNOWN)
+	if (variable->type != OVAL_VARIABLE_LOCAL)
 		return 0;
 
-        variable->values = oval_collection_new();
-        struct oval_component *component = oval_variable_get_component(variable);
+	var = (oval_variable_LOCAL_t *) variable;
+	if (var->flag != SYSCHAR_FLAG_UNKNOWN)
+		return 0;
+
+	component = var->component;
         if (component) {
-		variable->flag = oval_component_query(sess, component, variable->values);
+		if (!var->values)
+			var->values = oval_collection_new();
+		var->flag = oval_component_query(sess, component, var->values);
 	} else {
-		oscap_dlprintf(DBG_W, "NULL component bound to a variable, id: %s, type: %s.\n",
-			       oval_variable_type_get_text(variable->type), oval_variable_get_id(variable));
+		oscap_dlprintf(DBG_W, "NULL component bound to a variable, id: %s.\n", var->id);
 		return -1;
         }
 
-	flag = oval_variable_get_collection_flag(variable);
-	switch (flag) {
+	switch (var->flag) {
 	case SYSCHAR_FLAG_COMPLETE:
 	case SYSCHAR_FLAG_INCOMPLETE:
 		break;
@@ -210,19 +264,18 @@ int oval_probe_query_variable(oval_probe_session_t *sess, struct oval_variable *
 
 	val_itr = oval_variable_get_values(variable);
 	if (!oval_value_iterator_has_more(val_itr)) {
-		variable->flag = SYSCHAR_FLAG_ERROR;
+		oval_value_iterator_free(val_itr);
+		var->flag = SYSCHAR_FLAG_ERROR;
 		return 0;
 	}
 
-	var_dt = oval_variable_get_datatype(variable);
-
 	while (oval_value_iterator_has_more(val_itr)) {
 		struct oval_value *val;
 
 		val = oval_value_iterator_next(val_itr);
-		if (oval_value_cast(val, var_dt) != 0) {
+		if (oval_value_cast(val, var->datatype) != 0) {
 			oval_value_iterator_free(val_itr);
-			variable->flag = SYSCHAR_FLAG_ERROR;
+			var->flag = SYSCHAR_FLAG_ERROR;
 			return 0;
 		}
 	}
@@ -251,41 +304,42 @@ struct oval_variable *oval_variable_new(struct oval_definition_model *model, con
 {
 	oval_variable_t *variable;
 
-        if (model && oval_definition_model_is_locked(model)) {
-                oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
-                return NULL;
-        }
-
 	switch (type) {
 	case OVAL_VARIABLE_CONSTANT:{
-			variable = (oval_variable_t *) oscap_alloc(sizeof(oval_variable_CONEXT_t));
+			oval_variable_CONSTANT_t *cvar;
+
+			variable = (oval_variable_t *) oscap_alloc(sizeof(oval_variable_CONSTANT_t));
 			if (variable == NULL)
 				return NULL;
 
-			oval_variable_CONEXT_t *conext = (oval_variable_CONEXT_t *) variable;
-			conext->values = oval_collection_new();
-			conext->flag = SYSCHAR_FLAG_NOT_COLLECTED;
+			cvar = (oval_variable_CONSTANT_t *) variable;
+			cvar->values = NULL;
+			cvar->flag = SYSCHAR_FLAG_NOT_COLLECTED;
 		}
 		break;
 	case OVAL_VARIABLE_EXTERNAL:{
-			variable = (oval_variable_t *) oscap_alloc(sizeof(oval_variable_CONEXT_t));
+			oval_variable_EXTERNAL_t *evar;
+
+			variable = (oval_variable_t *) oscap_alloc(sizeof(oval_variable_EXTERNAL_t));
 			if (variable == NULL)
 				return NULL;
 
-			oval_variable_CONEXT_t *conext = (oval_variable_CONEXT_t *) variable;
-			conext->values = oval_collection_new();
-			conext->flag = SYSCHAR_FLAG_NOT_COLLECTED;
+			evar = (oval_variable_EXTERNAL_t *) variable;
+			evar->values_ref = NULL;
+			evar->flag = SYSCHAR_FLAG_NOT_COLLECTED;
 		}
 		break;
 	case OVAL_VARIABLE_LOCAL:{
+			oval_variable_LOCAL_t *lvar;
+
 			variable = (oval_variable_t *) oscap_alloc(sizeof(oval_variable_LOCAL_t));
 			if (variable == NULL)
 				return NULL;
 
-			oval_variable_LOCAL_t *local = (oval_variable_LOCAL_t *) variable;
-			local->component = NULL;
-			local->values = NULL;
-			local->flag = SYSCHAR_FLAG_UNKNOWN;
+			lvar = (oval_variable_LOCAL_t *) variable;
+			lvar->component = NULL;
+			lvar->values = NULL;
+			lvar->flag = SYSCHAR_FLAG_UNKNOWN;
 		}
 		break;
 	case OVAL_VARIABLE_UNKNOWN:{
@@ -293,10 +347,7 @@ struct oval_variable *oval_variable_new(struct oval_definition_model *model, con
 			if (variable == NULL)
 				return NULL;
 
-			oval_variable_UNKNOWN_t *unknwn = (oval_variable_UNKNOWN_t *) variable;
-			unknwn->component = NULL;
-			unknwn->values = NULL;
-			unknwn->flag = SYSCHAR_FLAG_UNKNOWN;
+			memset(variable, 0, sizeof(oval_variable_UNKNOWN_t));
 		};
 		break;
 	default:
@@ -316,51 +367,6 @@ struct oval_variable *oval_variable_new(struct oval_definition_model *model, con
 	return variable;
 }
 
-bool oval_variable_is_valid(struct oval_variable * variable)
-{
-        oval_variable_type_t type;
-
-        if (variable == NULL) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: NULL.\n");
-                return false;
-        }
-
-        if (oval_variable_get_datatype(variable) == OVAL_DATATYPE_UNKNOWN) {
-                oscap_dlprintf(DBG_W, "Argument is not valid: datatype == OVAL_DATATYPE_UNKNOWN.\n");
-                return false;
-        }
-
-        type = oval_variable_get_type(variable);
-        switch (type) {
-        case OVAL_VARIABLE_EXTERNAL:
-                break;
-        case OVAL_VARIABLE_CONSTANT:
-                break;
-        case OVAL_VARIABLE_LOCAL:
-                {
-                        oval_variable_LOCAL_t *var = (oval_variable_LOCAL_t *) variable;
-
-                        if (var->component == NULL) {
-                                oscap_dlprintf(DBG_W, "Argument is not valid: component == NULL.\n");
-                                return false;
-                        }
-                }
-                break;
-        default:
-                oscap_dlprintf(DBG_W, "Argument is not valid: wrong variable type: %d.\n", type);
-                return false;
-        }
-
-	return true;
-}
-
-bool oval_variable_is_locked(struct oval_variable * variable)
-{
-	__attribute__nonnull__(variable);
-
-	return oval_definition_model_is_locked(variable->model);
-}
-
 struct oval_variable *oval_variable_clone(struct oval_definition_model *new_model, struct oval_variable *old_variable) {
 	__attribute__nonnull__(old_variable);
 
@@ -374,22 +380,59 @@ struct oval_variable *oval_variable_clone(struct oval_definition_model *new_mode
 		oval_variable_set_datatype(new_variable, old_variable->datatype);
 		new_variable->flag = old_variable->flag;
 
-		if (old_variable->values) {
-			struct oval_value_iterator *old_values =
-			    (struct oval_value_iterator *)oval_collection_iterator(old_variable->values);
-			if (new_variable->values == NULL)
-				new_variable->values = oval_collection_new();
-			while (oval_value_iterator_has_more(old_values)) {
-				struct oval_value *value = oval_value_iterator_next(old_values);
-				/* char *text = oval_value_get_text(value); <-- unused */
-				oval_collection_add(new_variable->values, value);
-			}
-			oval_value_iterator_free(old_values);
-		}
+		switch (old_variable->type) {
+		case OVAL_VARIABLE_CONSTANT: {
+			oval_variable_CONSTANT_t *cvar;
+			struct oval_value_iterator *old_val_itr;
 
-		if (new_variable->type == OVAL_VARIABLE_LOCAL) {
-			struct oval_component *component = oval_variable_get_component(old_variable);
-			oval_variable_set_component(new_variable, oval_component_clone(new_model, component));
+			cvar = (oval_variable_CONSTANT_t *) new_variable;
+			old_val_itr = oval_variable_get_values(old_variable);
+			if (oval_value_iterator_has_more(old_val_itr))
+				cvar->values = oval_collection_new();
+			while (oval_value_iterator_has_more(old_val_itr)) {
+				struct oval_value *val;
+
+				val = oval_value_iterator_next(old_val_itr);
+				val = oval_value_clone(val);
+				oval_collection_add(cvar->values, val);
+			}
+			oval_value_iterator_free(old_val_itr);
+
+			break;
+		}
+		case OVAL_VARIABLE_EXTERNAL: {
+			oval_variable_EXTERNAL_t *evar, *old_evar;
+
+			evar = (oval_variable_EXTERNAL_t *) new_variable;
+			old_evar = (oval_variable_EXTERNAL_t *) old_variable;
+			evar->values_ref = old_evar->values_ref;
+
+			break;
+		}
+		case OVAL_VARIABLE_LOCAL: {
+			oval_variable_LOCAL_t *lvar, *old_lvar;
+			struct oval_value_iterator *old_val_itr;
+
+			lvar = (oval_variable_LOCAL_t *) new_variable;
+			old_val_itr = oval_variable_get_values(old_variable);
+			if (oval_value_iterator_has_more(old_val_itr))
+				lvar->values = oval_collection_new();
+			while (oval_value_iterator_has_more(old_val_itr)) {
+				struct oval_value *val;
+
+				val = oval_value_iterator_next(old_val_itr);
+				val = oval_value_clone(val);
+				oval_collection_add(lvar->values, val);
+			}
+			oval_value_iterator_free(old_val_itr);
+
+			old_lvar = (oval_variable_LOCAL_t *) old_variable;
+			lvar->component = oval_component_clone(new_model, old_lvar->component);
+
+			break;
+		}
+		default:
+			break;
 		}
 	}
 	return new_variable;
@@ -402,19 +445,41 @@ void oval_variable_free(struct oval_variable *variable)
 			oscap_free(variable->id);
 		if (variable->comment)
 			oscap_free(variable->comment);
-		oval_variable_CONEXT_t *conext = (oval_variable_CONEXT_t *) variable;
-		if (conext->values) {
-			oval_collection_free_items(conext->values, (oscap_destruct_func) oval_value_free);
-			conext->values = NULL;
+		variable->id = variable->comment = NULL;
+
+		switch (variable->type) {
+		case OVAL_VARIABLE_CONSTANT: {
+			oval_variable_CONSTANT_t *cvar;
+
+			cvar = (oval_variable_CONSTANT_t *) variable;
+			if (cvar->values)
+				oval_collection_free_items(cvar->values, (oscap_destruct_func) oval_value_free);
+			cvar->values = NULL;
+
+			break;
 		}
-		if (variable->type == OVAL_VARIABLE_LOCAL) {
-			oval_variable_LOCAL_t *local = (oval_variable_LOCAL_t *) variable;
-			if (local->component)
-				oval_component_free(local->component);
-			local->component = NULL;
+		case OVAL_VARIABLE_EXTERNAL: {
+			oval_variable_EXTERNAL_t *evar;
+
+			evar = (oval_variable_EXTERNAL_t *) variable;
+			evar->values_ref = NULL;
+
+			break;
+		}
+		case OVAL_VARIABLE_LOCAL: {
+			oval_variable_LOCAL_t *lvar;
+
+			lvar = (oval_variable_LOCAL_t *) variable;
+			if (lvar->values)
+				oval_collection_free_items(lvar->values, (oscap_destruct_func) oval_value_free);
+			lvar->values = NULL;
+			oval_component_free(lvar->component);
+
+			break;
+		}
+		default:
+			break;
 		}
-		variable->comment = NULL;
-		variable->id = NULL;
 
 		oscap_free(variable);
 	}
@@ -422,107 +487,154 @@ void oval_variable_free(struct oval_variable *variable)
 
 void oval_variable_set_datatype(struct oval_variable *variable, oval_datatype_t datatype)
 {
-	if (variable && !oval_variable_is_locked(variable)) {
-		variable->datatype = datatype;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	variable->datatype = datatype;
 }
 
-void oval_variable_set_type(struct oval_variable *variable, oval_variable_type_t type)
+void oval_variable_set_type(struct oval_variable *variable, oval_variable_type_t new_type)
 {
-	if (variable && !oval_variable_is_locked(variable)) {
-		if (variable->type == OVAL_VARIABLE_UNKNOWN) {
-			variable->type = type;
-			switch (type) {
-			case OVAL_VARIABLE_CONSTANT:
-			case OVAL_VARIABLE_EXTERNAL:
-				{
-					oval_variable_CONEXT_t *conext = (oval_variable_CONEXT_t *) variable;
-					conext->values = oval_collection_new();
-					conext->flag = SYSCHAR_FLAG_NOT_COLLECTED;
-				}
-				break;
-			case OVAL_VARIABLE_LOCAL:
-			case OVAL_VARIABLE_UNKNOWN:
-				variable->flag = SYSCHAR_FLAG_UNKNOWN;
-				break;
-			}
-		} else if (variable->type != type) {
-			/* TODO: Should we set and propagate error here ? */
-			oscap_dlprintf(DBG_E, "Attempt to reset valid variable type, oldtype: %s, newtype: %s.\n",
-				       oval_variable_type_get_text(variable->type), oval_variable_type_get_text(type));
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	if (variable->type != OVAL_VARIABLE_UNKNOWN) {
+		oscap_dlprintf(DBG_E, "Attempt to reset valid variable type, oldtype: %s, newtype: %s.\n",
+			       oval_variable_type_get_text(variable->type), oval_variable_type_get_text(new_type));
+		return;
+	}
+
+	variable->type = new_type;
+
+	switch (new_type) {
+	case OVAL_VARIABLE_CONSTANT: {
+		oval_variable_CONSTANT_t *cvar;
+
+		cvar = (oval_variable_CONSTANT_t *) variable;
+		cvar->values = NULL;
+		cvar->flag = SYSCHAR_FLAG_NOT_COLLECTED;
+
+		break;
+	}
+	case OVAL_VARIABLE_EXTERNAL: {
+		oval_variable_EXTERNAL_t *evar;
+
+		evar = (oval_variable_EXTERNAL_t *) variable;
+		evar->values_ref = NULL;
+		evar->flag = SYSCHAR_FLAG_NOT_COLLECTED;
+
+		break;
+	}
+	case OVAL_VARIABLE_LOCAL: {
+		oval_variable_LOCAL_t *lvar;
+
+		lvar = (oval_variable_LOCAL_t *) variable;
+		lvar->component = NULL;
+		lvar->values = NULL;
+		lvar->flag = SYSCHAR_FLAG_UNKNOWN;
+
+		break;
+	}
+	default:
+		break;
+	}
 }
 
 void oval_variable_set_comment(struct oval_variable *variable, char *comm)
 {
-	if (variable && !oval_variable_is_locked(variable)) {
-		if (variable->comment != NULL)
-			oscap_free(variable->comment);
-		variable->comment = oscap_strdup(comm);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(variable);
+
+	if (variable->comment != NULL)
+		oscap_free(variable->comment);
+	variable->comment = oscap_strdup(comm);
 
 }
 
 void oval_variable_set_deprecated(struct oval_variable *variable, bool deprecated)
 {
-	if (variable && !oval_variable_is_locked(variable)) {
-		variable->deprecated = deprecated;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(variable);
+	variable->deprecated = deprecated;
 }
 
 void oval_variable_set_version(struct oval_variable *variable, int version)
 {
-	if (variable && !oval_variable_is_locked(variable)) {
-		variable->version = version;
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(variable);
+	variable->version = version;
 }
 
 void oval_variable_add_value(struct oval_variable *variable, struct oval_value *value)
 {
-	if (variable && !oval_variable_is_locked(variable)) {
-		if (variable->type == OVAL_VARIABLE_CONSTANT || variable->type == OVAL_VARIABLE_EXTERNAL) {
-			/* char *text = oval_value_get_text(value); <-- unused */
-			oval_collection_add(variable->values, value);
-			variable->flag = SYSCHAR_FLAG_COMPLETE;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(variable);
+	oval_variable_CONSTANT_t *cvar;
+
+	if (variable->type != OVAL_VARIABLE_CONSTANT)
+		return;
+
+	cvar = (oval_variable_CONSTANT_t *) variable;
+	if (!cvar->values)
+		cvar->values = oval_collection_new();
+	oval_collection_add(cvar->values, value);
+	cvar->flag = SYSCHAR_FLAG_COMPLETE;
 }
 
 void oval_variable_clear_values(struct oval_variable *variable)
 {
-	oval_variable_CONEXT_t *conext_var;
+	__attribute__nonnull__(variable);
 
-	if (oval_variable_is_locked(variable)) {
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
-		return;
-	}
 	if (variable->type != OVAL_VARIABLE_CONSTANT && variable->type != OVAL_VARIABLE_EXTERNAL) {
 		oscap_dlprintf(DBG_W, "Wrong variable type for this operation: %d.\n", variable->type);
 		return;
         }
 
-	conext_var = (oval_variable_CONEXT_t *) variable;
-	oval_collection_free_items(conext_var->values, (oscap_destruct_func) oval_value_free);
-	conext_var->values = oval_collection_new();
-	variable->flag = SYSCHAR_FLAG_NOT_COLLECTED;
+	switch (variable->type) {
+	case OVAL_VARIABLE_CONSTANT: {
+		oval_variable_CONSTANT_t *cvar;
+
+		cvar = (oval_variable_CONSTANT_t *) variable;
+		if (cvar->values) {
+			oval_collection_free_items(cvar->values, (oscap_destruct_func) oval_value_free);
+			cvar->values = NULL;
+		}
+		cvar->flag = SYSCHAR_FLAG_NOT_COLLECTED;
+
+		break;
+	}
+	case OVAL_VARIABLE_EXTERNAL: {
+		oval_variable_EXTERNAL_t *evar;
+
+		evar = (oval_variable_EXTERNAL_t *) variable;
+		evar->values_ref = NULL;
+		evar->flag = SYSCHAR_FLAG_NOT_COLLECTED;
+
+		break;
+	}
+	default:
+		break;
+	}
+}
+
+int oval_variable_bind_ext_var(struct oval_variable *var, struct oval_variable_model *varmod, char *extvar_id)
+{
+	oval_variable_EXTERNAL_t *evar;
+
+	if (var->type != OVAL_VARIABLE_EXTERNAL) {
+		dW("Attemp to bind a non-external variable, id: %s, type: %s.\n", var->id, oval_variable_type_get_text(var->type));
+		return 2;
+	}
+
+	evar = (oval_variable_EXTERNAL_t *) var;
+	evar->values_ref = oval_variable_model_get_values_ref(varmod, extvar_id);
+	if (!evar->values_ref)
+		return 1;
+
+	evar->flag = SYSCHAR_FLAG_COMPLETE;
+	/* todo: store a reference to the variable model inside evar? */
+
+	return 0;
 }
 
 void oval_variable_set_component(struct oval_variable *variable, struct oval_component *component)
 {
-	if (variable && !oval_variable_is_locked(variable)) {
-		if (variable->type == OVAL_VARIABLE_LOCAL) {
-			oval_variable_LOCAL_t *local = (oval_variable_LOCAL_t *) variable;
-			local->component = component;
-		}
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(variable);
+
+	if (variable->type == OVAL_VARIABLE_LOCAL) {
+		oval_variable_LOCAL_t *local = (oval_variable_LOCAL_t *) variable;
+		local->component = component;
+	}
 }
 
 static void _oval_variable_parse_local_tag_component_consumer(struct oval_component *component, void *variable)
@@ -535,11 +647,9 @@ static int _oval_variable_parse_local_tag(xmlTextReaderPtr reader, struct oval_p
 	struct oval_variable *variable = (struct oval_variable *)user;
 	xmlChar *tagname = xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
-	int return_code = oval_component_parse_tag(reader, context, &_oval_variable_parse_local_tag_component_consumer,
-						   variable);
-	if (return_code != 1) {
-		oscap_dlprintf(DBG_W, "Parsing of %s terminated by an error at <%s>, line %d.\n",
-			       variable->id, tagname, xmlTextReaderGetParserLineNumber(reader));
+	int return_code = oval_component_parse_tag(reader, context, &_oval_variable_parse_local_tag_component_consumer, variable);
+	if (return_code != 0) {
+		dW("Parsing of %s terminated by an error at <%s>, line %d.\n", variable->id, tagname, xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(tagname);
 	oscap_free(namespace);
@@ -555,16 +665,16 @@ static void _oval_variable_parse_value(char *text_value, struct oval_variable *v
 	oval_variable_add_value(variable, value);
 }
 
-static int _oval_variable_parse_constant_tag(xmlTextReaderPtr reader,
-					     struct oval_parser_context *context, struct oval_variable *variable)
+static int _oval_variable_parse_constant_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *user)
 {
 	xmlChar *tagname = xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
-	int return_code = 1;
+	struct oval_variable *variable = (struct oval_variable *)user;
+	int return_code = 0;
 	if (strcmp("value", (char *)tagname) == 0 && strcmp(DEFINITION_NAMESPACE, (char *)namespace) == 0) {
 		oval_parser_text_value(reader, context, (oval_xml_value_consumer) _oval_variable_parse_value, variable);
 	} else {
-		oscap_dlprintf(DBG_W, "Invalid element <%s:%s> in constant variable %s on line %d.\n",
+		dW("Invalid element <%s:%s> in constant variable %s on line %d.\n",
 			       namespace, tagname, variable->id, xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(tagname);
@@ -572,11 +682,17 @@ static int _oval_variable_parse_constant_tag(xmlTextReaderPtr reader,
 	return return_code;
 }
 
-int oval_variable_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context)
+int oval_variable_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *usr)
 {
-	struct oval_definition_model *model = oval_parser_context_model(context);
-	char *tagname = (char *)xmlTextReaderLocalName(reader);
+	struct oval_definition_model *model = context->definition_model;
+	char *tagname = NULL;
+	char *id = NULL;
+	char *comm = NULL;
+	char *version = NULL;
+	int return_code = 0;
 	oval_variable_type_t type;
+
+	tagname = (char *)xmlTextReaderLocalName(reader);
 	if (strcmp(tagname, "constant_variable") == 0)
 		type = OVAL_VARIABLE_CONSTANT;
 	else if (strcmp(tagname, "external_variable") == 0)
@@ -585,36 +701,28 @@ int oval_variable_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context
 		type = OVAL_VARIABLE_LOCAL;
 	else {
 		type = OVAL_VARIABLE_UNKNOWN;
-		oscap_dlprintf(DBG_W, "Unhandled variable type: %s, line: %d.\n",
-			      tagname, xmlTextReaderGetParserLineNumber(reader));
+		dW("Unhandled variable type: %s, line: %d.\n", tagname, xmlTextReaderGetParserLineNumber(reader));
 	}
-	char *id = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "id");
+	id = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "id");
 	struct oval_variable *variable = oval_variable_get_new(model, id, type);
-	oscap_free(id);
-	id = variable->id;
 
-	char *comm = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "comment");
+	comm = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "comment");
 	if (comm != NULL) {
 		oval_variable_set_comment(variable, comm);
-		oscap_free(comm);
-		comm = NULL;
-
 	}
+
 	int deprecated = oval_parser_boolean_attribute(reader, "deprecated", 0);
 	oval_variable_set_deprecated(variable, deprecated);
-	char *version = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "version");
+
+	version = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "version");
 	oval_variable_set_version(variable, atoi(version));
-	oscap_free(version);
-	version = NULL;
 
 	oval_datatype_t datatype = oval_datatype_parse(reader, "datatype", OVAL_DATATYPE_UNKNOWN);
 	oval_variable_set_datatype(variable, datatype);
-	int return_code = 1;
+
 	switch (type) {
 	case OVAL_VARIABLE_CONSTANT:{
-			return_code =
-			    oval_parser_parse_tag(reader, context,
-						  (oval_xml_tag_parser) _oval_variable_parse_constant_tag, variable);
+			return_code = oval_parser_parse_tag(reader, context, _oval_variable_parse_constant_tag, variable);
 		}
 		break;
 	case OVAL_VARIABLE_LOCAL:{
@@ -629,58 +737,15 @@ int oval_variable_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context
 	default:
 		return_code = 1;
 	}
+
 	oscap_free(tagname);
+	oscap_free(id);
+	oscap_free(comm);
+	oscap_free(version);
+
 	return return_code;
 }
 
-void oval_variable_to_print(struct oval_variable *variable, char *indent, int idx)
-{
-	char nxtindent[100];
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%sVARIABLE.", indent);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%sVARIABLE[%d].", indent, idx);
-
-	oscap_dprintf("%sID         = %s\n", nxtindent, oval_variable_get_id(variable));
-	oscap_dprintf("%sVERSION    = %d\n", nxtindent, oval_variable_get_version(variable));
-	oscap_dprintf("%sCOMMENT    = %s\n", nxtindent, oval_variable_get_comment(variable));
-	oscap_dprintf("%sDEPRECATED = %d\n", nxtindent, oval_variable_get_deprecated(variable));
-	oscap_dprintf("%sTYPE       = %d\n", nxtindent, oval_variable_get_type(variable));
-	oscap_dprintf("%sDATATYPE   = %d\n", nxtindent, oval_variable_get_datatype(variable));
-	switch (oval_variable_get_type(variable)) {
-	case OVAL_VARIABLE_CONSTANT:{
-			struct oval_value_iterator *values = oval_variable_get_values(variable);
-			if (oval_value_iterator_has_more(values)) {
-				int i;
-				for (i = 0; oval_value_iterator_has_more(values); i++) {
-					struct oval_value *value = oval_value_iterator_next(values);
-					oval_value_to_print(value, nxtindent, i);
-				}
-			} else
-				oscap_dprintf("%sVALUES     = <<NO CONSTANTS BOUND>>\n", nxtindent);
-		}
-		break;
-	case OVAL_VARIABLE_EXTERNAL:{
-			oscap_dprintf("%sEXTERNAL   <<TODO>>\n", nxtindent);
-		}
-		break;
-	case OVAL_VARIABLE_LOCAL:{
-			struct oval_component *component = oval_variable_get_component(variable);
-			if (component == NULL)
-				oscap_dprintf("%sCOMPONENT  = <<NO COMPONENT BOUND>>\n", nxtindent);
-			else
-				oval_component_to_print(component, nxtindent, 0);
-		}
-		break;
-	case OVAL_VARIABLE_UNKNOWN:
-		break;
-	}
-}
-
 static xmlNode *_oval_VARIABLE_CONSTANT_to_dom(struct oval_variable *variable, xmlDoc * doc, xmlNode * parent) {
 	xmlNs *ns_definitions = xmlSearchNsByHref(doc, parent, OVAL_DEFINITIONS_NAMESPACE);
 	xmlNode *variable_node = xmlNewTextChild(parent, ns_definitions, BAD_CAST "constant_variable", NULL);
