@@ -307,6 +307,12 @@ char *SEXP_string_subcstr (const SEXP_t *s_exp, size_t beg, size_t len);
  */
 int SEXP_string_cmp (const SEXP_t *str_a, const SEXP_t *str_b);
 
+/**
+ * Try to cast the supplied sexp string to a boolean.
+ * @param s_ext the string to be cast
+ */
+bool SEXP_string_getb (const SEXP_t *s_exp);
+
 /*
  * list
  */
@@ -479,6 +485,8 @@ void       SEXP_listit_free (SEXP_it_t *it);
  */
 SEXP_t *SEXP_new (void);
 
+bool SEXP_emptyp(SEXP_t *sexp);
+
 /**
  * Create a new reference to a sexp object.
  * @param s_exp the object of which to increment the reference count
