@@ -20,16 +20,22 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+
 #include "generic/common.h"
 #include "public/sexp-manip.h"
 #include "_sexp-parser.h"
+#include "_seap-packetq.h"
 #include "_seap-packet.h"
 #include "_seap-scheme.h"
 #include "seap-descriptor.h"
-
+#include "public/seap-debug.h"
 #include "public/seap-message.h"
 #include "public/seap-command.h"
 #include "public/seap-error.h"
@@ -140,6 +146,7 @@ static int SEAP_packet_sexp2msg (SEXP_t *sexp_msg, SEAP_msg_t *seap_msg)
                                            "id", msg_n + 1, sexp_msg);
 
                                         sm_free (seap_msg->attrs);
+					SEXP_free(attr_name);
 
                                         return (SEAP_EUNEXP);
                                 }
@@ -158,11 +165,17 @@ static int SEAP_packet_sexp2msg (SEXP_t *sexp_msg, SEAP_msg_t *seap_msg)
                                                    (void *)attr_val, SEXP_strtype (attr_val));
 
                                                 sm_free (seap_msg->attrs);
+                                                SEXP_free(attr_val);
+                                                SEXP_free(attr_name);
+
                                                 return (SEAP_EINVAL);
                                         case EFAULT:
                                                 _D("id: not found\n");
 
                                                 sm_free (seap_msg->attrs);
+                                                SEXP_free(attr_val);
+                                                SEXP_free(attr_name);
+
                                                 return (SEAP_EINVAL);
                                         }
                                 }
@@ -186,6 +199,7 @@ static int SEAP_packet_sexp2msg (SEXP_t *sexp_msg, SEAP_msg_t *seap_msg)
                                         }
 
                                         sm_free (seap_msg->attrs);
+                                        SEXP_free(attr_name);
 
                                         return (SEAP_EINVAL);
                                 }
@@ -256,11 +270,9 @@ static SEXP_t *SEAP_packet_msg2sexp (SEAP_msg_t *msg)
                 SEXP_free(r0);
         }
 
-#if !defined(NDEBUG)
-        fprintf (stderr,   "--- pck out ---\n");
-        SEXP_fprintfa (stderr, sexp);
-        fprintf (stderr, "\n---------------\n");
-#endif
+	_I("MSG -> SEXP\n");
+	_S(sexp);
+	_I("packet size: %zu\n", SEXP_sizeof(sexp));
 
         return (sexp);
 }
@@ -360,6 +372,9 @@ static int SEAP_packet_sexp2cmd (SEXP_t *sexp_cmd, SEAP_cmd_t *seap_cmd)
         }
 
         if (item == NULL || mattrs < 1) {
+                if (item != NULL)
+                        SEXP_free(item);
+
                 errno = EINVAL;
                 return (-1);
         }
@@ -437,6 +452,10 @@ static SEXP_t *SEAP_packet_cmd2sexp (SEAP_cmd_t *cmd)
         if (cmd->args != NULL)
                 SEXP_list_add (sexp, cmd->args);
 
+	_I("CMD -> SEXP\n");
+	_S(sexp);
+	_I("packet size: %zu\n", SEXP_sizeof(sexp));
+
         SEXP_VALIDATE(sexp);
         return (sexp);
 }
@@ -522,6 +541,7 @@ static int SEAP_packet_sexp2err (SEXP_t *sexp_err, SEAP_err_t *seap_err)
                         return (-1);
                 }
 
+                SEXP_free(member);
                 ++n;
         }
 loop_exit:
@@ -554,6 +574,10 @@ static SEXP_t *SEAP_packet_err2sexp (SEAP_err_t *err)
         if (err->data != NULL)
                 SEXP_list_add (sexp, err->data);
 
+	_I("ERR -> SEXP\n");
+	_S(sexp);
+	_I("packet size: %zu\n", SEXP_sizeof(sexp));
+
         return (sexp);
 }
 
@@ -596,6 +620,8 @@ int SEAP_packet_recv (SEAP_CTX_t *ctx, int sd, SEAP_packet_t **packet)
         char        psym_cstr_b[16+1];
         char       *psym_cstr;
 
+	SEAP_packet_t *_packet;
+
         _LOGCALL_;
 
         dsc = SEAP_desc_get (ctx->sd_table, sd);
@@ -605,22 +631,9 @@ int SEAP_packet_recv (SEAP_CTX_t *ctx, int sd, SEAP_packet_t **packet)
                 return (-1);
         }
 
-        /*
-         * Check packet queue
-         */
-        if (pqueue_notempty (dsc->pck_queue)) {
-                /* TODO */
-                abort ();
-        }
+	if (SEAP_packetq_get(&dsc->pck_queue, packet) != -1)
+		return (0);
 
-        if (dsc->sexpbuf != NULL) {
-                if (SEXP_list_length (dsc->sexpbuf) > 0)
-                        goto sexp_buf_recv;
-                else {
-                        SEXP_free(dsc->sexpbuf);
-                        dsc->sexpbuf = NULL;
-                }
-        }
         /*
          * Event loop
          * The read mutex is not locked during the wait for an event.
@@ -710,7 +723,7 @@ eloop_exit:
                                 errno = ENETRESET;
                                 return (-1);
                         } else {
-                                errno = ECONNRESET;
+                                errno = ECONNABORTED;
                                 return (-1);
                         }
                 }
@@ -737,7 +750,19 @@ eloop_exit:
                                 _D("eloop_restart\n");
                                 goto eloop_start;
                         }
-                }
+                } else {
+			if (pstate == NULL || SEXP_pstate_errorp(pstate)) {
+				_D("FAIL: S-exp parsing error, buffer: length: %ld, content:\n%*.s\n",
+				   data_length, data_length, data_buffer);
+
+				SEXP_psetup_free(psetup);
+				SEXP_pstate_free(pstate);
+
+				errno = EILSEQ;
+
+				return (-1);
+			}
+		}
 
                 if (SCH_SELECT(dsc->scheme, dsc, SEAP_IO_EVREAD, ctx->recv_timeout, 0) != 0) {
                         switch (errno) {
@@ -746,6 +771,7 @@ eloop_exit:
                                         _D("FAIL: recv failed (timeout): dsc=%p, time=%hu, errno=%u, %s.\n",
                                            dsc, ctx->recv_timeout, errno, strerror (errno));
                                 }
+                                /* FALLTHROUGH */
                         default:
                                 protect_errno {
                                         _D("FAIL: recv failed: dsc=%p, errno=%u, %s.\n",
@@ -760,132 +786,141 @@ eloop_exit:
         }
 
         SEXP_psetup_free (psetup);
-        dsc->sexpbuf = sexp_buffer;
+	SEXP_VALIDATE(sexp_buffer);
+	(*packet) = NULL;
 
-sexp_buf_recv:
-        SEXP_VALIDATE(dsc->sexpbuf);
-        sexp_packet = SEXP_list_pop (dsc->sexpbuf);
+        while((sexp_packet = SEXP_list_pop (sexp_buffer)) != NULL) {
+		if (!SEXP_listp(sexp_packet)) {
+			_D("Invalid SEAP packet received: %s.\n", "not a list");
 
-        _A(sexp_packet != NULL);
+			SEXP_free (sexp_packet);
 
-        if (!SEXP_listp(sexp_packet)) {
-                _D("Invalid SEAP packet received: %s.\n", "not a list");
+			errno = EINVAL;
+			return (-1);
+		} else if (SEXP_list_length (sexp_packet) < 2) {
+			_D("Invalid SEAP packet received: %s.\n", "list length < 2");
 
-                SEXP_free (sexp_packet);
+			SEXP_free (sexp_packet);
 
-                errno = EINVAL;
-                return (-1);
-        } else if (SEXP_list_length (sexp_packet) < 2) {
-                _D("Invalid SEAP packet received: %s.\n", "list length < 2");
+			errno = EINVAL;
+			return (-1);
+		}
 
-                SEXP_free (sexp_packet);
+		psym_sexp = SEXP_list_first (sexp_packet);
 
-                errno = EINVAL;
-                return (-1);
-        }
+		if (!SEXP_stringp(psym_sexp)) {
+			_D("Invalid SEAP packet received: %s.\n", "first list item is not a string");
 
-        psym_sexp = SEXP_list_first (sexp_packet);
+			SEXP_free (psym_sexp);
+			SEXP_free (sexp_packet);
 
-        if (!SEXP_stringp(psym_sexp)) {
-                _D("Invalid SEAP packet received: %s.\n", "first list item is not a string");
+			errno = EINVAL;
+			return (-1);
+		} else if (SEXP_string_length (psym_sexp) != (strlen (SEAP_SYM_PREFIX) + 3)) {
+			_D("Invalid SEAP packet received: %s.\n", "invalid packet type symbol length");
 
-                SEXP_free (psym_sexp);
-                SEXP_free (sexp_packet);
+			SEXP_free (psym_sexp);
+			SEXP_free (sexp_packet);
 
-                errno = EINVAL;
-                return (-1);
-        } else if (SEXP_string_length (psym_sexp) != (strlen (SEAP_SYM_PREFIX) + 3)) {
-                _D("Invalid SEAP packet received: %s.\n", "invalid packet type symbol length");
+			errno = EINVAL;
+			return (-1);
+		} else if (SEXP_strncmp (psym_sexp, SEAP_SYM_PREFIX, strlen (SEAP_SYM_PREFIX)) != 0) {
+			_D("Invalid SEAP packet received: %s.\n", "invalid prefix");
 
-                SEXP_free (psym_sexp);
-                SEXP_free (sexp_packet);
+			SEXP_free (psym_sexp);
+			SEXP_free (sexp_packet);
 
-                errno = EINVAL;
-                return (-1);
-        } else if (SEXP_strncmp (psym_sexp, SEAP_SYM_PREFIX, strlen (SEAP_SYM_PREFIX)) != 0) {
-                _D("Invalid SEAP packet received: %s.\n", "invalid prefix");
+			errno = EINVAL;
+			return (-1);
+		}
 
-                SEXP_free (psym_sexp);
-                SEXP_free (sexp_packet);
+		SEXP_string_cstr_r (psym_sexp, psym_cstr_b, sizeof psym_cstr_b);
+		psym_cstr = psym_cstr_b + strlen (SEAP_SYM_PREFIX);
+		SEXP_free (psym_sexp);
 
-                errno = EINVAL;
-                return (-1);
-        }
+		switch (psym_cstr[0]) {
+		case 'm':
+			if (psym_cstr[1] == 's' &&
+			    psym_cstr[2] == 'g')
+			{
+				_packet = SEAP_packet_new ();
+				_packet->type = SEAP_PACKET_MSG;
 
-        SEXP_string_cstr_r (psym_sexp, psym_cstr_b, sizeof psym_cstr_b);
-        psym_cstr = psym_cstr_b + strlen (SEAP_SYM_PREFIX);
-        SEXP_free (psym_sexp);
+				if (SEAP_packet_sexp2msg (sexp_packet, &(_packet->data.msg)) != 0) {
+					/* error */
+					_D("Invalid SEAP packet received: %s.\n", "can't translate to msg struct");
 
-        switch (psym_cstr[0]) {
-        case 'm':
-                if (psym_cstr[1] == 's' &&
-                    psym_cstr[2] == 'g')
-                {
-                        (*packet) = SEAP_packet_new ();
-                        (*packet)->type = SEAP_PACKET_MSG;
+					SEXP_free (sexp_packet);
+					SEAP_packet_free(_packet);
 
-                        if (SEAP_packet_sexp2msg (sexp_packet, &((*packet)->data.msg)) != 0) {
-                                /* error */
-                                _D("Invalid SEAP packet received: %s.\n", "can't translate to msg struct");
+					errno = EINVAL;
+					return (-1);
+				}
+				break;
+			}
+			goto invalid;
+		case 'c':
+			if (psym_cstr[1] == 'm' &&
+			    psym_cstr[2] == 'd')
+			{
+				_packet = SEAP_packet_new ();
+				_packet->type = SEAP_PACKET_CMD;
 
-                                SEXP_free (sexp_packet);
+				if (SEAP_packet_sexp2cmd (sexp_packet, &(_packet->data.cmd)) != 0) {
+					/* error */
+					_D("Invalid SEAP packet received: %s.\n", "can't translate to cmd struct");
+					SEXP_free (sexp_packet);
+					SEAP_packet_free(_packet);
 
-                                errno = EINVAL;
-                                return (-1);
-                        }
-                        break;
-                }
-                goto invalid;
-        case 'c':
-                if (psym_cstr[1] == 'm' &&
-                    psym_cstr[2] == 'd')
-                {
-                        (*packet) = SEAP_packet_new ();
-                        (*packet)->type = SEAP_PACKET_CMD;
+					errno = EINVAL;
+					return (-1);
+				}
+				break;
+			}
+			goto invalid;
+		case 'e':
+			if (psym_cstr[1] == 'r' &&
+			    psym_cstr[2] == 'r')
+			{
+				_packet = SEAP_packet_new ();
+				_packet->type = SEAP_PACKET_ERR;
 
-                        if (SEAP_packet_sexp2cmd (sexp_packet, &((*packet)->data.cmd)) != 0) {
-                                /* error */
-                                _D("Invalid SEAP packet received: %s.\n", "can't translate to cmd struct");
-                                SEXP_free (sexp_packet);
+				if (SEAP_packet_sexp2err (sexp_packet, &(_packet->data.err)) != 0) {
+					/* error */
+					_D("Invalid SEAP packet received: %s.\n", "can't translate to err struct");
+					SEXP_free (sexp_packet);
+					SEAP_packet_free(_packet);
 
-                                errno = EINVAL;
-                                return (-1);
-                        }
-                        break;
-                }
-                goto invalid;
-        case 'e':
-                if (psym_cstr[1] == 'r' &&
-                    psym_cstr[2] == 'r')
-                {
-                        (*packet) = SEAP_packet_new ();
-                        (*packet)->type = SEAP_PACKET_ERR;
+					errno = EINVAL;
+					return (-1);
+				}
+				break;
+			}
+			/* FALLTHROUGH */
+		default:
+		invalid:
+			_D("Invalid SEAP packet received: %s.\n", "invalid packet type symbol");
+			SEXP_free (sexp_packet);
+			errno = EINVAL;
+			return (-1);
+		}
 
-                        if (SEAP_packet_sexp2err (sexp_packet, &((*packet)->data.err)) != 0) {
-                                /* error */
-                                _D("Invalid SEAP packet received: %s.\n", "can't translate to err struct");
-                                SEXP_free (sexp_packet);
 
-                                errno = EINVAL;
-                                return (-1);
-                        }
-                        break;
-                }
-        default:
-        invalid:
-                _D("Invalid SEAP packet received: %s.\n", "invalid packet type symbol");
-                SEXP_free (sexp_packet);
-                errno = EINVAL;
-                return (-1);
-        }
+		_I("Received packet\n");
+		_S(sexp_packet);
+		_I("packet size: %zu\n", SEXP_sizeof(sexp_packet));
 
-#if !defined(NDEBUG)
-        fprintf (stderr,   "--- pck in ---\n");
-        SEXP_fprintfa (stderr, sexp_packet);
-        fprintf (stderr, "\n--------------\n");
-#endif
+		SEXP_free(sexp_packet);
 
-        SEXP_free (sexp_packet);
+		if (*packet == NULL)
+			(*packet) = _packet;
+		else
+			SEAP_packetq_put(&dsc->pck_queue, _packet);
+
+		_packet = NULL;
+	}
+
+        SEXP_free(sexp_buffer);
 
         return (0);
 }
@@ -917,7 +952,7 @@ int SEAP_packet_recv_bytype (SEAP_CTX_t *ctx, int sd, SEAP_packet_t **packet, ui
                         return (ret);
                 else if (pck->type == type)
                         break;
-                else if (pqueue_add (dsc->pck_queue, pck) != 0)
+                else if (SEAP_packetq_put (&dsc->pck_queue, pck) == -1) /* XXX: infinite loop */
                         return (-1);
         }
 
