@@ -28,16 +28,24 @@
  *      Daniel Kopecek <dkopecek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <stdio.h>
 #include <string.h>
 #include <stdint.h>
 #include <stdbool.h>
 #include <errno.h>
+
 #include "common/assume.h"
+#include "public/sm_alloc.h"
 #include "_sexp-types.h"
 #include "_sexp-value.h"
 #include "_sexp-manip.h"
 #include "_sexp-rawptr.h"
-#include "public/sm_alloc.h"
+#include "public/sexp-manip.h"
+#include "public/sexp-manip_r.h"
 
 static void SEXP_free_lmemb (SEXP_t *s_exp);
 
@@ -50,7 +58,7 @@ SEXP_t *SEXP_number_new (SEXP_numtype_t t, const void *n)
         _LOGCALL_;
 
         if (t <= SEXP_NUM_UINT16) {
-                if (t <= SEXP_NUM_INT8) {
+                if (t <= SEXP_NUM_UINT8) {
                         switch (t) {
                         case SEXP_NUM_BOOL:  return SEXP_number_newb   (*(bool    *)(n));
                         case SEXP_NUM_INT8:  return SEXP_number_newi_8 (*(int8_t  *)(n));
@@ -83,26 +91,12 @@ SEXP_t *SEXP_number_new (SEXP_numtype_t t, const void *n)
 
 SEXP_t *SEXP_number_newb (bool n)
 {
-        SEXP_t    *s_exp;
-        SEXP_val_t v_dsc;
+        SEXP_t *sexp;
 
-        _LOGCALL_;
+        sexp = SEXP_new ();
+        sexp = SEXP_number_newb_r(sexp, n);
 
-        if (SEXP_val_new (&v_dsc, sizeof (SEXP_numtype_t) + sizeof (bool),
-                          SEXP_VALTYPE_NUMBER) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        SEXP_NCASTP(b,v_dsc.mem)->t = SEXP_NUM_BOOL;
-        SEXP_NCASTP(b,v_dsc.mem)->n = n;
-
-        s_exp = SEXP_new ();
-        s_exp->s_type = NULL;
-        s_exp->s_valp = v_dsc.ptr;
-
-        return (s_exp);
+        return (sexp);
 }
 
 SEXP_t *SEXP_number_newi_8  (int8_t n)
@@ -270,26 +264,12 @@ uint16_t SEXP_number_getu_16 (const SEXP_t *s_exp)
 
 SEXP_t *SEXP_number_newi_32 (int32_t n)
 {
-        SEXP_t    *s_exp;
-        SEXP_val_t v_dsc;
+        SEXP_t *sexp;
 
-        _LOGCALL_;
+        sexp = SEXP_new ();
+        sexp = SEXP_number_newi_32_r(sexp, n);
 
-        if (SEXP_val_new (&v_dsc, sizeof (SEXP_numtype_t) + sizeof (int32_t),
-                          SEXP_VALTYPE_NUMBER) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        SEXP_NCASTP(i32,v_dsc.mem)->t = SEXP_NUM_INT32;
-        SEXP_NCASTP(i32,v_dsc.mem)->n = n;
-
-        s_exp = SEXP_new ();
-        s_exp->s_type = NULL;
-        s_exp->s_valp = v_dsc.ptr;
-
-        return (s_exp);
+        return (sexp);
 }
 
 int32_t SEXP_number_geti_32 (const SEXP_t *s_exp)
@@ -335,26 +315,12 @@ int32_t SEXP_number_geti_32 (const SEXP_t *s_exp)
 
 SEXP_t *SEXP_number_newu_32 (uint32_t n)
 {
-        SEXP_t    *s_exp;
-        SEXP_val_t v_dsc;
+        SEXP_t *sexp;
 
-        _LOGCALL_;
+        sexp = SEXP_new ();
+        sexp = SEXP_number_newu_32_r(sexp, n);
 
-        if (SEXP_val_new (&v_dsc, sizeof (SEXP_numtype_t) + sizeof (uint32_t),
-                          SEXP_VALTYPE_NUMBER) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        SEXP_NCASTP(u32,v_dsc.mem)->t = SEXP_NUM_UINT32;
-        SEXP_NCASTP(u32,v_dsc.mem)->n = n;
-
-        s_exp = SEXP_new ();
-        s_exp->s_type = NULL;
-        s_exp->s_valp = v_dsc.ptr;
-
-        return (s_exp);
+        return (sexp);
 }
 
 uint32_t SEXP_number_getu_32 (const SEXP_t *s_exp)
@@ -469,26 +435,12 @@ int64_t SEXP_number_geti_64 (const SEXP_t *s_exp)
 
 SEXP_t *SEXP_number_newu_64 (uint64_t n)
 {
-        SEXP_t    *s_exp;
-        SEXP_val_t v_dsc;
+        SEXP_t *sexp;
 
-        _LOGCALL_;
+        sexp = SEXP_new();
+        sexp = SEXP_number_newu_64_r(sexp, n);
 
-        if (SEXP_val_new (&v_dsc, sizeof (SEXP_numtype_t) + sizeof (uint64_t),
-                          SEXP_VALTYPE_NUMBER) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        SEXP_NCASTP(u64,v_dsc.mem)->t = SEXP_NUM_UINT64;
-        SEXP_NCASTP(u64,v_dsc.mem)->n = n;
-
-        s_exp = SEXP_new ();
-        s_exp->s_type = NULL;
-        s_exp->s_valp = v_dsc.ptr;
-
-        return (s_exp);
+        return (sexp);
 }
 
 uint64_t SEXP_number_getu_64 (const SEXP_t *s_exp)
@@ -583,26 +535,12 @@ bool SEXP_number_getb (const SEXP_t *s_exp)
 
 SEXP_t *SEXP_number_newf (double n)
 {
-        SEXP_t    *s_exp;
-        SEXP_val_t v_dsc;
+        SEXP_t *sexp;
 
-        _LOGCALL_;
+        sexp = SEXP_new();
+        sexp = SEXP_number_newf_r(sexp, n);
 
-        if (SEXP_val_new (&v_dsc, sizeof (SEXP_numtype_t) + sizeof (double),
-                          SEXP_VALTYPE_NUMBER) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        SEXP_NCASTP(f,v_dsc.mem)->t = SEXP_NUM_DOUBLE;
-        SEXP_NCASTP(f,v_dsc.mem)->n = n;
-
-        s_exp = SEXP_new ();
-        s_exp->s_type = NULL;
-        s_exp->s_valp = v_dsc.ptr;
-
-        return (s_exp);
+        return (sexp);
 }
 
 double SEXP_number_getf (const SEXP_t *s_exp)
@@ -671,65 +609,25 @@ SEXP_numtype_t SEXP_number_type (const SEXP_t *s_exp)
 
 SEXP_t *SEXP_string_new  (const void *string, size_t length)
 {
-        SEXP_t    *s_exp;
-        SEXP_val_t v_dsc;
+        SEXP_t *sexp;
 
-        _LOGCALL_;
+        sexp = SEXP_new ();
+        sexp = SEXP_string_new_r(sexp, string, length);
 
-        if (SEXP_val_new (&v_dsc, sizeof (char) * length,
-                          SEXP_VALTYPE_STRING) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        memcpy (v_dsc.mem, string, sizeof (char) * length);
-
-        s_exp = SEXP_new ();
-        s_exp->s_type = NULL;
-        s_exp->s_valp = v_dsc.ptr;
-
-        return (s_exp);
+        return (sexp);
 }
 
 SEXP_t *SEXP_string_newf (const char *format, ...)
 {
-        va_list    alist;
-        SEXP_t    *s_exp;
-        SEXP_val_t v_dsc;
-        char      *v_string;
-        int        v_strlen;
+        va_list ap;
+        SEXP_t *sexp;
 
-        _A(format != NULL);
-        _LOGCALL_;
+        sexp = SEXP_new ();
+        va_start(ap, format);
+        sexp = SEXP_string_newf_rv(sexp, format, ap);
+        va_end(ap);
 
-        va_start (alist, format);
-        v_strlen = vasprintf (&v_string, format, alist);
-
-        _D("v_strlen = %d\n", v_strlen);
-
-        if (v_strlen < 0) {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        va_end (alist);
-
-        if (SEXP_val_new (&v_dsc, sizeof (char) * v_strlen,
-                          SEXP_VALTYPE_STRING) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        memcpy  (v_dsc.mem, v_string, sizeof (char) * v_strlen);
-        sm_free (v_string);
-
-        s_exp = SEXP_new ();
-        s_exp->s_type = NULL;
-        s_exp->s_valp = v_dsc.ptr;
-
-        return (s_exp);
+        return (sexp);
 }
 
 void SEXP_string_free (SEXP_t *s_exp)
@@ -837,12 +735,9 @@ int SEXP_strncmp (const SEXP_t *s_exp, const char *str, size_t n)
                 if (str[i] == '\0')
                         return (((unsigned char *)v_dsc.mem)[i] - '\0');
 
-                _D("%c ?= %c\n",  ((unsigned char *)v_dsc.mem)[i], ((unsigned char *)str)[i]);
-
                 c = ((unsigned char *)v_dsc.mem)[i] - ((unsigned char *)str)[i];
 
                 if (c != 0 || (i + 1) >= n) {
-                        _D("=> %d\n", c);
                         return (c);
                 }
         }
@@ -1029,64 +924,49 @@ int SEXP_string_cmp (const SEXP_t *str_a, const SEXP_t *str_b)
         return (c);
 }
 
+bool SEXP_string_getb (const SEXP_t *s_exp)
+{
+        SEXP_val_t v_dsc;
+        size_t     len;
+
+        _LOGCALL_;
+
+        if (s_exp == NULL) {
+                errno = EFAULT;
+                return (false);
+        }
+
+        SEXP_VALIDATE(s_exp);
+
+        SEXP_val_dsc (&v_dsc, s_exp->s_valp);
+
+        if (v_dsc.type != SEXP_VALTYPE_STRING) {
+                errno = EINVAL;
+                return (false);
+        }
+
+        len = v_dsc.hdr->size / sizeof (char);
+	if (!strncmp("1", v_dsc.mem, len) || !strncmp("true", v_dsc.mem, len))
+		return (true);
+
+        return (false);
+}
+
 /*
  * Lists
  */
 
 SEXP_t *SEXP_list_new (SEXP_t *memb, ...)
 {
-        va_list    alist;
-        SEXP_val_t v_dsc;
-        SEXP_t    *s_ptr[32];
-        size_t     s_cur;
-        uint8_t    b_exp;
-        SEXP_t    *s_exp;
+        SEXP_t *list;
+        va_list ap;
 
-        _LOGCALL_;
+        va_start(ap, memb);
+        list = SEXP_new ();
+        list = SEXP_list_new_rv(list, memb, ap);
+        va_end(ap);
 
-        va_start (alist, memb);
-
-        s_cur = 0;
-        s_ptr[s_cur] = memb;
-
-        while (s_ptr[s_cur] != NULL) {
-                _A(s_cur < (sizeof s_ptr / sizeof (SEXP_t *)));
-                SEXP_VALIDATE(s_ptr[s_cur]);
-
-                s_ptr[++s_cur] = va_arg (alist, SEXP_t *);
-        }
-
-        if (SEXP_val_new (&v_dsc, sizeof (void *) + sizeof (uint16_t),
-                          SEXP_VALTYPE_LIST) != 0)
-        {
-                /* TODO: handle this */
-                return (NULL);
-        }
-
-        if (s_cur > 0) {
-                for (b_exp = 0; (size_t)(1 << b_exp) < s_cur; ++b_exp);
-
-                SEXP_LCASTP(v_dsc.mem)->offset = 0;
-                SEXP_LCASTP(v_dsc.mem)->b_addr = (void *)SEXP_rawval_lblk_new (b_exp);
-
-                if (SEXP_rawval_lblk_fill ((uintptr_t)SEXP_LCASTP(v_dsc.mem)->b_addr,
-                                           s_ptr, s_cur) != ((uintptr_t)SEXP_LCASTP(v_dsc.mem)->b_addr))
-                {
-                        /* TODO: handle this */
-                        return (NULL);
-                }
-        } else {
-                SEXP_LCASTP(v_dsc.mem)->offset = 0;
-                SEXP_LCASTP(v_dsc.mem)->b_addr = NULL;
-        }
-
-        s_exp = SEXP_new ();
-        s_exp->s_type = NULL;
-        s_exp->s_valp = v_dsc.ptr;
-
-        SEXP_VALIDATE(s_exp);
-
-        return (s_exp);
+        return (list);
 }
 
 void SEXP_list_free (SEXP_t *s_exp)
@@ -1230,7 +1110,7 @@ SEXP_t *SEXP_list_rest  (const SEXP_t *list)
                 }
 
                 if (SEXP_VALP_LBLK(SEXP_LCASTP(v_dsc_r.mem)->b_addr) != NULL)
-                        SEXP_rawval_lblk_incref ((uintptr_t) SEXP_LCASTP(v_dsc_r.mem)->b_addr);
+                        SEXP_LCASTP(v_dsc_r.mem)->b_addr = (void *)SEXP_rawval_lblk_incref ((uintptr_t) SEXP_LCASTP(v_dsc_r.mem)->b_addr);
         }
 
         rest = SEXP_new ();
@@ -1645,6 +1525,14 @@ SEXP_t *SEXP_new (void)
         return (s_exp);
 }
 
+bool SEXP_emptyp(SEXP_t *sexp)
+{
+	if (sexp == NULL)
+		return (true);
+	else
+		return (sexp->s_valp == 0);
+}
+
 SEXP_t *SEXP_ref (const SEXP_t *s_exp_o)
 {
         SEXP_t *s_exp_r;
@@ -1698,7 +1586,7 @@ SEXP_t *SEXP_unref (SEXP_t *s_exp_o)
                         s_exp_o->__magic1 = SEXP_MAGIC1_INV;
 #endif
                         sm_free (s_exp_o);
-                        s_exp_o = NULL;
+			return (NULL);
                 }
 
 		s_exp_o->s_type = SEXP_rawptr_safemergeT(SEXP_datatypePtr_t, s_exp_o->s_type,
@@ -1834,58 +1722,23 @@ static void SEXP_free_lmemb (SEXP_t *s_exp)
 
 #if defined(NDEBUG)
 void SEXP_free (SEXP_t *s_exp)
-#else
-void __SEXP_free (SEXP_t *s_exp, const char *file, uint32_t line, const char *func)
-#endif
 {
-        _LOGCALL_;
-
-#if !defined(NDEBUG)
-        _D("s_exp=%p (%s:%u:%s)\n", s_exp, file, line, func);
-#endif
-
-        if (s_exp == NULL) {
-                _D("WARN: s_exp = NULL\n");
-                return;
+        if (s_exp != NULL) {
+                SEXP_free_r(s_exp);
+                sm_free(s_exp);
         }
-
-	if (!SEXP_softrefp(s_exp) && SEXP_typeof(s_exp) != SEXP_TYPE_EMPTY) {
-                SEXP_val_t v_dsc;
-
-                SEXP_VALIDATE(s_exp);
-                SEXP_val_dsc (&v_dsc, s_exp->s_valp);
-
-                if (SEXP_rawval_decref (s_exp->s_valp)) {
-                        switch (v_dsc.type) {
-                        case SEXP_VALTYPE_STRING:
-                                sm_free (v_dsc.hdr);
-                                break;
-                        case SEXP_VALTYPE_NUMBER:
-                                sm_free (v_dsc.hdr);
-                                break;
-                        case SEXP_VALTYPE_LIST:
-                                if (SEXP_LCASTP(v_dsc.mem)->b_addr != NULL)
-                                        SEXP_rawval_lblk_free ((uintptr_t)SEXP_LCASTP(v_dsc.mem)->b_addr, SEXP_free_lmemb);
-
-                                sm_free (v_dsc.hdr);
-                                break;
-                        default:
-                                abort ();
-                        }
-                }
-        }
-
-#if !defined(NDEBUG) || defined(VALIDATE_SEXP)
-        s_exp->s_valp = 0;
-        s_exp->s_type = NULL;
-        s_exp->__magic0 = SEXP_MAGIC0_INV;
-        s_exp->__magic1 = SEXP_MAGIC1_INV;
-#endif
-
-        sm_free (s_exp);
-
         return;
 }
+#else
+void __SEXP_free (SEXP_t *s_exp, const char *file, uint32_t line, const char *func)
+{
+        if (s_exp != NULL) {
+                __SEXP_free_r(s_exp, file, line, func);
+                sm_free (s_exp);
+        }
+        return;
+}
+#endif
 
 #if defined(NDEBUG)
 void SEXP_vfree (SEXP_t *s_exp, ...)
@@ -1895,7 +1748,7 @@ void __SEXP_vfree (const char *file, uint32_t line, const char *func, SEXP_t *s_
 {
         va_list ap;
 
-#if !defined(NDEBUG)
+#if !defined(NDEBUG) && defined(SEAP_VERBOSE_DEBUG)
         _D("s_exp=%p (%s:%u:%s)\n", s_exp, file, line, func);
 #endif
 
@@ -1977,7 +1830,7 @@ int SEXP_datatype_set_nth (SEXP_t *list, uint32_t n, const char *name)
         if (v_dsc.type != SEXP_VALTYPE_LIST)
                 return (-1);
 
-        s_nth = SEXP_rawval_lblk_nth (list->s_valp, n);
+        s_nth = SEXP_rawval_lblk_nth ((uintptr_t)SEXP_LCASTP(v_dsc.mem)->b_addr, n);
 
         if (s_nth == NULL)
                 return (-1);
