@@ -28,6 +28,10 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stddef.h>
 #include <string.h>
@@ -40,6 +44,10 @@
 #if defined(__linux__)
 # include <mntent.h>
 # include <unistd.h>
+#elif defined(__SVR4) && defined(__sun)
+# include <sys/mnttab.h>
+# include <sys/mntent.h>
+# include <sys/unistd.h>
 #elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 # include <sys/param.h>
 # include <sys/ucred.h>
@@ -204,6 +212,75 @@ static fsdev_t *__fsdev_init(fsdev_t * lfs, const char **fs, size_t fs_cnt)
 
 	return (lfs);
 }
+#elif defined(__SVR4) && defined(__sun)
+
+#define DEVID_ARRAY_SIZE 16
+#define DEVID_ARRAY_ADD  8
+
+static fsdev_t *__fsdev_init(fsdev_t * lfs, const char **fs, size_t fs_cnt)
+{
+	int e;
+	FILE *fp;
+	size_t i;
+
+	struct mnttab *ment;
+	struct stat st;
+
+	fp = fopen(MNTTAB, "r");
+	if (fp == NULL) {
+		e = errno;
+		free(lfs);
+		errno = e;
+		return (NULL);
+	}
+
+	lfs->ids = malloc(sizeof(dev_t) * DEVID_ARRAY_SIZE);
+
+	if (lfs->ids == NULL) {
+		e = errno;
+		free(lfs);
+		errno = e;
+		return (NULL);
+	}
+
+	lfs->cnt = DEVID_ARRAY_SIZE;
+	i = 0;
+
+	if (fs == NULL) {
+		while ((getmntent(fp, ment)) != 0) {
+                        /* TODO: Is this check reliable? */
+                        if (stat (ment->mnt_special, &st) == 0 && (st.st_mode & S_IFCHR)) {
+
+				if (i >= lfs->cnt) {
+					lfs->cnt += DEVID_ARRAY_ADD;
+					lfs->ids = realloc(lfs->ids, sizeof(dev_t) * lfs->cnt);
+				}
+
+				memcpy(&(lfs->ids[i++]), &st.st_dev, sizeof(dev_t));
+			}
+		}
+	} else {
+		while ((getmntent(fp, ment)) != 0) {
+
+			if (match_fs(ment->mnt_fstype, fs, fs_cnt)) {
+
+				if (i >= lfs->cnt) {
+					lfs->cnt += DEVID_ARRAY_ADD;
+					lfs->ids = realloc(lfs->ids, sizeof(dev_t) * lfs->cnt);
+				}
+
+				memcpy(&(lfs->ids[i++]), &st.st_dev, sizeof(dev_t));
+			}
+		}
+	}
+
+	fclose(fp);
+
+	lfs->ids = realloc(lfs->ids, sizeof(dev_t) * i);
+	lfs->cnt = (lfs->ids == NULL ? 0 : i);
+
+	return (lfs);
+}
 #endif
 
 fsdev_t *fsdev_init(const char **fs, size_t fs_cnt)
@@ -211,12 +288,15 @@ fsdev_t *fsdev_init(const char **fs, size_t fs_cnt)
 	fsdev_t *lfs;
 
 	lfs = malloc(sizeof(fsdev_t));
-	lfs = __fsdev_init(lfs, fs, fs_cnt);
 
 	if (lfs == NULL)
 		return (NULL);
 
-	qsort(lfs->ids, lfs->cnt, sizeof(dev_t), fsdev_cmp);
+	if (__fsdev_init(lfs, fs, fs_cnt) == NULL)
+		return (NULL);
+
+        if (lfs->ids != NULL && lfs->cnt > 1)
+                qsort(lfs->ids, lfs->cnt, sizeof(dev_t), fsdev_cmp);
 
 	return (lfs);
 }
@@ -261,7 +341,8 @@ fsdev_t *fsdev_strinit(const char *fs_names)
 		}
 	}
 
-	qsort(fs_arr, fs_cnt, sizeof(char *), fsname_cmp);
+	if (fs_arr != NULL && fs_cnt > 0)
+		qsort(fs_arr, fs_cnt, sizeof(char *), fsname_cmp);
 
 	lfs = fsdev_init((const char **)fs_arr, fs_cnt);
 	e = errno;
