@@ -44,16 +44,20 @@
  *    [0..1] string family
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <seap.h>
 #include <string.h>
 #include <probe-api.h>
 
-int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg)
+int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
-	SEXP_t *item, *v_fm;
+	SEXP_t *item;
 
         (void)arg;
+        (void)filters;
 
 	const char *family =
 #if defined(_WIN32)
@@ -72,14 +76,12 @@ int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg)
 		return PROBE_EINVAL;
 	}
 
-        item  = probe_item_creat ("family_item", NULL,
-                                  /* entities */
-                                  "family", NULL, v_fm = SEXP_string_new (family, strlen (family)),
-                                  NULL);
-	probe_cobj_add_item(probe_out, item);
+        item = probe_item_create(OVAL_INDEPENDENT_FAMILY, NULL,
+                                 "family", OVAL_DATATYPE_STRING, family,
+                                 NULL);
 
+	probe_cobj_add_item(probe_out, item);
         SEXP_free (item);
-        SEXP_free (v_fm);
 
 	return (0);
 }
