@@ -30,6 +30,10 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <seap.h>
 #include <probe-api.h>
 #include <stdlib.h>
@@ -41,6 +45,7 @@
 #include <pthread.h>
 #include <errno.h>
 #include <crapi/crapi.h>
+
 #include "oval_fts.h"
 
 #define FILE_SEPARATOR '/'
@@ -69,7 +74,7 @@ static int mem2hex (uint8_t *mem, size_t mlen, char *str, size_t slen)
 
 static int filehash_cb (const char *p, const char *f, void *ptr)
 {
-        SEXP_t *itm, *r0, *r1, *r2, *r3;
+        SEXP_t *itm, *r0, *r1;
         SEXP_t *res = (SEXP_t *) ptr;
 
         char   pbuf[PATH_MAX+1];
@@ -157,18 +162,13 @@ static int filehash_cb (const char *p, const char *f, void *ptr)
                 /*
                  * Create and add the item
                  */
-                itm = probe_item_creat ("filehash_item", NULL,
-                                        /* entities */
-                                        "path", NULL,
-                                        r0 = SEXP_string_newf (p, plen),
-                                        "filename", NULL,
-                                        r1 = SEXP_string_newf (f, flen),
-                                        "md5", NULL,
-                                        r2 = SEXP_string_newf (md5_str, sizeof md5_str - 1),
-                                        "sha1", NULL,
-                                        r3 = SEXP_string_newf (sha1_str, sizeof sha1_str - 1),
+                itm = probe_item_create(OVAL_INDEPENDENT_FILE_HASH, NULL,
+                                        "filepath", OVAL_DATATYPE_STRING, pbuf,
+                                        "path",     OVAL_DATATYPE_STRING, p,
+                                        "filename", OVAL_DATATYPE_STRING, f,
+                                        "md5",      OVAL_DATATYPE_STRING, md5_str,
+                                        "sha1",     OVAL_DATATYPE_STRING, sha1_str,
                                         NULL);
-                SEXP_vfree (r0, r1, r2, r3, NULL);
         }
 
 	probe_cobj_add_item(res, itm);
@@ -212,15 +212,15 @@ void probe_fini (void *arg)
         return;
 }
 
-int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex)
+int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex, SEXP_t *filters)
 {
-        SEXP_t *path, *filename, *behaviors;
-        SEXP_t *r0, *r1, *r2;
-        int     filecnt;
+        SEXP_t *path, *filename, *behaviors, *filepath;
 
 	OVAL_FTS    *ofts;
 	OVAL_FTSENT *ofts_ent;
 
+        (void)filters;
+
 	if (probe_in == NULL || probe_out == NULL) {
 		return (PROBE_EINVAL);
 	}
@@ -234,46 +234,25 @@ int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex)
         path      = probe_obj_getent (probe_in, "path",      1);
         filename  = probe_obj_getent (probe_in, "filename",  1);
         behaviors = probe_obj_getent (probe_in, "behaviors", 1);
+        filepath = probe_obj_getent (probe_in, "filepath", 1);
 
-        if (path == NULL || filename == NULL) {
+        /* we want either path+filename or filepath */
+        if ( (path == NULL || filename == NULL) && filepath==NULL ) {
                 SEXP_free (behaviors);
                 SEXP_free (path);
                 SEXP_free (filename);
+		SEXP_free (filepath);
 
 		return (PROBE_ENOELM);
         }
 
-        if (behaviors == NULL) {
-                SEXP_t *bh_list;
-
-                bh_list = probe_ent_creat ("behaviors",
-                                           r0 = probe_attr_creat ("max_depth",
-                                                                  r1 = SEXP_string_newf ("1"),
-                                                                  "recurse_direction",
-                                                                  r2 = SEXP_string_newf ("none"),
-                                                                  NULL),
-                                           NULL /* val */,
-                                           NULL /* end */);
-
-                behaviors = SEXP_list_first (bh_list);
-
-                SEXP_vfree (bh_list,
-                            r0, r1, r2, NULL);
-        } else {
-                if (!probe_ent_attrexists (behaviors, "max_depth")) {
-                        probe_ent_attr_add (behaviors,
-                                            "max_depth", r0 = SEXP_string_newf ("-1"));
-                        SEXP_free (r0);
-                }
-
-                if (!probe_ent_attrexists (behaviors, "recurse_direction")) {
-                        probe_ent_attr_add (behaviors,
-                                            "recurse_direction", r0 = SEXP_string_newf ("none"));
-                        SEXP_free (r0);
-                }
+        /* behaviours are not important if filepath is used */
+        if(filepath != NULL && behaviors != NULL) {
+                SEXP_free (behaviors);
+                behaviors = NULL;
         }
 
-        _A(behaviors != NULL);
+	probe_filebehaviors_canonicalize(&behaviors);
 
         switch (pthread_mutex_lock (&__filehash_probe_mutex)) {
         case 0:
@@ -281,13 +260,16 @@ int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex)
         default:
                 _D("Can't lock mutex(%p): %u, %s.\n", &__filehash_probe_mutex, errno, strerror (errno));
 
+		SEXP_free (behaviors);
+		SEXP_free (path);
+		SEXP_free (filename);
+		SEXP_free (filepath);
+
 		return (PROBE_EFATAL);
         }
 
-	filecnt = 0;
-
-	if ((ofts = oval_fts_open(path, filename, NULL, behaviors)) != NULL) {
-		for (; (ofts_ent = oval_fts_read(ofts)) != NULL; ++filecnt) {
+	if ((ofts = oval_fts_open(path, filename, filepath, behaviors)) != NULL) {
+		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
 			filehash_cb(ofts_ent->path, ofts_ent->file, probe_out);
 			oval_ftsent_free(ofts_ent);
 		}
@@ -295,20 +277,10 @@ int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex)
 		oval_fts_close(ofts);
 	}
 
-        if (filecnt < 0) {
-		char s[50];
-		SEXP_t *msg;
-
-		snprintf(s, sizeof (s), "find_files returned error: %d", filecnt);
-		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, s);
-		probe_cobj_add_msg(probe_out, msg);
-		SEXP_free(msg);
-		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
-        }
-
         SEXP_free (behaviors);
         SEXP_free (path);
         SEXP_free (filename);
+        SEXP_free (filepath);
 
         switch (pthread_mutex_unlock (&__filehash_probe_mutex)) {
         case 0:
