@@ -49,8 +49,13 @@
  *    string value_of
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -60,6 +65,7 @@
 #include <seap.h>
 #include <probe-api.h>
 #include <oval_fts.h>
+#include <common/debug_priv.h>
 
 #define FILE_SEPARATOR '/'
 
@@ -93,17 +99,14 @@ void probe_fini(void *arg)
 static int process_file(const char *path, const char *filename, void *arg)
 {
 	struct pfdata *pfd = (struct pfdata *) arg;
-	int i, ret = 0, path_len, filename_len, node_cnt;
+	int ret = 0, path_len, filename_len;
 	char *whole_path = NULL;
-	xmlDocPtr doc = NULL;
-	xmlXPathContextPtr xpath_ctx = NULL;
-	xmlXPathObjectPtr xpath_obj = NULL;
-	xmlNodeSetPtr nodes;
-	xmlNodePtr cur_node, *node_tab;
-	xmlChar *value;
-	SEXP_t *item;
-
-        SEXP_t *r0, *r1, *r2;
+	xmlDoc *doc = NULL;
+	xmlXPathContext *xpath_ctx = NULL;
+	xmlXPathObject *xpath_obj = NULL;
+	SEXP_t *item = NULL;
+        SEXP_t *r0;
+        char filepath[PATH_MAX+1];
 
 	if (filename == NULL)
 		goto cleanup;
@@ -141,45 +144,99 @@ static int process_file(const char *path, const char *filename, void *arg)
 		goto cleanup;
 	}
 
-	nodes = xpath_obj->nodesetval;
-	if (nodes == NULL) {
-		ret = -4;
-		goto cleanup;
+        snprintf(filepath, PATH_MAX, "%s%c%s", path, FILE_SEPARATOR, filename);
+
+        item = probe_item_create(OVAL_INDEPENDENT_XML_FILE_CONTENT, NULL,
+                                 "filepath", OVAL_DATATYPE_STRING, filepath,
+                                 "path",     OVAL_DATATYPE_STRING, path,
+                                 "filename", OVAL_DATATYPE_STRING, filename,
+                                 "xpath",    OVAL_DATATYPE_STRING, pfd->xpath,
+                                 NULL);
+
+	dI("xpath obj type: %d.\n", xpath_obj->type);
+	switch(xpath_obj->type) {
+	case XPATH_BOOLEAN:
+	{
+		SEXP_t *val;
+		int b;
+
+		b = xmlXPathCastToBoolean(xpath_obj);
+		val = SEXP_number_newb(b);
+		probe_item_ent_add(item, "value_of", NULL, val);
+		SEXP_free(val);
+		break;
 	}
+	case XPATH_NUMBER:
+	{
+		SEXP_t *val;
+		double d;
 
-        item = probe_item_creat("xmlfilecontent_item", NULL,
-                                /* entities */
-                                "path",     NULL, r0 = SEXP_string_new(path, strlen (path)),
-                                "filename", NULL, r1 = SEXP_string_new(filename, strlen (filename)),
-                                "xpath",    NULL, r2 = SEXP_string_new(pfd->xpath, strlen (pfd->xpath)),
-                                NULL);
+		d = xmlXPathCastToNumber(xpath_obj);
+		val = SEXP_number_newf(d);
+		probe_item_ent_add(item, "value_of", NULL, val);
+		SEXP_free(val);
+		break;
+	}
+	case XPATH_STRING:
+	{
+		SEXP_t *val;
+		char *s;
 
-        SEXP_vfree (r0, r1, r2, NULL);
+		s = (char *) xmlXPathCastToString(xpath_obj);
+		val = SEXP_string_newf("%s", s);
+		xmlFree(s);
+		probe_item_ent_add(item, "value_of", NULL, val);
+		SEXP_free(val);
+		break;
+	}
+	case XPATH_NODESET:
+	{
+		int node_cnt, i;
+		xmlNodeSet *nodes;
+		xmlNode *cur_node, **node_tab;
 
-	node_cnt = nodes->nodeNr;
-	if (node_cnt == 0) {
-		probe_item_setstatus(item, OVAL_STATUS_DOESNOTEXIST);
-		probe_item_ent_add(item, "value_of", NULL, NULL);
-                probe_itement_setstatus(item, "value_of", 1, OVAL_STATUS_DOESNOTEXIST);
-        } else {
-		node_tab = nodes->nodeTab;
-		for (i = 0; i < node_cnt; ++i) {
-			cur_node = nodes->nodeTab[i];
-			if (cur_node->type == XML_ATTRIBUTE_NODE ||
-			    cur_node->type == XML_TEXT_NODE)
-                        {
-				value = xmlNodeGetContent(cur_node);
-				probe_item_ent_add(item, "value_of", NULL, r0 = SEXP_string_new ((char *) value, strlen ((char *) value)));
-				xmlFree(value);
-                                SEXP_free (r0);
+		nodes = xpath_obj->nodesetval;
+		if (nodes == NULL) {
+			ret = -4;
+			goto cleanup;
+		}
+
+		node_cnt = nodes->nodeNr;
+		dI("node_cnt: %d.\n", node_cnt);
+		if (node_cnt == 0) {
+			probe_item_setstatus(item, OVAL_STATUS_DOESNOTEXIST);
+			probe_item_ent_add(item, "value_of", NULL, NULL);
+			probe_itement_setstatus(item, "value_of", 1, OVAL_STATUS_DOESNOTEXIST);
+		} else {
+			node_tab = nodes->nodeTab;
+			for (i = 0; i < node_cnt; ++i) {
+				cur_node = node_tab[i];
+				dI("node[%d] line: %d, name: '%s', type: %d.\n",
+				   i, cur_node->line, cur_node->name, cur_node->type);
+				if (cur_node->type == XML_ATTRIBUTE_NODE
+				    || cur_node->type == XML_TEXT_NODE) {
+					xmlChar *value;
+
+					value = xmlNodeGetContent(cur_node);
+					probe_item_ent_add(item, "value_of", NULL,
+							   r0 = SEXP_string_newf ("%s", (char *) value));
+					xmlFree(value);
+					SEXP_free (r0);
+				}
 			}
 		}
+		break;
+	}
+	default:
+		probe_item_setstatus(item, OVAL_STATUS_DOESNOTEXIST);
+		break;
 	}
 
 	probe_cobj_add_item(pfd->cobj, item);
-        SEXP_free (item);
 
  cleanup:
+	if (item != NULL)
+		SEXP_free(item);
 	if (xpath_obj != NULL)
 		xmlXPathFreeObject(xpath_obj);
 	if (xpath_ctx != NULL)
@@ -192,60 +249,47 @@ static int process_file(const char *path, const char *filename, void *arg)
 	return ret;
 }
 
-int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg)
+int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
-	SEXP_t *path_ent, *filename_ent, *xpath_ent, *behaviors_ent;
-        SEXP_t *r0, *r1, *r2;
+	SEXP_t *path_ent, *filename_ent, *xpath_ent, *behaviors_ent, *filepath_ent;
+	SEXP_t *r0;
 
 	OVAL_FTS    *ofts;
 	OVAL_FTSENT *ofts_ent;
 
         (void)arg;
+        (void)filters;
 
 	if (probe_in == NULL || probe_out == NULL) {
 		return(PROBE_EINVAL);
 	}
 
-        if ((path_ent = probe_obj_getent(probe_in, "path", 1)) == NULL)
-		return(PROBE_ENOELM);
-        if ((filename_ent = probe_obj_getent(probe_in, "filename", 1)) == NULL) {
-                SEXP_free(path_ent);
-		return(PROBE_ENOELM);
-        }
-        if ((xpath_ent = probe_obj_getent(probe_in, "xpath", 1)) == NULL) {
-                SEXP_vfree(path_ent, filename_ent, NULL);
-		return(PROBE_ENOELM);
-        }
-
-	/* canonicalize behaviors */
+        path_ent = probe_obj_getent(probe_in, "path", 1);
+	filename_ent = probe_obj_getent(probe_in, "filename", 1);
+	xpath_ent = probe_obj_getent(probe_in, "xpath", 1);
+	filepath_ent = probe_obj_getent(probe_in, "filepath", 1);
 	behaviors_ent = probe_obj_getent(probe_in, "behaviors", 1);
-        if (behaviors_ent == NULL) {
-		SEXP_t *behaviors_new;
 
-		behaviors_new = probe_ent_creat("behaviors",
-                                                r0 = probe_attr_creat("max_depth", r1 = SEXP_string_newf ("1"),
-                                                                      "recurse_direction", r2 = SEXP_string_newf ("none"),
-                                                                      NULL),
-                                                NULL /* val */,
-                                                NULL /* end */);
+        /* we want (path+filename or filepath) + xpath */
+        if ( ((path_ent == NULL || filename_ent == NULL) && filepath_ent==NULL ) || 
+             xpath_ent==NULL) {
+                SEXP_free (path_ent);
+                SEXP_free (filename_ent);
+                SEXP_free (filepath_ent);
+                SEXP_free (xpath_ent);
+		SEXP_free (behaviors_ent);
 
-                SEXP_vfree (r0, r1, r2, NULL);
+                return PROBE_ENOELM;
+        }
 
-		behaviors_ent = SEXP_list_first(behaviors_new);
-                SEXP_free (behaviors_new);
-	} else {
-		if (!probe_ent_attrexists (behaviors_ent, "max_depth")) {
-			probe_ent_attr_add (behaviors_ent,"max_depth", r0 = SEXP_string_newf ("1"));
-                        SEXP_free (r0);
-                }
+        /* behaviours are not important if filepath is used */
+        if(filepath_ent != NULL && behaviors_ent != NULL) {
+                SEXP_free (behaviors_ent);
+                behaviors_ent = NULL;
+        }
 
-		if (!probe_ent_attrexists (behaviors_ent, "recurse_direction")) {
-			probe_ent_attr_add (behaviors_ent,"recurse_direction", r0 = SEXP_string_newf ("none"));
-                        SEXP_free (r0);
-                }
-	}
+	probe_filebehaviors_canonicalize(&behaviors_ent);
 
-	int fcnt;
 	struct pfdata pfd;
 
 	pfd.xpath = SEXP_string_cstr(r0 = probe_ent_getval(xpath_ent));
@@ -254,10 +298,8 @@ int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg)
 	pfd.filename_ent = filename_ent;
 	pfd.cobj = probe_out;
 
-	fcnt = 0;
-
-	if ((ofts = oval_fts_open(path_ent, filename_ent, NULL, behaviors_ent)) != NULL) {
-		for (; (ofts_ent = oval_fts_read(ofts)) != NULL; ++fcnt) {
+	if ((ofts = oval_fts_open(path_ent, filename_ent, filepath_ent, behaviors_ent)) != NULL) {
+		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
 			process_file(ofts_ent->path, ofts_ent->file, &pfd);
 			oval_ftsent_free(ofts_ent);
 		}
@@ -265,21 +307,11 @@ int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg)
 		oval_fts_close(ofts);
 	}
 
-	if (fcnt < 0) {
-		char s[50];
-		SEXP_t *msg;
-
-		snprintf(s, sizeof (s), "find_files returned error: %d", fcnt);
-		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, s);
-		probe_cobj_add_msg(probe_out, msg);
-		SEXP_free(msg);
-		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
-	}
-
         SEXP_free (path_ent);
         SEXP_free (filename_ent);
         SEXP_free (xpath_ent);
         SEXP_free (behaviors_ent);
+        SEXP_free (filepath_ent);
 
 	return 0;
 }
