@@ -30,19 +30,17 @@
 typedef struct {
 	FTS    *ofts_fts;
 
-	SEXP_t  *ofts_path;
 	char   **ofts_st_path;       /**< starting paths - NULL terminated */
 	uint16_t ofts_st_path_count; /**< number of starting paths */
 	uint16_t ofts_st_path_index; /**< current starting path */
 
 	pcre       *ofts_path_regex;
 	pcre_extra *ofts_path_regex_extra;
-	bool        ofts_path_regex_Opartial; /**< do "partial match" optimization? */
+	uint32_t ofts_path_op;
 
-	SEXP_t *ofts_filename;
-	SEXP_t *ofts_behaviors;
-
-	bool ofts_nilfilename;
+	SEXP_t *ofts_spath;
+	SEXP_t *ofts_sfilename;
+	SEXP_t *ofts_sfilepath;
 
 	int max_depth;
 	int direction;
@@ -68,16 +66,10 @@ typedef struct {
 #define OVAL_RECURSE_FS_ALL     2 /* default */
 
 typedef struct {
-	char  *filepath; /* path + file */
-	size_t filepath_len;
-
-	char  *file;     /* file - shares memory with filepath! */
+	char *file;
 	size_t file_len;
-
-	char  *path;     /* path - separate memory */
+	char *path;
 	size_t path_len;
-
-	bool  nilfile;
 } OVAL_FTSENT;
 
 /*
