@@ -7,7 +7,7 @@
  * @author "Tomas Heinrich" <theinric@redhat.com>
  */
 /*
- * Copyright 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2011 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -29,14 +29,24 @@
  *      Tomas Heinrich <theinric@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
+
 #include "debug_priv.h"
 #include "_probe-api.h"
 #include "assume.h"
 #include "memusage.h"
 #include "sysinfo.h"
+#include "oval_probe_impl.h"
+
+extern probe_rcache_t  *OSCAP_GSYM(pcache);
+extern probe_ncache_t  *OSCAP_GSYM(encache);
+extern struct id_desc_t OSCAP_GSYM(id_desc);
 
 /*
  * items
@@ -57,7 +67,7 @@ SEXP_t *probe_item_creat(const char *name, SEXP_t * attrs, ...)
 		attrs = va_arg(ap, SEXP_t *);
 		val = va_arg(ap, SEXP_t *);
 
-                ns  = encache_ref (OSCAP_GSYM(encache), name);
+                ns  = probe_ncache_ref (OSCAP_GSYM(encache), name);
 		ent = SEXP_list_new(NULL);
 
 		if (attrs != NULL) {
@@ -278,7 +288,7 @@ SEXP_t *probe_obj_creat(const char *name, SEXP_t * attrs, ...)
 		attrs = va_arg(ap, SEXP_t *);
 		val = va_arg(ap, SEXP_t *);
 
-                ns  = encache_ref (OSCAP_GSYM(encache), name);
+                ns  = probe_ncache_ref (OSCAP_GSYM(encache), name);
 		ent = SEXP_list_new(NULL);
 
 		if (attrs != NULL) {
@@ -313,7 +323,7 @@ SEXP_t *probe_obj_new(const char *name, SEXP_t * attrs)
 	_LOGCALL_;
 
 	obj = SEXP_list_new(NULL);
-	ns  = encache_ref (OSCAP_GSYM(encache), name);
+	ns  = probe_ncache_ref (OSCAP_GSYM(encache), name);
 
 	if (attrs != NULL) {
 		SEXP_t *nl, *nj;
@@ -356,7 +366,7 @@ SEXP_t *probe_obj_getent(const SEXP_t * obj, const char *name, uint32_t n)
 		}
 
 		if (SEXP_stringp(ent_name)) {
-#if !defined(NDEBUG)
+#if !defined(NDEBUG) && defined(SEAP_VERBOSE_DEBUG)
 			char buf[128];
 			SEXP_string_cstr_r(ent_name, buf, sizeof buf);
 			_D("1=\"%s\", 2=\"%s\", n=%u\n", buf, name, n);
@@ -416,8 +426,6 @@ SEXP_t *probe_obj_getattrval(const SEXP_t * obj, const char *name)
 	obj_name = SEXP_list_first(obj);
 	name_len = snprintf(name_buf, sizeof name_buf, ":%s", name);
 
-	_D("an=%s\n", name_buf);
-
 	_A(name_len < sizeof name_buf);
 
 	if (SEXP_listp(obj_name)) {
@@ -837,6 +845,7 @@ SEXP_t *probe_msg_creat(oval_message_level_t level, char *message)
 {
 	SEXP_t *lvl, *str, *msg;
 
+	_D(message);
 	lvl = SEXP_number_newu(level);
 	str = SEXP_string_newf("%s", message);
 	msg = SEXP_list_new(lvl, str, NULL);
@@ -845,6 +854,31 @@ SEXP_t *probe_msg_creat(oval_message_level_t level, char *message)
 	return msg;
 }
 
+SEXP_t *probe_msg_creatf(oval_message_level_t level, const char *fmt, ...)
+{
+	va_list alist;
+	int len;
+	char *cstr;
+	SEXP_t *lvl, *str, *msg;
+
+	_LOGCALL_;
+
+	va_start(alist, fmt);
+	len = vasprintf(&cstr, fmt, alist);
+	va_end(alist);
+	if (len < 0)
+		return NULL;
+
+	_D(cstr);
+	str = SEXP_string_new(cstr, len);
+	oscap_free(cstr);
+	lvl = SEXP_number_newu(level);
+	msg = SEXP_list_new(lvl, str, NULL);
+	SEXP_vfree(lvl, str, NULL);
+
+	return msg;
+}
+
 /*
  * entities
  */
@@ -879,7 +913,7 @@ SEXP_t *probe_ent_creat1(const char *name, SEXP_t * attrs, SEXP_t * val)
 	_LOGCALL_;
 
 	ent = SEXP_list_new(NULL);
-	ns  = encache_ref (OSCAP_GSYM(encache), name);
+	ns  = probe_ncache_ref (OSCAP_GSYM(encache), name);
 
 	if (attrs != NULL) {
 		SEXP_t *nl, *nj;
@@ -910,12 +944,22 @@ SEXP_t *probe_ent_attr_add(SEXP_t * ent, const char *name, SEXP_t * val)
 
 int probe_ent_getvals(const SEXP_t * ent, SEXP_t ** res)
 {
+	SEXP_t *val_lst;
+	int len;
+
 	_LOGCALL_;
 	if (probe_ent_attrexists(ent, "var_ref"))
-		(*res) = SEXP_list_nth(ent, 2);
+		val_lst = SEXP_list_nth(ent, 2);
 	else
-		(*res) = SEXP_list_rest(ent);
-	return (SEXP_list_length(*res));
+		val_lst = SEXP_list_rest(ent);
+	len = SEXP_list_length(val_lst);
+
+	if (res != NULL)
+		(*res) = val_lst;
+	else
+		SEXP_free(val_lst);
+
+	return (len);
 }
 
 SEXP_t *probe_ent_getval(const SEXP_t * ent)
@@ -997,115 +1041,78 @@ bool probe_ent_attrexists(const SEXP_t * ent, const char *name)
 	return probe_obj_attrexists(ent, name);
 }
 
-int probe_ent_setdatatype(SEXP_t * ent, oval_datatype_t type)
+int probe_ent_setdatatype(SEXP_t *ent, oval_datatype_t type)
 {
-	SEXP_t *val;
+	const char *strtype;
 
 	_A(ent != NULL);
 	_LOGCALL_;
 
-	val = probe_ent_getval(ent);
-
-	if (val == NULL)
-		return (-1);
-
 	switch (type) {
-	case OVAL_DATATYPE_BINARY:
-		return SEXP_datatype_set(val, "binary");
 	case OVAL_DATATYPE_BOOLEAN:
-		return SEXP_datatype_set(val, "bool");
-	case OVAL_DATATYPE_EVR_STRING:
-		return SEXP_datatype_set(val, "evr_str");
-	case OVAL_DATATYPE_FILESET_REVISION:
-		return SEXP_datatype_set(val, "fset_rev");
 	case OVAL_DATATYPE_FLOAT:
-		/* TODO */
-		return (-1);
-	case OVAL_DATATYPE_IOS_VERSION:
-		return SEXP_datatype_set(val, "ios_ver");
-	case OVAL_DATATYPE_VERSION:
-		return SEXP_datatype_set(val, "version");
 	case OVAL_DATATYPE_INTEGER:
-		/* TODO */
-		return (-1);
 	case OVAL_DATATYPE_STRING:
-		return (SEXP_stringp(val) ? 0 : -1);
+		/* these are stored directly in each SEXP */
+		return 0;
 	default:
-		return (-1);
+		/* set the SEXP to a user datatype */
+		strtype = oval_datatype_get_text(type);
+		return SEXP_datatype_set(ent, strtype);
 	}
+}
 
-	return (0);
+static oval_datatype_t _sexp_val_getdatatype(const SEXP_t *val)
+{
+	SEXP_type_t sdt;
+	SEXP_numtype_t sndt;
+
+	sdt = SEXP_typeof(val);
+
+	switch (sdt) {
+	case SEXP_TYPE_STRING:
+		return OVAL_DATATYPE_STRING;
+	case SEXP_TYPE_NUMBER:
+		sndt = SEXP_number_type(val);
+		switch (sndt) {
+		case SEXP_NUM_BOOL:
+			return OVAL_DATATYPE_BOOLEAN;
+		case SEXP_NUM_FLOAT:
+			return OVAL_DATATYPE_FLOAT;
+		default: /* everything else is considered an integer */
+			return OVAL_DATATYPE_INTEGER;
+		}
+	default:
+		dE("Unexpected SEXP datatype: %d, '%s'.\n", sdt, SEXP_strtype(val));
+		return OVAL_DATATYPE_UNKNOWN;
+	}
 }
 
 oval_datatype_t probe_ent_getdatatype(const SEXP_t * ent)
 {
-	SEXP_t *val;
 	const char *str;
-	oval_datatype_t ret;
 
 	_A(ent != NULL);
 	_LOGCALL_;
 
-	val = probe_ent_getval(ent);
-	ret = OVAL_DATATYPE_UNKNOWN;
-
-	if (val == NULL)
-		return (OVAL_DATATYPE_UNKNOWN);
-
-	str = SEXP_datatype(val);
-
+	str = SEXP_datatype(ent);
 	if (str != NULL) {
-		switch (str[0]) {
-		case 'b':
-			if (strcmp(str + 1, "ool") == 0)
-				ret = OVAL_DATATYPE_BOOLEAN;
-			else if (strcmp(str + 1, "inary") == 0)
-				ret = OVAL_DATATYPE_BINARY;
-			break;
-		case 'e':
-			if (strcmp(str + 1, "vr_str") == 0)
-				ret = OVAL_DATATYPE_EVR_STRING;
-			break;
-		case 'f':
-			if (strcmp(str + 1, "set_rev") == 0)
-				ret = OVAL_DATATYPE_FILESET_REVISION;
-			break;
-		case 'i':
-			if (strcmp(str + 1, "os_ver") == 0)
-				ret = OVAL_DATATYPE_IOS_VERSION;
-			break;
-		case 'v':
-			if (strcmp(str + 1, "ersion") == 0)
-				ret = OVAL_DATATYPE_VERSION;
-			break;
-		}
+		/* user datatype */
+		return oval_datatype_from_text(str);
 	} else {
-		switch (SEXP_typeof(val)) {
-		case SEXP_TYPE_NUMBER:
+		/* SEXP datatype */
+		SEXP_t *val;
+		oval_datatype_t dt;
 
-			switch (SEXP_number_type(val)) {
-			case SEXP_NUM_FLOAT:
-				ret = OVAL_DATATYPE_FLOAT;
-				break;
-			case SEXP_NUM_BOOLEAN:
-				ret = OVAL_DATATYPE_BOOLEAN;
-				break;
-			case SEXP_NUM_NONE:
-				abort();
-			default:
-				ret = OVAL_DATATYPE_INTEGER;
-			}
+		val = probe_ent_getval(ent);
+		if (val == NULL)
+			return OVAL_DATATYPE_UNKNOWN;
 
-			break;
-		case SEXP_TYPE_STRING:
-			ret = OVAL_DATATYPE_STRING;
-			break;
-		}
+		dt = _sexp_val_getdatatype(val);
+		SEXP_free(val);
+
+		return dt;
 	}
-
-	SEXP_free(val);
-
-	return (ret);
 }
 
 int probe_ent_setmask(SEXP_t * ent, bool mask)
@@ -1249,6 +1256,75 @@ void probe_free(SEXP_t * obj)
 	SEXP_free(obj);
 }
 
+void probe_filebehaviors_canonicalize(SEXP_t **behaviors)
+{
+	SEXP_t *bhs, *r0;
+
+	_A(behaviors != NULL);
+
+	bhs = *behaviors;
+
+	if (!bhs) {
+		r0 = SEXP_list_new(NULL);
+		bhs = *behaviors = probe_ent_creat1("behaviors", r0, NULL);
+		SEXP_free(r0);
+	}
+
+	if (!probe_ent_attrexists(bhs, "max_depth")) {
+		r0 = SEXP_string_newf("-1");
+		probe_ent_attr_add(bhs, "max_depth", r0);
+		SEXP_free(r0);
+	}
+	if (!probe_ent_attrexists(bhs, "recurse")) {
+		r0 = SEXP_string_newf("symlinks and directories");
+		probe_ent_attr_add(bhs, "recurse", r0);
+		SEXP_free(r0);
+	}
+	if (!probe_ent_attrexists(bhs, "recurse_direction")) {
+		r0 = SEXP_string_newf("none");
+		probe_ent_attr_add(bhs, "recurse_direction", r0);
+		SEXP_free(r0);
+	}
+	if (!probe_ent_attrexists(bhs, "recurse_file_system")) {
+		r0 = SEXP_string_newf("all");
+		probe_ent_attr_add(bhs, "recurse_file_system", r0);
+		SEXP_free(r0);
+	}
+}
+
+void probe_tfc54behaviors_canonicalize(SEXP_t **behaviors)
+{
+	SEXP_t *bhs, *r0;
+
+	_A(behaviors != NULL);
+
+	bhs = *behaviors;
+
+	if (!bhs) {
+		r0 = SEXP_list_new(NULL);
+		bhs = *behaviors = probe_ent_creat1("behaviors", r0, NULL);
+		SEXP_free(r0);
+	}
+
+	probe_filebehaviors_canonicalize(&bhs);
+
+	if (!probe_ent_attrexists(bhs, "ignore_case")) {
+		r0 = SEXP_string_newf("0");
+		probe_ent_attr_add(bhs, "ignore_case", r0);
+		SEXP_free(r0);
+	}
+	if (!probe_ent_attrexists(bhs, "multiline")) {
+		r0 = SEXP_string_newf("1");
+		probe_ent_attr_add(bhs, "multiline", r0);
+		SEXP_free(r0);
+	}
+	if (!probe_ent_attrexists(bhs, "singleline")) {
+		r0 = SEXP_string_newf("0");
+		probe_ent_attr_add(bhs, "singleline", r0);
+		SEXP_free(r0);
+	}
+}
+
 /**
  * Return a copy of the supplied item with optimized memory representation
  */
@@ -1258,7 +1334,7 @@ static SEXP_t *probe_item_optimize(const SEXP_t *item)
 	return SEXP_ref(item);
 }
 
-#define PROBE_RESULT_MEMCHECK_CTRESHOLD  65535  /* item count */
+#define PROBE_RESULT_MEMCHECK_CTRESHOLD  32768  /* item count */
 #define PROBE_RESULT_MEMCHECK_MINFREEMEM 128    /* MiB */
 #define PROBE_RESULT_MEMCHECK_MAXRATIO   0.80   /* max. memory usage ratio - used/total */
 
@@ -1270,7 +1346,7 @@ static int probe_cobj_memcheck(size_t item_cnt)
 	if (item_cnt > PROBE_RESULT_MEMCHECK_CTRESHOLD) {
 		struct memusage mu;
 		struct sysinfo  si;
-		float  c_ratio;
+		double c_ratio;
 
 		// todo: add an error message to the collected object?
 
@@ -1280,7 +1356,7 @@ static int probe_cobj_memcheck(size_t item_cnt)
 		if (sysinfo (&si) != 0)
 			return (-1);
 
-		c_ratio = (float)mu.mu_data/(float)((si.totalram * si.mem_unit) / 1024);
+		c_ratio = (double)mu.mu_data/(double)((si.totalram * si.mem_unit) / 1024);
 
 		if (c_ratio > PROBE_RESULT_MEMCHECK_MAXRATIO) {
 			dW("Memory usage ratio limit reached! limit=%f, current=%f\n",
@@ -1300,6 +1376,160 @@ static int probe_cobj_memcheck(size_t item_cnt)
 	return (0);
 }
 
-#undef PROBE_RESULT_MEMCHECK_CTRESHOLD
-#undef PROBE_RESULT_MEMCHECK_MINFREEMEM
-#undef PROBE_RESULT_MEMCHECK_MAXRATIO
+SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attributes[],
+                          /* const char *value_name, oval_datatype_t value_type, void *value, */ ...)
+{
+        va_list ap;
+	SEXP_t *item, *name_sexp, *value_sexp = NULL, *entity;
+        SEXP_t value_sexp_mem, entity_mem;
+	const char *value_name, *subtype_name;
+        char item_name[64];
+        oval_datatype_t value_type;
+
+        char   *value_str, **value_stra;
+        int64_t value_int;
+        double  value_flt;
+        bool    value_bool;
+        bool    free_value = true;
+        int     value_i, multiply;
+
+        subtype_name = oval_subtype2str(item_subtype);
+
+        if (subtype_name == NULL) {
+                dE("Invalid/Unknown subtype: %d\n", (int)item_subtype);
+                return (NULL);
+        }
+
+        if (strlen(subtype_name) + strlen("_item") < sizeof item_name) {
+                strcpy(item_name, subtype_name);
+                strcat(item_name, "_item");
+                item_name[sizeof item_name - 1] = '\0';
+        } else {
+                dE("item name too long: no buffer space available\n");
+                return (NULL);
+        }
+
+	va_start(ap, item_attributes);
+
+	item       = probe_item_new(item_name, NULL);
+	value_name = va_arg(ap, const char *);
+
+        while (value_name != NULL) {
+                value_i    = 0;
+                multiply   = 1;
+		value_type = va_arg(ap, oval_datatype_t);
+
+                switch (value_type) {
+                case OVAL_DATATYPE_STRING:
+                        value_str  = va_arg(ap, char *);
+
+                        if (value_str == NULL)
+                                goto skip;
+
+                        value_sexp = SEXP_string_new_r(&value_sexp_mem, value_str, strlen(value_str));
+                        break;
+                case OVAL_DATATYPE_STRING_M:
+                        value_type = OVAL_DATATYPE_STRING;
+                        value_stra = va_arg(ap, char **);
+                        multiply   = 0;
+
+                        if (value_stra == NULL)
+                                goto skip;
+
+                        while (value_stra[multiply] != NULL)
+                                ++multiply;
+
+                        value_sexp = oscap_alloc(sizeof(SEXP_t) * multiply);
+
+                        for (value_i = 0; value_i < multiply; ++value_i)
+                                SEXP_string_new_r(value_sexp + value_i, value_stra[value_i], strlen(value_stra[value_i]));
+
+                        value_i = 0;
+                        break;
+                case OVAL_DATATYPE_BOOLEAN:
+                        value_bool = (bool)va_arg(ap, int);
+                        value_sexp = SEXP_number_newb_r(&value_sexp_mem, value_bool);
+                        break;
+                case OVAL_DATATYPE_INTEGER:
+                        value_int  = va_arg(ap, int64_t);
+                        value_sexp = SEXP_number_newi_64_r(&value_sexp_mem, value_int);
+                        break;
+                case OVAL_DATATYPE_FLOAT:
+                        value_flt  = va_arg(ap, double);
+                        value_sexp = SEXP_number_newf_r(&value_sexp_mem, value_flt);
+                        break;
+                case OVAL_DATATYPE_SEXP:
+                        value_sexp = va_arg(ap, SEXP_t *);
+
+                        if (value_sexp == NULL)
+                                goto skip;
+                        else
+                                free_value = false;
+
+			value_type = _sexp_val_getdatatype(value_sexp);
+                        break;
+                case OVAL_DATATYPE_RECORD:
+			entity = va_arg(ap, SEXP_t *);
+			SEXP_list_add(item, entity);
+			goto skip;
+                case OVAL_DATATYPE_EVR_STRING:
+                case OVAL_DATATYPE_FILESET_REVISION:
+                case OVAL_DATATYPE_IOS_VERSION:
+                case OVAL_DATATYPE_VERSION:
+                        value_str  = va_arg(ap, char *);
+                        value_sexp = SEXP_string_new_r(&value_sexp_mem, value_str, strlen(value_str));
+
+                        break;
+                        /* TODO */
+                case OVAL_DATATYPE_BINARY:
+                case OVAL_DATATYPE_UNKNOWN:
+			dE("Unknown or unsupported OVAL datatype: %d, '%s', name: '%s'.\n",
+			   value_type, oval_datatype_get_text(value_type), value_name);
+                        SEXP_free(item);
+
+                        return (NULL);
+                }
+
+                name_sexp = probe_ncache_ref(OSCAP_GSYM(encache), value_name);
+
+                while(value_i < multiply) {
+                        entity = SEXP_list_new_r(&entity_mem, name_sexp, value_sexp + value_i, NULL);
+
+                        if (probe_ent_setdatatype(entity, value_type) != 0) {
+                                SEXP_free_r(&entity_mem);
+                                SEXP_free(name_sexp);
+                                SEXP_free(item);
+
+                                if (free_value) {
+                                        while(value_i < multiply)
+                                                SEXP_free_r(value_sexp + value_i++);
+                                        if (multiply > 1)
+                                                oscap_free(value_sexp);
+                                }
+
+                                return (NULL);
+                        }
+
+                        assume_d(item       != NULL, NULL);
+                        assume_d(entity     != NULL, NULL);
+                        assume_d(name_sexp  != NULL, NULL);
+
+                        SEXP_list_add(item, entity);
+                        SEXP_free_r(&entity_mem);
+
+                        if (free_value)
+                                SEXP_free_r(value_sexp + value_i);
+                        ++value_i;
+                }
+
+                SEXP_free(name_sexp);
+
+                if (multiply > 1)
+                        oscap_free(value_sexp);
+        skip:
+                value_name = va_arg(ap, const char *);
+		free_value = true;
+        }
+
+        return (item);
+}
