@@ -66,6 +66,7 @@
 #include <oval_definitions.h>
 #include <oval_system_characteristics.h>
 #include <oval_results.h>
+#include <oval_types.h>
 
 /*
  * items
@@ -290,8 +291,21 @@ oval_syschar_collection_flag_t probe_cobj_compute_flag(SEXP_t *cobj);
  * messages
  */
 
+/**
+ * Create a new message that can be added to a collected object.
+ * @param level the level associated with the new message
+ * @param message the text of the new message
+ */
 SEXP_t *probe_msg_creat(oval_message_level_t level, char *message);
 
+/**
+ * Create a new message that can be added to a collected object.
+ * @param level the level associated with the new message
+ * @param fmt printf-like format string that produces the text of the new message
+ * @param ... arguments for the format
+ */
+SEXP_t *probe_msg_creatf(oval_message_level_t level, const char *fmt, ...) __attribute__((format(printf, 2, 3), nonnull(2)));
+
 /*
  * entities
  */
@@ -414,6 +428,22 @@ size_t probe_ent_getname_r(const SEXP_t * ent, char *buffer, size_t buflen);
  */
 void probe_free(SEXP_t * obj);
 
+/**
+ * Set all of the missing attributes of the 'behaviors' entity
+ * to default values. If the referenced pointer contains NULL,
+ * a new entity is created and stored in the referenced pointer.
+ * @param behaviors address of the pointer to the 'behaviors' entity, must not be NULL
+ */
+void probe_filebehaviors_canonicalize(SEXP_t **behaviors);
+
+/**
+ * Set all of the missing attributes of the 'behaviors' entity
+ * to default values. If the referenced pointer contains NULL,
+ * a new entity is created and stored in the referenced pointer.
+ * @param behaviors address of the pointer to the 'behaviors' entity, must not be NULL
+ */
+void probe_tfc54behaviors_canonicalize(SEXP_t **behaviors);
+
 #define PROBE_EINVAL     1	/**< Invalid type/value/format */
 #define PROBE_ENOELM     2	/**< Missing element OBSOLETE: use ENOENT */
 #define PROBE_ENOVAL     3	/**< Missing value */
@@ -424,9 +454,12 @@ void probe_free(SEXP_t * obj);
 #define PROBE_ERANGE     8	/**< Out of range */
 #define PROBE_EDOM       9	/**< Out of domain */
 #define PROBE_EFAULT    10	/**< Memory fault/NULL value */
-#define PROBE_EACCES    11	/**< Operation not perimitted */
+#define PROBE_EACCESS   11	/**< Operation not permitted */
 #define PROBE_ESETEVAL  12	/**< Set evaluation failed */
 #define PROBE_ENOENT    13      /**< Missing entity */
+#define PROBE_ENOOBJ    14      /**< Missing object */
+#define PROBE_ECONNABORTED 15   /**< Evaluation aborted */
+#define PROBE_ESYSTEM  253      /**< System call failure */
 #define PROBE_EFATAL   254	/**< Unrecoverable error */
 #define PROBE_EUNKNOWN 255	/**< Unknown/Unexpected error */
 
@@ -442,16 +475,22 @@ void probe_free(SEXP_t * obj);
 
 void *probe_init(void) __attribute__ ((unused));
 void probe_fini(void *) __attribute__ ((unused));
-int probe_main(SEXP_t *, SEXP_t *, void *) __attribute__ ((nonnull(1, 2)));
+int probe_main(SEXP_t *, SEXP_t *, void *, SEXP_t *) __attribute__ ((nonnull(1, 2)));
 
 #define PROBE_VARREF_HANDLING 0
 #define PROBE_RESULT_CACHING  1
 
 int probe_setoption(int option, ...);
 
-SEXP_t *probe_prepare_filters(SEXP_t *obj);
 bool probe_item_filtered(SEXP_t *item, SEXP_t *filters);
 
 int probe_result_additem(SEXP_t *result, SEXP_t *item);
 
+typedef struct {
+        oval_datatype_t type;
+        void           *value;
+} probe_elmatr_t;
+
+SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attributes[], ...);
+
 #endif				/* PROBE_API_H */
