@@ -29,6 +29,11 @@
  * Authors:
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <seap.h>
 #include <probe-api.h>
 #include <stdlib.h>
@@ -37,6 +42,12 @@
 #include <pthread.h>
 #include <errno.h>
 #include <limits.h>
+
+#if defined(HAVE_LIBACL)
+# include <sys/types.h>
+# include <acl/libacl.h>
+#endif
+
 #include "oval_fts.h"
 #include "SEAP/generic/rbt/rbt.h"
 
@@ -74,8 +85,9 @@
 static SEXP_t *gr_true   = NULL, *gr_false  = NULL, *gr_t_reg  = NULL;
 static SEXP_t *gr_t_dir  = NULL, *gr_t_lnk  = NULL, *gr_t_blk  = NULL;
 static SEXP_t *gr_t_fifo = NULL, *gr_t_sock = NULL, *gr_t_char = NULL;
+static SEXP_t  gr_lastpath;
 
-static SEXP_t *strfiletype (mode_t mode)
+static SEXP_t *se_filetype (mode_t mode)
 {
         switch (mode & S_IFMT) {
         case S_IFREG:  return (gr_t_reg);
@@ -93,7 +105,6 @@ static SEXP_t *strfiletype (mode_t mode)
 }
 
 struct cbargs {
-        SEXP_t *filename_ent;
         SEXP_t *cobj;
 	SEXP_t *filters;
 	int     error;
@@ -109,10 +120,10 @@ static SEXP_t *ID_cache_get(int32_t id)
 	if (rbt_i32_get(g_ID_cache, id, (void *)&s_id) == 0)
 		return SEXP_ref(s_id); /* cache hit (first attempt) */
 
-	s_id = SEXP_string_newf("%u", (uint32_t)(id));
+	s_id = SEXP_number_newu_32(id);
 
 	if (g_ID_cache_max == 0 || rbt_i32_size(g_ID_cache) < g_ID_cache_max) {
-		if (rbt_i32_add(g_ID_cache, id, (void *)s_id) == 0)
+		if (rbt_i32_add(g_ID_cache, id, (void *)s_id, NULL) == 0)
 			return SEXP_ref(s_id); /* insert succeeded */
 
 		if (rbt_i32_get(g_ID_cache, id, (void *)&s_id2) == 0) {
@@ -143,6 +154,71 @@ static void ID_cache_free(void)
 	g_ID_cache_max = 0;
 }
 
+static SEXP_t *get_atime(struct stat *st, SEXP_t *sexp)
+{
+	return SEXP_number_newu_64_r (sexp,
+#if defined(OS_FREEBSD)
+# if (__STDC_VERSION__ >= 199901L)
+		(uint64_t) st->st_atimespec.tv_sec
+# else
+		(uint64_t) st->st_atimespec.tv_sec
+# endif
+#elif defined(OS_LINUX) || defined(OS_SOLARIS)
+		(uint64_t) st->st_atim.tv_sec
+#endif
+		);
+}
+
+static SEXP_t *get_ctime(struct stat *st, SEXP_t *sexp)
+{
+	return SEXP_number_newu_64_r (sexp,
+#if defined(OS_FREEBSD)
+# if (__STDC_VERSION__ >= 199901L)
+		(uint64_t) st->st_ctimespec.tv_sec
+# else
+		(uint64_t) st->st_ctimespec.tv_sec
+# endif
+#elif defined(OS_LINUX) || defined(OS_SOLARIS)
+		(uint64_t) st->st_ctim.tv_sec
+#endif
+		);
+}
+
+static SEXP_t *get_mtime(struct stat *st, SEXP_t *sexp)
+{
+	return SEXP_number_newu_64_r (sexp,
+#if defined(OS_FREEBSD)
+# if (__STDC_VERSION__ >= 199901L)
+		(uint64_t) st->st_mtimespec.tv_sec
+# else
+		(uint64_t) st->st_mtimespec.tv_sec
+# endif
+#elif defined(OS_LINUX) || defined(OS_SOLARIS)
+		(uint64_t) st->st_mtim.tv_sec
+#endif
+		);
+}
+
+static SEXP_t *get_size(struct stat *st, SEXP_t *sexp)
+{
+#if defined(_FILE_OFFSET_BITS)
+# if   _FILE_OFFSET_BITS == 64
+	return SEXP_number_newu_64_r(sexp, st->st_size);
+# elif _FILE_OFFSET_BITS == 32
+	return SEXP_number_newu_32_r(sexp, st->st_size);
+# else
+#  error "Invalid _FILE_OFFSET_BITS value"
+	return NULL;
+# endif
+#elif defined(LARGE_FILE_SOURCE)
+	return SEXP_number_newu_64_r(sexp, st->st_size);
+#else
+	return SEXP_number_newu_32_r(sexp, st->st_size);
+#endif
+}
+
+#define MODEP(statp, bit) ((statp)->st_mode & (bit) ? gr_true : gr_false)
+
 static int file_cb (const char *p, const char *f, void *ptr)
 {
         char path_buffer[PATH_MAX];
@@ -154,129 +230,65 @@ static int file_cb (const char *p, const char *f, void *ptr)
         cobj = args->cobj;
 	filters = args->filters;
 
-        if (f == NULL)
-		return (0);
-
-        snprintf (path_buffer, sizeof path_buffer, "%s/%s", p, f);
-        st_path = path_buffer;
+	if (f == NULL) {
+		st_path = p;
+	} else {
+		snprintf (path_buffer, sizeof path_buffer, "%s/%s", p, f);
+		st_path = path_buffer;
+	}
 
         if (lstat (st_path, &st) == -1) {
                 _D("FAIL: errno=%u, %s.\n", errno, strerror (errno));
                 return (-1);
         } else {
-                SEXP_t *r0, *r1, *r3, *r4;
-                SEXP_t *r5, *r6, *r7, *r8;
+                SEXP_t *se_usr_id, *se_grp_id;
+                SEXP_t  se_atime_mem, se_ctime_mem, se_mtime_mem, se_size_mem;
 
-		r3 = ID_cache_get(st.st_uid);
-		r4 = st.st_gid != st.st_uid ? ID_cache_get(st.st_gid) : SEXP_ref(r3);
+		se_usr_id = ID_cache_get(st.st_uid);
+		se_grp_id = st.st_gid != st.st_uid ? ID_cache_get(st.st_gid) : SEXP_ref(se_usr_id);
 
-                item = probe_item_creat ("file_item", NULL,
-                                        /* entities */
-                                        "path", NULL,
-                                        r0 = SEXP_string_newf ("%s", p),
+		if (!SEXP_emptyp(&gr_lastpath)) {
+			if (SEXP_strcmp(&gr_lastpath, p) != 0) {
+				SEXP_free_r(&gr_lastpath);
+				SEXP_string_new_r(&gr_lastpath, p, strlen(p));
+			}
+		} else
+			SEXP_string_new_r(&gr_lastpath, p, strlen(p));
 
-                                        "filename", NULL,
-                                        r1 = (f != NULL ? SEXP_string_newf ("%s", f) : NULL),
-
-                                        "type", NULL,
-                                        strfiletype (st.st_mode),
-
-					"group_id", NULL, r4,
-					"user_id",  NULL, r3,
-
-                                        "a_time", NULL,
-                                        r5 = SEXP_string_newf (
-#if defined(OS_FREEBSD)
-# if (__STDC_VERSION__ >= 199901L)
-						               "%jd", (intmax_t) st.st_atimespec.tv_sec
-# else
-							       "%lld", (unsigned long long) st.st_atimespec.tv_sec
-# endif
-#elif defined(OS_LINUX) || defined(OS_SOLARIS)
-                                                               "%u", (unsigned int)st.st_atim.tv_sec
+                item = probe_item_create(OVAL_UNIX_FILE, NULL,
+                                         "filepath", OVAL_DATATYPE_STRING, f == NULL ? NULL : st_path,
+                                         "path",     OVAL_DATATYPE_SEXP,  &gr_lastpath,
+                                         "filename", OVAL_DATATYPE_STRING, f == NULL ? "" : f,
+                                         "type",     OVAL_DATATYPE_SEXP, se_filetype(st.st_mode),
+                                         "group_id", OVAL_DATATYPE_SEXP, se_grp_id,
+                                         "user_id",  OVAL_DATATYPE_SEXP, se_usr_id,
+                                         "a_time",   OVAL_DATATYPE_SEXP, get_atime(&st, &se_atime_mem),
+                                         "c_time",   OVAL_DATATYPE_SEXP, get_ctime(&st, &se_ctime_mem),
+                                         "m_time",   OVAL_DATATYPE_SEXP, get_mtime(&st, &se_mtime_mem),
+                                         "size",     OVAL_DATATYPE_SEXP, get_size(&st, &se_size_mem),
+                                         "suid",     OVAL_DATATYPE_SEXP, MODEP(&st, S_ISUID),
+                                         "sgid",     OVAL_DATATYPE_SEXP, MODEP(&st, S_ISGID),
+                                         "sticky",   OVAL_DATATYPE_SEXP, MODEP(&st, S_ISVTX),
+                                         "uread",    OVAL_DATATYPE_SEXP, MODEP(&st, S_IRUSR),
+                                         "uwrite",   OVAL_DATATYPE_SEXP, MODEP(&st, S_IWUSR),
+                                         "uexec",    OVAL_DATATYPE_SEXP, MODEP(&st, S_IXUSR),
+                                         "gread",    OVAL_DATATYPE_SEXP, MODEP(&st, S_IRGRP),
+                                         "gwrite",   OVAL_DATATYPE_SEXP, MODEP(&st, S_IWGRP),
+                                         "gexec",    OVAL_DATATYPE_SEXP, MODEP(&st, S_IXGRP),
+                                         "oread",    OVAL_DATATYPE_SEXP, MODEP(&st, S_IROTH),
+                                         "owrite",   OVAL_DATATYPE_SEXP, MODEP(&st, S_IWOTH),
+                                         "oexec",    OVAL_DATATYPE_SEXP, MODEP(&st, S_IXOTH),
+#if defined(HAVE_LIBACL)
+                                         "has_extended_acl", OVAL_DATATYPE_SEXP, acl_extended_file(st_path) ? gr_true : gr_false,
 #endif
-                                                ),
+                                         NULL);
 
-                                        "c_time", NULL,
-                                        r6 = SEXP_string_newf (
-#if defined(OS_FREEBSD)
-# if (__STDC_VERSION__ >= 199901L)
-						               "%jd", (intmax_t) st.st_ctimespec.tv_sec
-# else
-							       "%lld", (unsigned long long) st.st_ctimespec.tv_sec
-# endif
-#elif defined(OS_LINUX) || defined(OS_SOLARIS)
-                                                               "%u", (unsigned int)st.st_ctim.tv_sec
-#endif
-                                                ),
-
-                                        "m_time", NULL,
-                                        r7 = SEXP_string_newf (
-#if defined(OS_FREEBSD)
-# if (__STDC_VERSION__ >= 199901L)
-						               "%jd", (intmax_t) st.st_mtimespec.tv_sec
-# else
-							       "%lld", (unsigned long long) st.st_mtimespec.tv_sec
-# endif
-#elif defined(OS_LINUX) || defined(OS_SOLARIS)
-                                                               "%u", (unsigned int)st.st_mtim.tv_sec
-#endif
-                                                ),
-
-                                        "size", NULL,
-#if defined(_FILE_OFFSET_BITS)
-# if   _FILE_OFFSET_BITS == 64
-                                        r8 = SEXP_number_newu_64 (st.st_size),
-# elif _FILE_OFFSET_BITS == 32
-                                        r8 = SEXP_number_newu_32 (st.st_size),
-# else
-#  error "Invalid _FILE_OFFSET_BITS value"
-                                        r8 = NULL,
-# endif
-#elif defined(LARGE_FILE_SOURCE)
-                                        r8 = SEXP_number_newu_64 (st.st_size),
-#else
-                                        r8 = SEXP_number_newu_32 (st.st_size),
-#endif
-                                        "suid", NULL,
-                                        (st.st_mode & S_ISUID ? gr_true : gr_false),
-
-                                        "sgid", NULL,
-                                        (st.st_mode & S_ISGID ? gr_true : gr_false),
-
-                                        "sticky", NULL,
-                                        (st.st_mode & S_ISVTX ? gr_true : gr_false),
-
-                                        "uread", NULL,
-                                        (st.st_mode & S_IRUSR ? gr_true : gr_false),
-
-                                        "uwrite", NULL,
-                                        (st.st_mode & S_IWUSR ? gr_true : gr_false),
-
-                                        "uexec", NULL,
-                                        (st.st_mode & S_IXUSR ? gr_true : gr_false),
-
-                                        "gread", NULL,
-                                        (st.st_mode & S_IRGRP ? gr_true : gr_false),
-
-                                        "gwrite", NULL,
-                                        (st.st_mode & S_IWGRP ? gr_true : gr_false),
-
-                                        "gexec", NULL,
-                                        (st.st_mode & S_IXGRP ? gr_true : gr_false),
-
-                                        "oread", NULL,
-                                        (st.st_mode & S_IROTH ? gr_true : gr_false),
-
-                                        "owrite", NULL,
-                                        (st.st_mode & S_IWOTH ? gr_true : gr_false),
-
-                                        "oexec", NULL,
-                                        (st.st_mode & S_IXOTH ? gr_true : gr_false),
-
-                                        NULL);
-
-		SEXP_vfree(r0, r1, r3, r4, r5, r6, r7, r8, NULL);
+                SEXP_free(se_grp_id);
+                SEXP_free(se_usr_id);
+                SEXP_free_r(&se_atime_mem);
+                SEXP_free_r(&se_ctime_mem);
+                SEXP_free_r(&se_mtime_mem);
+                SEXP_free_r(&se_size_mem);
 
 		if (!probe_item_filtered(item, filters)) {
 			if (probe_cobj_add_item(cobj, item) != 0) {
@@ -326,6 +338,8 @@ void *probe_init (void)
         gr_t_sock = SEXP_string_new (STRLEN_PAIR(STR_SOCKET));
         gr_t_char = SEXP_string_new (STRLEN_PAIR(STR_CHARSPEC));
 
+	SEXP_init(&gr_lastpath);
+
 	/*
 	 * Initialize ID cache
 	 */
@@ -359,6 +373,9 @@ void probe_fini (void *arg)
                     gr_t_fifo, gr_t_sock, gr_t_char,
                     NULL);
 
+	if (!SEXP_emptyp(&gr_lastpath))
+		SEXP_free_r(&gr_lastpath);
+
 	/*
 	 * Free ID cache
 	 */
@@ -372,11 +389,10 @@ void probe_fini (void *arg)
         return;
 }
 
-int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex)
+int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex, SEXP_t *filters)
 {
-        SEXP_t *path, *filename, *behaviors;
-        SEXP_t *r0, *r1, *r2, *r3, *r4;
-        int     filecnt, err;
+        SEXP_t *path, *filename, *behaviors, *filepath;
+	int err;
         struct cbargs cbargs;
 	OVAL_FTS    *ofts;
 	OVAL_FTSENT *ofts_ent;
@@ -394,62 +410,25 @@ int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex)
         path      = probe_obj_getent (probe_in, "path",      1);
         filename  = probe_obj_getent (probe_in, "filename",  1);
         behaviors = probe_obj_getent (probe_in, "behaviors", 1);
+        filepath =  probe_obj_getent (probe_in, "filepath", 1);
 
-        if (path == NULL || filename == NULL) {
+	/* we want either path+filename or filepath */
+        if ( (path == NULL || filename == NULL) && filepath==NULL ) {
                 SEXP_free (behaviors);
                 SEXP_free (path);
                 SEXP_free (filename);
+                SEXP_free (filepath);
 
                 return PROBE_ENOELM;
         }
 
-        if (behaviors == NULL) {
-                SEXP_t *bh_list;
+	/* behaviours are not important if filepath is used */
+	if(filepath != NULL && behaviors != NULL) {
+		SEXP_free (behaviors);
+		behaviors = NULL;
+	}
 
-                bh_list = probe_ent_creat ("behaviors",
-                                           r0 = probe_attr_creat ("max_depth",
-                                                                  r1 = SEXP_string_newf ("-1"),
-                                                                  "recurse",
-                                                                  r2 = SEXP_string_newf ("symlinks and directories"),
-                                                                  "recurse_direction",
-                                                                  r3 = SEXP_string_newf ("none"),
-                                                                  "recurse_file_system",
-                                                                  r4 = SEXP_string_newf ("all"),
-                                                                  NULL),
-                                           NULL /* val */,
-                                           NULL /* end */);
-
-                behaviors = SEXP_list_first (bh_list);
-
-                SEXP_vfree (bh_list,
-                            r0, r1, r2, r3, r4, NULL);
-        } else {
-                if (!probe_ent_attrexists (behaviors, "max_depth")) {
-                        probe_ent_attr_add (behaviors,
-                                            "max_depth", r0 = SEXP_string_newf ("-1"));
-                        SEXP_free (r0);
-                }
-
-                if (!probe_ent_attrexists (behaviors, "recurse")) {
-                        probe_ent_attr_add (behaviors,
-                                            "recurse", r0 = SEXP_string_newf ("symlinks and directories"));
-                        SEXP_free (r0);
-                }
-
-                if (!probe_ent_attrexists (behaviors, "recurse_direction")) {
-                        probe_ent_attr_add (behaviors,
-                                            "recurse_direction", r0 = SEXP_string_newf ("none"));
-                        SEXP_free (r0);
-                }
-
-                if (!probe_ent_attrexists (behaviors, "recurse_file_system")) {
-                        probe_ent_attr_add (behaviors,
-                                            "recurse_file_system", r0 = SEXP_string_newf ("all"));
-                        SEXP_free (r0);
-                }
-        }
-
-        _A(behaviors != NULL);
+	probe_filebehaviors_canonicalize(&behaviors);
 
         switch (pthread_mutex_lock (&__file_probe_mutex)) {
         case 0:
@@ -457,41 +436,32 @@ int probe_main (SEXP_t *probe_in, SEXP_t *probe_out, void *mutex)
         default:
                 _D("Can't lock mutex(%p): %u, %s.\n", &__file_probe_mutex, errno, strerror (errno));
 
+		SEXP_free(path);
+		SEXP_free(filename);
+		SEXP_free(filepath);
+		SEXP_free(behaviors);
+
                 return PROBE_EFATAL;
         }
 
-        cbargs.cobj = probe_out;
-        cbargs.filename_ent = filename;
-	cbargs.filters = probe_prepare_filters(probe_in);
-	cbargs.error = 0;
+        cbargs.cobj    = probe_out;
+	cbargs.filters = filters;
+	cbargs.error   = 0;
 
-	filecnt = 0;
-
-	if ((ofts = oval_fts_open(path, filename, NULL, behaviors)) != NULL) {
-		for (; (ofts_ent = oval_fts_read(ofts)) != NULL; ++filecnt) {
+	if ((ofts = oval_fts_open(path, filename, filepath, behaviors)) != NULL) {
+		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
 			file_cb(ofts_ent->path, ofts_ent->file, &cbargs);
 			oval_ftsent_free(ofts_ent);
 		}
-
 		oval_fts_close(ofts);
 	}
 
 	err = 0;
 
-        if (filecnt < 0) {
-		char s[50];
-		SEXP_t *msg;
-
-		snprintf(s, sizeof (s), "find_files returned error: %d", filecnt);
-		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, s);
-		probe_cobj_add_msg(probe_out, msg);
-		SEXP_free(msg);
-		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
-
-		err = cbargs.error;
-        }
-
-	SEXP_vfree(behaviors, path, filename, cbargs.filters, NULL);
+	SEXP_free(path);
+	SEXP_free(filename);
+	SEXP_free(filepath);
+	SEXP_free(behaviors);
 
         switch (pthread_mutex_unlock (&__file_probe_mutex)) {
         case 0:
