@@ -33,26 +33,33 @@
  */
 
 /*
- * interface probe:
+ * interface probe
  *
- *   interface_object:
- *     string name
+ *   reference:
+ *     http://oval.mitre.org/language/version5.6/ovaldefinition/documentation/unix-definitions-schema.html#interface_object
+ *     http://oval.mitre.org/language/version5.6/ovalsc/documentation/unix-system-characteristics-schema.html#interface_item
  *
- *   interface_item:
- *     string name
- *     interfacetype type
- *     string hardware_addr
- *     string inet_addr
- *     string broadcast_addr
- *     string netmask
- *     [0..*] string flag
+ *   object:
+ *     name oval-def:EntityObjectStringType 1 1
+ *
+ *   item:
+ *     name           oval-sc:EntityItemStringType    0 1
+ *     type           unix-sc:EntityItemInterfaceType 0 1
+ *     hardware_addr  oval-sc:EntityItemStringType    0 1
+ *     inet_addr      oval-sc:EntityItemStringType    0 1
+ *     broadcast_addr oval-sc:EntityItemStringType    0 1
+ *     netmask        oval-sc:EntityItemStringType    0 1
+ *     flag           oval-sc:EntityItemStringType    0 unbounded
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <stdio.h>
 
 #include <probe-api.h>
-#include <probe-entcmp.h>
+#include <probe/entcmp.h>
 
 #if defined(__linux__)
 #include <unistd.h>
@@ -62,15 +69,19 @@
 #include <sys/ioctl.h>
 #include <string.h>
 #include <net/if.h>
+#include <net/if_arp.h>
 
 static int fd=-1;
 
-static char *get_mac(const struct ifaddrs *ifa)
+static void get_l2_info(const struct ifaddrs *ifa, char **mp, char **tp)
 {
 	struct ifreq ifr;
 	unsigned char mac[6];
 	static char mac_buf[20];
 
+	*mp = mac_buf;
+	*tp = "";
+
 	memset(&ifr, 0, sizeof(struct ifreq));
 	strcpy(ifr.ifr_name, ifa->ifa_name);
 	if (ioctl(fd, SIOCGIFHWADDR, &ifr) >= 0) {
@@ -78,19 +89,39 @@ static char *get_mac(const struct ifaddrs *ifa)
 		snprintf(mac_buf, sizeof(mac_buf),
 			"%02X:%02X:%02X:%02X:%02X:%02X",
 			mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
+		switch (ifr.ifr_hwaddr.sa_family) {
+		case ARPHRD_ETHER:
+			*tp = "ARPHRD_ETHER";
+			break;
+		case ARPHRD_FDDI:
+			*tp = "ARPHRD_FDDI";
+			break;
+		case ARPHRD_LOOPBACK:
+			*tp = "ARPHRD_LOOPBACK";
+			break;
+		case ARPHRD_PPP:
+			*tp = "ARPHRD_PPP";
+			break;
+		case ARPHRD_PRONET:
+			*tp = "ARPHRD_PRONET";
+			break;
+		case ARPHRD_SLIP:
+			*tp = "ARPHRD_SLIP";
+			break;
+		case ARPHRD_VOID:
+			*tp = "ARPHRD_VOID";
+			break;
+		}
 	} else
 		mac_buf[0] = 0;
-
-	return mac_buf;
 }
 
 static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
 {
 	struct ifaddrs *ifaddr, *ifa;
 	int family, rc=1;
-	char host[NI_MAXHOST], broad[NI_MAXHOST], mask[NI_MAXHOST], *mac;
+	char host[NI_MAXHOST], broad[NI_MAXHOST], mask[NI_MAXHOST], *mac, *type;
 	SEXP_t *item;
-	SEXP_t *r0, *r1, *r2, *r3, *r4;
 
 	if (getifaddrs(&ifaddr) == -1) {
 		SEXP_t *msg;
@@ -134,7 +165,7 @@ static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
 		}
 		SEXP_free(sname);
 
-		mac = get_mac(ifa);
+		get_l2_info(ifa, &mac, &type);
 		rc = getnameinfo(ifa->ifa_addr, (family == AF_INET) ?
 			sizeof(struct sockaddr_in) :
 			sizeof(struct sockaddr_in6), host, NI_MAXHOST,
@@ -184,21 +215,17 @@ static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
 		} else
 			*broad = '\0';
 
-		item = probe_item_creat("interface_item", NULL,
-				"name", NULL,
-                                r0 = SEXP_string_new (ifa->ifa_name, strlen (ifa->ifa_name)),
-				"hardware_addr", NULL,
-                                r1 = SEXP_string_new (mac, strlen (mac)),
-				"inet_addr", NULL,
-                                r2 = SEXP_string_new (host, strlen (host)),
-				"broadcast_addr", NULL,
-                                r3 = SEXP_string_new (broad, strlen (broad)),
-				"netmask", NULL,
-                                r4 = SEXP_string_new (mask, strlen (mask)),
-				NULL);
+                item = probe_item_create(OVAL_UNIX_INTERFACE, NULL,
+                                         "name",           OVAL_DATATYPE_STRING, ifa->ifa_name,
+                                         "type",           OVAL_DATATYPE_STRING, type,
+                                         "hardware_addr",  OVAL_DATATYPE_STRING, mac,
+                                         "inet_addr",      OVAL_DATATYPE_STRING, host,
+                                         "broadcast_addr", OVAL_DATATYPE_STRING, broad,
+                                         "netmask",        OVAL_DATATYPE_STRING, mask,
+                                         NULL);
 
 		probe_cobj_add_item(probe_out, item);
-		SEXP_vfree(r0, r1, r2, r3, r4, item, NULL);
+                SEXP_free(item);
 	}
 
 	rc = 0;
@@ -212,34 +239,29 @@ leave1:
 static int get_ifs(SEXP_t *name_ent, SEXP_t *probe_out)
 {
 	/* todo */
-
 	SEXP_t *item;
-	SEXP_t *r0, *r1, *r2, *r3, *r4;
 
-	item = probe_item_creat("interface_item", NULL,
-				"name", NULL,
-				r0 = SEXP_string_newf("dummy0"),
-				"hardware_addr", NULL,
-				r1 = SEXP_string_newf("aa:bb:cc:dd:ee:ff"),
-				"inet_addr", NULL,
-				r2 = SEXP_string_newf("1.2.3.4"),
-				"broadcast_addr", NULL,
-				r3 = SEXP_string_newf("1.2.3.255"),
-				"netmask", NULL,
-				r4 = SEXP_string_newf("255.255.255.0"),
-				NULL);
+        item = probe_item_create(OVAL_UNIX_INTERFACE, NULL,
+                                 "name",           OVAL_DATATYPE_STRING, "dummy0"
+                                 "hardware_addr",  OVAL_DATATYPE_STRING, "aa:bb:cc:dd:ee:ff",
+                                 "inet_addr",      OVAL_DATATYPE_STRING, "1.2.3.4",
+                                 "broadcast_addr", OVAL_DATATYPE_STRING, "1.2.3.254",
+                                 "netmask",        OVAL_DATATYPE_STRING, "255.255.255.0",
+                                 NULL);
 
 	probe_cobj_add_item(probe_out, item);
-	SEXP_vfree(r0, r1, r2, r3, r4, item, NULL);
+        SEXP_free(item);
 
 	return (0);
 }
 #endif
 
-int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg)
+int probe_main(SEXP_t *probe_in, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
 	SEXP_t *name_ent;
 
+        (void)filters;
+
 	if (probe_in == NULL || probe_out == NULL) {
 		return (PROBE_EINVAL);
 	}
