@@ -45,11 +45,10 @@
  * user_id
  */
 
-#include "config.h"
-#include "seap.h"
-#include "probe-api.h"
-#include "probe-entcmp.h"
-#include "alloc.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -61,6 +60,11 @@
 #include <arpa/inet.h>
 #include <regex.h>
 
+#include "seap.h"
+#include "probe-api.h"
+#include "probe/entcmp.h"
+#include "alloc.h"
+
 /* This structure contains the information OVAL is asking or requesting */
 struct server_info {
 	SEXP_t *protocol_ent;
@@ -184,7 +188,7 @@ static int collect_process_info(llist *l)
 		int pid, ppid;
 		char buf[100];
 		char *tmp, cmd[16], state, *text = NULL;
-		int fd, len, euid;
+		int fd, len, euid = 0;
 
 		// Skip non-process dir entries
 		if(*ent->d_name<'0' || *ent->d_name>'9')
@@ -220,9 +224,7 @@ static int collect_process_info(llist *l)
 		// Get the effective uid
 		snprintf(buf, 32, "/proc/%d/status", pid);
 		sf = fopen(buf, "rt");
-		if (sf == NULL)
-			euid = 0;
-		else {
+		if (sf) {
 			int line = 0;
 			__fsetlocking(sf, FSETLOCKING_BYCALLER);
 			while (fgets(buf, sizeof(buf), sf)) {
@@ -257,12 +259,14 @@ static int collect_process_info(llist *l)
 			char line[256], ln[256], *s, *e;
 			unsigned long inode;
 			lnode node;
+			int lnlen;
 
 			if (ent->d_name[0] == '.')
 				continue;
 			snprintf(ln, 256, "%s/%s", buf, ent->d_name);
-			if (readlink(ln, line, 255) < 0)
+			if ((lnlen = readlink(ln, line, sizeof(line)-1)) < 0)
 				continue;
+			line[lnlen] = 0;
 
 			// Only look at the socket entries
 			if (memcmp(line, "socket:", 7) == 0) {
@@ -329,29 +333,35 @@ static int eval_data(const char *type, const char *local_address,
 
 static void report_finding(struct result_info *res, llist *l, SEXP_t *probe_out)
 {
-	SEXP_t *r0, *r1, *r2, *r3, *r4, *r5, *r6, *r7, *r8, *r9, *item_sexp;
+        SEXP_t *item;
+        SEXP_t se_lport_mem, se_rport_mem, se_lfull_mem, se_ffull_mem, se_uid_mem;
 	lnode *n = list_get_cur(l);
 
-	item_sexp = probe_item_creat("inetlisteningservers_item", NULL,
-		/* entities */
-		"protocol", NULL, r0 = SEXP_string_newf("%s", res->proto),
-		"local_address", NULL, r1 = SEXP_string_newf("%s",
-						res->laddr),
-		"local_port", NULL, r2 = SEXP_string_newf("%u", res->lport),
-		"local_full_address", NULL, r3 = SEXP_string_newf("%s:%u",
-						res->laddr, res->lport),
-		"program_name", NULL, r4 = SEXP_string_newf("%s", n->cmd),
-		"foreign_address", NULL, r5 = SEXP_string_newf("%s",
-						res->raddr),
-		"foreign_port", NULL, r6 = SEXP_string_newf("%u", res->rport),
-		"foreign_full_address", NULL, r7 = SEXP_string_newf("%s:%u",
-						res->raddr, res->rport),
-		"pid", NULL, r8 = SEXP_number_newi(n->pid),
-		"user_id", NULL, r9 = SEXP_string_newf("%u", n->uid),
-		NULL);
-	SEXP_vfree(r0, r1, r2, r3, r4, r5, r6, r7, r8, r9, NULL);
-	probe_cobj_add_item(probe_out, item_sexp);
-	SEXP_free(item_sexp);
+        item = probe_item_create(OVAL_LINUX_INET_LISTENING_SERVERS, NULL,
+                                 "protocol",             OVAL_DATATYPE_STRING,  res->proto,
+                                 "local_address",        OVAL_DATATYPE_STRING,  res->laddr,
+                                 "local_port",           OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_lport_mem,
+                                                                                                   "%u", res->lport),
+                                 "local_full_address",   OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_lfull_mem,
+                                                                                                   "%s:%u", res->laddr, res->lport),
+                                 "program_name",         OVAL_DATATYPE_STRING,  n->cmd,
+                                 "foreign_address",      OVAL_DATATYPE_STRING,  res->raddr,
+                                 "foreign_port",         OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_rport_mem,
+                                                                                                   "%u", res->rport),
+                                 "foreign_full_address", OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_ffull_mem,
+                                                                                                   "%s:%u", res->raddr, res->rport),
+                                 "pid",                  OVAL_DATATYPE_INTEGER, (int64_t)n->pid,
+                                 "user_id",              OVAL_DATATYPE_SEXP,    SEXP_string_newf_r(&se_uid_mem,
+                                                                                                   "%u", n->uid),
+                                 NULL);
+
+	probe_cobj_add_item(probe_out, item);
+	SEXP_free(item);
+        SEXP_free_r(&se_lport_mem);
+        SEXP_free_r(&se_rport_mem);
+        SEXP_free_r(&se_lfull_mem);
+        SEXP_free_r(&se_ffull_mem);
+        SEXP_free_r(&se_uid_mem);
 }
 
 static void addr_convert(const char *src, char *dest, int size)
@@ -518,11 +528,13 @@ static int read_raw(const char *proc, const char *type, llist *l,
 	return 0;
 }
 
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
+int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
 	int err;
 	llist ll;
 
+        (void)filters;
+
 	if (object == NULL || probe_out == NULL) {
 		return (PROBE_EINVAL);
 	}
@@ -548,12 +560,9 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
 	// Now start collecting the info
 	list_create(&ll);
 	if (collect_process_info(&ll) || perm_warn) {
-		char *s = "Permission error.\n";
 		SEXP_t *msg;
 
-		_D(s);
-
-		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, s);
+		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "Permission error.");
 		probe_cobj_add_msg(probe_out, msg);
 		SEXP_free(msg);
 		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
