@@ -44,6 +44,10 @@
  *                string signature_keyid)
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -62,10 +66,9 @@
 /* SEAP */
 #include <seap.h>
 #include <probe-api.h>
-#include "probe-entcmp.h"
+#include "probe/entcmp.h"
 #include <alloc.h>
 #include <common/assume.h>
-#include <config.h>
 
 #ifdef HAVE_LIBRPM44
 #include <rpm/header.h>
@@ -94,7 +97,7 @@ struct rpminfo_rep {
 };
 
 struct rpminfo_global {
-        rpmdb           rpmdb;
+        rpmts           rpmts;
         pthread_mutex_t mutex;
 };
 
@@ -173,7 +176,7 @@ static int get_rpminfo (struct rpminfo_req *req, struct rpminfo_rep **rep)
 
         switch (req->op) {
         case OVAL_OPERATION_EQUALS:
-                match = rpmdbInitIterator (g_rpm.rpmdb, RPMTAG_NAME, (const void *)req->name, 0);
+                match = rpmtsInitIterator (g_rpm.rpmts, RPMTAG_NAME, (const void *)req->name, 0);
 
                 if (match == NULL) {
                         ret = 0;
@@ -184,7 +187,7 @@ static int get_rpminfo (struct rpminfo_req *req, struct rpminfo_rep **rep)
 
                 break;
         case OVAL_OPERATION_PATTERN_MATCH:
-                match = rpmdbInitIterator (g_rpm.rpmdb, RPMDBI_PACKAGES, NULL, 0);
+                match = rpmtsInitIterator (g_rpm.rpmts, RPMDBI_PACKAGES, NULL, 0);
 
                 if (match == NULL) {
                         ret = 0;
@@ -245,8 +248,7 @@ void *probe_init (void)
                 return (NULL);
         }
 
-        /* XXX: check retval */
-        rpmdbOpen (NULL, &g_rpm.rpmdb, O_RDONLY, 0644);
+        g_rpm.rpmts = rpmtsCreate();
 
         pthread_mutex_init (&(g_rpm.mutex), NULL);
 
@@ -257,25 +259,26 @@ void probe_fini (void *ptr)
 {
         struct rpminfo_global *r = (struct rpminfo_global *)ptr;
 
-        rpmdbClose (r->rpmdb);
+        rpmtsFree(r->rpmts);
 	rpmFreeCrypto();
         rpmFreeRpmrc();
         rpmFreeMacros(NULL);
         rpmlogClose();
-        rpmFreeFilesystems();
         pthread_mutex_destroy (&(r->mutex));
 
         return;
 }
 
-int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg)
+int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
-        SEXP_t *val, *item_sexp, *r0, *r1, *ent;
+        SEXP_t *val, *item, *ent;
 	int rpmret, i;
 
         struct rpminfo_req request_st;
         struct rpminfo_rep *reply_st;
 
+        (void)filters;
+
 	if (object == NULL || probe_out == NULL) {
 		return (PROBE_EINVAL);
 	}
@@ -319,6 +322,7 @@ int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg)
         }
 
         if (request_st.name == NULL) {
+		SEXP_free (ent);
                 switch (errno) {
                 case EINVAL:
                         _D("%s: invalid value type\n", "name");
@@ -343,48 +347,42 @@ int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg)
         case -1: /* Error */
                 _D("get_rpminfo failed\n");
 
-                item_sexp = probe_item_creat ("rpminfo_item", NULL,
-                                              "name", NULL,
-                                              r0 = SEXP_string_newf(request_st.name),
-                                              NULL);
+                item = probe_item_create(OVAL_LINUX_RPM_INFO, NULL,
+                                         "name", OVAL_DATATYPE_STRING, request_st.name,
+                                         NULL);
 
-                probe_item_setstatus (item_sexp, OVAL_STATUS_ERROR);
-
-		probe_cobj_add_item(probe_out, item_sexp);
-                SEXP_free (item_sexp);
-                SEXP_free (r0);
+                probe_item_setstatus (item, OVAL_STATUS_ERROR);
 
+		probe_cobj_add_item(probe_out, item);
+                SEXP_free (item);
                 break;
         default: /* Ok */
                 _A(rpmret   >= 0);
                 _A(reply_st != NULL);
                 {
-			SEXP_t *r2, *r3, *r4, *r5, *r6;
+                        SEXP_t *name;
 
                         for (i = 0; i < rpmret; ++i) {
-				r0 = SEXP_string_newf("%s", reply_st[i].name);
-				if (probe_entobj_cmp(ent, r0) != OVAL_RESULT_TRUE) {
-					SEXP_free(r0);
+				name = SEXP_string_newf("%s", reply_st[i].name);
+
+				if (probe_entobj_cmp(ent, name) != OVAL_RESULT_TRUE) {
+					SEXP_free(name);
 					continue;
 				}
 
-                                item_sexp = probe_item_creat ("rpminfo_item", NULL,
-					"name", NULL, r0,
-					"arch", NULL,
-					r1 = SEXP_string_newf ("%s", reply_st[i].arch),
-					"epoch", NULL,
-					r2 = SEXP_string_newf ("%s", reply_st[i].epoch),
-					"release", NULL,
-					r3 = SEXP_string_newf ("%s", reply_st[i].release),
-					"version", NULL,
-					r4 = SEXP_string_newf ("%s", reply_st[i].version),
-					"evr", NULL,
-					r5 = SEXP_string_newf ("%s", reply_st[i].evr),
-					"signature_keyid", NULL,
-					r6 = SEXP_string_newf ("%s", reply_st[i].signature_keyid),
-					NULL);
-                                probe_cobj_add_item(probe_out, item_sexp);
-				SEXP_vfree(item_sexp, r0, r1, r2, r3, r4, r5, r6, NULL);
+                                item = probe_item_create(OVAL_LINUX_RPM_INFO, NULL,
+                                                         "name",    OVAL_DATATYPE_SEXP, name,
+                                                         "arch",    OVAL_DATATYPE_STRING, reply_st[i].arch,
+                                                         "epoch",   OVAL_DATATYPE_STRING, reply_st[i].epoch,
+                                                         "release", OVAL_DATATYPE_STRING, reply_st[i].release,
+                                                         "version", OVAL_DATATYPE_STRING, reply_st[i].version,
+                                                         "evr",     OVAL_DATATYPE_EVR_STRING, reply_st[i].evr,
+                                                         "signature_keyid", OVAL_DATATYPE_STRING, reply_st[i].signature_keyid,
+                                                         NULL);
+
+				/* only for >= 5.8: probe_itement_setdatatype(item_sexp, "evr", OVAL_DATATYPE_EVR_STRING); */
+                                probe_cobj_add_item(probe_out, item);
+				SEXP_vfree(item, name, NULL);
 
                                 __rpminfo_rep_free (&(reply_st[i]));
                         }
