@@ -44,11 +44,10 @@
  * user_id
  */
 
-#include "config.h"
-#include "seap.h"
-#include "probe-api.h"
-#include "probe-entcmp.h"
-#include "alloc.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -62,7 +61,12 @@
 #include <sched.h>
 #include <time.h>
 
-#if defined(__linux)
+#include "seap.h"
+#include "probe-api.h"
+#include "probe/entcmp.h"
+#include "alloc.h"
+
+#if defined(__linux__)
 /* Convenience structure for the results being reported */
 struct result_info {
         const char *command;
@@ -80,23 +84,28 @@ unsigned long ticks, boot;
 
 static void report_finding(struct result_info *res, SEXP_t *probe_out)
 {
-	SEXP_t *r0, *r1, *r2, *r3, *r4, *r5, *r6, *r7, *r8, *item_sexp;
+        SEXP_t *item;
+        SEXP_t se_pid_mem, se_ppid_mem, se_tty_mem, se_pri_mem, se_uid_mem;
 
-	item_sexp = probe_item_creat("process_item", NULL,
-		/* entities */
-		"command", NULL, r0 = SEXP_string_newf("%s", res->command),
-		"exec_time", NULL, r1 = SEXP_string_newf("%s", res->exec_time),
-		"pid", NULL, r2 = SEXP_string_newf("%u", res->pid),
-		"ppid", NULL, r3 = SEXP_string_newf("%u", res->ppid),
-		"priority", NULL, r4 = SEXP_string_newf("%ld", res->priority),
-		"scheduling_class", NULL, r5 = SEXP_string_newf("%s", res->scheduling_class),
-		"start_time", NULL, r6 = SEXP_string_newf("%s",res->start_time),
-		"tty", NULL, r7 = SEXP_string_newf("%d", res->tty),
-		"user_id", NULL, r8 = SEXP_string_newf("%u", res->user_id),
-		NULL);
-	SEXP_vfree(r0, r1, r2, r3, r4, r5, r6, r7, r8, NULL);
-	probe_cobj_add_item(probe_out, item_sexp);
-	SEXP_free(item_sexp);
+        item = probe_item_create(OVAL_UNIX_PROCESS, NULL,
+                                 "command",   OVAL_DATATYPE_STRING, res->command,
+                                 "exec_time", OVAL_DATATYPE_STRING, res->exec_time,
+                                 "pid",       OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_pid_mem, "%u", res->pid),
+                                 "ppid",      OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_ppid_mem, "%u", res->ppid),
+                                 "priority",  OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_pri_mem, "%ld", res->priority),
+                                 "scheduling_class", OVAL_DATATYPE_STRING, res->scheduling_class,
+                                 "start_time", OVAL_DATATYPE_STRING, res->start_time,
+                                 "tty",        OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_tty_mem, "%d", res->tty),
+                                 "user_id",    OVAL_DATATYPE_SEXP, SEXP_string_newf_r(&se_uid_mem, "%u", res->user_id),
+                                 NULL);
+
+	probe_cobj_add_item(probe_out, item);
+	SEXP_free(item);
+        SEXP_free_r(&se_pid_mem);
+        SEXP_free_r(&se_ppid_mem);
+        SEXP_free_r(&se_tty_mem);
+        SEXP_free_r(&se_pri_mem);
+        SEXP_free_r(&se_uid_mem);
 }
 
 static void get_boot_time(void)
@@ -128,14 +137,12 @@ static void get_boot_time(void)
 static unsigned get_effective_id(int pid)
 {
 	char buf[100];
-	unsigned euid;
+	unsigned euid = 0;
 	FILE *sf;
 
 	snprintf(buf, sizeof(buf), "/proc/%d/status", pid);
 	sf = fopen(buf, "rt");
-	if (sf == NULL)
-		euid = 0;
-	else {
+	if (sf) {
 		int line = 0;
 		__fsetlocking(sf, FSETLOCKING_BYCALLER);
 		while (fgets(buf, sizeof(buf), sf)) {
@@ -307,10 +314,12 @@ static int read_process(SEXP_t *cmd_ent, SEXP_t *probe_out)
 	return err;
 }
 
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
+int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
 	SEXP_t *ent;
 
+        (void)filters;
+
 	if (object == NULL || probe_out == NULL) {
 		return (PROBE_EINVAL);
 	}
@@ -322,7 +331,7 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
 
 	if (read_process(ent, probe_out)) {
 		SEXP_free(ent);
-		return PROBE_EACCES;
+		return PROBE_EACCESS;
 	}
 
 	SEXP_free(ent);
@@ -338,7 +347,7 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
 		return (PROBE_EINVAL);
 	}
 
-	item_sexp = probe_obj_creat ("process_item", NULL, NULL);
+	item_sexp = probe_item_creat ("process_item", NULL, NULL);
         probe_item_setstatus (item_sexp, OVAL_STATUS_NOTCOLLECTED);
 	probe_cobj_add_item(probe_out, item_sexp);
         SEXP_free (item_sexp);
