@@ -44,6 +44,10 @@
  *                bool   kill)
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #define _BSD_SOURCE
 #include <stdio.h>
 #include <stdbool.h>
@@ -57,8 +61,9 @@
 
 #include <seap.h>
 #include <probe-api.h>
-#include <probe-entcmp.h>
+#include <probe/entcmp.h>
 #include <alloc.h>
+#include "common/debug_priv.h"
 
 #ifndef _A
 #define _A(x) assert(x)
@@ -338,11 +343,13 @@ static int get_runlevel (struct runlevel_req *req, struct runlevel_rep **rep)
 # error "Sorry, your OS isn't supported."
 #endif
 
-int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg)
+int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
         struct runlevel_req request_st;
         struct runlevel_rep *reply_st = NULL;
 
+        (void)filters;
+
 	if (object == NULL || probe_out == NULL) {
 		return (PROBE_EINVAL);
 	}
@@ -363,38 +370,29 @@ int probe_main (SEXP_t *object, SEXP_t *probe_out, void *arg)
 	}
 
 	if (get_runlevel(&request_st, &reply_st) == -1) {
-		char *s = "get_runlevel failed.\n";
 		SEXP_t *msg;
 
-                _D(s);
-
-		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, s);
+		msg = probe_msg_creat(OVAL_MESSAGE_LEVEL_ERROR, "get_runlevel failed.");
 		probe_cobj_add_msg(probe_out, msg);
 		SEXP_free(msg);
 		probe_cobj_set_flag(probe_out, SYSCHAR_FLAG_ERROR);
 	} else {
 		struct runlevel_rep *next_rep;
-		SEXP_t *item_sexp;
+		SEXP_t *item;
 
 		while (reply_st != NULL) {
-			SEXP_t *r0, *r1, *r2, *r3;
-
-			_D("get_runlevel: [0]=\"%s\", [1]=\"%s\", [2]=\"%d\", [3]=\"%d\"\n",
+			dI("get_runlevel: [0]=\"%s\", [1]=\"%s\", [2]=\"%d\", [3]=\"%d\"\n",
 			   reply_st->service_name, reply_st->runlevel, reply_st->start, reply_st->kill);
 
-			item_sexp = probe_item_creat("runlevel_item", NULL,
-						     /* entities */
-						     "service_name", NULL,
-						     r0 = SEXP_string_newf("%s", reply_st->service_name),
-						     "runlevel", NULL,
-						     r1 = SEXP_string_newf("%s", reply_st->runlevel),
-						     "start", NULL,
-						     r2 = SEXP_number_newb(reply_st->start),
-						     "kill", NULL,
-						     r3 = SEXP_number_newb(reply_st->kill),
-						     NULL);
-			probe_cobj_add_item(probe_out, item_sexp);
-			SEXP_vfree(r0, r1, r2, r3, item_sexp, NULL);
+                        item = probe_item_create(OVAL_UNIX_RUNLEVEL, NULL,
+                                                 "service_name", OVAL_DATATYPE_STRING,  reply_st->service_name,
+                                                 "runlevel",     OVAL_DATATYPE_STRING,  reply_st->runlevel,
+                                                 "start",        OVAL_DATATYPE_BOOLEAN, reply_st->start,
+                                                 "kill",         OVAL_DATATYPE_BOOLEAN, reply_st->kill,
+                                                 NULL);
+
+			probe_cobj_add_item(probe_out, item);
+                        SEXP_free(item);
 
 			next_rep = reply_st->next;
 			oscap_free(reply_st->service_name);
