@@ -41,21 +41,27 @@
  * processor_type
  */
 
-#include "config.h"
-#include "seap.h"
-#include "probe-api.h"
-#include "alloc.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #ifdef HAVE_STDIO_EXT_H
 # include <stdio_ext.h>
 #endif
 #include <string.h>
 #include <sys/utsname.h>
 
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
+#include "seap.h"
+#include "probe-api.h"
+#include "alloc.h"
+
+int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
 	const char *processor;
 	struct utsname buf;
-	SEXP_t *r0, *r1, *r2, *r3, *r4, *r5, *item_sexp;
+        SEXP_t *item;
+
+        (void)filters;
 
 	if (object == NULL || probe_out == NULL) {
 		return (PROBE_EINVAL);
@@ -64,7 +70,7 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
 	// Collect the info
 	uname(&buf);
 	processor = buf.machine;
-#ifdef __linux
+#ifdef __linux__
 	if (strcmp(processor, "i686") == 0) {
 		// See if we have an athlon processor
 		FILE *f = fopen("/proc/cpu", "rt");
@@ -85,18 +91,18 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
 		}
 	}
 #endif
-	item_sexp = probe_obj_creat("uname_item", NULL,
-		/* entities */
-		"machine_class", NULL, r0 = SEXP_string_newf("%s", buf.machine),
-		"node_name", NULL, r1 = SEXP_string_newf("%s", buf.nodename),
-		"os_name", NULL, r2 = SEXP_string_newf("%s", buf.sysname),
-		"os_release", NULL, r3 = SEXP_string_newf("%s", buf.release),
-		"os_version", NULL, r4 = SEXP_string_newf("%s", buf.version),
-		"processor_type", NULL, r5 = SEXP_string_newf("%s", processor),
-		NULL);
-	SEXP_vfree(r0, r1, r2, r3, r4, r5, NULL);
-	probe_cobj_add_item(probe_out, item_sexp);
-	SEXP_free(item_sexp);
+
+        item = probe_item_create(OVAL_UNIX_UNAME, NULL,
+                                 "machine_class",  OVAL_DATATYPE_STRING, buf.machine,
+                                 "node_name",      OVAL_DATATYPE_STRING, buf.nodename,
+                                 "os_name",        OVAL_DATATYPE_STRING, buf.sysname,
+                                 "os_release",     OVAL_DATATYPE_STRING, buf.release,
+                                 "os_version",     OVAL_DATATYPE_STRING, buf.version,
+                                 "processor_type", OVAL_DATATYPE_STRING, processor,
+                                 NULL);
+
+	probe_cobj_add_item(probe_out, item);
+	SEXP_free(item);
 
 	return 0;
 }
