@@ -27,7 +27,10 @@
  *   Daniel Kopecek <dkopecek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <probe-api.h>
 #include <errno.h>
 #include <unistd.h>
@@ -1329,7 +1332,7 @@ int op_assign_disabled(void *var, char *val)
 		if (*tok == '\0')
 			continue;
 
-		rbt_str_get(xiconf->stree, tok, (void *)srv);
+		rbt_str_get(xiconf->stree, tok, (void *)&srv);
 
 		if (srv == NULL) {
 			srv = xiconf_service_new();
@@ -1370,7 +1373,7 @@ int op_assign_enabled(void *var, char *val)
 		if (*tok == '\0')
 			continue;
 
-		rbt_str_get(xiconf->stree, tok, (void *)srv);
+		rbt_str_get(xiconf->stree, tok, (void *)&srv);
 
 		if (srv == NULL) {
 			srv = xiconf_service_new();
@@ -1419,7 +1422,7 @@ void probe_fini(void *arg)
 		xiconf_free(arg);
 }
 
-int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
+int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg, SEXP_t *filters)
 {
 	SEXP_t *eval;
 	char    srv_name[256];
@@ -1429,6 +1432,8 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
 	xiconf_strans_t  *xres;
 	xiconf_t         *xcfg = (xiconf_t *)arg;
 
+        (void)filters;
+
 	if (object == NULL || probe_out == NULL) {
 		err = PROBE_EINVAL;
 		goto fail;
@@ -1456,7 +1461,6 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
 	eval = probe_obj_getentval(object, "protocol", 1);
 
 	if (eval == NULL) {
-		oscap_free(srv_name);
 		err = PROBE_ENOVAL;
 		goto fail;
 	}
@@ -1483,34 +1487,27 @@ int probe_main(SEXP_t *object, SEXP_t *probe_out, void *arg)
 
 	if (xres != NULL) {
 		SEXP_t *item;
-		SEXP_t *r[0xC+1];
-		register unsigned int i, l;
-
-#define SEXP_string_new1(str) SEXP_string_new((str), strlen(str))
+		register unsigned int l;
 
 		for (l = 0; l < xres->cnt; ++l) {
-			item = probe_item_creat("xinetd_item", NULL,
-						"protocol",         NULL, r[0x0]= SEXP_string_new1(xres->srv[l]->protocol),
-						"service_name",     NULL, r[0x1]= SEXP_string_new1(xres->srv[l]->name),
-						"flags",            NULL, r[0x2]= SEXP_string_new1(xres->srv[l]->flags),
-						"no_access",        NULL, r[0x3]= SEXP_string_new1(xres->srv[l]->no_access),
-						"only_from",        NULL, r[0x4]= SEXP_string_new1(xres->srv[l]->only_from),
-						"port",             NULL, r[0x5]= SEXP_number_newu(xres->srv[l]->port),
-						"server",           NULL, r[0x6]= SEXP_string_new1(xres->srv[l]->server),
-						"server_arguments", NULL, r[0x7]= SEXP_string_new1(xres->srv[l]->server_args),
-						"socket_type",      NULL, r[0x8]= SEXP_string_new1(xres->srv[l]->socket_type),
-						"type",             NULL, r[0x9]= SEXP_string_new1(xres->srv[l]->type),
-						"user",             NULL, r[0xA]= SEXP_string_new1(xres->srv[l]->user),
-						"wait",             NULL, r[0xB]= SEXP_number_newb(xres->srv[l]->wait),
-						"disabled",         NULL, r[0xC]= SEXP_number_newb(xres->srv[l]->disable),
+                        item = probe_item_create(OVAL_UNIX_XINETD, NULL,
+						"protocol",         OVAL_DATATYPE_STRING,  xres->srv[l]->protocol,
+						"service_name",     OVAL_DATATYPE_STRING,  xres->srv[l]->name,
+						"flags",            OVAL_DATATYPE_STRING,  xres->srv[l]->flags,
+						"no_access",        OVAL_DATATYPE_STRING,  xres->srv[l]->no_access,
+						"only_from",        OVAL_DATATYPE_STRING,  xres->srv[l]->only_from,
+                                                "port",             OVAL_DATATYPE_INTEGER, (int64_t)xres->srv[l]->port,
+						"server",           OVAL_DATATYPE_STRING,  xres->srv[l]->server,
+						"server_arguments", OVAL_DATATYPE_STRING,  xres->srv[l]->server_args,
+						"socket_type",      OVAL_DATATYPE_STRING,  xres->srv[l]->socket_type,
+						"type",             OVAL_DATATYPE_STRING,  xres->srv[l]->type,
+						"user",             OVAL_DATATYPE_STRING,  xres->srv[l]->user,
+						"wait",             OVAL_DATATYPE_BOOLEAN, xres->srv[l]->wait,
+						"disabled",         OVAL_DATATYPE_BOOLEAN, xres->srv[l]->disable,
 						NULL);
 
 			probe_cobj_add_item(probe_out, item);
 			SEXP_free(item);
-
-			/* TODO: get rid of useless S-exp refs */
-			for (i = 0; i < 0xC+1; ++i)
-				SEXP_free(r[i]);
 		}
 	}
 
