@@ -44,8 +44,9 @@
 #define OVAL_PDFLAG_NORECONN 0x0002	/**< don't try to reconnect on fatal errors */
 #define OVAL_PDGLAG_RUNALL   0x0004	/**< execute all probes when executing the first */
 #define OVAL_PDFLAG_RUNNOW   0x0008	/**< execute all probes immediately */
+#define OVAL_PDFLAG_SLAVE    0x0010
 
-#define OVAL_PDFLAG_MASK (0x0001|0x0002|0x0004|0x0008)
+#define OVAL_PDFLAG_MASK (0x0001|0x0002|0x0004|0x0008|0x0010)
 
 /**
  * Evaluate system info probe
@@ -61,13 +62,6 @@ int oval_probe_query_sysinfo(oval_probe_session_t *sess, struct oval_sysinfo **o
  */
 int oval_probe_query_object(oval_probe_session_t *psess, struct oval_object *object, int flags, struct oval_syschar **out_syschar) __attribute__ ((nonnull(1, 2)));
 
-/**
- * Probe all objects and update system characteristic model in the session
- * @param sess probe session
- * @return 0 on success
- */
-int oval_probe_query_objects(oval_probe_session_t *sess) __attribute__ ((nonnull(1)));
-
 /**
  * Probe objects required for the evalatuation of the specified definition and update the system characteristics model associated with the session
  * @param sess probe session
