@@ -175,7 +175,10 @@ typedef enum {
 	OVAL_INDEPENDENT_TEXT_FILE_CONTENT = OVAL_FAMILY_INDEPENDENT + 7,
 	OVAL_INDEPENDENT_UNKNOWN = OVAL_FAMILY_INDEPENDENT + 8,
 	OVAL_INDEPENDENT_VARIABLE = OVAL_FAMILY_INDEPENDENT + 9,
-	OVAL_INDEPENDENT_XML_FILE_CONTENT = OVAL_FAMILY_INDEPENDENT + 10
+	OVAL_INDEPENDENT_XML_FILE_CONTENT = OVAL_FAMILY_INDEPENDENT + 10,
+        OVAL_INDEPENDENT_LDAP57 = OVAL_FAMILY_INDEPENDENT + 11,
+	OVAL_INDEPENDENT_LDAP = OVAL_FAMILY_INDEPENDENT + 12,
+        OVAL_INDEPENDENT_SQL57 = OVAL_FAMILY_INDEPENDENT + 13
 } oval_independent_subtype_t;
 
 /// IOS subtypes
@@ -193,7 +196,12 @@ typedef enum {
 	OVAL_LINUX_DPKG_INFO = OVAL_FAMILY_LINUX + 1,
 	OVAL_LINUX_INET_LISTENING_SERVERS = OVAL_FAMILY_LINUX + 2,
 	OVAL_LINUX_RPM_INFO = OVAL_FAMILY_LINUX + 3,
-	OVAL_LINUX_SLACKWARE_PKG_INFO_TEST = OVAL_FAMILY_LINUX + 4
+	OVAL_LINUX_SLACKWARE_PKG_INFO_TEST = OVAL_FAMILY_LINUX + 4,
+        OVAL_LINUX_PARTITION = OVAL_FAMILY_LINUX + 5,
+	OVAL_LINUX_IFLISTENERS = OVAL_FAMILY_LINUX + 6,
+	OVAL_LINUX_RPMVERIFY = OVAL_FAMILY_LINUX + 7,
+	OVAL_LINUX_SELINUXBOOLEAN = OVAL_FAMILY_LINUX + 8,
+	OVAL_LINUX_SELINUXSECURITYCONTEXT = OVAL_FAMILY_LINUX + 9
 } oval_linux_subtype_t;
 
 /// MacOS subtypes
@@ -229,7 +237,13 @@ typedef enum {
 	OVAL_UNIX_SCCS = OVAL_FAMILY_UNIX + 7,
 	OVAL_UNIX_SHADOW = OVAL_FAMILY_UNIX + 8,
 	OVAL_UNIX_UNAME = OVAL_FAMILY_UNIX + 9,
-	OVAL_UNIX_XINETD = OVAL_FAMILY_UNIX + 10
+	OVAL_UNIX_XINETD = OVAL_FAMILY_UNIX + 10,
+        OVAL_UNIX_DNSCACHE = OVAL_FAMILY_UNIX + 11,
+        OVAL_UNIX_SYSCTL = OVAL_FAMILY_UNIX + 12,
+	OVAL_UNIX_PROCESS58 = OVAL_FAMILY_UNIX + 13,
+	OVAL_UNIX_FILEEXTENDEDATTRIBUTE = OVAL_FAMILY_UNIX + 14,
+	OVAL_UNIX_GCONF = OVAL_FAMILY_UNIX + 15,
+	OVAL_UNIX_ROUTINGTABLE = OVAL_FAMILY_UNIX + 16
 } oval_unix_subtype_t;
 
 /// Windows subtypes
