@@ -20,9 +20,14 @@
  *      Lukas Kuklinek <lkuklinek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
-#include "item.h"
 #include <text.h>
+
+#include "item.h"
 #include "helpers.h"
 #include "xccdf_impl.h"
 #include "common/_error.h"
@@ -248,10 +253,7 @@ xmlNode *xccdf_benchmark_to_dom(struct xccdf_benchmark *benchmark, xmlDocPtr doc
 		xmlNewTextChild(root_node, ns_xccdf, BAD_CAST "version", BAD_CAST version);
 
 	const char *metadata = xccdf_benchmark_get_metadata(benchmark);
-	if (metadata) {
-		xmlNode *metadata_node = xmlNewTextChild(root_node, ns_xccdf, BAD_CAST "metadata", NULL);
-		xmlNewTextChild(metadata_node, ns_xccdf, NULL, BAD_CAST metadata);
-	}
+	if (metadata) oscap_xmlstr_to_dom(root_node, "metadata", metadata);
 
 	OSCAP_FOR(xccdf_model, model, xccdf_benchmark_get_models(benchmark)) {
 		xmlNode *model_node = xmlNewTextChild(root_node, ns_xccdf, BAD_CAST "model", NULL);
@@ -389,7 +391,7 @@ struct xccdf_notice *xccdf_notice_new(void)
 {
     struct xccdf_notice *notice = oscap_calloc(1, sizeof(struct xccdf_notice));
     notice->text = oscap_text_new_full(XCCDF_TEXT_NOTICE, NULL, NULL);
-    return NULL;
+    return notice;
 }
 
 struct xccdf_notice *xccdf_notice_clone(const struct xccdf_notice * notice)
@@ -545,7 +547,9 @@ bool xccdf_benchmark_rename_item(struct xccdf_item *item, const char *newid)
 	if (bench != NULL) {
 		if (newid != NULL && xccdf_benchmark_get_item(XBENCHMARK(bench), newid) != NULL)
 			return false; // ID already assigned
-		xccdf_benchmark_unregister_item(item);
+
+		if (xccdf_item_get_id(item) != NULL)
+			xccdf_benchmark_unregister_item(item);
 
 		if (newid != NULL)
 			oscap_htable_add(bench->sub.benchmark.dict, newid, item);
@@ -590,4 +594,3 @@ void xccdf_plain_text_free(struct xccdf_plain_text *plain)
 
 OSCAP_ACCESSOR_STRING(xccdf_plain_text, id)
 OSCAP_ACCESSOR_STRING(xccdf_plain_text, text)
-
