@@ -22,14 +22,24 @@
  * 	Josh Adams <jadams@tresys.com>
  */
 
-#include "item.h"
-#include "helpers.h"
-#include "xccdf_impl.h"
-#include "../common/util.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <time.h>
 #include <math.h>
 
+#include <libxml/tree.h>
+#include <libxml/xpath.h>
+#include <libxml/parser.h>
+#include <libxml/xpathInternals.h>
+
+#include "item.h"
+#include "helpers.h"
+#include "xccdf_impl.h"
+#include "common/util.h"
+
 const struct oscap_string_map XCCDF_OPERATOR_MAP[] = {
 	{XCCDF_OPERATOR_EQUALS, "equals"},
 	{XCCDF_OPERATOR_NOT_EQUAL, "not equal"},
@@ -47,7 +57,7 @@ const struct oscap_string_map XCCDF_LEVEL_MAP[] = {
 	{XCCDF_LOW, "low"},
 	{XCCDF_MEDIUM, "medium"},
 	{XCCDF_HIGH, "high"},
-	{0, NULL}
+	{XCCDF_UNKNOWN, NULL}
 };
 
 static const struct oscap_string_map XCCDF_STATUS_MAP[] = {
@@ -59,6 +69,18 @@ static const struct oscap_string_map XCCDF_STATUS_MAP[] = {
 	{XCCDF_STATUS_NOT_SPECIFIED, NULL}
 };
 
+static const struct oscap_string_map XCCDF_WARNING_MAP[] = {
+        {XCCDF_WARNING_GENERAL, "general"},
+        {XCCDF_WARNING_FUNCTIONALITY, "functionality"},
+        {XCCDF_WARNING_PERFORMANCE, "performance"},
+        {XCCDF_WARNING_HARDWARE, "hardware"},
+        {XCCDF_WARNING_LEGAL, "legal"},
+        {XCCDF_WARNING_REGULATORY, "regulatory"},
+        {XCCDF_WARNING_MANAGEMENT, "management"},
+        {XCCDF_WARNING_AUDIT, "audit"},
+        {XCCDF_WARNING_DEPENDENCY, "dependency"},
+	{XCCDF_WARNING_NOT_SPECIFIED, NULL}
+};
 
 struct xccdf_item *xccdf_item_new(xccdf_type_t type, struct xccdf_item *parent)
 {
@@ -329,13 +351,31 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 	}
 	xccdf_status_iterator_free(statuses);
 
+    time_t vt = xccdf_item_get_version_time(item);
+	const char *version = xccdf_item_get_version(item);
+	if (xccdf_item_get_type(item) != XCCDF_BENCHMARK && (version || vt || xccdf_item_get_version_update(item))) {
+		xmlNode* version_node = xmlNewTextChild(item_node, ns_xccdf, BAD_CAST "version", BAD_CAST version);
+        if (xccdf_item_get_version_update(item))
+		    xmlNewProp(version_node, BAD_CAST "update", BAD_CAST xccdf_item_get_version_update(item));
+        if (vt) {
+            struct tm *lt = localtime(&vt);
+            char timestamp[] = "yyyy-mm-ddThh:mm:ss";
+            snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
+                 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
+            xmlNewProp(version_node, BAD_CAST "time", BAD_CAST timestamp);
+        }
+    }
+
 	/* Handle generic item child nodes */
 	xccdf_texts_to_dom(xccdf_item_get_title(item), item_node, "title");
 	xccdf_texts_to_dom(xccdf_item_get_description(item), item_node, "description");
 
-	const char *version = xccdf_item_get_version(item);
-	if (version && xccdf_item_get_type(item) != XCCDF_BENCHMARK)
-		xmlNewTextChild(item_node, ns_xccdf, BAD_CAST "version", BAD_CAST version);
+	struct xccdf_warning_iterator *warnings = xccdf_item_get_warnings(item);
+	while (xccdf_warning_iterator_has_more(warnings)) {
+		struct xccdf_warning *warning = xccdf_warning_iterator_next(warnings);
+		xccdf_warning_to_dom(warning, doc, item_node);
+	}
+	xccdf_warning_iterator_free(warnings);
 
 	xccdf_texts_to_dom(xccdf_item_get_question(item), item_node, "question");
 
@@ -392,6 +432,18 @@ xmlNode *xccdf_profile_note_to_dom(struct xccdf_profile_note *note, xmlDoc *doc,
 	return note_node;
 }
 
+xmlNode *xccdf_warning_to_dom(struct xccdf_warning *warning, xmlDoc *doc, xmlNode *parent)
+{
+	xmlNode *warning_node = NULL;
+	xccdf_warning_category_t category = xccdf_warning_get_category(warning);
+
+	warning_node = oscap_text_to_dom(xccdf_warning_get_text(warning), parent, "warning");
+	if (category != XCCDF_WARNING_NOT_SPECIFIED)
+	    xmlNewProp(warning_node, BAD_CAST "category", BAD_CAST XCCDF_WARNING_MAP[category - 1].string);
+
+	return warning_node;
+}
+
 xmlNode *xccdf_status_to_dom(struct xccdf_status *status, xmlDoc *doc, xmlNode *parent)
 {
 	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
@@ -424,11 +476,11 @@ xmlNode *xccdf_fixtext_to_dom(struct xccdf_fixtext *fixtext, xmlDoc *doc, xmlNod
 
 	xccdf_level_t complexity = xccdf_fixtext_get_complexity(fixtext);
         if (complexity != 0)
-	        xmlNewProp(fixtext_node, BAD_CAST "complexity", BAD_CAST XCCDF_LEVEL_MAP[complexity].string);
+	        xmlNewProp(fixtext_node, BAD_CAST "complexity", BAD_CAST XCCDF_LEVEL_MAP[complexity-1].string);
 
 	xccdf_level_t disruption = xccdf_fixtext_get_disruption(fixtext);
         if (disruption != 0)
-	        xmlNewProp(fixtext_node, BAD_CAST "disruption", BAD_CAST XCCDF_LEVEL_MAP[disruption].string);
+	        xmlNewProp(fixtext_node, BAD_CAST "disruption", BAD_CAST XCCDF_LEVEL_MAP[disruption-1].string);
 
 	xccdf_strategy_t strategy = xccdf_fixtext_get_strategy(fixtext);
         if (strategy != 0)
@@ -447,6 +499,9 @@ xmlNode *xccdf_fix_to_dom(struct xccdf_fix *fix, xmlDoc *doc, xmlNode *parent)
 	const char *sys = xccdf_fix_get_system(fix);
 	if (sys != NULL) xmlNewProp(fix_node, BAD_CAST "system", BAD_CAST sys);
 
+	const char *platform = xccdf_fix_get_platform(fix);
+	if (platform != NULL) xmlNewProp(fix_node, BAD_CAST "platform", BAD_CAST platform);
+
 	if (xccdf_fix_get_reboot(fix))
 		xmlNewProp(fix_node, BAD_CAST "reboot", BAD_CAST "True");
 
@@ -468,7 +523,7 @@ xmlNode *xccdf_fix_to_dom(struct xccdf_fix *fix, xmlDoc *doc, xmlNode *parent)
 	// This is in the XCCDF Spec, but not implemented in OpenSCAP
 	//const char *instance = xccdf_fix_get_instance(fix);
 	//xmlNewTextChild(fix_node, ns_xccdf, BAD_CAST "instance", BAD_CAST instance);
-	
+
 	return fix_node;
 }
 
@@ -616,14 +671,25 @@ bool xccdf_item_process_element(struct xccdf_item * item, xmlTextReaderPtr reade
         }
         break;
     }
-	case XCCDFE_VERSION:
-		item->item.version_time = oscap_get_datetime(xccdf_attribute_get(reader, XCCDFA_TIME));
-		item->item.version_update = xccdf_attribute_copy(reader, XCCDFA_UPDATE);
-		item->item.version = oscap_element_string_copy(reader);
-		break;
+	case XCCDFE_VERSION: {
+        xmlNode *ver = xmlTextReaderExpand(reader);
+        char *vt = (char*) xmlGetProp(ver, BAD_CAST "time");
+        if (vt) item->item.version_time = oscap_get_datetime(vt);
+        oscap_free(vt);
+        item->item.version_update = (char*) xmlGetProp(ver, BAD_CAST "update");
+        item->item.version = (char *) xmlNodeGetContent(ver);
+        if (oscap_streq(item->item.version, "")) {
+            oscap_free(item->item.version);
+            item->item.version = NULL;
+        }
+		return true;
+    }
 	case XCCDFE_RATIONALE:
         oscap_list_add(item->item.rationale, oscap_text_new_parse(XCCDF_TEXT_HTMLSUB, reader));
 		return true;
+        case XCCDFE_PLATFORM:
+        oscap_list_add(item->item.platforms, xccdf_attribute_copy(reader, XCCDFA_IDREF));
+                return true;
 	case XCCDFE_QUESTION:
         oscap_list_add(item->item.question, oscap_text_new_parse(XCCDF_TEXT_PLAIN, reader));
 		return true;
@@ -828,19 +894,6 @@ struct xccdf_model *xccdf_model_new_xml(xmlTextReaderPtr reader)
 
 OSCAP_ACCESSOR_STRING(xccdf_model, system)
 
-static const struct oscap_string_map XCCDF_WARNING_MAP[] = {
-	{ XCCDF_WARNING_GENERAL, "general" },
-	{ XCCDF_WARNING_FUNCTIONALITY, "functionality" },
-	{ XCCDF_WARNING_PERFORMANCE, "performance" },
-	{ XCCDF_WARNING_HARDWARE, "hardware" },
-	{ XCCDF_WARNING_LEGAL, "legal" },
-	{ XCCDF_WARNING_REGULATORY, "regulatory" },
-	{ XCCDF_WARNING_MANAGEMENT, "management" },
-	{ XCCDF_WARNING_AUDIT, "audit" },
-	{ XCCDF_WARNING_DEPENDENCY, "dependency" },
-	{ XCCDF_WARNING_GENERAL, NULL }
-};
-
 void xccdf_model_free(struct xccdf_model *model)
 {
 	if (model) {
@@ -1099,3 +1152,90 @@ const struct oscap_text_traits XCCDF_TEXT_HTMLSUB  = { .html = true, .can_overri
 const struct oscap_text_traits XCCDF_TEXT_NOTICE   = { .html = true };
 const struct oscap_text_traits XCCDF_TEXT_PROFNOTE = { .html = true, .can_substitute = true };
 
+
+// text substitution
+char* oscap_text_xccdf_substitute(const char *text, xccdf_substitution_func cb, void *arg)
+{
+    if (text == NULL)
+        return NULL;
+    if (cb == NULL)
+        return oscap_strdup(text);
+
+    char *result                = NULL;
+    char *result_tmp            = NULL;
+    char *input                 = NULL;
+    xmlDocPtr doc               = NULL;
+    xmlXPathContextPtr xpathCtx = NULL;
+    xmlXPathObjectPtr xpathObj  = NULL;
+    xmlNodeSetPtr nodes         = NULL;
+    const char *xpath = "//cdf:sub";
+    size_t size;
+
+    input = oscap_sprintf("<x xmlns='http://www.w3.org/1999/xhtml'>%s</x>", text);
+
+    doc = xmlParseMemory(input, strlen(input));
+    if (doc == NULL) goto cleanup;
+
+    xpathCtx = xmlXPathNewContext(doc);
+    if (xpathCtx == NULL) goto cleanup;
+
+    if (xmlXPathRegisterNs(xpathCtx, BAD_CAST "cdf", BAD_CAST XCCDF_XMLNS) != 0) goto cleanup;
+
+    xpathObj = xmlXPathEvalExpression(BAD_CAST xpath, xpathCtx);
+    if (xpathObj == NULL) goto cleanup;
+
+    nodes = xpathObj->nodesetval;
+    size = (nodes) ? nodes->nodeNr : 0;
+
+    for (size_t i = 0; i < size; ++i) {
+        xmlNodePtr cur = nodes->nodeTab[i];
+        char *sub = NULL; // text to be substituted
+
+        if (cur->type == XML_ELEMENT_NODE) {
+            if (oscap_streq((const char *) cur->name, "sub")) {
+                const char *id = (const char *) xmlGetProp(cur, BAD_CAST "idref");
+                if (id) sub = cb(XCCDF_SUBST_SUB, id, arg);
+            }
+            else if (oscap_streq((const char *) cur->name, "instance")) {
+                // TODO
+            }
+        }
+
+        // do the substitution
+        if (sub) {
+            xmlNodePtr sub_node = xmlNewText(BAD_CAST sub);
+            xmlReplaceNode(cur, sub_node);
+            oscap_free(sub);
+        }
+    }
+
+    // get resulting string
+    xmlDocDumpMemory(doc, (xmlChar**) &result_tmp, (int*) &size);
+    if (result_tmp) {
+        result = strchr(result_tmp, '>');          // skip <?xml ... ?>
+
+        if (result) {
+            ++result;
+            result = strchr(result, '>');  // skip root element
+        }
+        else result = NULL;
+
+        if (result) {
+            ++result; // skip the '>' itself
+            char *end = strrchr(result, '<');
+            if (end) *end = '\0';
+            else result = NULL;
+        }
+
+        result = oscap_strdup(result); // duplicate selected part
+    }
+
+cleanup:
+    xmlXPathFreeObject(xpathObj);
+    xmlXPathFreeContext(xpathCtx);
+    xmlFreeDoc(doc);
+    oscap_free(input);
+    oscap_free(result_tmp);
+    return result;
+}
+
