@@ -20,6 +20,10 @@
  *      Lukas Kuklinek <lkuklinek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "item.h"
 #include "helpers.h"
 #include "xccdf_impl.h"
@@ -241,7 +245,9 @@ struct xccdf_item *xccdf_profile_parse(xmlTextReaderPtr reader, struct xccdf_ite
 				struct xccdf_refine_rule *rr = xccdf_refine_rule_new();
 				rr->item = oscap_strdup(id);
 				rr->selector = xccdf_attribute_copy(reader, XCCDFA_SELECTOR);
-				rr->weight = xccdf_attribute_get_float(reader, XCCDFA_WEIGHT);
+				if (xccdf_attribute_has(reader, XCCDFA_WEIGHT))
+				    rr->weight = xccdf_attribute_get_float(reader, XCCDFA_WEIGHT);
+                                else rr->weight = -1.0;
 				if (xccdf_attribute_has(reader, XCCDFA_ROLE))
 					rr->role =
 					    oscap_string_to_enum(XCCDF_ROLE_MAP,
@@ -355,7 +361,7 @@ void xccdf_profile_to_dom(struct xccdf_profile *profile, xmlNode *profile_node,
 	struct xccdf_refine_rule_iterator *refine_rules = xccdf_profile_get_refine_rules(profile);
 	while (xccdf_refine_rule_iterator_has_more(refine_rules)) {
 		struct xccdf_refine_rule *refine_rule = xccdf_refine_rule_iterator_next(refine_rules);
-		xmlNode *refrule_node = xmlNewTextChild(profile_node, ns_xccdf, BAD_CAST "refine-value", NULL);
+		xmlNode *refrule_node = xmlNewTextChild(profile_node, ns_xccdf, BAD_CAST "refine-rule", NULL);
 
 		const char *idref = xccdf_refine_rule_get_item(refine_rule);
 		if (idref)
@@ -371,7 +377,7 @@ void xccdf_profile_to_dom(struct xccdf_profile *profile, xmlNode *profile_node,
 
 		xccdf_level_t severity = xccdf_refine_rule_get_severity(refine_rule);
 		if (severity != 0)
-			xmlNewProp(refrule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity].string);
+			xmlNewProp(refrule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
 
 		float weight = xccdf_refine_rule_get_weight(refine_rule);
 		char *weight_str = oscap_sprintf("%f", weight);
@@ -433,6 +439,3 @@ OSCAP_ACCESSOR_SIMPLE(xccdf_operator_t, xccdf_refine_value, oper)
 OSCAP_IGETINS(oscap_text, xccdf_refine_value, remarks, remark)
 OSCAP_ACCESSOR_STRING(xccdf_setvalue, item)
 OSCAP_ACCESSOR_STRING(xccdf_setvalue, value)
-
-OSCAP_ITERATOR_RESET(xccdf_setvalue)
-OSCAP_ITERATOR_RESET(xccdf_refine_value)
