@@ -22,12 +22,18 @@
  *      Josh Adams <jadams@tresys.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <assert.h>
+#include <string.h>
+
 #include "item.h"
 #include "elements.h"
 #include "helpers.h"
 #include "xccdf_impl.h"
-#include <assert.h>
-#include <string.h>
+
 
 bool xccdf_content_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
 {
@@ -99,7 +105,7 @@ static bool xccdf_item_parse_deps(xmlTextReaderPtr reader, struct xccdf_item *it
 			struct oscap_list *reqs = oscap_list_new();
 			char *ids = xccdf_attribute_copy(reader, XCCDFA_IDREF), *idsstr = ids, *id;
 
-			while ((id = oscap_strsep(&ids, " ")) != NULL) {
+			while ((id = strsep(&ids, " ")) != NULL) {
 				if (strcmp(id, "") == 0) continue;
 				oscap_list_add(reqs, oscap_strdup(id));
 			}
@@ -402,6 +408,16 @@ struct xccdf_ident *xccdf_ident_new_fill(const char *id, const char *sys)
 	return ident;
 }
 
+void xccdf_ident_set_id(struct xccdf_ident * ident, const char *id)
+{
+        ident->id = oscap_strdup(id);
+}
+
+void xccdf_ident_set_system(struct xccdf_ident * ident, const char *sys)
+{
+        ident->system = oscap_strdup(sys);
+}
+
 struct xccdf_ident *xccdf_ident_parse(xmlTextReaderPtr reader)
 {
 	XCCDF_ASSERT_ELEMENT(reader, XCCDFE_IDENT);
@@ -672,13 +688,13 @@ void xccdf_check_export_free(struct xccdf_check_export *item)
 
 const struct oscap_string_map XCCDF_STRATEGY_MAP[] = {
 	{XCCDF_STRATEGY_CONFIGURE, "configure"},
-	{XCCDF_STRATEGY_COMBINATION, "combination"},
 	{XCCDF_STRATEGY_DISABLE, "disable"},
 	{XCCDF_STRATEGY_ENABLE, "enable"},
 	{XCCDF_STRATEGY_PATCH, "patch"},
 	{XCCDF_STRATEGY_POLICY, "policy"},
 	{XCCDF_STRATEGY_RESTRICT, "restrict"},
 	{XCCDF_STRATEGY_UPDATE, "update"},
+	{XCCDF_STRATEGY_COMBINATION, "combination"},
 	{XCCDF_STRATEGY_UNKNOWN, NULL}
 };
 
@@ -743,6 +759,10 @@ struct xccdf_fixtext *xccdf_fixtext_parse(xmlTextReaderPtr reader)
 	struct xccdf_fixtext *fix = xccdf_fixtext_new();
 	fix->fixref = xccdf_attribute_copy(reader, XCCDFA_FIXREF);
 	fix->text = oscap_text_new_parse(XCCDF_TEXT_HTMLSUB, reader);
+	fix->reboot     = xccdf_attribute_get_bool(reader, XCCDFA_REBOOT);
+	fix->strategy   = oscap_string_to_enum(XCCDF_STRATEGY_MAP, xccdf_attribute_get(reader, XCCDFA_STRATEGY));
+	fix->disruption = oscap_string_to_enum(XCCDF_LEVEL_MAP, xccdf_attribute_get(reader, XCCDFA_DISRUPTION));
+	fix->complexity = oscap_string_to_enum(XCCDF_LEVEL_MAP, xccdf_attribute_get(reader, XCCDFA_COMPLEXITY));
 	return fix;
 }
 
@@ -798,6 +818,43 @@ struct oscap_stringlist_iterator *xccdf_group_get_requires(const struct xccdf_gr
 	return oscap_iterator_new(XITEM(group)->sub.group.requires);
 }
 
+bool xccdf_rule_add_requires(struct xccdf_rule* rule, struct oscap_stringlist* requires)
+{
+    oscap_list_add(XITEM(rule)->sub.rule.requires, requires);
+    return true;
+}
+bool xccdf_group_add_requires(struct xccdf_group* group, struct oscap_stringlist* requires)
+{
+    oscap_list_add(XITEM(group)->sub.group.requires, requires);
+    return true;
+}
+bool xccdf_item_add_requires(struct xccdf_item* item, struct oscap_stringlist* requires)
+{
+	if (item == NULL) return NULL;
+	if (item->type == XCCDF_RULE)  return xccdf_rule_add_requires ( XRULE(item), requires);
+	if (item->type == XCCDF_GROUP) return xccdf_group_add_requires(XGROUP(item), requires);
+    return false;
+}
+
+bool xccdf_rule_add_conflicts(struct xccdf_rule* rule, const char* conflicts)
+{
+    oscap_stringlist_add_string((struct oscap_stringlist*)(XITEM(rule)->sub.rule.conflicts), conflicts);
+    return true;
+}
+bool xccdf_group_add_conflicts(struct xccdf_group* group, const char* conflicts)
+{
+    oscap_stringlist_add_string((struct oscap_stringlist*)(XITEM(group)->sub.group.conflicts), conflicts);
+    return true;
+}
+bool xccdf_item_add_conflicts(struct xccdf_item* item, const char* conflicts)
+{
+	if (item == NULL) return NULL;
+	if (item->type == XCCDF_RULE)  return xccdf_rule_add_conflicts ( XRULE(item), conflicts);
+	if (item->type == XCCDF_GROUP) return xccdf_group_add_conflicts(XGROUP(item), conflicts);
+    return false;
+}
+
+
 void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc, xmlNode *parent)
 {
 	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
@@ -841,7 +898,8 @@ void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc,
 	struct oscap_string_iterator *platforms = xccdf_rule_get_platforms(rule);
 	while (oscap_string_iterator_has_more(platforms)) {
 		const char *platform = oscap_string_iterator_next(platforms);
-		xmlNewTextChild(rule_node, ns_xccdf, BAD_CAST "platform", BAD_CAST platform);
+		xmlNode * child = xmlNewTextChild(rule_node, ns_xccdf, BAD_CAST "platform", BAD_CAST NULL);
+                xmlNewProp(child, BAD_CAST "idref", BAD_CAST platform);
 	}
 	oscap_string_iterator_free(platforms);
 
@@ -851,7 +909,8 @@ void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc,
 		struct oscap_string_iterator *strings = oscap_stringlist_get_strings(list);
 		while (oscap_string_iterator_has_more(strings)) {
 			const char *requires = oscap_string_iterator_next(strings);
-			xmlNewTextChild(rule_node, ns_xccdf, BAD_CAST "requires", BAD_CAST requires);
+			xmlNode * child = xmlNewTextChild(rule_node, ns_xccdf, BAD_CAST "requires", BAD_CAST NULL);
+                        xmlNewProp(child, BAD_CAST "idref", BAD_CAST requires);
 		}
 		oscap_string_iterator_free(strings);
 	}
@@ -860,7 +919,8 @@ void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc,
 	struct oscap_string_iterator *conflicts = xccdf_rule_get_conflicts(rule);
 	while (oscap_string_iterator_has_more(conflicts)) {
 		const char *conflict = oscap_string_iterator_next(conflicts);
-		xmlNewTextChild(rule_node, ns_xccdf, BAD_CAST "conflicts", BAD_CAST conflict);
+		xmlNode * child = xmlNewTextChild(rule_node, ns_xccdf, BAD_CAST "conflicts", BAD_CAST NULL);
+                xmlNewProp(child, BAD_CAST "idref", BAD_CAST conflict);
 	}
 	oscap_string_iterator_free(conflicts);
 
@@ -931,7 +991,8 @@ void xccdf_group_to_dom(struct xccdf_group *group, xmlNode *group_node, xmlDoc *
 	struct oscap_string_iterator *platforms = xccdf_group_get_platforms(group);
 	while (oscap_string_iterator_has_more(platforms)) {
 		const char *platform = oscap_string_iterator_next(platforms);
-		xmlNewTextChild(group_node, ns_xccdf, BAD_CAST "platform", BAD_CAST platform);
+		xmlNode * child = xmlNewTextChild(group_node, ns_xccdf, BAD_CAST "platform", BAD_CAST NULL);
+                xmlNewProp(child, BAD_CAST "idref", BAD_CAST platform);
 	}
 	oscap_string_iterator_free(platforms);
 
@@ -941,7 +1002,8 @@ void xccdf_group_to_dom(struct xccdf_group *group, xmlNode *group_node, xmlDoc *
 		struct oscap_string_iterator *strings = oscap_stringlist_get_strings(list);
 		while (oscap_string_iterator_has_more(strings)) {
 			const char *requires = oscap_string_iterator_next(strings);
-			xmlNewTextChild(group_node, ns_xccdf, BAD_CAST "requires", BAD_CAST requires);
+			xmlNode * child = xmlNewTextChild(group_node, ns_xccdf, BAD_CAST "requires", BAD_CAST NULL);
+                        xmlNewProp(child, BAD_CAST "idref", BAD_CAST requires);
 		}
 		oscap_string_iterator_free(strings);
 	}
@@ -950,7 +1012,8 @@ void xccdf_group_to_dom(struct xccdf_group *group, xmlNode *group_node, xmlDoc *
 	struct oscap_string_iterator *conflicts = xccdf_group_get_conflicts(group);
 	while (oscap_string_iterator_has_more(conflicts)) {
 		const char *conflict = oscap_string_iterator_next(conflicts);
-		xmlNewTextChild(group_node, ns_xccdf, BAD_CAST "conflicts", BAD_CAST conflict);
+		xmlNode * child = xmlNewTextChild(group_node, ns_xccdf, BAD_CAST "conflicts", BAD_CAST NULL);
+                xmlNewProp(child, BAD_CAST "idref", BAD_CAST conflict);
 	}
 	oscap_string_iterator_free(conflicts);
 
@@ -1026,7 +1089,8 @@ OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fixtext, disruption)
 OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fixtext, complexity)
 OSCAP_ACCESSOR_SIMPLE(bool, xccdf_fixtext, reboot)
 OSCAP_ACCESSOR_TEXT(xccdf_fixtext, text)
-OSCAP_ACCESSOR_STRING(xccdf_fixtext, fixref) XCCDF_ITERATOR_GEN_S(fixtext)
+OSCAP_ACCESSOR_STRING(xccdf_fixtext, fixref)
+XCCDF_ITERATOR_GEN_S(fixtext)
 
 OSCAP_ACCESSOR_SIMPLE(xccdf_strategy_t, xccdf_fix, strategy)
 OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fix, disruption)
