@@ -20,13 +20,18 @@
  *      Lukas Kuklinek <lkuklinek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <math.h>
+#include <string.h>
+
 #include "item.h"
 #include "elements.h"
 #include "helpers.h"
 #include "xccdf_impl.h"
-#include "../common/elements.h"
-#include <math.h>
-#include <string.h>
+#include "common/elements.h"
 
 static struct xccdf_value_instance *xccdf_value_instance_new(xccdf_value_type_t type);
 
@@ -126,11 +131,13 @@ struct xccdf_item *xccdf_value_parse(xmlTextReaderPtr reader, struct xccdf_item
 		case XCCDFE_CHOICES:
 			val->flags.must_match = xccdf_attribute_get_bool(reader, XCCDFA_MUSTMATCH);
 			val->flags.must_match_given = true;
+                        xmlTextReaderRead(reader); /* Move to the next node (subnode of <choices>)*/
 			while (oscap_to_start_element(reader, depth + 1)) {
 				if (xccdf_element_get(reader) == XCCDFE_CHOICE)
 					oscap_list_add(val->choices, oscap_element_string_copy(reader));
 				xmlTextReaderRead(reader);
 			}
+                        break;
 		default:
 			xccdf_item_process_element(value, reader);
 		}
@@ -195,7 +202,7 @@ void xccdf_value_to_dom(struct xccdf_value *value, xmlNode *value_node, xmlDoc *
 			inst->type == XCCDF_TYPE_STRING && inst->match != NULL, ns_xccdf, "match", inst->selector);
         if (inst->choices->first) {
             xmlNode *choices = xmlNewNode(ns_xccdf, BAD_CAST "choices");
-            if (inst->flags.must_match_given)
+            if ((inst->flags.must_match_given) && (inst->flags.must_match))
                 xmlNewProp(choices, BAD_CAST "mustMatch", BAD_CAST oscap_enum_to_string(OSCAP_BOOL_MAP, inst->flags.must_match));
             if ((inst->selector)&&(strlen(inst->selector) > 0)) xmlNewProp(choices, BAD_CAST "selector", BAD_CAST inst->selector);
             OSCAP_FOR_STR(ch, xccdf_value_instance_get_choices(inst))
@@ -231,11 +238,11 @@ void xccdf_value_free(struct xccdf_item *val)
     }
 }
 
-bool xccdf_value_set_oper(struct xccdf_item * value, xccdf_operator_t oper)
+bool xccdf_value_set_oper(struct xccdf_value * value, xccdf_operator_t oper)
 {
         __attribute__nonnull__(value);
 
-        value->sub.value.oper = oper;
+        xccdf_value_to_item(value)->sub.value.oper = oper;
         return true;
 
 }
@@ -340,17 +347,16 @@ struct xccdf_value_instance *xccdf_value_new_instance(struct xccdf_value *val)
 #define XCCDF_VALUE_INSTANCE_VALUE_ACCESSOR_STR(WHAT) \
 	const char *xccdf_value_instance_get_##WHAT##_string(const struct xccdf_value_instance *inst) { return inst->WHAT; } \
 	bool xccdf_value_instance_set_##WHAT##_string(struct xccdf_value_instance *inst, const char *newval) { \
-		oscap_free(inst->WHAT); inst->WHAT = oscap_strdup(newval); return true; }
+		oscap_free(inst->WHAT); inst->WHAT = oscap_strdup(newval); inst->flags.WHAT##_given = true; return true; }
 #define XCCDF_VALUE_INSTANCE_VALUE_ACCESSOR_NUM(WHAT) \
-	xccdf_numeric xccdf_value_instance_get_##WHAT##_number(const struct xccdf_value_instance *inst) { return strtol(inst->WHAT, NULL, 10); } \
+	xccdf_numeric xccdf_value_instance_get_##WHAT##_number(const struct xccdf_value_instance *inst) { return oscap_strtol(inst->WHAT, NULL, 10); } \
 	bool xccdf_value_instance_set_##WHAT##_number(struct xccdf_value_instance *inst, xccdf_numeric newval) { \
-		oscap_free(inst->WHAT); inst->WHAT = oscap_sprintf("%f", newval); return true; }
+		oscap_free(inst->WHAT); inst->WHAT = oscap_sprintf("%f", newval); inst->flags.WHAT##_given = true; return true; }
 #define XCCDF_VALUE_INSTANCE_VALUE_ACCESSOR_BOOL(WHAT) \
 	bool xccdf_value_instance_get_##WHAT##_boolean(const struct xccdf_value_instance *inst) \
         { return oscap_string_to_enum(OSCAP_BOOL_MAP, inst->WHAT); } \
 	bool xccdf_value_instance_set_##WHAT##_boolean(struct xccdf_value_instance *inst, bool newval) \
-		{ oscap_free(inst->WHAT); inst->WHAT = oscap_strdup(oscap_enum_to_string(OSCAP_BOOL_MAP, newval)); return true; }
-
+		{ oscap_free(inst->WHAT); inst->WHAT = oscap_strdup(oscap_enum_to_string(OSCAP_BOOL_MAP, newval)); inst->flags.WHAT##_given = true; return true; }
 
 OSCAP_ACCESSOR_STRING(xccdf_value_instance, selector)
 OSCAP_GETTER(xccdf_value_type_t, xccdf_value_instance, type)
