@@ -23,19 +23,23 @@
  *              Maros Barabas <mbarabas@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <math.h> /* For NAN <- TODO */
 #include <time.h> /* For timestamps in rule results and TestResult */
 
 #include "public/xccdf_policy.h"
-#include "../XCCDF/public/xccdf.h"
+#include "XCCDF/public/xccdf.h"
 
-#include "../common/list.h"
-#include "../_error.h"
-#include "../common/public/text.h"
-#include "../common/debug_priv.h"
-#include "../common/reporter_priv.h"
+#include "common/list.h"
+#include "common/_error.h"
+#include "common/public/text.h"
+#include "common/debug_priv.h"
+#include "common/reporter_priv.h"
 
 /**
  * Typedef of callback structure with system identificator, callback function and usr data (optional) 
@@ -107,7 +111,6 @@ OSCAP_GETTER(struct xccdf_profile *, xccdf_policy, profile)
 OSCAP_IGETINS(xccdf_select, xccdf_policy, selects, select)
 OSCAP_IGETINS_GEN(xccdf_value_binding, xccdf_policy, values, value)
 OSCAP_IGETINS(xccdf_result, xccdf_policy, results, result)
-OSCAP_ITERATOR_RESET(xccdf_policy)
 
 /**
  * XCCDF value binding structure is binding between Refine values, Set values, 
@@ -131,7 +134,6 @@ OSCAP_GETTER(char *, xccdf_value_binding, name)
 OSCAP_GETTER(char *, xccdf_value_binding, value)
 OSCAP_GETTER(char *, xccdf_value_binding, setvalue)
 OSCAP_GETTER(xccdf_operator_t, xccdf_value_binding, operator)
-OSCAP_ITERATOR_RESET(xccdf_value_binding)
 
 /**
  * XCCDF Default score structure represents Default XCCDF Score model
@@ -167,39 +169,39 @@ typedef struct xccdf_flat_score {
  * Get callback from callback structure when system is selected 
  * Parameter policy handle callback structure and sysname the name of system specified in XCCDF file
  */
-static callback * xccdf_policy_get_callback(struct xccdf_policy * policy, const char * sysname) {
+/*static callback * xccdf_policy_get_callback(struct xccdf_policy * policy, const char * sysname) {*/
 
-        struct oscap_list * cbs = policy->model->callbacks;
+/*struct oscap_list * cbs = policy->model->callbacks;*/
 
-        struct oscap_iterator * callback_it = oscap_iterator_new(cbs);
-        while (oscap_iterator_has_more(callback_it)) {
-            callback * cb = oscap_iterator_next(callback_it);
+/*struct oscap_iterator * callback_it = oscap_iterator_new(cbs);*/
+/*while (oscap_iterator_has_more(callback_it)) {*/
+/*callback * cb = oscap_iterator_next(callback_it);*/
 
             /* If system string of callback is NULL and sysname 
              * is NULL we can't call strcmp, because it will raise 
              * SIGSEG so we want to avoid such behavior
              */
-            if ((cb->system == NULL) && (sysname == NULL)) {
-                oscap_iterator_free(callback_it);
-                return cb;
-            } else if ((cb->system == NULL) || (sysname == NULL)) continue;
+/*if ((cb->system == NULL) && (sysname == NULL)) {*/
+/*oscap_iterator_free(callback_it);*/
+/*return cb;*/
+/*} else if ((cb->system == NULL) || (sysname == NULL)) continue;*/
 
             /* If both structures have system strings, compare 
              * them to equality
              */
-            else if (!strcmp(cb->system, sysname)) {
-                oscap_iterator_free(callback_it);
-                return cb;
-            }
+/*else if (!strcmp(cb->system, sysname)) {*/
+/*oscap_iterator_free(callback_it);*/
+/*return cb;*/
+/*}*/
 
-        }
-        oscap_iterator_free(callback_it);
+/*}*/
+/*oscap_iterator_free(callback_it);*/
         
         /* There are no callbacks
          */
-        return NULL;
+/*return NULL;*/
         
-}
+/*}*/
 
 /**
  * Filter function returning true if the item is selected, false otherwise
@@ -239,34 +241,34 @@ static bool xccdf_policy_filter_select(void *item, void *selectid)
  * false otherwise.
  * This function is only called from iterator constructor
  */
-static bool xccdf_policy_filter_callback(void *cb, void *sysname)
-{
+/*static bool xccdf_policy_filter_callback(void *cb, void *sysname)*/
+/*{*/
         /* If system string of callback is NULL and sysname 
          * is NULL we can't call strcmp, because it will raise 
          * SIGSEG so we want to avoid such behavior
          */
-        if ((((callback *)cb)->system == NULL) && ( (const char *) sysname == NULL))
-            return true;
-        else if ((((callback *)cb)->system == NULL) || ((const char *) sysname == NULL))
-            return false;
+/*if ((((callback *)cb)->system == NULL) && ( (const char *) sysname == NULL))*/
+/*return true;*/
+/*else if ((((callback *)cb)->system == NULL) || ((const char *) sysname == NULL))*/
+/*return false;*/
 
         /* If both structures have system strings, compare 
          * them to equality
          */
-        else if (!strcmp(((callback *)cb)->system, (const char *) sysname)) {
-            return true;
-        } else return false;
-}
+/*else if (!strcmp(((callback *)cb)->system, (const char *) sysname)) {*/
+/*return true;*/
+/*} else return false;*/
+/*}*/
 
 /**
  * Get callbacks that match sysname. Call this function to get iterator over list of callbacks
  * that have the same system name
  */
-static struct oscap_iterator * xccdf_policy_get_callbacks_by_sysname(struct xccdf_policy * policy, const char * sysname) {
+/*static struct oscap_iterator * xccdf_policy_get_callbacks_by_sysname(struct xccdf_policy * policy, const char * sysname) {
 
     return oscap_iterator_new_filter( policy->model->callbacks, (oscap_filter_func) xccdf_policy_filter_callback,
             (void *) sysname);
-}
+}*/
 
 /**
  * Return true if given policy has the select, false otherwise.
@@ -536,7 +538,7 @@ xccdf_policy_evaluate_cb(struct xccdf_policy * policy, const char * sysname, con
             oscap_iterator_free(cb_it);
             return XCCDF_RESULT_NOT_CHECKED;
         }
-        if (strcmp(cb->system, sysname))
+        if (oscap_strcmp(cb->system, sysname))
             continue;
 
         struct xccdf_value_binding_iterator * binding_it = (struct xccdf_value_binding_iterator *) oscap_iterator_new(bindings);
@@ -562,7 +564,7 @@ static int xccdf_policy_report_cb(struct xccdf_policy * policy, const char * sys
         /* Check if the callback match sysname, 
          * continue otherwise
          */
-        if (strcmp(cb->system, sysname))
+        if (oscap_strcmp(cb->system, sysname))
             continue;
 
         /* Report by oscap_reporter_message
@@ -615,7 +617,7 @@ static struct oscap_list * xccdf_policy_check_get_value_bindings(struct xccdf_po
 
             /* Apply related setvalue from policy profile */
             s_value = xccdf_policy_get_setvalue(policy, xccdf_value_get_id(value));
-            if (s_value != NULL) binding->setvalue = (char *) xccdf_setvalue_get_value(s_value);
+            if (s_value != NULL) binding->setvalue = oscap_strdup((char *) xccdf_setvalue_get_value(s_value));
 
             /* Apply related refine value from policy profile */
             const char * selector = NULL;
@@ -1026,7 +1028,7 @@ static struct oscap_stringlist * xccdf_check_get_files(struct xccdf_check * chec
     return names;
 }
 
-static struct oscap_stringlist * xccdf_item_get_files(struct xccdf_item * item)
+struct oscap_stringlist * xccdf_item_get_files(struct xccdf_item * item)
 {
 
     struct xccdf_item_iterator  * child_it;
@@ -1261,7 +1263,8 @@ struct xccdf_policy * xccdf_policy_new(struct xccdf_policy_model * model, struct
         policy->model = model;
 
         /* Create selects from benchmark model */
-        sel_it = xccdf_profile_get_selects(profile);
+        if (profile != NULL)
+            sel_it = xccdf_profile_get_selects(profile);
         /* Iterate through selects in profile */
         while (xccdf_select_iterator_has_more(sel_it)) {
 
@@ -1285,7 +1288,10 @@ struct xccdf_policy * xccdf_policy_new(struct xccdf_policy_model * model, struct
          * hash table
          */
         sel_it = xccdf_policy_get_selects(policy);
+        const int HTABLE_MIN_SIZE = 256;
         int s_count = oscap_iterator_get_itemcount((struct oscap_iterator *) sel_it);
+        if (s_count < HTABLE_MIN_SIZE)
+            s_count = HTABLE_MIN_SIZE;
         policy->ht_selects = oscap_htable_new1((oscap_compare_func) strcmp, s_count);
         if (policy->ht_selects == NULL) {
 	    oscap_dlprintf(DBG_E, "Can't make hash table for policy selects\n");
@@ -1333,13 +1339,14 @@ struct xccdf_select * xccdf_policy_get_select_by_id(struct xccdf_policy * policy
         __attribute__nonnull__(policy);
         
         struct xccdf_select_iterator    * sel_it;
-        struct xccdf_select             * sel;
+        struct xccdf_select             * sel = NULL;
 
         if (policy->ht_selects != NULL) {
             /* We have hash table -> faster
              */
-            return oscap_htable_get(policy->ht_selects, item_id);
-        } else {
+            sel = oscap_htable_get(policy->ht_selects, item_id);
+        }
+        if (sel == NULL) {
             /* We don't have hash table :( -> do it old way
              */
             sel_it = xccdf_policy_get_selects(policy);
@@ -1351,7 +1358,7 @@ struct xccdf_select * xccdf_policy_get_select_by_id(struct xccdf_policy * policy
                     }
             }
             xccdf_select_iterator_free(sel_it);
-        }
+        } else return sel;
 
         return NULL;
 }
@@ -1672,6 +1679,8 @@ struct xccdf_item * xccdf_policy_tailor_item(struct xccdf_policy * policy, struc
                 xccdf_rule_set_role((struct xccdf_rule *) new_item, xccdf_refine_rule_get_role(r_rule));
             if (xccdf_refine_rule_get_severity(r_rule) != NAN)
                 xccdf_rule_set_severity((struct xccdf_rule *) new_item, xccdf_refine_rule_get_severity(r_rule));
+            if (xccdf_refine_rule_get_weight(r_rule) != -1.0)
+                xccdf_rule_set_weight((struct xccdf_rule *) new_item, xccdf_refine_rule_get_weight(r_rule));
             break;
         }
         case XCCDF_GROUP: {
@@ -1699,14 +1708,14 @@ struct xccdf_item * xccdf_policy_tailor_item(struct xccdf_policy * policy, struc
             struct xccdf_value_instance_iterator * instance_it = xccdf_value_get_instances((struct xccdf_value *) new_item);
             while (xccdf_value_instance_iterator_has_more(instance_it)) {
                 instance = xccdf_value_instance_iterator_next(instance_it);
-                if (!strcmp(xccdf_value_instance_get_value(instance), value))
+                if (!oscap_strcmp(xccdf_value_instance_get_value(instance), value))
                     selector = xccdf_value_instance_get_selector(instance);
             }
             xccdf_value_instance_iterator_free(instance_it);
             instance_it = xccdf_value_get_instances((struct xccdf_value *) new_item);
             while (xccdf_value_instance_iterator_has_more(instance_it)) {
                 instance = xccdf_value_instance_iterator_next(instance_it);
-                if (strcmp(xccdf_value_instance_get_selector(instance), selector))
+                if (oscap_strcmp(xccdf_value_instance_get_selector(instance), selector))
                     xccdf_value_instance_iterator_remove(instance_it);
             }
             xccdf_value_instance_iterator_free(instance_it);
@@ -1717,7 +1726,7 @@ struct xccdf_item * xccdf_policy_tailor_item(struct xccdf_policy * policy, struc
 
             int oper = xccdf_policy_get_refine_value_oper(policy, item);
             if (oper == -1) break;
-            xccdf_value_set_oper(item, (xccdf_operator_t) oper);
+            xccdf_value_set_oper((struct xccdf_value *) item, (xccdf_operator_t) oper);
             break;
         }
         default:
@@ -1731,21 +1740,6 @@ struct oscap_stringlist * xccdf_policy_model_get_files(struct xccdf_policy_model
     return xccdf_item_get_files((struct xccdf_item *) xccdf_policy_model_get_benchmark(policy_model));
 }
 
-
-
-void xccdf_policy_export_variables(struct xccdf_policy *policy, char *export_namespace, const char *file) {
-}
-
-void xccdf_policy_export_controls (struct xccdf_policy *policy, char *export_namespace, const char *file) {
-}
-
-void xccdf_policy_import_results(struct xccdf_policy *policy, char *import_namespace, const char *file) {
-}
-
-void xccdf_policy_export_results(struct xccdf_policy *policy, char *scoring_model_namespace, const char *file) {
-}
-
-
 void xccdf_policy_model_free(struct xccdf_policy_model * model) {
 
 	oscap_list_free(model->policies, (oscap_destruct_func) xccdf_policy_free);
@@ -1777,3 +1771,51 @@ void xccdf_value_binding_free(struct xccdf_value_binding * binding) {
         oscap_free(binding->setvalue);
         oscap_free(binding);
 }
+
+// textual substitiution implementation
+
+static char* xccdf_subst_callback(xccdf_subst_type_t type, const char *id, void *arg) {
+
+    if (id == NULL || arg == NULL)
+        return NULL;
+
+    struct xccdf_policy *policy = arg;
+    struct xccdf_policy_model *model = xccdf_policy_get_model(policy);
+    if (model == NULL) return NULL;
+    struct xccdf_benchmark *bench = xccdf_policy_model_get_benchmark(model);
+    if (bench == NULL) return NULL;
+
+    switch (type) {
+        case XCCDF_SUBST_SUB: {
+
+            // try substitute a value form <cdf:plain_text> element
+            const char *subst = xccdf_benchmark_get_plain_text(bench, id);
+            if (subst) return oscap_strdup(subst);
+
+            // try substitute a Value value
+            struct xccdf_item *value = xccdf_benchmark_get_item(bench, id);
+            if (value && xccdf_item_get_type(value) == XCCDF_VALUE) {
+                char *ret = oscap_strdup("");
+                struct xccdf_value *value_tailored = xccdf_item_to_value(xccdf_policy_tailor_item(policy, value));
+                struct xccdf_value_instance_iterator *it = xccdf_value_get_instances(value_tailored);
+                if (xccdf_value_instance_iterator_has_more(it)) {
+                    struct xccdf_value_instance *inst = xccdf_value_instance_iterator_next(it);
+                    oscap_free(ret);
+                    ret = oscap_strdup(xccdf_value_instance_get_value(inst));
+                }
+                xccdf_value_instance_iterator_free(it);
+                xccdf_item_free(xccdf_value_to_item(value_tailored));
+                return ret;
+            }
+            break;
+        }
+        default: return NULL; // TODO implement other substitution types
+    }
+
+    return NULL;
+}
+
+char* xccdf_policy_substitute(const char *text, struct xccdf_policy *policy) {
+    return oscap_text_xccdf_substitute(text, xccdf_subst_callback, policy);
+}
+
