@@ -21,10 +21,15 @@
  *       Lukas Kuklinek <lkuklinek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stddef.h>
 #include <unistd.h>
 #include <stdint.h>
 #include <assert.h>
+
 #include "bfind.h"
 
 void *oscap_bfind(void *ptr, size_t nmemb, size_t size, void *key, int cmpfn(void *, void *))
