@@ -24,13 +24,16 @@
  *      Maros Barabas <mbarabas@redhat.com>
  */
 
-#include "elements.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <string.h>
 
+#include "public/oscap.h"
 #include "util.h"
 #include "list.h"
-#include "public/oscap.h"
+#include "elements.h"
 
 
 const struct oscap_string_map OSCAP_BOOL_MAP[] = {
@@ -60,11 +63,18 @@ bool oscap_to_start_element(xmlTextReaderPtr reader, int depth)
 
 char *oscap_element_string_copy(xmlTextReaderPtr reader)
 {
-	if (xmlTextReaderNodeType(reader) == 1 || xmlTextReaderNodeType(reader) == 2)
+	int t;
+
+	if (xmlTextReaderIsEmptyElement(reader))
+		return NULL;
+
+	t = xmlTextReaderNodeType(reader);
+	if (t == XML_ELEMENT_NODE || t == XML_ATTRIBUTE_NODE)
 		xmlTextReaderRead(reader);
 	if (xmlTextReaderHasValue(reader))
 		return (char *)xmlTextReaderValue(reader);
-	return NULL;
+	else
+		return (char *) calloc(1,1);
 }
 
 const char *oscap_element_string_get(xmlTextReaderPtr reader)
@@ -126,8 +136,9 @@ time_t oscap_get_datetime(const char *date)
 
 xmlNode *oscap_xmlstr_to_dom(xmlNode *parent, const char *elname, const char *content)
 {
-	char *str = oscap_sprintf("<x>%s</x>", content);
-	xmlDoc *doc = xmlReadMemory(str, strlen(str), NULL, NULL, XML_PARSE_RECOVER | XML_PARSE_NOERROR | XML_PARSE_NOWARNING | XML_PARSE_NONET);
+	char *str = oscap_sprintf("<x xmlns:xhtml='http://www.w3.org/1999/xhtml'>%s</x>", content);
+	xmlDoc *doc = xmlReadMemory(str, strlen(str), NULL, NULL,
+		XML_PARSE_RECOVER | XML_PARSE_NOERROR | XML_PARSE_NOWARNING | XML_PARSE_NONET | XML_PARSE_NSCLEAN);
 	xmlNode *text_node = xmlCopyNode(xmlDocGetRootElement(doc), 1);
 	xmlNodeSetName(text_node, BAD_CAST elname);
 	xmlAddChild(parent, text_node);
