@@ -20,14 +20,11 @@
  *      Lukas Kuklinek <lkuklinek@redhat.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libxml/parser.h>
-#include "public/oscap.h"
-#include "_error.h"
-#include "util.h"
-#include "list.h"
-#include "elements.h"
-#include "reporter_priv.h"
-#include "config.h"
 #include <libxml/xmlschemas.h>
 #include <libxslt/xslt.h>
 #include <libxslt/xsltInternals.h>
@@ -35,10 +32,25 @@
 #include <libxslt/xsltutils.h>
 #include <libexslt/exslt.h>
 #include <string.h>
-
-#include <config.h>
-#if HAVE_UNISTD_H
 #include <unistd.h>
+
+#include "public/oscap.h"
+#include "_error.h"
+#include "util.h"
+#include "list.h"
+#include "elements.h"
+#include "reporter_priv.h"
+
+#ifndef OSCAP_DEFAULT_SCHEMA_PATH
+const char * const OSCAP_SCHEMA_PATH = "/usr/local/share/openscap/schemas";
+#else
+const char * const OSCAP_SCHEMA_PATH = OSCAP_DEFAULT_SCHEMA_PATH;
+#endif
+
+#ifndef OSCAP_DEFAULT_XSLT_PATH
+const char * const OSCAP_XSLT_PATH = "/usr/local/share/openscap/xsl";
+#else
+const char * const OSCAP_XSLT_PATH = OSCAP_DEFAULT_XSLT_PATH;
 #endif
 
 void oscap_init(void)
@@ -58,20 +70,13 @@ void oscap_cleanup(void)
 const char *oscap_get_version(void) { return VERSION; }
 
 #ifdef WIN32
-const char *OSCAP_OS_PATH_DELIM  = "\\";
+const char * const OSCAP_OS_PATH_DELIM  = "\\";
 #else
-const char *OSCAP_OS_PATH_DELIM  = "/";
+const char * const OSCAP_OS_PATH_DELIM  = "/";
 #endif
 
 const char *OSCAP_PATH_SEPARATOR = ":";
 
-#ifndef OSCAP_DEFAULT_SCHEMA_PATH
-#define OSCAP_DEFAULT_SCHEMA_PATH "/usr/local/share/openscap/schemas"
-#endif
-#ifndef OSCAP_DEFAULT_XSLT_PATH
-#define OSCAP_DEFAULT_XSLT_PATH "/usr/local/share/openscap/xsl"
-#endif
-
 bool oscap_file_exists(const char *path, int mode)
 {
 	if (path == NULL) return false;
@@ -103,6 +108,9 @@ char *oscap_find_file(const char *filename, int mode, const char *pathvar, const
 
 	while (*paths) {
 		if (oscap_streq(*paths, "")) continue;
+		oscap_rtrim(*paths, '/');  // strip slases at the end of the path
+		if (oscap_streq(*paths, "")) **paths = '/';
+
 		char *curpath = oscap_sprintf("%s%s%s", *paths, OSCAP_OS_PATH_DELIM, filename);
 		if (oscap_file_exists(curpath, mode)) {
 			ret = curpath;
@@ -119,7 +127,7 @@ char *oscap_find_file(const char *filename, int mode, const char *pathvar, const
 
 static char *oscap_get_schema_path(const char *filename)
 {
-	return oscap_find_file(filename, R_OK, "OSCAP_SCHEMA_PATH", OSCAP_DEFAULT_SCHEMA_PATH);
+	return oscap_find_file(filename, R_OK, "OSCAP_SCHEMA_PATH", OSCAP_SCHEMA_PATH);
 }
 
 static void oscap_xml_validity_handler(void *user, xmlErrorPtr error)
@@ -190,9 +198,10 @@ struct oscap_schema_table_entry {
 
 // TODO do not hardcode versions... (duplicities)
 struct oscap_schema_table_entry OSCAP_SCHEMAS_TABLE[] = {
-	{ OSCAP_DOCUMENT_OVAL_DEFINITIONS, "oval",  "5.5",   "oval-definitions-schema.xsd"            },
-	{ OSCAP_DOCUMENT_OVAL_RESULTS,     "oval",  "5.5",   "oval-results-schema.xsd"                },
-	{ OSCAP_DOCUMENT_OVAL_SYSCHAR,     "oval",  "5.5",   "oval-system-characteristics-schema.xsd" },
+	{ OSCAP_DOCUMENT_OVAL_DEFINITIONS, "oval",  "5.8",   "oval-definitions-schema.xsd"            },
+	{ OSCAP_DOCUMENT_OVAL_VARIABLES,   "oval",  "5.8",   "oval-variables-schema.xsd"              },
+	{ OSCAP_DOCUMENT_OVAL_RESULTS,     "oval",  "5.8",   "oval-results-schema.xsd"                },
+	{ OSCAP_DOCUMENT_OVAL_SYSCHAR,     "oval",  "5.8",   "oval-system-characteristics-schema.xsd" },
 	{ OSCAP_DOCUMENT_XCCDF,            "xccdf", "1.1.4", "xccdf-schema.xsd"                       },
 	{ 0, NULL, NULL, NULL }
 };
@@ -222,10 +231,10 @@ bool oscap_validate_document(const char *xmlfile, oscap_document_type_t doctype,
 	return ret;
 }
 
-bool oscap_apply_xslt(const char *xmlfile, const char *xsltfile, const char *outfile, const char **params)
+bool oscap_apply_xslt_var(const char *xmlfile, const char *xsltfile, const char *outfile, const char **params, const char *pathvar, const char *defpath)
 {
     bool ret = false;
-    char *xsltpath = oscap_find_file(xsltfile, R_OK, "OSCAP_XSLT_PATH", OSCAP_DEFAULT_XSLT_PATH);
+    char *xsltpath = oscap_find_file(xsltfile, R_OK, pathvar, defpath);
     xsltStylesheetPtr cur = NULL;
     xmlDocPtr doc = NULL, res = NULL;
     FILE *f = NULL;
@@ -233,7 +242,7 @@ bool oscap_apply_xslt(const char *xmlfile, const char *xsltfile, const char *out
     char *args[argc+1]; memset(args, 0, sizeof(char*) * (argc + 1));
 
     if (xsltpath == NULL) {
-        oscap_seterr(OSCAP_EFAMILY_OSCAP, 0, "XSLT file to by used by the transformation was not found.");
+        oscap_seterr(OSCAP_EFAMILY_OSCAP, 0, "XSLT file to be used by the transformation was not found.");
         goto cleanup;
     }
 
@@ -285,3 +294,8 @@ cleanup:
     return ret;
 }
 
+bool oscap_apply_xslt(const char *xmlfile, const char *xsltfile, const char *outfile, const char **params)
+{
+	return oscap_apply_xslt_var(xmlfile, xsltfile, outfile, params, "OSCAP_XSLT_PATH", OSCAP_XSLT_PATH);
+}
+
