@@ -130,6 +130,15 @@
 
 /** @} */
 
+/// OS-specific filesystem path delimiter
+extern const char * const OSCAP_OS_PATH_DELIM;
+
+/// Default XML Schema path (if not overridden by the environment variable)
+extern const char * const OSCAP_SCHEMA_PATH;
+
+/// Default XSLT path (if not overridden by the environment variable)
+extern const char * const OSCAP_XSLT_PATH;
+
 
 /**
  * Initialize OpenSCAP library.
@@ -164,6 +173,7 @@ const char *oscap_get_version(void);
 /// SCAP document type identifiers
 typedef enum oscap_document_type {
 	OSCAP_DOCUMENT_OVAL_DEFINITIONS = 1,  ///< OVAL Definitions file
+	OSCAP_DOCUMENT_OVAL_VARIABLES,        ///< OVAL Variables
 	OSCAP_DOCUMENT_OVAL_SYSCHAR,          ///< OVAL system characteristics file
 	OSCAP_DOCUMENT_OVAL_RESULTS,          ///< OVAL results file
 	OSCAP_DOCUMENT_XCCDF,                 ///< XCCDF benchmark file
@@ -206,6 +216,21 @@ bool oscap_validate_document(const char *xmlfile, oscap_document_type_t doctype,
  */
 bool oscap_apply_xslt(const char *xmlfile, const char *xsltfile, const char *outfile, const char **params);
 
+/**
+ * Apply XSLT stylesheet to a XML file.
+ *
+ * This function lets user specify environment variable with
+ * a XSL stylesheet search path(s) and a fallback path if the variable is not defined.
+ * Except for this it is completely identical to oscap_apply_xslt().
+ *
+ * @param xmlfile File to be transformed.
+ * @param xsltfile XSLT filename
+ * @param outfile Result file shall be written here (NULL for stdout).
+ * @param params list of key-value pairs to pass to the stylesheet.
+ * @return Success or failure.
+ */
+bool oscap_apply_xslt_var(const char *xmlfile, const char *xsltfile, const char *outfile, const char **params, const char *pathvar, const char *defpath);
+
 /************************************************************/
 /** @} validation group end */
 
