@@ -153,7 +153,7 @@ bool oscap_text_get_overrides(const struct oscap_text *text);
  * Set whether this text overrides parent content.
  * @memberof oscap_text
  */
-//bool oscap_text_set_overrides(struct oscap_text *text, bool overrides);
+bool oscap_text_set_overrides(struct oscap_text *text, bool overrides);
 
 /// @memberof oscap_text
 bool oscap_text_set_text(struct oscap_text *text, const char * string);
@@ -190,6 +190,8 @@ bool oscap_text_iterator_has_more(struct oscap_text_iterator *it);
 void oscap_text_iterator_free(struct oscap_text_iterator *it);
 /// @memberof oscap_text_iterator
 void oscap_text_iterator_remove(struct oscap_text_iterator *it);
+/// @memberof oscap_text_iterator
+void oscap_text_iterator_reset(struct oscap_text_iterator *it);
 
 /**
  * @struct oscap_string_iterator
@@ -207,6 +209,8 @@ bool oscap_string_iterator_has_more(struct oscap_string_iterator *it);
 void oscap_string_iterator_free(struct oscap_string_iterator *it);
 /// @memberof oscap_string_iterator
 void oscap_string_iterator_remove(struct oscap_string_iterator *it);
+/// @memberof oscap_string_iterator
+void oscap_string_iterator_reset(struct oscap_string_iterator *it);
 
 /**
  * @struct oscap_stringlist_iterator
@@ -222,7 +226,8 @@ bool oscap_stringlist_iterator_has_more(struct oscap_stringlist_iterator *it);
 void oscap_stringlist_iterator_free(struct oscap_stringlist_iterator *it);
 /// @memberof oscap_stringlist_iterator
 void oscap_stringlist_iterator_remove(struct oscap_stringlist_iterator *it);
-
+/// @memberof oscap_stringlist_iterator
+void oscap_stringlist_iterator_reset(struct oscap_stringlist_iterator *it);
 
 /************************************************************/
 /** @} End of Iterators group */
