@@ -21,13 +21,19 @@
  *      Lukas Kuklinek <lkuklinek@redhat.com>
  */
 
-#include "util.h"
-#include "_error.h"
-#include "oscap.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <ctype.h>
 #include <limits.h>
 #include <stdarg.h>
+#include <math.h>
+
+#include "util.h"
+#include "_error.h"
+#include "oscap.h"
 
 
 int oscap_string_to_enum(const struct oscap_string_map *map, const char *str)
@@ -58,25 +64,20 @@ char *oscap_strdup(const char *str)
 
 	m = strdup(str);
 
-	if ((m == NULL) && (strlen(m) > 0))
+	if (m == NULL)
 		oscap_seterr(OSCAP_EFAMILY_GLIBC, errno, "Memory allocation failed.");
 
 	return m;
 }
 
-char *oscap_strsep(char **str, const char *delim)
-{
-	if (str == NULL || *str == NULL)
-		return NULL;
-	char *ret = *str;
-	*str = strstr(*str, delim);
-	if (*str) {
-		**str = '\0';
-		(*str)++;
-	}
-	return ret;
+float oscap_strtol(const char *str, char **endptr, int base){
+    if (str == NULL) {
+        return NAN;
+    } else
+        return strtol(str, endptr, base);
 }
 
+
 static const size_t CPE_SPLIT_INIT_ALLOC = 8;
 
 char **oscap_split(char *str, const char *delim)
@@ -101,7 +102,7 @@ char **oscap_split(char *str, const char *delim)
 				return NULL;
 			}
 		}
-		fields[i++] = oscap_strsep(stringp, delim);
+		fields[i++] = strsep(stringp, delim);
 	}
 	fields[i] = NULL;
 
@@ -168,3 +169,13 @@ char *oscap_sprintf(const char *fmt, ...)
     return ret;
 }
 
+char *oscap_rtrim(char *str, char ch)
+{
+	if (str == NULL) return NULL;
+	for (char *pos = str + strlen(str) - 1; pos >= str; --pos) {
+		if (*pos == ch) *pos = '\0';
+		else break;
+	}
+	return str;
+}
+
