@@ -125,7 +125,7 @@ typedef void (*oscap_consumer_func) (void *, void *);
 #define OSCAP_ITERATOR_FREE(n) void n##_iterator_free(OSCAP_ITERATOR(n) it) { oscap_iterator_free(ITERATOR_CAST(it)); }
 #define OSCAP_ITERATOR_RESET(n) void n##_iterator_reset(OSCAP_ITERATOR(n) it) { oscap_iterator_reset(ITERATOR_CAST(it)); }
 #define OSCAP_ITERATOR_DETACH(t,n) t n##_iterator_detach(OSCAP_ITERATOR(n) it) { return oscap_iterator_detach(ITERATOR_CAST(it)); }
-#define OSCAP_ITERATOR_GEN_T(t,n) OSCAP_ITERATOR_FWD(n) OSCAP_ITERATOR_HAS_MORE(n) OSCAP_ITERATOR_NEXT(t,n) OSCAP_ITERATOR_FREE(n)
+#define OSCAP_ITERATOR_GEN_T(t,n) OSCAP_ITERATOR_FWD(n) OSCAP_ITERATOR_HAS_MORE(n) OSCAP_ITERATOR_RESET(n) OSCAP_ITERATOR_NEXT(t,n) OSCAP_ITERATOR_FREE(n)
 #define OSCAP_ITERATOR_GEN(n) OSCAP_ITERATOR_GEN_T(struct n*,n)
 
 #define OSCAP_ITERATOR_REMOVE_T(t,n,destructor) \
@@ -327,12 +327,12 @@ const char *oscap_enum_to_string(const struct oscap_string_map *map, int val);
 char *oscap_strdup(const char *str);
 
 /**
- * Use strsep on string
- * We can't use strsep from string.h because of compatibility issues
- * @param str String we want to split
- * @param delim Delimiter of string parts
+ * Use strtol on string, if string is NULL, return NaN
+ * @param str String we want to duplicate
+ * @param endptr see man strtol
+ * @param base see man strtol
  */
-char *oscap_strsep(char **str, const char *delim);
+float oscap_strtol(const char *str, char **endptr, int base);
 
 /**
  * Split a string.
@@ -357,6 +357,8 @@ char *oscap_vsprintf(const char *fmt, va_list ap);
 char *oscap_sprintf(const char *fmt, ...);
 /// In a list of key-value pairs (odd indicies are keys, even values), find a value for given key
 const char *oscap_strlist_find_value(char ** const kvalues, const char *key);
+/// Right trim @a ch characters (modifies its first argument!)
+char *oscap_rtrim(char *str, char ch);
 
 // check pointer equality
 bool oscap_ptr_cmp(void *node1, void *node2);
