@@ -43,6 +43,13 @@
 #ifdef ENABLE_XCCDF
 #include <xccdf.h>
 #endif
+#ifdef ENABLE_CPE
+#include <cpedict.h>
+#include <cpeuri.h>
+#endif
+
+#define INVALID_DOCUMENT_MSG "oscap was unable to validate the XML document you provided.\n"\
+			     "Please ensure that the XML document is valid and well-formed, and try again."
 
 struct oscap_action;
 struct oscap_module;
@@ -98,29 +105,43 @@ struct cvss_metrics {
 };
 #endif
 
+#ifdef ENABLE_CPE
+struct cpe_action {
+	char * name;
+	char * dict;
+};
+#endif
+
 struct oscap_action {
-        int doctype;
         struct oscap_module *module;
+	/* files */
         char *f_xccdf;
         char *f_oval;
+        char **f_ovals;
+	char *f_syschar;
         char *f_results;
         char *f_report;
-        char *url_xccdf;
-        char *url_oval;
-        char **urls_oval;
+	char *f_variables;
+	/* others */
         char *profile;
-        char *file_version;
         char *show;
         char *format;
         const char *tmpl;
         char *id;
+        char *oval_template;
         int verbosity;
         int hide_profile_info;
 #ifdef ENABLE_CVSS
         struct cvss_metrics *cvss_metrics;
 #endif
-	bool force;
-	bool oval_results;
+#ifdef ENABLE_CPE
+	struct cpe_action * cpe_action;
+#endif
+        int doctype;
+	int force;
+	int validate;
+	int oval_results;
+	int export_variables;
 };
 
 int app_validate_xml(const struct oscap_action *action);
@@ -140,4 +161,7 @@ extern struct oscap_module OSCAP_CVSS_MODULE;
 #ifdef ENABLE_OVAL
 extern struct oscap_module OSCAP_OVAL_MODULE;
 #endif
+#ifdef ENABLE_CPE
+extern struct oscap_module OSCAP_CPE_MODULE;
+#endif
 
