@@ -42,7 +42,7 @@ static int app_evaluate_xccdf(const struct oscap_action *action);
 static int app_xccdf_resolve(const struct oscap_action *action);
 static bool getopt_xccdf(int argc, char **argv, struct oscap_action *action);
 static bool getopt_generate(int argc, char **argv, struct oscap_action *action);
-static int xccdf_gen_report(const char *infile, const char *id, const char *outfile, const char *show);
+static int xccdf_gen_report(const char *infile, const char *id, const char *outfile, const char *show, const char *oval_template);
 static int app_xccdf_xslt(const struct oscap_action *action);
 
 static struct oscap_module* XCCDF_SUBMODULES[];
@@ -86,9 +86,11 @@ static struct oscap_module XCCDF_EVAL = {
         "Options:\n"
         "   --profile <name>\r\t\t\t\t - The name of Profile to be evaluated.\n"
         "   --oval-results\r\t\t\t\t - Save OVAL results as well.\n"
-        "   --result-file <file>\r\t\t\t\t - Write XCCDF Results into file.\n"
-        "   --report-file <file>\r\t\t\t\t - Write HTML report into file.\n"
-                                 "\t\t\t\t   (--result-file must be also specified for this to work)\n",
+        "   --export-variables\r\t\t\t\t - Export OVAL external variables provided by XCCDF.\n"
+        "   --results <file>\r\t\t\t\t - Write XCCDF Results into file.\n"
+        "   --report <file>\r\t\t\t\t - Write HTML report into file.\n"
+        "   --skip-valid \r\t\t\t\t - Skip validation.\n"
+                                 "\t\t\t\t   (--results must be also specified for this to work)\n",
     .opt_parser = getopt_xccdf,
     .func = app_evaluate_xccdf
 };
@@ -231,22 +233,27 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 	struct xccdf_policy_model *policy_model = NULL;
         void **def_models = NULL;
         void **sessions = NULL;
+	char ** oval_files = NULL;
+	int idx = 0;
 	
 	/* Validate documents */
-        if (!oscap_validate_document(action->f_xccdf, OSCAP_DOCUMENT_XCCDF, NULL, 
-				     (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
-                if (oscap_err()) {
-                        fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-                        return OSCAP_FAIL;
-                }
-                return OSCAP_ERROR;
-        }
-
+	if( action->validate ) {
+        	if (!oscap_validate_document(action->f_xccdf, OSCAP_DOCUMENT_XCCDF, NULL, 
+					     (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+        	        if (oscap_err()) {
+        	                fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
+        	                return OSCAP_FAIL;
+        	        }
+			fprintf(stdout, "%s\n", INVALID_DOCUMENT_MSG);
+        	        return OSCAP_ERROR;
+        	}
+	}
 
 	/* Load XCCDF model and XCCDF Policy model */
 	benchmark = xccdf_benchmark_import(action->f_xccdf);
-        if(benchmark==NULL && oscap_err()) {
-                fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
+	if (benchmark == NULL) {
+		if (oscap_err())
+			fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
                 return OSCAP_ERROR;
         }
 
@@ -256,30 +263,37 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 	 * report error and return -1 */
 	if (action->profile != NULL) {
 		policy = xccdf_policy_model_get_policy_by_id(policy_model, action->profile);
+
+		if(policy == NULL) {
+                        fprintf(stderr, "Profile \"%s\" was not found.\n", action->profile);
+                        return OSCAP_ERROR;
+                }
 	} else {
 		policy_it = xccdf_policy_model_get_policies(policy_model);
 		if (xccdf_policy_iterator_has_more(policy_it)) {
 			policy = xccdf_policy_iterator_next(policy_it);
 		}
 		xccdf_policy_iterator_free(policy_it);
+
+		if (policy == NULL) {
+			fprintf(stderr, "No Policy to evaluate. \n");
+			return OSCAP_ERROR;
+		}
 	}
 
-	if (policy == NULL) {
-		fprintf(stderr, "No Policy to evaluate. \n");
-		return OSCAP_ERROR;
-	}
 
 	/* Register callback */
 	xccdf_policy_model_register_start_callback(policy_model, scallback, (void*) action);
 	xccdf_policy_model_register_output_callback(policy_model, callback, (void*) action);
 
 	/* NO OVAL files? get ones from policy model */
-	char ** oval_files = NULL;
-	if (action->urls_oval == NULL) {
+	if (action->f_ovals == NULL) {
 		struct stat sb;
-		int idx=0;
-		int files_cnt = 3;
-		oval_files = malloc(files_cnt * sizeof(char *));
+		char * tmp_path;
+		
+		idx = 0;
+		oval_files = malloc(sizeof(char *));
+		oval_files[idx] = NULL;
 
 		char * pathcopy =  strdup(action->f_xccdf);
 		char * path = dirname(pathcopy);
@@ -287,51 +301,53 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 		struct oscap_stringlist * files = xccdf_policy_model_get_files(policy_model);
 		struct oscap_string_iterator *files_it = oscap_stringlist_get_strings(files);
 		while (oscap_string_iterator_has_more(files_it)) {
-			if ( idx + 1 == files_cnt) {
-				files_cnt = files_cnt * 2;
-				oval_files = realloc(oval_files, files_cnt * sizeof(char *));
+			tmp_path = malloc(PATH_MAX * sizeof(char));
+			sprintf(tmp_path, "%s/%s", path, oscap_string_iterator_next(files_it));
+
+			if (stat(tmp_path, &sb)) {
+				fprintf(stderr, "WARNING: Skipping %s file which is referenced from XCCDF content\n", tmp_path);
+				free(tmp_path);
 			}
-			oval_files[idx] = malloc(PATH_MAX * sizeof(char));
-			sprintf(oval_files[idx], "%s/%s", path, oscap_string_iterator_next(files_it));
-			if (stat(oval_files[idx], &sb)) {
-				fprintf(stderr, "WARNING: Skipping %s file which is referenced from XCCDF content\n", oval_files[idx]);
-				free(oval_files[idx]);
+			else { 
+				oval_files[idx] = tmp_path;
+				idx++;
+				oval_files = realloc(oval_files, (idx + 1) * sizeof(char *));
+				oval_files[idx] = NULL;
 			}
-			else idx++; 
 		}
 		oscap_string_iterator_free(files_it);
 		oscap_stringlist_free(files);
 		free(pathcopy);
-		oval_files[idx] = NULL;
 	} else
-		oval_files = action->urls_oval;
+		oval_files = action->f_ovals;
 
 	
-	/* register oval sessions */
-        if (oval_files != NULL) {
-	    int i;
-	    /* Validate documents */
-            for (i=0; oval_files[i]; i++) {
-		if (!oscap_validate_document(oval_files[i], OSCAP_DOCUMENT_OVAL_DEFINITIONS, NULL, 
-					     (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
-			if (oscap_err()) {
-				fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
-				return OSCAP_FAIL;
+	/* Validate OVAL files */
+	if (action->validate) {
+        	for (idx=0; oval_files[idx]; idx++) {
+			if (!oscap_validate_document(oval_files[idx], OSCAP_DOCUMENT_OVAL_DEFINITIONS, NULL,
+						     (action->verbosity >= 0 ? oscap_reporter_fd : NULL), stdout)) {
+				if (oscap_err()) {
+					fprintf(stderr, "ERROR: %s\n", oscap_err_desc());
+					return OSCAP_FAIL;
+				}
+				fprintf(stdout, "%s\n", INVALID_DOCUMENT_MSG);
+				return OSCAP_ERROR;
 			}
-			return OSCAP_ERROR;
-		}
-	    }
-	    /* create sessions */
-            def_models = malloc((i+1) * sizeof(struct oval_definition_model *));
-            sessions = malloc((i+1) * sizeof(struct oval_agent_session *));
-            for (i=0; oval_files[i]; i++) {
-                struct oval_definition_model *tmp_def_model = oval_definition_model_import(oval_files[i]);
-		if(tmp_def_model==NULL && oscap_err()) {
-			fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
-			return OSCAP_ERROR;
 		}
+	}
 
-                struct oval_agent_session *tmp_sess = oval_agent_new_session(tmp_def_model, basename(oval_files[i]));
+	/* Create OVAL sessions */
+        def_models = malloc((idx+1) * sizeof(struct oval_definition_model *));
+        sessions = malloc((idx+1) * sizeof(struct oval_agent_session *));
+        for (idx=0; oval_files[idx]; idx++) {
+	        struct oval_definition_model *tmp_def_model = oval_definition_model_import(oval_files[idx]);
+		if (tmp_def_model == NULL) {
+			if (oscap_err())
+				fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
+			return OSCAP_ERROR;
+		}
+                struct oval_agent_session *tmp_sess = oval_agent_new_session(tmp_def_model, basename(oval_files[idx]));
 		if (tmp_sess == NULL) {
 			if (oscap_err())
 				fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
@@ -339,12 +355,11 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 			return OSCAP_ERROR;
 		}
 	        xccdf_policy_model_register_engine_oval(policy_model, tmp_sess);
-                def_models[i] = tmp_def_model;
-                sessions[i] = tmp_sess;
-            }
-	    def_models[i] = NULL;
-	    sessions[i] = NULL;
-        } 
+                def_models[idx] = tmp_def_model;
+                sessions[idx] = tmp_sess;
+	}
+	def_models[idx] = NULL;
+	sessions[idx] = NULL;
 	
 	
 	/* Perform evaluation */
@@ -352,7 +367,7 @@ int app_evaluate_xccdf(const struct oscap_action *action)
         if (ritem == NULL) return OSCAP_ERROR;
 
 	/* Write results into XCCDF Test Result model */
-	xccdf_result_set_benchmark_uri(ritem, action->url_xccdf);
+	xccdf_result_set_benchmark_uri(ritem, action->f_xccdf);
 	struct oscap_text *title = oscap_text_new();
 	oscap_text_set_text(title, "OSCAP Scan Result");
 	xccdf_result_add_title(ritem, title);
@@ -361,7 +376,10 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 		if (id != NULL)
 			xccdf_result_set_profile(ritem, id);
 	}
-	oval_agent_export_sysinfo_to_xccdf_result(sessions[0], ritem);
+
+	/* Use sysinfo from the first oval session, if exist */
+	if( sessions[0] )
+		oval_agent_export_sysinfo_to_xccdf_result(sessions[0], ritem);
 
 	struct xccdf_model_iterator *model_it = xccdf_benchmark_get_models(benchmark);
 	while (xccdf_model_iterator_has_more(model_it)) {
@@ -371,13 +389,6 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 	}
 	xccdf_model_iterator_free(model_it);
 
-	/* Export results */
-	if (action->f_results != NULL) {
-	        xccdf_benchmark_add_result(benchmark, xccdf_result_clone(ritem));
-       		xccdf_benchmark_export(benchmark, action->f_results);
-        	if (action->f_report != NULL)
-        		xccdf_gen_report(action->f_results, xccdf_result_get_id(ritem), action->f_report, "");
-        }
 	/* Export OVAL results */
 	if (action->oval_results == true) {
         	for (int i=0; sessions[i]; i++) {
@@ -408,6 +419,42 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 			free(name);
 		}
 	}
+	/* Export results */
+	if (action->f_results != NULL) {
+		xccdf_benchmark_add_result(benchmark, xccdf_result_clone(ritem));
+		xccdf_benchmark_export(benchmark, action->f_results);
+		if (action->f_report != NULL)
+			xccdf_gen_report(action->f_results, xccdf_result_get_id(ritem), action->f_report, "", (action->oval_results ? "%.result.xml" : ""));
+	}
+
+	/* Export variables */
+	if (action->export_variables) {
+		int i;
+
+		for (i = 0; sessions[i]; ++i) {
+			int j;
+			char *sname;
+			struct oval_results_model *resmod;
+			struct oval_definition_model *defmod;
+			struct oval_variable_model_iterator *varmod_itr;
+
+			sname = (char *) oval_agent_get_filename(sessions[i]);
+			resmod = oval_agent_get_results_model(sessions[i]);
+			defmod = oval_results_model_get_definition_model(resmod);
+
+			j = 0;
+			varmod_itr = oval_definition_model_get_variable_models(defmod);
+			while (oval_variable_model_iterator_has_more(varmod_itr)) {
+				char fname[strlen(sname) + 32];
+				struct oval_variable_model *varmod;
+
+				varmod = oval_variable_model_iterator_next(varmod_itr);
+				snprintf(fname, sizeof(fname), "%s-%d.variables-%d.xml", sname, i, j++);
+				oval_variable_model_export(varmod, fname);
+			}
+			oval_variable_model_iterator_free(varmod_itr);
+		}
+	}
 
 	/* Get the result from TestResult model and decide if end with error or with correct return code */
 	int retval = OSCAP_OK;
@@ -426,7 +473,7 @@ int app_evaluate_xccdf(const struct oscap_action *action)
         }
 	free(def_models);
         free(sessions);
-	if (oval_files != action->urls_oval) {
+	if (oval_files != action->f_ovals) {
 		for(int i=0; oval_files[i]; i++) {
 			free(oval_files[i]);
 		}
@@ -448,8 +495,9 @@ int app_xccdf_resolve(const struct oscap_action *action)
 	}
 
 	struct xccdf_benchmark *bench = xccdf_benchmark_import(action->f_xccdf);
-        if(bench==NULL && oscap_err()) {
-	        fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
+	if (bench == NULL) {
+		if (oscap_err())
+			fprintf(stderr, "Error: (%d) %s\n", oscap_err_code(), oscap_err_desc());
 	        return OSCAP_ERROR;
 	}
 
@@ -471,10 +519,10 @@ int app_xccdf_resolve(const struct oscap_action *action)
 	return ret;
 }
 
-static int xccdf_gen_report(const char *infile, const char *id, const char *outfile, const char *show)
+static int xccdf_gen_report(const char *infile, const char *id, const char *outfile, const char *show, const char *oval_template)
 {
-    const char *params[] = { "result-id", id, "show", show, "verbosity", "", NULL };
-    return app_xslt(infile, "xccdf-results-report.xsl", outfile, params);
+    const char *params[] = { "result-id", id, "show", show, "verbosity", "", "oval-template", oval_template, NULL };
+    return app_xslt(infile, "xccdf-report.xsl", outfile, params);
 }
 
 int app_xccdf_xslt(const struct oscap_action *action)
@@ -486,6 +534,7 @@ int app_xccdf_xslt(const struct oscap_action *action)
         "profile",           action->profile,
         "template",          action->tmpl,
         "format",            action->format,
+        "oval-template",     action->oval_template,
         "verbosity",         action->verbosity >= 0 ? "1" : "",
         "hide-profile-info", action->hide_profile_info ? "yes" : NULL,
         NULL };
@@ -511,43 +560,60 @@ bool getopt_generate(int argc, char **argv, struct oscap_action *action)
     return true;
 }
 
+enum oval_opt {
+    XCCDF_OPT_RESULT_FILE = 1,
+    XCCDF_OPT_PROFILE,
+    XCCDF_OPT_REPORT_FILE,
+    XCCDF_OPT_SHOW,
+    XCCDF_OPT_TEMPLATE,
+    XCCDF_OPT_FORMAT,
+    XCCDF_OPT_OVAL_TEMPLATE,
+    XCCDF_OPT_FILE_VERSION,
+    XCCDF_OPT_OUTPUT = 'o',
+    XCCDF_OPT_RESULT_ID = 'i'
+};
+
 bool getopt_xccdf(int argc, char **argv, struct oscap_action *action)
 {
-    assert(action != NULL);
+	assert(action != NULL);
 
 	action->doctype = OSCAP_DOCUMENT_XCCDF;
 
 	/* Command-options */
 	const struct option long_options[] = {
-		{"force", 0, 0, 'f'},
-		{"output", 1, 0, 'o'},
-		{"result-id", 1, 0, 'i'},
-		{"result-file", 1, 0, 0},
-		{"xccdf-profile", 1, 0, 1},
-		{"file-version", 1, 0, 2},
-		{"profile", 1, 0, 3},
-		{"report-file", 1, 0, 4},
-		{"oval-results", 0, 0, 5},
-		{"show", 1, 0, 6},
-		{"template", 1, 0, 7},
-		{"format", 1, 0, 8},
-		{"hide-profile-info", 0, &action->hide_profile_info, 1},
+	// options
+		{"output",		required_argument, NULL, XCCDF_OPT_OUTPUT},
+		{"results", 		required_argument, NULL, XCCDF_OPT_RESULT_FILE},
+		{"profile", 		required_argument, NULL, XCCDF_OPT_PROFILE},
+		{"result-id",		required_argument, NULL, XCCDF_OPT_RESULT_ID},
+		{"report", 		required_argument, NULL, XCCDF_OPT_REPORT_FILE},
+		{"show", 		required_argument, NULL, XCCDF_OPT_SHOW},
+		{"template", 		required_argument, NULL, XCCDF_OPT_TEMPLATE},
+		{"format", 		required_argument, NULL, XCCDF_OPT_FORMAT},
+		{"oval-template", 	required_argument, NULL, XCCDF_OPT_OVAL_TEMPLATE},
+	// flags
+		{"force",		no_argument, &action->force, 1},
+		{"oval-results",	no_argument, &action->oval_results, 1},
+		{"skip-valid",		no_argument, &action->validate, 0},
+		{"hide-profile-info",	no_argument, &action->hide_profile_info, 1},
+		{"export-variables",	no_argument, &action->export_variables, 1},
+	// end
 		{0, 0, 0, 0}
 	};
 
 	int c;
-	while ((c = getopt_long(argc, argv, "o:i:f", long_options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "", long_options, NULL)) != -1) {
 		switch (c) {
-		case 'o': case 0: action->f_results = optarg; break;
-		case 3: case 1: action->profile = optarg; break;
-        case 'i': action->id = optarg; break;
-		case 2: action->file_version = optarg; break;
-     		case 4: action->f_report = optarg; break;
-		case 'f': action->force = true; break;
-		case 5: action->oval_results = true; break;
-		case 6: action->show = optarg; break;
-		case 7: action->tmpl = optarg; break;
-		case 8: action->format = optarg; break;
+		case XCCDF_OPT_OUTPUT: 
+		case XCCDF_OPT_RESULT_FILE:	action->f_results = optarg;	break;
+		case XCCDF_OPT_PROFILE:		action->profile = optarg;	break;
+		case XCCDF_OPT_RESULT_ID:	action->id = optarg;		break;
+		case XCCDF_OPT_REPORT_FILE:	action->f_report = optarg; 	break;
+		case XCCDF_OPT_SHOW:		action->show = optarg;		break;
+		case XCCDF_OPT_TEMPLATE:	action->tmpl = optarg;		break;
+		case XCCDF_OPT_FORMAT:		action->format = optarg;	break;
+		case XCCDF_OPT_OVAL_TEMPLATE:	action->oval_template = optarg; break;
+		case 0: break;
 		default: return oscap_module_usage(action->module, stderr, NULL);
 		}
 	}
@@ -559,24 +625,24 @@ bool getopt_xccdf(int argc, char **argv, struct oscap_action *action)
 			return oscap_module_usage(action->module, stderr, "XCCDF file need to be specified!");
 		}
                 if (action->f_report && !action->f_results)
-                    return oscap_module_usage(action->module, stderr, "Please specify --result-file if you want to generate a HTML report.");
+                    return oscap_module_usage(action->module, stderr, "Please specify --result if you want to generate a HTML report.");
 
-                action->url_xccdf = argv[optind];
+                action->f_xccdf = argv[optind];
                 if (argc > (optind+1)) {
-                    action->urls_oval = malloc((argc-(optind+1)+1) * sizeof(char *));
+                    action->f_ovals = malloc((argc-(optind+1)+1) * sizeof(char *));
                     int i = 1;
                     while (argc > (optind+i)) {
-                        action->urls_oval[i-1] = argv[optind + i];
+                        action->f_ovals[i-1] = argv[optind + i];
                         i++;
                     }
-                    action->urls_oval[i-1] = NULL;
+                    action->f_ovals[i-1] = NULL;
                 } else {
-                    action->urls_oval = NULL;
+                    action->f_ovals = NULL;
                 }
 	} else {
 		if (optind >= argc)
 			return oscap_module_usage(action->module, stderr, "XCCDF file needs to be specified!");
-		action->url_xccdf = argv[optind];
+		action->f_xccdf = argv[optind];
 	}
 
 	return true;
